@@ -435,7 +435,7 @@ vgzZAxisSearch(int id, void *res, void *data)
   struct vgzSearchState *state = (struct vgzSearchState *)data;
   (void)res;
   if (zaxisCompare(id, state->zaxistype, state->nlevels, state->lbounds,
-                   state->levels, NULL, NULL, 0)
+                   state->levels, NULL, NULL, 0, -1)
       == false)
     {
       state->resIDValue = id;
@@ -463,7 +463,7 @@ int vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, const double *
     {
       zaxisID = vlistptr->zaxisIDs[index];
 
-      if ( zaxisCompare(zaxisID, zaxistype, nlevels, has_bounds, levels, NULL, NULL, 0) == false )
+      if ( zaxisCompare(zaxisID, zaxistype, nlevels, has_bounds, levels, NULL, NULL, 0, -1) == false )
         {
           zaxisdefined = true;
           break;
@@ -591,8 +591,6 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
 	      {
 		int nvct = 0;
                 double *levels = NULL;
-                char **cvals1 = NULL, **cvals2 = NULL;
-                size_t clen2 = 0;
 		double *lbounds = NULL, *ubounds = NULL;
 		const double *vct = NULL;
                 char ctemp[CDI_MAX_NAME];
@@ -627,9 +625,12 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                     vct  = zaxisInqVctPtr(zaxisID);
                   }
 
+                size_t clen2 = 0;
+                char **cvals2 = NULL;
+#ifndef USE_MPI
                 if ( zaxisType == ZAXIS_CHAR )
                   {
-                    cvals1 = zaxisInqCValsPtr(zaxisID);
+                    char **cvals1 = zaxisInqCValsPtr(zaxisID);
                     size_t clen1 = (size_t)zaxisInqCLen(zaxisID);
                     for ( int levID = 0; levID < nlevs; ++levID )
                       if ( vars1[varID].levinfo[levID].flag )
@@ -646,11 +647,12 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                     for ( int levID = 0; levID < nlevs; ++levID )
                       if ( vars1[varID].levinfo[levID].flag )
                         {
-                          cvals2[levID2] = Malloc((size_t)(clen2) * sizeof (char));
+                          cvals2[levID2] = (char*) Malloc((size_t)(clen2) * sizeof(char));
                           memcpy(cvals2[levID2], cvals1[levID], clen2*sizeof(char));
                           levID2++;
                         }
                   }
+#endif
 
                 if ( zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL) )
                   {
@@ -692,6 +694,7 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                 zaxisInqUnits(zaxisID, ctemp);
                 zaxisDefUnits(zaxisID2, ctemp);
                 zaxisDefDatatype(zaxisID2, zaxisInqDatatype(zaxisID));
+                zaxisDefPositive(zaxisID2, zaxisInqPositive(zaxisID));
 
                 if ( zaxisType == ZAXIS_CHAR )
                   {
@@ -703,6 +706,8 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
 
                 if ( zaxisType == ZAXIS_GENERIC ) zaxisDefLtype(zaxisID2, zaxisInqLtype(zaxisID));
 
+                cdiCopyAtts(zaxisID, CDI_GLOBAL, zaxisID2, CDI_GLOBAL);
+
 		zaxisID = zaxisID2;
 		vars2[varID2].zaxisID = zaxisID2;
 	      }
@@ -1610,8 +1615,6 @@ void resize_opt_grib_entries(var_t *var, int nentries)
 {
   if (var->opt_grib_kvpair_size >= nentries)
     {
-      if ( CDI_Debug )
-        Message("data structure has size %d, no resize to %d needed.", var->opt_grib_kvpair_size, nentries);
       return;   /* nothing to do; array is still large enough */
     }
   else
