@@ -1,7 +1,3 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
-#endif
-
 #include <string.h>
 #include <math.h>
 #include <float.h>
@@ -14,7 +10,6 @@
 #include "cdi_uuid.h"
 #include "resource_handle.h"
 #include "resource_unpack.h"
-#include "varscan.h"
 #include "namespace.h"
 #include "serialize.h"
 #include "zaxis.h"
@@ -128,8 +123,10 @@ void zaxis_init(zaxis_t *zaxisptr)
   zaxisptr->p0name[0]     = 0;
   zaxisptr->p0value.defined = false;
   zaxisptr->vals          = NULL;
+#ifndef USE_MPI
   zaxisptr->cvals         = NULL;
   zaxisptr->clength       = 0;
+#endif
   zaxisptr->ubounds       = NULL;
   zaxisptr->lbounds       = NULL;
   zaxisptr->weights       = NULL;
@@ -181,7 +178,7 @@ void zaxisInit(void)
 static
 void zaxis_copy(zaxis_t *zaxisptr2, zaxis_t *zaxisptr1)
 {
-  int zaxisID2 = zaxisptr2->self;
+  const int zaxisID2 = zaxisptr2->self;
   memcpy(zaxisptr2, zaxisptr1, sizeof(zaxis_t));
   zaxisptr2->self = zaxisID2;
 }
@@ -204,7 +201,7 @@ int zaxisCreate_(int zaxistype, int size, int id)
   if ( zaxistype >= CDI_NumZaxistype || zaxistype < 0 )
     Error("Internal problem! zaxistype > CDI_MaxZaxistype");
 
-  int zaxisID = zaxisptr->self;
+  const int zaxisID = zaxisptr->self;
   zaxisDefName(zaxisID, ZaxistypeEntry[zaxistype].name);
   if ( zaxistype != ZAXIS_GENERIC ) zaxisDefLongname(zaxisID, ZaxistypeEntry[zaxistype].longname);
   zaxisDefUnits(zaxisID, ZaxistypeEntry[zaxistype].units);
@@ -261,6 +258,7 @@ int zaxisCreate(int zaxistype, int size)
 {
   if ( CDI_Debug ) Message("zaxistype: %d size: %d ", zaxistype, size);
 
+  xassert(size);
   zaxisInit();
 
   return zaxisCreate_(zaxistype, size, CDI_UNDEFID);
@@ -271,15 +269,17 @@ void zaxisDestroyKernel( zaxis_t * zaxisptr )
 {
   xassert ( zaxisptr );
 
-  int id = zaxisptr->self;
+  const int id = zaxisptr->self;
 
-  if ( zaxisptr->vals )    Free( zaxisptr->vals );
+  if ( zaxisptr->vals ) Free( zaxisptr->vals );
+#ifndef USE_MPI
   if ( zaxisptr->cvals )
     {
       for ( int i=0; i<zaxisptr->size; i++)
         Free(zaxisptr->cvals[i]);
       Free( zaxisptr->cvals );
     }
+#endif
   if ( zaxisptr->lbounds ) Free( zaxisptr->lbounds );
   if ( zaxisptr->ubounds ) Free( zaxisptr->ubounds );
   if ( zaxisptr->weights ) Free( zaxisptr->weights );
@@ -332,17 +332,17 @@ static inline
 void zaxisSetString(char *zaxisstrname, const char *name, size_t len)
 {
   if ( len > CDI_MAX_NAME ) len = CDI_MAX_NAME;
-  strncpy(zaxisstrname, name, len);
+  strncpy(zaxisstrname, name, len - 1);
   zaxisstrname[len-1] = 0;
 }
 
 static inline
 void zaxisGetString(char *name, const char *zaxisstrname, size_t len)
 {
-  size_t slen = strlen(zaxisstrname)+1;
+  size_t slen = strlen(zaxisstrname) + 1;
   if ( slen > len ) slen = len;
   if ( slen > CDI_MAX_NAME ) slen = CDI_MAX_NAME;
-  strncpy(name, zaxisstrname, slen);
+  strncpy(name, zaxisstrname, slen - 1);
   name[slen-1] = 0;
 }
 
@@ -765,12 +765,13 @@ The function @func{zaxisDefLevels} defines the levels of a Z-axis.
 */
 void zaxisDefLevels(int zaxisID, const double *levels)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  size_t size = (size_t)zaxisptr->size;
-
   if ( levels )
     {
-      if ( zaxisptr->vals == NULL )
+      zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
+      const size_t size = (size_t)zaxisptr->size;
+      xassert(size);
+
+      if (zaxisptr->vals == NULL && size)
         zaxisptr->vals = (double*) Malloc(size*sizeof(double));
 
       double *vals = zaxisptr->vals;
@@ -785,21 +786,26 @@ void zaxisDefLevels(int zaxisID, const double *levels)
 
 void zaxisDefCvals(int zaxisID, const char **cvals, int clen)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  int size = zaxisptr->size;
-
+#ifndef USE_MPI
   if ( cvals && clen )
     {
-      zaxisptr->clength = clen;
-      zaxisptr->cvals = (char**) Malloc((size_t)size*sizeof(char *));
+      zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
+      const size_t size = zaxisptr->size;
+      xassert(size);
 
-      for ( int ilev = 0; ilev < size; ++ilev )
+      zaxisptr->clength = clen;
+      if (size) zaxisptr->cvals = (char**) Malloc(size*sizeof(char *));
+
+      for ( size_t ilev = 0; ilev < size; ++ilev )
         {
-          zaxisptr->cvals[ilev] = Malloc((size_t)clen*sizeof(char));
-          memcpy(zaxisptr->cvals[ilev],cvals[ilev], (size_t)clen*sizeof(char));
+          zaxisptr->cvals[ilev] = (char*) Malloc(clen*sizeof(char));
+          memcpy(zaxisptr->cvals[ilev], cvals[ilev], clen*sizeof(char));
         }
       reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
     }
+#else
+  Error("This function was disabled!");
+#endif
 }
 
 /*
@@ -820,9 +826,11 @@ The function @func{zaxisDefLevel} defines one level of a Z-axis.
 void zaxisDefLevel(int zaxisID, int levelID, double level)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  int size = zaxisptr->size;
+  const int size = zaxisptr->size;
+  xassert(size);
+  xassert(levelID >= 0 && levelID < size);
 
-  if ( zaxisptr->vals == NULL )
+  if (zaxisptr->vals == NULL && size)
     zaxisptr->vals = (double*) Malloc((size_t)size*sizeof(double));
 
   if ( levelID >= 0 && levelID < size )
@@ -955,9 +963,9 @@ The function @func{zaxisInqLevel} returns one level of a Z-axis.
 */
 double zaxisInqLevel(int zaxisID, int levelID)
 {
-  double level = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  double level = 0;
   if ( zaxisptr->vals && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->vals[levelID];
 
@@ -967,9 +975,9 @@ double zaxisInqLevel(int zaxisID, int levelID)
 
 double zaxisInqLbound(int zaxisID, int levelID)
 {
-  double level = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  double level = 0;
   if ( zaxisptr->lbounds && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->lbounds[levelID];
 
@@ -979,9 +987,9 @@ double zaxisInqLbound(int zaxisID, int levelID)
 
 double zaxisInqUbound(int zaxisID, int levelID)
 {
-  double level = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  double level = 0;
   if ( zaxisptr->ubounds && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->ubounds[levelID];
 
@@ -996,13 +1004,13 @@ const double *zaxisInqLevelsPtr(int zaxisID)
 }
 
 
+#ifndef USE_MPI
 char **zaxisInqCValsPtr(int zaxisID)
 {
-  char **cvals = NULL;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  cvals = zaxisptr->cvals;
-  return cvals;
+  return zaxisptr->cvals;
 }
+#endif
 
 /*
 @Function  zaxisInqLevels
@@ -1023,9 +1031,9 @@ The function @func{zaxisInqLevels} returns all levels of a Z-axis.
 */
 int zaxisInqLevels(int zaxisID, double *levels)
 {
-  int size = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int size = 0;
   if ( zaxisptr->vals )
     {
       size = zaxisptr->size;
@@ -1040,34 +1048,38 @@ int zaxisInqLevels(int zaxisID, double *levels)
 
 int zaxisInqCLen(int zaxisID)
 {
-  int size = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int clen = 0;
+#ifndef USE_MPI
   if ( zaxisptr->cvals && zaxisptr->clength)
-    size = zaxisptr->clength;
-  return size;
+    clen = zaxisptr->clength;
+#endif
+
+  return clen;
 }
 
 int zaxisInqCVals(int zaxisID, char ***clevels)
 {
-  int size = 0;
-  size_t clen = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int size = 0;
+#ifndef USE_MPI
   if ( zaxisptr->cvals )
     {
       size = zaxisptr->size;
-      clen = zaxisptr->clength;
+      const size_t clen = zaxisptr->clength;
       if ( size && clen )
         {
-          (*clevels) = Malloc(size*sizeof(char*));
+          (*clevels) = (char**) Malloc(size*sizeof(char*));
           for ( int i = 0; i < size; i++ )
             {
-              (*clevels)[i] = Malloc(clen*sizeof(char));
+              (*clevels)[i] = (char*) Malloc(clen*sizeof(char));
               memcpy((*clevels)[i], zaxisptr->cvals[i], clen*sizeof(char));
             }
           }
     }
+#endif
 
   return size;
 }
@@ -1075,9 +1087,9 @@ int zaxisInqCVals(int zaxisID, char ***clevels)
 
 int zaxisInqLbounds(int zaxisID, double *lbounds)
 {
-  int size = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int size = 0;
   if ( zaxisptr->lbounds )
     {
       size = zaxisptr->size;
@@ -1093,9 +1105,9 @@ int zaxisInqLbounds(int zaxisID, double *lbounds)
 
 int zaxisInqUbounds(int zaxisID, double *ubounds)
 {
-  int size = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int size = 0;
   if ( zaxisptr->ubounds )
     {
       size = zaxisptr->size;
@@ -1111,9 +1123,9 @@ int zaxisInqUbounds(int zaxisID, double *ubounds)
 
 int zaxisInqWeights(int zaxisID, double *weights)
 {
-  int size = 0;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int size = 0;
   if ( zaxisptr->weights )
     {
       size = zaxisptr->size;
@@ -1129,13 +1141,12 @@ int zaxisInqWeights(int zaxisID, double *weights)
 
 int zaxisInqLevelID(int zaxisID, double level)
 {
-  int levelID = CDI_UNDEFID;
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
+  int levelID = CDI_UNDEFID;
   if ( zaxisptr->vals )
     {
-      int size = zaxisptr->size;
-
+      const int size = zaxisptr->size;
       for ( int i = 0; i < size; i++ )
         if ( fabs(level-zaxisptr->vals[i]) < DBL_EPSILON )
           {
@@ -1207,7 +1218,7 @@ void cdiCheckZaxis(int zaxisID)
 
   if ( zaxisInqType(zaxisID) == ZAXIS_GENERIC && zaxisptr->vals )
     {
-      int size = zaxisptr->size;
+      const int size = zaxisptr->size;
       if ( size > 1 )
         {
           /* check direction */
@@ -1277,7 +1288,7 @@ void zaxisDefLbounds(int zaxisID, const double *lbounds)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
-  size_t size = (size_t)zaxisptr->size;
+  const size_t size = (size_t)zaxisptr->size;
 
   if ( CDI_Debug )
     if ( zaxisptr->lbounds != NULL )
@@ -1295,7 +1306,7 @@ void zaxisDefUbounds(int zaxisID, const double *ubounds)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
-  size_t size = (size_t)zaxisptr->size;
+  const size_t size = (size_t)zaxisptr->size;
 
   if ( CDI_Debug )
     if ( zaxisptr->ubounds != NULL )
@@ -1313,7 +1324,7 @@ void zaxisDefWeights(int zaxisID, const double *weights)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
-  size_t size = (size_t)zaxisptr->size;
+  const size_t size = (size_t)zaxisptr->size;
 
   if ( CDI_Debug )
     if ( zaxisptr->weights != NULL )
@@ -1351,10 +1362,10 @@ int zaxisDuplicate(int zaxisID)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
 
-  int zaxistype = zaxisInqType(zaxisID);
-  int zaxissize = zaxisInqSize(zaxisID);
+  const int zaxistype = zaxisInqType(zaxisID);
+  const int zaxissize = zaxisInqSize(zaxisID);
 
-  int zaxisIDnew = zaxisCreate(zaxistype, zaxissize);
+  const int zaxisIDnew = zaxisCreate(zaxistype, zaxissize);
   zaxis_t *zaxisptrnew = zaxis_to_pointer(zaxisIDnew);
 
   zaxis_copy(zaxisptrnew, zaxisptr);
@@ -1365,28 +1376,28 @@ int zaxisDuplicate(int zaxisID)
 
   if ( zaxisptr->vals )
     {
-      size_t size = (size_t)zaxissize;
+      const size_t size = (size_t)zaxissize;
       zaxisptrnew->vals = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->vals, zaxisptr->vals, size * sizeof (double));
     }
 
   if ( zaxisptr->lbounds )
     {
-      size_t size = (size_t)zaxissize;
+      const size_t size = (size_t)zaxissize;
       zaxisptrnew->lbounds = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->lbounds, zaxisptr->lbounds, size * sizeof(double));
     }
 
   if ( zaxisptr->ubounds )
     {
-      size_t size = (size_t)zaxissize;
+      const size_t size = (size_t)zaxissize;
       zaxisptrnew->ubounds = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->ubounds, zaxisptr->ubounds, size * sizeof (double));
     }
 
   if ( zaxisptr->vct )
     {
-      size_t size = (size_t)zaxisptr->vctsize;
+      const size_t size = (size_t)zaxisptr->vctsize;
       if ( size )
         {
           zaxisptrnew->vctsize = (int)size;
@@ -1395,6 +1406,8 @@ int zaxisDuplicate(int zaxisID)
         }
     }
 
+  cdiCopyAtts(zaxisID, CDI_GLOBAL, zaxisIDnew, CDI_GLOBAL);
+
   return zaxisIDnew;
 }
 
@@ -1403,12 +1416,11 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
 {
   xassert(zaxisptr);
 
-  int zaxisID = zaxisptr->self;
-  int type    = zaxisptr->type;
-  int nlevels = zaxisptr->size;
-  int datatype = zaxisptr->datatype;
+  const int type    = zaxisptr->type;
+  const int nlevels = zaxisptr->size;
+  const int datatype = zaxisptr->datatype;
 
-  int dig = (datatype == CDI_DATATYPE_FLT64) ? 15 : 7;
+  const int dig = (datatype == CDI_DATATYPE_FLT64) ? 15 : 7;
 
   int nbyte;
   int nbyte0 = 0;
@@ -1416,7 +1428,7 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
   fprintf(fp, "size      = %d\n", nlevels);
   if ( nlevels == 1 )
     {
-      bool zscalar = (bool)zaxisptr->scalar;
+      const bool zscalar = (bool)zaxisptr->scalar;
       if ( zscalar ) fprintf(fp, "scalar    = true\n");
     }
   if ( zaxisptr->name[0]     ) fprintf(fp, "name      = %s\n", zaxisptr->name);
@@ -1440,21 +1452,6 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
       fprintf(fp, "\n");
     }
 
-  if ( zaxisptr->cvals )
-    {
-      dig = datatype;
-      nbyte0 = fprintf(fp, "types     = ");
-      nbyte = nbyte0;
-      for ( int levelID = 0; levelID < nlevels; levelID++ )
-        {
-          fprintf(fp, "\n");
-          fprintf(fp, "%*s", nbyte0, "");
-          nbyte = nbyte0;
-          nbyte += fprintf(fp, "%.*s [%d]", dig, zaxisptr->cvals[levelID], levelID+1);
-        }
-      fprintf(fp, "\n");
-    }
-
   if ( zaxisptr->lbounds && zaxisptr->ubounds )
     {
       nbyte0 = fprintf(fp, "lbounds   = ");
@@ -1488,7 +1485,7 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
 
   if ( type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF )
     {
-      int vctsize = zaxisptr->vctsize;
+      const int vctsize = zaxisptr->vctsize;
       const double *vct = zaxisptr->vct;
       fprintf(fp, "vctsize   = %d\n", vctsize);
       if ( vctsize )
@@ -1505,42 +1502,8 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
               nbyte += fprintf(fp, "%.15g ", vct[i]);
             }
           fprintf(fp, "\n");
-          /*
-          nbyte0 = fprintf(fp, "vct_b     = ");
-          nbyte  = nbyte0;
-          for ( i = 0; i < vctsize/2; i++ )
-            {
-              if ( nbyte > 70 )
-                {
-                  fprintf(fp, "\n%*s", nbyte0, "");
-                  nbyte = nbyte0;
-                }
-              nbyte += fprintf(fp, "%.15g ", vct[vctsize/2+i]);
-            }
-          fprintf(fp, "\n");
-          */
         }
     }
-
-  if ( type == ZAXIS_REFERENCE )
-    {
-      unsigned char uuid[CDI_UUID_SIZE];
-      zaxisInqUUID(zaxisID, uuid);
-      if ( *uuid )
-        {
-          const unsigned char *d = uuid;
-          fprintf(fp, "uuid      = %02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x\n",
-                  d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7],
-                  d[8], d[9], d[10], d[11], d[12], d[13], d[14], d[15]);
-        }
-    }
-}
-
-
-void zaxisPrint(int zaxisID)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  zaxisPrintKernel(zaxisptr, stdout);
 }
 
 
