@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,13 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "dmemory.h"
+#include "functs.h"
+#include "process_int.h"
+#include "griddes.h"
 #include "commandline.h"
+#include "cdo_cdiWrapper.h"
+#include "cdo_defaultValues.h"
 
 #ifdef HAVE_LIBNETCDF
 #include "netcdf.h"
@@ -79,9 +83,9 @@ inivar(VAR *var, int gridtype, int zaxistype, int code, const char *name, const
   var->gridtype = gridtype;
   var->zaxistype = zaxistype;
   var->code = code;
-  var->name = NULL;
-  var->longname = NULL;
-  var->units = NULL;
+  var->name = nullptr;
+  var->longname = nullptr;
+  var->units = nullptr;
   if (name) var->name = strdup(name);
   if (longname) var->longname = strdup(longname);
   if (units) var->units = strdup(units);
@@ -123,15 +127,14 @@ import_e5ml(const char *filename, VAR **vars)
   size_t count[3];
   int nlon, nlat, nlev, nlevp1, nvct, nsp, i, iv;
   int gridIDgp, gridIDsp, zaxisIDml, zaxisIDsfc;
-  int gridtype, zaxistype;
+  int gridtype;
   int nc_file_id;
   char filetype[256];
   size_t attlen;
   double *xvals, *yvals, *vct, *levs;
 
-  /* open file and check file type */
-  /* nce(nc_open(filename, NC_NOWRITE, &nc_file_id)); */
-  nc_file_id = cdf_openread(filename);
+  // open file and check file type
+  nc_file_id = cdo_cdf_openread(filename);
 
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "file_type", filetype));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "file_type", &attlen));
@@ -209,9 +212,6 @@ import_e5ml(const char *filename, VAR **vars)
       nvals = 0;
 
       gridtype = (*vars)[iv].gridtype;
-      zaxistype = (*vars)[iv].zaxistype;
-
-      UNUSED(zaxistype);
 
       if (gridtype == GRID_GAUSSIAN)
         {
@@ -275,8 +275,8 @@ import_e5ml(const char *filename, VAR **vars)
   nce(nc_inq_varid(nc_file_id, "STP", &nc_var_id));
   nce(nc_get_vara_double(nc_file_id, nc_var_id, start, count, (*vars)[nvars_ml].ptr));
 
-  /*close input file */
-  nce(nc_close(nc_file_id));
+  // close input file
+  cdo_cdf_close(nc_file_id);
 
   nvars = nvars_ml + 1;
 
@@ -300,7 +300,7 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
   int lon, lat;
   int nlon, nlat, nlev, nlevp1 /*, nvct*/, nsp, n2, i, nvclev;
   int lat_dimid, lon_dimid, nlev_dimid, nlevp1_dimid, nsp_dimid, nvclev_dimid, n2_dimid;
-  int gridIDgp = -1, gridIDsp, zaxisIDml = -1;
+  int gridIDgp = -1, zaxisIDml = -1;
   int gridtype, zaxistype;
   int nc_file_id;
   int nc_stpid, lspid;
@@ -316,7 +316,7 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
   int writemode = NC_CLOBBER;
   unsigned long data_size;
 
-  date_and_time_in_sec = time(NULL);
+  date_and_time_in_sec = time(nullptr);
   timestr[0] = 0;
 
   if (date_and_time_in_sec != -1)
@@ -326,10 +326,10 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
     }
 
   username = getenv("LOGNAME");
-  if (username == NULL)
+  if (username == nullptr)
     {
       username = getenv("USER");
-      if (username == NULL) username = "unknown";
+      if (username == nullptr) username = "unknown";
     }
 
   n2 = 2;
@@ -353,8 +353,6 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
         }
       else if (gridtype == GRID_SPECTRAL && nsp == 0)
         {
-          gridIDsp = vars[varid].gridID;
-          UNUSED(gridIDsp);
           nsp = gridInqSize(vars[varid].gridID);
           nsp = nsp / 2;
         }
@@ -647,7 +645,7 @@ read_gg3d(int nc_file_id, const char *name, VAR *var, int gridID, int zaxisID)
   gridtype = gridInqType(gridID);
   zaxistype = zaxisInqType(zaxisID);
 
-  inivar(var, gridtype, zaxistype, 0, name, NULL, NULL);
+  inivar(var, gridtype, zaxistype, 0, name, nullptr, nullptr);
 
   nlon = gridInqXsize(gridID);
   nlat = gridInqYsize(gridID);
@@ -689,7 +687,7 @@ read_fc4d(int nc_file_id, const char *name, VAR *var, int gridID, int zaxisID, i
   gridtype = gridInqType(gridID);
   zaxistype = zaxisInqType(zaxisID);
 
-  inivar(var, gridtype, zaxistype, 0, name, NULL, NULL);
+  inivar(var, gridtype, zaxistype, 0, name, nullptr, nullptr);
 
   nfc = gridInqSize(gridID);
   nlev = zaxisInqSize(zaxisID);
@@ -743,8 +741,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
   int lon_dimid, lat_dimid, nhgl_dimid, nlevp1_dimid, spc_dimid, nvclev_dimid;
   int complex_dimid, nmp1_dimid, belowsurface_dimid, lev_dimid, ilev_dimid;
   int /* surface_dimid, height2m_dimid, height10m_dimid,*/ n2_dimid;
-  int lon, lat, nhgl, nlevp1, spc, nvclev;
-  int complex, nmp1, belowsurface, lev, ilev;
+  int lon, lat, nhgl, nlevp1, nvclev;
+  int complex, nmp1, belowsurface, lev ;
   int /* surface, height2m, height10m,*/ n2;
   int iatt;
   int attint;
@@ -755,9 +753,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
   const int attstringlen = 8192;
   char attstring[8192];
 
-  /* open file and check file type */
-  /* nce(nc_open(filename, NC_NOWRITE, &nc_file_id)); */
-  nc_file_id = cdf_openread(filename);
+  // open file and check file type
+  nc_file_id = cdo_cdf_openread(filename);
 
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "file_type", filetype));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "file_type", &attlen));
@@ -785,9 +782,7 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
 
   nce(nc_inq_dimid(nc_file_id, "spc", &spc_dimid));
   nce(nc_inq_dimlen(nc_file_id, spc_dimid, &dimlen));
-  spc = (int) dimlen;
 
-  UNUSED(spc);
 
   nce(nc_inq_dimid(nc_file_id, "nvclev", &nvclev_dimid));
   nce(nc_inq_dimlen(nc_file_id, nvclev_dimid, &dimlen));
@@ -811,9 +806,7 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
 
   nce(nc_inq_dimid(nc_file_id, "ilev", &ilev_dimid));
   nce(nc_inq_dimlen(nc_file_id, ilev_dimid, &dimlen));
-  ilev = (int) dimlen;
 
-  UNUSED(ilev);
   /*
   nce(nc_inq_dimid(nc_file_id, "surface", &surface_dimid));
   nce(nc_inq_dimlen(nc_file_id, surface_dimid, &surface));
@@ -1029,7 +1022,7 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
             {
               nvals = nlon * nlat;
 
-              inivar(&(*vars)[varid], GRID_GAUSSIAN, ZAXIS_SURFACE, 0, name, NULL, NULL);
+              inivar(&(*vars)[varid], GRID_GAUSSIAN, ZAXIS_SURFACE, 0, name, nullptr, nullptr);
 
               (*vars)[varid].gridID = gridIDgp;
               (*vars)[varid].zaxisID = zaxisIDsfc;
@@ -1047,7 +1040,7 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
             {
               nvals = nhgl;
 
-              inivar(&(*vars)[varid], GRID_GENERIC, ZAXIS_HYBRID, 0, name, NULL, NULL);
+              inivar(&(*vars)[varid], GRID_GENERIC, ZAXIS_HYBRID, 0, name, nullptr, nullptr);
 
               (*vars)[varid].gridID = gridIDhgl;
               (*vars)[varid].zaxisID = zaxisIDml;
@@ -1099,8 +1092,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
         }
     }
 
-  /*close input file */
-  nce(nc_close(nc_file_id));
+  // close input file
+  cdo_cdf_close(nc_file_id);
 
   nvars = varid;
 
@@ -1493,48 +1486,46 @@ export_e5res(const char *filename, VAR *vars, int nvars)
 void *
 Echam5ini(void *process)
 {
-  int streamID1, streamID2 = CDI_UNDEFID;
   int nrecs = 0;
   int varID, levelID;
-  int vlistID1, vlistID2;
   int nvars = 0;
   int iv, nlev;
   size_t gridsize;
   size_t nmiss;
-  int taxisID, tsID;
+  int tsID;
 
   cdoInitialize(process);
 
   // clang-format off
-  int IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, NULL);
-  int IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, NULL);
-  int EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, NULL);
-  int EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, NULL);
+  const auto IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, nullptr);
+  const auto IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, nullptr);
+  const auto EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, nullptr);
+  const auto EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   if (operatorID == EXPORT_E5ML && processSelf().m_ID != 0) cdoAbort("This operator can't be linked with other operators!");
 
   if (operfunc == func_read)
     {
-      VAR *vars = NULL;
+      VAR *vars = nullptr;
       ATTS atts;
       int iatt;
 
       iniatts(&atts);
 
       if (operatorID == IMPORT_E5ML)
-        nvars = import_e5ml(cdoGetStreamName(0).c_str(), &vars);
+        nvars = import_e5ml(cdoGetStreamName(0), &vars);
       else if (operatorID == IMPORT_E5RES)
-        nvars = import_e5res(cdoGetStreamName(0).c_str(), &vars, &atts);
+        nvars = import_e5res(cdoGetStreamName(0), &vars, &atts);
       else
         cdoAbort("Operator not implemented!");
 
       if (nvars == 0) cdoAbort("Unsupported file type!");
 
-      vlistID2 = vlistCreate();
+      const auto vlistID2 = vlistCreate();
       vlistDefNtsteps(vlistID2, 0);
 
       for (iv = 0; iv < nvars; iv++)
@@ -1557,17 +1548,17 @@ Echam5ini(void *process)
           cdiDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt], (int) strlen(atts.atxtentry[iatt]) + 1, atts.atxtentry[iatt]);
         }
 
-      taxisID = taxisCreate(TAXIS_ABSOLUTE);
+      const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
       vlistDefTaxis(vlistID2, taxisID);
 
-      if (cdoDefaultFileType == CDI_UNDEFID) cdoDefaultFileType = CDI_FILETYPE_NC;
+      if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
-      streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+      const auto streamID2 = cdoOpenWrite(1);
 
-      pstreamDefVlist(streamID2, vlistID2);
+      cdoDefVlist(streamID2, vlistID2);
 
       tsID = 0;
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
@@ -1576,30 +1567,30 @@ Echam5ini(void *process)
 
           for (levelID = 0; levelID < nlev; levelID++)
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, vars[varID].ptr + levelID * gridsize, 0);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, vars[varID].ptr + levelID * gridsize, 0);
             }
         }
 
-      pstreamClose(streamID2);
+      cdoStreamClose(streamID2);
 
       vlistDestroy(vlistID2);
     }
   else if (operfunc == func_write)
     {
-      VAR *vars = NULL;
+      VAR *vars = nullptr;
       int code, gridID, zaxisID, gridtype, zaxistype, gridsize, nlev;
       char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
       int64_t vdate;
-      int taxisID, vtime;
+      int vtime;
       int ntr = 0;
 
-      streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+      const auto streamID1 = cdoOpenRead(0);
 
-      vlistID1 = cdoStreamInqVlist(streamID1);
-      taxisID = vlistInqTaxis(vlistID1);
+      const auto vlistID1 = cdoStreamInqVlist(streamID1);
+      const auto taxisID = vlistInqTaxis(vlistID1);
 
-      nvars = vlistNvars(vlistID1);
+      const auto nvars = vlistNvars(vlistID1);
 
       vars = (VAR *) Malloc(nvars * sizeof(VAR));
 
@@ -1687,20 +1678,20 @@ Echam5ini(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           gridID = vlistInqVarGrid(vlistID1, varID);
           gridsize = gridInqSize(gridID);
 
-          pstreamReadRecord(streamID1, vars[varID].ptr + levelID * gridsize, &nmiss);
+          cdoReadRecord(streamID1, vars[varID].ptr + levelID * gridsize, &nmiss);
         }
 
-      pstreamClose(streamID1);
+      cdoStreamClose(streamID1);
 
       if (operatorID == EXPORT_E5ML)
-        export_e5ml(cdoGetStreamName(1).c_str(), vars, nvars, vdate, vtime, ntr);
+        export_e5ml(cdoGetStreamName(1), vars, nvars, vdate, vtime, ntr);
       else if (operatorID == EXPORT_E5RES)
-        export_e5res(cdoGetStreamName(1).c_str(), vars, nvars);
+        export_e5res(cdoGetStreamName(1), vars, nvars);
       else
         cdoAbort("Operator not implemented!");
     }
@@ -1715,5 +1706,5 @@ Echam5ini(void *process)
   */
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
