@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,15 +34,16 @@
 
 #include "cdi.h"
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
 #include "statistic.h"
 #include "timer.h"
 #include "datetime.h"
+#include "cdo_options.h"
+#include "eof_mode.h"
 
-enum T_EIGEN_MODE get_eigenmode(void);
-enum T_WEIGHT_MODE get_weightmode(void);
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
 
@@ -68,10 +69,10 @@ EOF3d(void *process)
   int calendar = CALENDAR_STANDARD;
 
   double sum_w;
-  double **cov = NULL; /* TODO: covariance matrix / eigenvectors after solving */
+  double **cov = nullptr; /* TODO: covariance matrix / eigenvectors after solving */
   double *eigv;
 
-  if (cdoTimer)
+  if (Options::Timer)
     {
       timer_cov = timer_new("Timeof cov");
       timer_eig = timer_new("Timeof eig");
@@ -80,16 +81,16 @@ EOF3d(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("eof3d",        EOF3D_,        0, NULL);
-  cdoOperatorAdd("eof3dtime",    EOF3D_TIME,    0, NULL);
-  cdoOperatorAdd("eof3dspatial", EOF3D_SPATIAL, 0, NULL);
+  cdoOperatorAdd("eof3d",        EOF3D_,        0, nullptr);
+  cdoOperatorAdd("eof3dtime",    EOF3D_TIME,    0, nullptr);
+  cdoOperatorAdd("eof3dspatial", EOF3D_SPATIAL, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg("Number of eigen functions to write out");
-  int n_eig = parameter2int(operatorArgv()[0]);
+  auto n_eig = parameter2int(cdoOperatorArgv(0));
 
   enum T_EIGEN_MODE eigen_mode = get_eigenmode();
   enum T_WEIGHT_MODE weight_mode = get_weightmode();
@@ -98,48 +99,47 @@ EOF3d(void *process)
 
   if (operfunc == EOF3D_SPATIAL) cdoAbort("Operator not Implemented - use eof3d or eof3dtime instead");
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  auto streamID1 = cdoOpenRead(0);
+  auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   /* COUNT NUMBER OF TIMESTEPS if EOF3D_ or EOF3D_TIME */
-  int nts = vlistNtsteps(vlistID1);
+  auto nts = vlistNtsteps(vlistID1);
   if (nts == -1)
     {
       nts = 0;
       while (cdoStreamInqTimestep(streamID1, nts)) nts++;
 
-      if (cdoVerbose) cdoPrint("Counted %i timeSteps", nts);
+      if (Options::cdoVerbose) cdoPrint("Counted %i timeSteps", nts);
 
-      pstreamClose(streamID1);
+      cdoStreamClose(streamID1);
 
-      streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+      streamID1 = cdoOpenRead(0);
       vlistID1 = cdoStreamInqVlist(streamID1);
     }
-  else if (cdoVerbose)
+  else if (Options::cdoVerbose)
     cdoPrint("Found %i timeSteps", nts);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
 
   /* reset the requested number of eigen-function to the maximum if neccessary
    */
   if (n_eig > nts)
     {
       cdoWarning("Solving in time-space:");
-      cdoWarning("Number of eigen-functions to write out is bigger than number "
-                 "of time-steps.");
+      cdoWarning("Number of eigen-functions to write out is bigger than number of time-steps.");
       cdoWarning("Setting n_eig to %i.", nts);
       n_eig = nts;
     }
 
   n = nts;
 
-  if (cdoVerbose) cdoPrint("counted %i timesteps", n);
+  if (Options::cdoVerbose) cdoPrint("counted %i timesteps", n);
 
-  int nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
   int nrecs;
 
-  int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridID1 = vlistInqVarGrid(vlistID1, 0);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   /* allocation of temporary fields and output structures */
   double *in = (double *) Malloc(gridsizemax * sizeof(double));
@@ -151,10 +151,10 @@ EOF3d(void *process)
   size_t maxlevs = 0;
   for (varID = 0; varID < nvars; ++varID)
     {
-      size_t gridsize = vlistGridsizeMax(vlistID1);
+      const auto gridsize = vlistGridsizeMax(vlistID1);
       nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       temp_size = gridsize * nlevs;
-      double missval = vlistInqVarMissval(vlistID1, varID);
+      const auto missval = vlistInqVarMissval(vlistID1, varID);
 
       if (nlevs > maxlevs) maxlevs = nlevs;
 
@@ -185,10 +185,9 @@ EOF3d(void *process)
         }
     }
 
-  if (cdoVerbose)
-    cdoPrint("Allocated eigenvalue/eigenvector with nts=%i, n=%i, gridsize=%zu "
-             "for processing in %s",
-             nts, n, gridsizemax, "time_space");
+  if (Options::cdoVerbose)
+    cdoPrint("Allocated eigenvalue/eigenvector with nts=%i, n=%i, gridsize=%zu for processing in %s", nts, n, gridsizemax,
+             "time_space");
 
   double *weight = (double *) Malloc(maxlevs * gridsizemax * sizeof(double));
   for (size_t i = 0; i < maxlevs * gridsizemax; ++i) weight[i] = 1.;
@@ -211,21 +210,21 @@ EOF3d(void *process)
   int tsID = 0;
 
   /* read the data and create covariance matrices for each var & level */
-  while (TRUE)
+  while (true)
     {
       nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
 
-          pstreamReadRecord(streamID1, in, &nmiss);
+          cdoReadRecord(streamID1, in, &nmiss);
 
-          size_t offset = gridsize * levelID;
+          const auto offset = gridsize * levelID;
           for (size_t i = 0; i < gridsize; ++i)
             {
               if (!DBL_IS_EQUAL(in[i], missval))
@@ -236,11 +235,10 @@ EOF3d(void *process)
               else
                 {
                   if (datacounts[varID][offset + i] != 0) cdoAbort("Missing values unsupported!");
-                  if (missval_warning == false)
+                  if (!missval_warning)
                     {
-                      // cdoWarning("Missing Value Support not checked for this
-                      // Operator!"); cdoWarning("Does not work with changing
-                      // locations of missing values in time.");
+                      // cdoWarning("Missing Value Support not checked for this Operator!");
+                      // cdoWarning("Does not work with changing locations of missing values in time.");
                       missval_warning = true;
                     }
                   datafields[varID][tsID][i + offset] = 0;
@@ -250,29 +248,28 @@ EOF3d(void *process)
       tsID++;
     }
 
-  if (cdoVerbose) cdoPrint("Read data for %i variables", nvars);
+  if (Options::cdoVerbose) cdoPrint("Read data for %i variables", nvars);
 
   size_t *pack = (size_t *) Malloc(temp_size * sizeof(size_t));  // TODO
 
   for (varID = 0; varID < nvars; varID++)
     {
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+      const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
       nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       temp_size = gridsize * nlevs;
-      double missval = vlistInqVarMissval(vlistID1, varID);
+      auto missval = vlistInqVarMissval(vlistID1, varID);
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         {
           char vname[64];
           vlistInqVarName(vlistID1, varID, &vname[0]);
-          cdoPrint("==========================================================="
-                   "=================");
+          cdoPrint("============================================================================");
           cdoPrint("Calculating covariance matrix and SVD for var%i (%s)", varID, vname);
         }
 
       npack = 0;  // TODO already set to 0
 
-      if (cdoTimer) timer_start(timer_cov);
+      if (Options::Timer) timer_start(timer_cov);
 
       for (size_t i = 0; i < temp_size; i++)
         {
@@ -294,9 +291,7 @@ EOF3d(void *process)
         {
           char vname[64];
           vlistInqVarName(vlistID1, varID, &vname[0]);
-          cdoWarning("Refusing to calculate EOF from a single time step for "
-                     "var%i (%s)",
-                     varID + 1, &vname[0]);
+          cdoWarning("Refusing to calculate EOF from a single time step for var%i (%s)", varID + 1, &vname[0]);
           continue;
         }
 
@@ -304,7 +299,7 @@ EOF3d(void *process)
       for (int j1 = 0; j1 < nts; j1++) cov[j1] = (double *) Malloc(nts * sizeof(double));
       eigv = (double *) Malloc(n * sizeof(double));
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         {
           cdoPrint("varID %i allocated eigv and cov with nts=%i and n=%i", varID, nts, n);
           cdoPrint("   npack=%zu, nts=%i temp_size=%zu", npack, nts, temp_size);
@@ -325,14 +320,14 @@ EOF3d(void *process)
             }
         }
 
-      if (cdoVerbose) cdoPrint("calculated cov-matrix");
+      if (Options::cdoVerbose) cdoPrint("calculated cov-matrix");
 
       /* SOLVE THE EIGEN PROBLEM */
-      if (cdoTimer) timer_stop(timer_cov);
+      if (Options::Timer) timer_stop(timer_cov);
 
-      if (cdoTimer) timer_start(timer_eig);
+      if (Options::Timer) timer_start(timer_eig);
 
-      if (cdoVerbose) cdoPrint("Processed correlation matrix for var %2i | npack: %zu", varID, n);
+      if (Options::cdoVerbose) cdoPrint("Processed correlation matrix for var %2i | npack: %zu", varID, n);
 
       if (eigen_mode == JACOBI)
         parallel_eigen_solution_of_symmetric_matrix(cov, eigv, n, __func__);
@@ -340,11 +335,11 @@ EOF3d(void *process)
         eigen_solution_of_symmetric_matrix(cov, eigv, n, __func__);
       /* NOW: cov contains the eigenvectors, eigv the eigenvalues */
 
-      if (cdoVerbose) cdoPrint("Processed SVD decomposition for var %d from %d x %d matrix", varID, n, n);
+      if (Options::cdoVerbose) cdoPrint("Processed SVD decomposition for var %d from %d x %d matrix", varID, n, n);
 
       for (int eofID = 0; eofID < n; eofID++) eigenvalues[varID][eofID][0] = eigv[eofID];
 
-      if (cdoTimer) timer_stop(timer_eig);
+      if (Options::Timer) timer_stop(timer_eig);
 
       for (int eofID = 0; eofID < n_eig; eofID++)
         {
@@ -371,7 +366,7 @@ EOF3d(void *process)
 
           if (sum > 0)
             {
-              sum = sqrt(sum);
+              sum = std::sqrt(sum);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(sum, npack, eigenvec, pack)
 #endif
@@ -394,15 +389,15 @@ EOF3d(void *process)
   /* write files with eigenvalues (ID3) and eigenvectors (ID2) */
 
   /*  eigenvalues */
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const auto streamID2 = cdoOpenWrite(1);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisDefRdate(taxisID2, 0);
   taxisDefRtime(taxisID2, 0);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int gridID2 = gridCreate(GRID_LONLAT, 1);
+  const auto gridID2 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID2, 1);
   gridDefYsize(gridID2, 1);
   double xvals = 0, yvals = 0;
@@ -412,65 +407,65 @@ EOF3d(void *process)
   ngrids = vlistNgrids(vlistID2);
   for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2, i, gridID2);
 
-  int zaxisID2 = zaxisCreate(ZAXIS_GENERIC, 1);
+  const auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, 1);
   double zvals = 0;
   zaxisDefLevels(zaxisID2, &zvals);
   zaxisDefName(zaxisID2, "zaxis_Reduced");
   zaxisDefLongname(zaxisID2, "Reduced zaxis from EOF3D - only one eigen value per 3D eigen vector");
 
-  int nzaxis = vlistNzaxis(vlistID2);
+  const auto nzaxis = vlistNzaxis(vlistID2);
   for (int i = 0; i < nzaxis; i++) vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
 
   /*  eigenvectors */
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
+  const auto streamID3 = cdoOpenWrite(2);
 
-  int vlistID3 = vlistDuplicate(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   taxisDefRdate(taxisID3, 0);
   taxisDefRtime(taxisID3, 0);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID3, vlistID3);
 
-  int vdate = 10101;
+  int64_t vdate = 10101;
   int vtime = 0;
-  juldate_t juldate = juldate_encode(calendar, vdate, vtime);
+  auto juldate = julianDateEncode(calendar, vdate, vtime);
   for (tsID = 0; tsID < n; tsID++)
     {
-      juldate = juldate_add_seconds(60, juldate);
-      juldate_decode(calendar, juldate, &vdate, &vtime);
+      juldate = julianDateAddSeconds(60, juldate);
+      julianDateDecode(calendar, juldate, vdate, vtime);
 
       taxisDefVdate(taxisID2, vdate);
       taxisDefVtime(taxisID2, vtime);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       if (tsID < n_eig)
         {
           taxisDefVdate(taxisID3, vdate);
           taxisDefVtime(taxisID3, vtime);
-          pstreamDefTimestep(streamID3, tsID);
+          cdoDefTimestep(streamID3, tsID);
         }
 
       for (varID = 0; varID < nvars; varID++)
         {
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
           nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < (int) nlevs; levelID++)
             {
-              size_t offset = levelID * gridsizemax;
+              const auto offset = levelID * gridsizemax;
               if (tsID < n_eig)
                 {
                   nmiss = arrayNumMV(gridsizemax, &eigenvectors[varID][tsID][offset], missval);
-                  pstreamDefRecord(streamID3, varID, levelID);
-                  pstreamWriteRecord(streamID3, &eigenvectors[varID][tsID][offset], nmiss);
+                  cdoDefRecord(streamID3, varID, levelID);
+                  cdoWriteRecord(streamID3, &eigenvectors[varID][tsID][offset], nmiss);
                 }
             }
 
           nmiss = (DBL_IS_EQUAL(eigenvalues[varID][tsID][0], missval)) ? 1 : 0;
 
-          pstreamDefRecord(streamID2, varID, 0);
-          pstreamWriteRecord(streamID2, eigenvalues[varID][tsID], nmiss);
+          cdoDefRecord(streamID2, varID, 0);
+          cdoWriteRecord(streamID2, eigenvalues[varID][tsID], nmiss);
         }  // for ( varID = 0; ... )
     }      // for ( tsID = 0; ... )
 
@@ -498,11 +493,11 @@ EOF3d(void *process)
   Free(pack);
   Free(weight);
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
