@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,53 +24,91 @@
       Exprf      aexprf          Append evaluated expressions from script file
 */
 /*
-Operatoren: +, -, *, \, ^, ==, !=, >, <, >=, <=, <=>, &&, ||, ?:
-Functions: sqrt, exp, log, log10, sin, cos, tan, asin, acos, atan
-Functions: min, max, avg, std, var
-Constansts: M_PI, M_E
+  Operatoren: +, -, *, \, ^, ==, !=, >, <, >=, <=, <=>, &&, ||, ?:
+  Functions: sqrt, exp, log, log10, sin, cos, tan, asin, acos, atan
+  Functions: min, max, avg, std, var
+  Constansts: M_PI, M_E
 */
 
-#include <sys/types.h> /* stat */
-#include <sys/stat.h>  /* stat */
-#include <unistd.h>    /* stat */
+#include <sys/stat.h> /* stat */
+#include <cassert>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_math.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "expr.h"
 #include "datetime.h"
+#include "cdo_zaxis.h"
+#include "cdi_lockedIO.h"
 
 void grid_cell_area(int gridID, double *array);
 int getSurfaceID(int vlistID);
 
 static char *
-exprs_from_arg(const char *arg)
+exprsFromArgument(const std::vector<std::string> &exprArgv)
 {
-  size_t slen = strlen(arg);
-  char *exprs = (char *) Malloc(slen + 2);
-  strcpy(exprs, operatorArgv()[0]);
-  if (exprs[slen - 1] != ';')
+  char *exprs = nullptr;
+  int exprArgc = exprArgv.size();
+  auto firstArg = exprArgv[0].c_str();
+
+  if (exprArgc == 1)
     {
-      exprs[slen] = ';';
-      exprs[slen + 1] = 0;
+      size_t slen = strlen(firstArg);
+      exprs = (char *) Malloc(slen + 2);
+      strcpy(exprs, firstArg);
+      if (exprs[slen - 1] != ';')
+        {
+          exprs[slen] = ';';
+          exprs[slen + 1] = 0;
+        }
+    }
+  else if (exprArgc > 1)
+    {
+      size_t slen = exprArgc + 1;
+      for (int i = 0; i < exprArgc; ++i) slen += exprArgv[i].size();
+      exprs = (char *) Malloc(slen);
+      strcpy(exprs, firstArg);
+      char *pexprs = exprs + strlen(firstArg);
+      for (int i = 1; i < exprArgc; ++i)
+        {
+          *pexprs++ = ',';
+          strcpy(pexprs, exprArgv[i].c_str());
+          pexprs += exprArgv[i].size();
+        }
+      if (*pexprs != ';')
+        {
+          *pexprs++ = ';';
+          *pexprs = 0;
+        }
+    }
+  else
+    {
+      operatorCheckArgc(1);
     }
 
   return exprs;
 }
 
 static char *
-exprs_from_file(const char *exprf)
+exprsFromFile(const std::vector<std::string> &exprArgv)
 {
+  int exprArgc = exprArgv.size();
+  if (exprArgc != 1) operatorCheckArgc(1);
+  auto exprf = exprArgv[0].c_str();
+
   /* Open expr script file for reading */
-  FILE *fp = fopen(exprf, "r");
-  if (fp == NULL) cdoAbort("Open failed on %s", exprf);
+  auto fp = fopen(exprf, "r");
+  if (fp == nullptr) cdoAbort("Open failed on %s", exprf);
 
   struct stat filestat;
   if (stat(exprf, &filestat) != 0) cdoAbort("Stat failed on %s", exprf);
 
-  size_t fsize = (size_t) filestat.st_size;
+  const size_t fsize = (size_t) filestat.st_size;
   char *exprs = (char *) Malloc(fsize + 1);
 
   int ichar, ipos = 0;
@@ -86,21 +124,21 @@ exprs_from_file(const char *exprf)
 
 #define MAX_PARAMS 4096
 
-char *
+static char *
 str_replace(char *target, const char *needle, const char *replacement)
 {
   char buffer[1024] = { 0 };
   char *insert_point = &buffer[0];
-  const char *tmp = target;
-  size_t needle_len = strlen(needle);
-  size_t repl_len = strlen(replacement);
+  auto tmp = target;
+  const auto needle_len = strlen(needle);
+  const auto repl_len = strlen(replacement);
 
   while (1)
     {
-      const char *p = strstr(tmp, needle);
+      char *p = strstr(tmp, needle);
 
       // walked past last occurrence of needle; copy remaining part
-      if (p == NULL)
+      if (p == nullptr)
         {
           strcpy(insert_point, tmp);
           break;
@@ -125,9 +163,9 @@ str_replace(char *target, const char *needle, const char *replacement)
 }
 
 static char *
-exprs_expand(char *exprs, int vlistID)
+exprsExpand(char *exprs, int vlistID)
 {
-  size_t len = strlen(exprs);
+  const auto len = strlen(exprs);
   unsigned nequal = 0, nsemi = 0;
   for (size_t i = 0; i < len; ++i)
     {
@@ -139,7 +177,7 @@ exprs_expand(char *exprs, int vlistID)
   if (nequal == 0 && nsemi == 1 && strstr(exprs, needle))
     {
       char varname[CDI_MAX_NAME];
-      int nvars = vlistNvars(vlistID);
+      const auto nvars = vlistNvars(vlistID);
       const size_t bufsize = 1024;
       char *sbuf = (char *) Malloc(bufsize);
       char *buf = (char *) Malloc(nvars * bufsize);
@@ -167,21 +205,20 @@ params_new(int vlistID)
   paramType *params = (paramType *) Malloc(MAX_PARAMS * sizeof(paramType));
   memset(params, 0, MAX_PARAMS * sizeof(paramType));
 
-  int nvars1 = vlistNvars(vlistID);
+  const auto nvars1 = vlistNvars(vlistID);
 
-  char name[CDI_MAX_NAME];
-  char longname[CDI_MAX_NAME];
-  char units[CDI_MAX_NAME];
+  char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
 
   for (int varID = 0; varID < nvars1; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID, varID);
-      int datatype = vlistInqVarDatatype(vlistID, varID);
-      int steptype = vlistInqVarTimetype(vlistID, varID);
-      size_t ngp = gridInqSize(gridID);
-      size_t nlev = zaxisInqSize(zaxisID);
-      double missval = vlistInqVarMissval(vlistID, varID);
+      const auto gridID = vlistInqVarGrid(vlistID, varID);
+      const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+      const auto datatype = vlistInqVarDatatype(vlistID, varID);
+      const auto steptype = vlistInqVarTimetype(vlistID, varID);
+      const auto nlat = gridInqYsize(gridID);
+      const auto ngp = gridInqSize(gridID);
+      const auto nlev = zaxisInqSize(zaxisID);
+      const auto missval = vlistInqVarMissval(vlistID, varID);
 
       vlistInqVarName(vlistID, varID, name);
       vlistInqVarLongname(vlistID, varID, longname);
@@ -196,11 +233,12 @@ params_new(int vlistID)
       params[varID].zaxisID = zaxisID;
       params[varID].datatype = datatype;
       params[varID].steptype = steptype;
+      params[varID].nlat = nlat;
       params[varID].ngp = ngp;
       params[varID].nlev = nlev;
       params[varID].missval = missval;
       params[varID].nmiss = 0;
-      params[varID].data = NULL;
+      params[varID].data = nullptr;
       params[varID].name = strdup(name);
       params[varID].longname = strdup(longname);
       params[varID].units = strdup(units);
@@ -212,16 +250,16 @@ params_new(int vlistID)
 static void
 params_add_coord(parseParamType *parse_arg, int coord, int cdiID, size_t size, const char *units, const char *longname)
 {
-  int ncoords = parse_arg->ncoords;
+  const auto ncoords = parse_arg->ncoords;
   if (ncoords >= parse_arg->maxcoords) cdoAbort("Too many coordinates (limit=%d)", parse_arg->maxcoords);
 
   parse_arg->coords[ncoords].needed = false;
   parse_arg->coords[ncoords].coord = coord;
   parse_arg->coords[ncoords].cdiID = cdiID;
   parse_arg->coords[ncoords].size = size;
-  parse_arg->coords[ncoords].units = NULL;
-  parse_arg->coords[ncoords].longname = NULL;
-  parse_arg->coords[ncoords].data = NULL;
+  parse_arg->coords[ncoords].units = nullptr;
+  parse_arg->coords[ncoords].longname = nullptr;
+  parse_arg->coords[ncoords].data = nullptr;
   if (units) parse_arg->coords[ncoords].units = strdup(units);
   if (longname) parse_arg->coords[ncoords].longname = strdup(longname);
 
@@ -231,7 +269,7 @@ params_add_coord(parseParamType *parse_arg, int coord, int cdiID, size_t size, c
 int
 params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
 {
-  int ncoords = parse_arg->ncoords;
+  const auto ncoords = parse_arg->ncoords;
   for (int coordID = 0; coordID < ncoords; ++coordID)
     {
       if (parse_arg->coords[coordID].coord == coord && parse_arg->coords[coordID].cdiID == cdiID) return coordID;
@@ -248,25 +286,25 @@ params_add_coordinates(int vlistID, parseParamType *parse_arg)
   char longname[CDI_MAX_NAME];
   char units[CDI_MAX_NAME];
 
-  int ngrids = vlistNgrids(vlistID);
+  const int ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
-      int gridID = vlistGrid(vlistID, index);
-      size_t size = gridInqSize(gridID);
+      const auto gridID = vlistGrid(vlistID, index);
+      const auto size = gridInqSize(gridID);
       gridInqXunits(gridID, units);
       params_add_coord(parse_arg, 'x', gridID, size, units, "longitude");
       gridInqYunits(gridID, units);
       params_add_coord(parse_arg, 'y', gridID, size, units, "latitude");
 
       params_add_coord(parse_arg, 'a', gridID, size, "m^2", "grid cell area");
-      params_add_coord(parse_arg, 'w', gridID, size, NULL, "grid cell area weights");
+      params_add_coord(parse_arg, 'w', gridID, size, nullptr, "grid cell area weights");
     }
 
-  int nzaxis = vlistNzaxis(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
-      int zaxisID = vlistZaxis(vlistID, index);
-      int size = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID, index);
+      const auto size = zaxisInqSize(zaxisID);
       zaxisInqUnits(zaxisID, units);
       zaxisInqLongname(zaxisID, longname);
       params_add_coord(parse_arg, 'z', zaxisID, size, units, longname);
@@ -277,7 +315,7 @@ static int
 params_add_ts(parseParamType *parse_arg)
 {
   int varID = -1;
-  paramType *params = parse_arg->params;
+  auto params = parse_arg->params;
   if (params)
     {
       varID = parse_arg->nparams;
@@ -291,6 +329,7 @@ params_add_ts(parseParamType *parse_arg)
       params[varID].nlev = 1;
 
       parse_arg->nparams++;
+      parse_arg->cnparams++;
     }
 
   return varID;
@@ -313,20 +352,22 @@ params_delete(paramType *params)
 }
 
 static void
-parseParamInit(parseParamType *parse_arg, int vlistID, int pointID, int surfaceID, paramType *params)
+parseParamInit(parseParamType *parse_arg, int vlistID, int pointID, int zonalID, int surfaceID, paramType *params)
 {
-  int nvars = vlistNvars(vlistID);
-  int ngrids = vlistNgrids(vlistID);
-  int nzaxis = vlistNzaxis(vlistID);
-  int maxcoords = ngrids * 4 + nzaxis;
+  const auto nvars = vlistNvars(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
+  const auto maxcoords = ngrids * 4 + nzaxis;
 
   parse_arg->maxparams = MAX_PARAMS;
   parse_arg->nparams = nvars;
+  parse_arg->cnparams = nvars;
   parse_arg->nvars1 = nvars;
   parse_arg->init = true;
-  parse_arg->debug = cdoVerbose != 0;
+  parse_arg->debug = Options::cdoVerbose != 0;
   parse_arg->params = params;
   parse_arg->pointID = pointID;
+  parse_arg->zonalID = zonalID;
   parse_arg->surfaceID = surfaceID;
   parse_arg->needed = (bool *) Malloc(nvars * sizeof(bool));
   parse_arg->coords = (coordType *) Malloc(maxcoords * sizeof(coordType));
@@ -334,6 +375,27 @@ parseParamInit(parseParamType *parse_arg, int vlistID, int pointID, int surfaceI
   parse_arg->ncoords = 0;
 }
 
+static int
+genZonalID(int vlistID)
+{
+  int zonalID = -1;
+
+  const auto ngrids = vlistNgrids(vlistID);
+  for (int index = 0; index < ngrids; index++)
+    {
+      const auto gridID = vlistGrid(vlistID, index);
+      const auto gridtype = gridInqType(gridID);
+      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GENERIC)
+        if (gridInqXsize(gridID) > 1 && gridInqYsize(gridID) >= 1)
+          {
+            zonalID = gridToZonal(gridID);
+            break;
+          }
+    }
+
+  return zonalID;
+}
+
 void *
 Expr(void *process)
 {
@@ -353,33 +415,39 @@ Expr(void *process)
   cdoOperatorAdd("aexprf", 0, 0, "expr script filename");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool replacesVariables = cdoOperatorF1(operatorID);
-  bool readsCommandLine = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const bool replacesVariables = cdoOperatorF1(operatorID);
+  const bool readsCommandLine = cdoOperatorF2(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  char *exprs = readsCommandLine ? exprs_from_arg(operatorArgv()[0]) : exprs_from_file(operatorArgv()[0]);
+  const auto exprArgv = cdoGetOperArgv();
 
-  int streamID1 = cdoStreamOpenRead(0);
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  auto exprs = readsCommandLine ? exprsFromArgument(exprArgv) : exprsFromFile(exprArgv);
 
-  exprs = exprs_expand(exprs, vlistID1);
-  if (cdoVerbose) cdoPrint(exprs);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  int nvars1 = vlistNvars(vlistID1);
+  exprs = exprsExpand(exprs, vlistID1);
+  if (Options::cdoVerbose) cdoPrint(exprs);
 
-  int pointID = gridCreate(GRID_GENERIC, 1);
-  int surfaceID = getSurfaceID(vlistID1);
+  const auto nvars1 = vlistNvars(vlistID1);
 
-  paramType *params = params_new(vlistID1);
+  const auto pointID = gridCreate(GRID_GENERIC, 1);
+  const auto zonalID = genZonalID(vlistID1);
+  const auto surfaceID = getSurfaceID(vlistID1);
 
-  parseParamInit(&parse_arg, vlistID1, pointID, surfaceID, params);
+  auto params = params_new(vlistID1);
+
+  parseParamInit(&parse_arg, vlistID1, pointID, zonalID, surfaceID, params);
 
   // Set all input variables to 'needed' if replacing is switched off
   for (int varID = 0; varID < nvars1; varID++) parse_arg.needed[varID] = !replacesVariables;
 
-  int vartsID = params_add_ts(&parse_arg);
+  // init function rand()
+  srand(Options::Random_Seed);
+
+  const auto vartsID = params_add_ts(&parse_arg);
   parse_arg.tsID = vartsID;
   params_add_coordinates(vlistID1, &parse_arg);
 
@@ -390,120 +458,131 @@ Expr(void *process)
 
   parse_arg.init = false;
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     for (int varID = 0; varID < nvars1; varID++)
       if (parse_arg.needed[varID]) cdoPrint("Needed var: %d %s", varID, params[varID].name);
 
-  for (int varID = 0; varID < parse_arg.nparams; varID++)
-    {
-      if (params[varID].ngp == 0 && params[varID].nlev == 0)
-        {
-          params[varID].ngp = 1;
-          params[varID].nlev = 1;
-          params[varID].type = PARAM_CONST;
-        }
-    }
-
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     for (int varID = 0; varID < parse_arg.nparams; varID++)
       cdoPrint("var: %d %s ngp=%zu nlev=%zu coord=%c", varID, params[varID].name, params[varID].ngp, params[varID].nlev,
                params[varID].coord == 0 ? ' ' : params[varID].coord);
 
-  int *varIDmap = (int *) Malloc(parse_arg.nparams * sizeof(int));
+  std::vector<int> varIDmap(parse_arg.nparams);
 
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
+  vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
+  vlistClearFlag(vlistID1);
   if (!replacesVariables)
     {
-      vlistClearFlag(vlistID1);
       int pidx = 0;
       for (int varID = 0; varID < nvars1; varID++)
         {
           params[varID].select = false;
-          if (params[varID].remove == false)
+          if (!params[varID].remove)
             {
               varIDmap[pidx++] = varID;
-              int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              for (int levID = 0; levID < nlevs; levID++) vlistDefFlag(vlistID1, varID, levID, TRUE);
+              const auto nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+              // printf("Replace %d nlevs %d\n", varID, nlevs);
+              for (int levID = 0; levID < nlevs; levID++) vlistDefFlag(vlistID1, varID, levID, true);
             }
         }
-      cdoVlistCopyFlag(vlistID2, vlistID1);
     }
+  cdoVlistCopyFlag(vlistID2, vlistID1);  // Copy global attributes
 
+  //  printf("parse_arg.nparams %d\n", parse_arg.nparams);
   for (int pidx = 0; pidx < parse_arg.nparams; pidx++)
     {
-      if (pidx < nvars1 && params[pidx].select == false) continue;
-      if (pidx >= nvars1 && params[pidx].type == PARAM_CONST) continue;
-      if (pidx >= nvars1 && params[pidx].name[0] == '_') continue;
-      if (pidx >= nvars1 && params[pidx].remove == true) continue;
-      if (pidx >= nvars1 && params[pidx].coord) continue;
+      if (pidx < nvars1 && !params[pidx].select) continue;
+      if (pidx >= nvars1)
+        {
+          if (params[pidx].type == PARAM_CONST) continue;
+          if (params[pidx].name[0] == '_') continue;
+          if (params[pidx].remove) continue;
+          if (params[pidx].coord) continue;
+        }
 
-      int varID = vlistDefVar(vlistID2, params[pidx].gridID, params[pidx].zaxisID, params[pidx].steptype);
+      // printf("gridID %d zaxisID %d\n",  params[pidx].gridID, params[pidx].zaxisID);
+      const auto varID = vlistDefVar(vlistID2, params[pidx].gridID, params[pidx].zaxisID, params[pidx].steptype);
       vlistDefVarName(vlistID2, varID, params[pidx].name);
+      // printf("add: %d %s %d levs %d\n", pidx,  params[pidx].name, varID, zaxisInqSize(params[pidx].zaxisID));
       if (params[pidx].lmiss) vlistDefVarMissval(vlistID2, varID, params[pidx].missval);
       if (params[pidx].units) vlistDefVarUnits(vlistID2, varID, params[pidx].units);
       if (params[pidx].longname) vlistDefVarLongname(vlistID2, varID, params[pidx].longname);
-      if (memcmp(params[pidx].name, "var", 3) == 0)
+      const auto len = strlen(params[pidx].name);
+      if (len > 3 && memcmp(params[pidx].name, "var", 3) == 0)
         {
-          if (strlen(params[pidx].name) > 3 && isdigit(params[pidx].name[3]))
+          if (isdigit(params[pidx].name[3]))
             {
-              int code = atoi(params[pidx].name + 3);
+              const int code = atoi(params[pidx].name + 3);
               vlistDefVarCode(vlistID2, varID, code);
             }
         }
       varIDmap[varID] = pidx;
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       for (int varID = 0; varID < nvars1; varID++)
         if (parse_arg.needed[varID]) printf("needed: %d %s\n", varID, parse_arg.params[varID].name);
       cdoPrint("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
     }
 
-  int nvars2 = vlistNvars(vlistID2);
+  const auto nvars2 = vlistNvars(vlistID2);
   if (nvars2 == 0) cdoAbort("No output variable found!");
 
   for (int varID = 0; varID < nvars1; varID++)
     {
       if (parse_arg.needed[varID])
         {
-          size_t ngp = params[varID].ngp;
-          size_t nlev = params[varID].nlev;
-          size_t nItems = MAX(4, ngp*nlev);
-          params[varID].data = (double*) Malloc(nItems*sizeof(double));
+          const auto ngp = params[varID].ngp;
+          const auto nlev = params[varID].nlev;
+          const auto nItems = cdo::max((size_t) 4, ngp * nlev);
+          params[varID].data = (double *) Malloc(nItems * sizeof(double));
         }
     }
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      size_t ngp = params[varID].ngp;
-      size_t nlev = params[varID].nlev;
-      size_t nItems = MAX(4, ngp*nlev);
-      params[varID].data = (double*) Malloc(nItems*sizeof(double));
+      const auto ngp = params[varID].ngp;
+      const auto nlev = params[varID].nlev;
+      const auto nItems = cdo::max((size_t) 4, ngp * nlev);
+      params[varID].data = (double *) Malloc(nItems * sizeof(double));
     }
 
   for (int i = 0; i < parse_arg.ncoords; i++)
     {
       if (parse_arg.coords[i].needed)
         {
-          int coord = parse_arg.coords[i].coord;
+          const int coord = parse_arg.coords[i].coord;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              int gridID = parse_arg.coords[i].cdiID;
-              size_t ngp = parse_arg.coords[i].size;
+              auto gridID = parse_arg.coords[i].cdiID;
+              const auto ngp = parse_arg.coords[i].size;
               double *data = (double *) Malloc(ngp * sizeof(double));
               parse_arg.coords[i].data = data;
               if (coord == 'x' || coord == 'y')
                 {
-                  if (gridInqType(gridID) == GRID_GENERIC) cdoAbort("Grid has no geographical coordinates!");
-                  if (gridInqType(gridID) == GRID_GME) gridID = gridToUnstructured(gridID, 0);
-                  if (gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR)
-                    gridID = gridToCurvilinear(gridID, 0);
+                  const auto gridtype = gridInqType(gridID);
+                  if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
+                  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
-                  if (coord == 'x')
-                    gridInqXvals(gridID, data);
-                  else if (coord == 'y')
-                    gridInqYvals(gridID, data);
+                  if (gridtype == GRID_UNSTRUCTURED)
+                    {
+                      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+                        {
+                          if (gridInqNumber(gridID) > 0)
+                            {
+                              gridID = referenceToGrid(gridID);
+                              if (gridID == -1) cdoAbort("Reference to source grid not found!");
+                            }
+                        }
+                    }
+
+                  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+                    cdoAbort("Cell center coordinates missing!");
+
+                  if (coord == 'x') gridInqXvals(gridID, data);
+                  if (coord == 'y') gridInqYvals(gridID, data);
 
                   if (gridID != parse_arg.coords[i].cdiID) gridDestroy(gridID);
                 }
@@ -517,16 +596,14 @@ Expr(void *process)
                   if (ngp > 1)
                     {
                       int wstatus = gridWeights(gridID, data);
-                      if (wstatus)
-                        cdoWarning("Grid cell bounds not available, using "
-                                   "constant grid cell area weights!");
+                      if (wstatus) cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
                     }
                 }
             }
           else if (coord == 'z')
             {
-              int zaxisID = parse_arg.coords[i].cdiID;
-              size_t nlev = parse_arg.coords[i].size;
+              const auto zaxisID = parse_arg.coords[i].cdiID;
+              const auto nlev = parse_arg.coords[i].size;
               double *data = (double *) Malloc(nlev * sizeof(double));
               parse_arg.coords[i].data = data;
               cdoZaxisInqLevels(zaxisID, data);
@@ -538,30 +615,30 @@ Expr(void *process)
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      int coord = params[varID].coord;
+      const int coord = params[varID].coord;
       if (coord)
         {
           char *varname = strdup(params[varID].name);
           varname[strlen(varname) - 2] = 0;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              int coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
-              int gridID = parse_arg.coords[coordID].cdiID;
-              size_t ngp = parse_arg.coords[coordID].size;
+              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
+              const auto gridID = parse_arg.coords[coordID].cdiID;
+              const auto ngp = parse_arg.coords[coordID].size;
               double *data = parse_arg.coords[coordID].data;
               assert(gridID == params[varID].gridID);
-              assert(data != NULL);
+              assert(data != nullptr);
 
               arrayCopy(ngp, data, params[varID].data);
             }
           else if (coord == 'z')
             {
-              int coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
-              int zaxisID = parse_arg.coords[coordID].cdiID;
-              size_t nlev = parse_arg.coords[coordID].size;
+              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
+              const auto zaxisID = parse_arg.coords[coordID].cdiID;
+              const auto nlev = parse_arg.coords[coordID].size;
               double *data = parse_arg.coords[coordID].data;
               assert(zaxisID == params[varID].zaxisID);
-              assert(data != NULL);
+              assert(data != nullptr);
 
               arrayCopy(nlev, data, params[varID].data);
             }
@@ -572,32 +649,33 @@ Expr(void *process)
         }
     }
 
-  if (cdoVerbose) vlistPrint(vlistID2);
+  if (Options::cdoVerbose) vlistPrint(vlistID2);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(1, cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int vdate0 = 0, vtime0 = 0;
-  int calendar = taxisInqCalendar(taxisID1);
+  int64_t vdate0 = 0;
+  int vtime0 = 0;
+  const auto calendar = taxisInqCalendar(taxisID1);
 
   int nrecs;
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       double jdelta = 0;
 
       if (tsID)
         {
-          juldate_t juldate0 = juldate_encode(calendar, vdate0, vtime0);
-          juldate_t juldate = juldate_encode(calendar, vdate, vtime);
-          jdelta = juldate_to_seconds(juldate_sub(juldate, juldate0));
+          auto juldate0 = julianDateEncode(calendar, vdate0, vtime0);
+          auto juldate = julianDateEncode(calendar, vdate, vtime);
+          jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
         }
 
       vdate0 = vdate;
@@ -622,7 +700,7 @@ Expr(void *process)
 
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int varID = 0; varID < nvars1; varID++)
         if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].nmiss = 0;
@@ -630,56 +708,57 @@ Expr(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           if (parse_arg.needed[varID])
             {
-              size_t offset = params[varID].ngp * levelID;
-              double *vardata = params[varID].data + offset;
+              const auto offset = params[varID].ngp * levelID;
+              auto vardata = params[varID].data + offset;
               size_t nmiss;
-              pstreamReadRecord(streamID1, vardata, &nmiss);
+              cdoReadRecord(streamID1, vardata, &nmiss);
               params[varID].nmiss += nmiss;
             }
         }
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          int pidx = varIDmap[varID];
+          const auto pidx = varIDmap[varID];
           if (pidx < nvars1) continue;
-          size_t ngp = params[pidx].ngp;
-          size_t nlev = params[pidx].nlev;
+          const auto ngp = params[pidx].ngp;
+          const auto nlev = params[pidx].nlev;
 
           params[pidx].nmiss = 0;
-          arrayFill(ngp * nlev, params[pidx].data, 0.0);
+          varrayFill(ngp * nlev, params[pidx].data, 0.0);
         }
 
+      parse_arg.cnparams = vartsID + 1;
       yy_scan_string(exprs, scanner);
       yyparse(&parse_arg, scanner);
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          int pidx = varIDmap[varID];
+          const auto pidx = varIDmap[varID];
 
           if (tsID > 0 && params[pidx].steptype == TIME_CONSTANT) continue;
 
-          double missval = vlistInqVarMissval(vlistID2, varID);
+          const auto missval = vlistInqVarMissval(vlistID2, varID);
 
-          size_t ngp = params[pidx].ngp;
-          int nlev = (int) params[pidx].nlev;
+          const auto ngp = params[pidx].ngp;
+          const int nlev = (int) params[pidx].nlev;
           for (int levelID = 0; levelID < nlev; levelID++)
             {
-              size_t offset = ngp * levelID;
+              const auto offset = ngp * levelID;
               double *vardata = params[pidx].data + offset;
-              size_t nmiss = arrayNumMV(ngp, vardata, missval);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, vardata, nmiss);
+              const auto nmiss = arrayNumMV(ngp, vardata, missval);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, vardata, nmiss);
             }
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   vlistDestroy(vlistID2);
 
@@ -700,11 +779,11 @@ Expr(void *process)
         }
       Free(parse_arg.coords);
     }
-  if (varIDmap) Free(varIDmap);
 
   gridDestroy(pointID);
+  if (zonalID != -1) gridDestroy(zonalID);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
