@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,14 +25,18 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include <mpim_grid.h>
+#include "cdo_cdiWrapper.h"
+#include "util_string.h"
 
-void cdo_print_zaxis(int zaxisID);
+
+void cdoPrintZaxis(int zaxisID);
 
 void
-cdo_print_attributes(FILE *fp, int cdiID, int varID, int nblanks)
+cdoPrintAttributes(FILE *fp, int cdiID, int varID, int nblanks)
 {
 #define MAXATT 8192
   int natts;
@@ -66,22 +70,23 @@ cdo_print_attributes(FILE *fp, int cdiID, int varID, int nblanks)
       else if (atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64)
         {
           if (attlen > MAXATT) attlen = MAXATT;
-          cdiInqAttFlt(cdiID, varID, attname, MAXATT, attflt);
+          cdiInqAttFlt(cdiID, varID, attname, attlen, attflt);
           fprintf(fp, "%*s", nblanks, "");
           fprintf(fp, "%s = ", attname);
           for (int i = 0; i < attlen; ++i)
             {
               if (i > 0) fprintf(fp, ", ");
               if (atttype == CDI_DATATYPE_FLT32)
-                fprintf(fp, "%sf", double_to_attstr(CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
+                fprintf(fp, "%sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
               else
-                fprintf(fp, "%s", double_to_attstr(CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
+                fprintf(fp, "%s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
             }
           fprintf(fp, "\n");
         }
       else if (atttype == CDI_DATATYPE_TXT)
         {
-          cdiInqAttTxt(cdiID, varID, attname, sizeof(atttxt), atttxt);
+          if (attlen > MAXATT) attlen = MAXATT;
+          cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt);
           atttxt[attlen] = 0;
           fprintf(fp, "%*s", nblanks, "");
           fprintf(fp, "%s = \"%s\"\n", attname, atttxt);
@@ -90,16 +95,16 @@ cdo_print_attributes(FILE *fp, int cdiID, int varID, int nblanks)
 }
 
 static void
-printHistory(FILE *fp, int streamID)
+printHistory(FILE *fp, CdoStreamID streamID)
 {
-  int fileID = pstreamFileID(streamID);
+  const int fileID = cdoFileID(streamID);
   size_t historysize = (size_t) streamInqHistorySize(fileID);
   if (historysize > 0)
     {
       char *history = (char *) Malloc(historysize + 1);
       history[historysize] = 0;
       streamInqHistoryString(fileID, history);
-      fprintf(fp, "  history = %s\n", history);
+      fprintf(fp, "  history = \"%s\"\n", history);
       Free(history);
     }
 }
@@ -107,26 +112,96 @@ printHistory(FILE *fp, int streamID)
 static void
 printSource(FILE *fp, int vlistID, int varID)
 {
-  /* institute info */
-  const char *instptr = institutInqLongnamePtr(vlistInqVarInstitut(vlistID, varID));
+  // institute info
+  auto instptr = institutInqLongnamePtr(vlistInqVarInstitut(vlistID, varID));
   if (instptr) fprintf(fp, "  institution = %s\n", instptr);
 
-  /* source info */
-  const char *modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
+  // source info
+  auto modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
   if (modelptr) fprintf(fp, "  source = %s\n", modelptr);
 }
 
 static void
-partab(FILE *fp, int streamID, int option)
+printVCT(int vlistID, bool lvct)
 {
-  int vlistID = cdoStreamInqVlist(streamID);
+  const auto nzaxis = vlistNzaxis(vlistID);
+  for (int index = 0; index < nzaxis; index++)
+    {
+      const auto zaxisID = vlistZaxis(vlistID, index);
+      const auto type = zaxisInqType(zaxisID);
+      if (type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF)
+        {
+          const auto vctsize = zaxisInqVctSize(zaxisID);
+          const auto vct = zaxisInqVctPtr(zaxisID);
+
+          if (vctsize % 2 == 0)
+            {
+              if (lvct)
+                {
+                  fprintf(stdout, "#   k         vct_a(k) [Pa]             vct_b(k) []\n");
+                  for (int i = 0; i < vctsize / 2; i++) fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize / 2 + i]);
+                }
+              else
+                {
+                  fprintf(stdout, "vctsize   = %d\n", vctsize);
+                  int nbyte0 = fprintf(stdout, "vct       = ");
+                  int nbyte = nbyte0;
+                  for (int i = 0; i < vctsize; i++)
+                    {
+                      if (nbyte > 70 || i == vctsize / 2)
+                        {
+                          fprintf(stdout, "\n%*s", nbyte0, "");
+                          nbyte = nbyte0;
+                        }
+                      nbyte += fprintf(stdout, "%.9g ", vct[i]);
+                    }
+                  fprintf(stdout, "\n");
+                }
+            }
+          else
+            for (int i = 0; i < vctsize; i++) fprintf(stdout, "%5d %25.17f\n", i, vct[i]);
+
+          break;
+        }
+    }
+}
+
+static void
+printCodeTable(int vlistID)
+{
+  char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
+
+  const auto nvars = vlistNvars(vlistID);
+  for (int varID = 0; varID < nvars; varID++)
+    {
+      varname[0] = 0;
+      varlongname[0] = 0;
+      varunits[0] = 0;
+      auto code = vlistInqVarCode(vlistID, varID);
+      vlistInqVarName(vlistID, varID, varname);
+      vlistInqVarLongname(vlistID, varID, varlongname);
+      vlistInqVarUnits(vlistID, varID, varunits);
+      fprintf(stdout, "%4d  %-12s", code, varname);
+      if (varlongname[0])
+        {
+          fprintf(stdout, "  %s", varlongname);
+          if (varunits[0]) fprintf(stdout, " [%s]", varunits);
+        }
+      fprintf(stdout, "\n");
+    }
+}
+
+static void
+partab(FILE *fp, CdoStreamID streamID, const int option)
+{
+  const auto vlistID = cdoStreamInqVlist(streamID);
   int varID, datatype = -1;
   char pstr[32];
   char paramstr[32];
   char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varstdname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
 
-  int nvars = vlistNvars(vlistID);
-  bool linebreak = (option == 4) ? false : true;
+  const int nvars = vlistNvars(vlistID);
+  const bool linebreak = option != 4;
 
   if (option == 2)
     {
@@ -138,7 +213,7 @@ partab(FILE *fp, int streamID, int option)
           fprintf(fp, "  name=_GLOBAL_\n");
           printHistory(fp, streamID);
           printSource(fp, vlistID, 0);
-          cdo_print_attributes(fp, vlistID, CDI_GLOBAL, 2);
+          cdoPrintAttributes(fp, vlistID, CDI_GLOBAL, 2);
           fprintf(fp, "/\n");
         }
     }
@@ -178,8 +253,8 @@ partab(FILE *fp, int streamID, int option)
       varname[0] = 0;
       varlongname[0] = 0;
       varunits[0] = 0;
-      int param = vlistInqVarParam(vlistID, varID);
-      double missval = vlistInqVarMissval(vlistID, varID);
+      const auto param = vlistInqVarParam(vlistID, varID);
+      const auto missval = vlistInqVarMissval(vlistID, varID);
       vlistInqVarName(vlistID, varID, varname);
       vlistInqVarStdname(vlistID, varID, varstdname);
       vlistInqVarLongname(vlistID, varID, varlongname);
@@ -218,24 +293,25 @@ partab(FILE *fp, int streamID, int option)
             if (linebreak) fprintf(fp, "\n");
           }
 
-      int chunktype = vlistInqVarChunkType(vlistID, varID);
-      if (chunktype == CDI_CHUNK_AUTO)
+      int uvRelativeToGrid = 0;
+      if (cdiInqKeyInt(vlistID, varID, CDI_KEY_UVRELATIVETOGRID, &uvRelativeToGrid) == CDI_NOERR)
         {
-          fprintf(fp, "  chunktype=auto");
-          if (linebreak) fprintf(fp, "\n");
-        }
-      else if (chunktype == CDI_CHUNK_GRID)
-        {
-          fprintf(fp, "  chunktype=grid");
-          if (linebreak) fprintf(fp, "\n");
-        }
-      if (chunktype == CDI_CHUNK_LINES)
-        {
-          fprintf(fp, "  chunktype=lines");
+          fprintf(fp, "  uvRelativeToGrid=%d", uvRelativeToGrid);
           if (linebreak) fprintf(fp, "\n");
         }
 
-      if (option == 2) cdo_print_attributes(fp, vlistID, varID, 2);
+      const auto chunktype = vlistInqVarChunkType(vlistID, varID);
+      const char *chunkname = (chunktype == CDI_CHUNK_AUTO)
+                                  ? "auto"
+                                  : ((chunktype == CDI_CHUNK_GRID) ? "grid" : ((chunktype == CDI_CHUNK_LINES) ? "lines" : nullptr));
+
+      if (chunkname)
+        {
+          fprintf(fp, "  chunktype=%s", chunkname);
+          if (linebreak) fprintf(fp, "\n");
+        }
+
+      if (option == 2) cdoPrintAttributes(fp, vlistID, varID, 2);
       if (option == 2) fprintf(fp, "  missing_value=%g\n", missval);
 
       if (!linebreak) fprintf(fp, "  ");
@@ -244,10 +320,10 @@ partab(FILE *fp, int streamID, int option)
 }
 
 static void
-filedes(int streamID)
+filedes(CdoStreamID streamID)
 {
   printf("\n");
-  int filetype = pstreamInqFiletype(streamID);
+  const auto filetype = cdoInqFiletype(streamID);
   switch (filetype)
     {
     case CDI_FILETYPE_GRB: printf("  GRIB data\n"); break;
@@ -269,7 +345,7 @@ filedes(int streamID)
     case CDI_FILETYPE_EXT:
     case CDI_FILETYPE_IEG:
       {
-        int byteorder = pstreamInqByteorder(streamID);
+        const auto byteorder = cdoInqByteorder(streamID);
         switch (byteorder)
           {
           case CDI_BIGENDIAN: printf("  byteorder is BIGENDIAN\n"); break;
@@ -288,95 +364,48 @@ Filedes(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int GRIDDES  = cdoOperatorAdd("griddes",   0, 0, NULL);
-  int GRIDDES2 = cdoOperatorAdd("griddes2",  0, 0, NULL);
-  int ZAXISDES = cdoOperatorAdd("zaxisdes",  0, 0, NULL);
-  int VCT      = cdoOperatorAdd("vct",       0, 0, NULL);
-  int VCT2     = cdoOperatorAdd("vct2",      0, 0, NULL);
-  int CODETAB  = cdoOperatorAdd("codetab",   0, 0, NULL);
-  int FILEDES  = cdoOperatorAdd("filedes",   0, 0, NULL);
-  int VLIST    = cdoOperatorAdd("vlist",     0, 0, NULL);
-  int SPARTAB  = cdoOperatorAdd("spartab",   0, 0, NULL);
-  int PARTAB   = cdoOperatorAdd("partab",    0, 0, NULL);
-  int PARTAB2  = cdoOperatorAdd("partab2",   0, 0, NULL);
+  const auto GRIDDES  = cdoOperatorAdd("griddes",   0, 0, nullptr);
+  const auto GRIDDES2 = cdoOperatorAdd("griddes2",  0, 0, nullptr);
+  const auto ZAXISDES = cdoOperatorAdd("zaxisdes",  0, 0, nullptr);
+  const auto VCT      = cdoOperatorAdd("vct",       0, 0, nullptr);
+  const auto VCT2     = cdoOperatorAdd("vct2",      0, 0, nullptr);
+  const auto CODETAB  = cdoOperatorAdd("codetab",   0, 0, nullptr);
+  const auto FILEDES  = cdoOperatorAdd("filedes",   0, 0, nullptr);
+  const auto VLIST    = cdoOperatorAdd("vlist",     0, 0, nullptr);
+  const auto SPARTAB  = cdoOperatorAdd("spartab",   0, 0, nullptr);
+  const auto PARTAB   = cdoOperatorAdd("partab",    0, 0, nullptr);
+  const auto PARTAB2  = cdoOperatorAdd("partab2",   0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
-
-  int vlistID = cdoStreamInqVlist(streamID);
-
-  int nvars = vlistNvars(vlistID);
-  int ngrids = vlistNgrids(vlistID);
-  int nzaxis = vlistNzaxis(vlistID);
+  const auto streamID = cdoOpenRead(0);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
   if (operatorID == GRIDDES || operatorID == GRIDDES2)
     {
-      int opt = (operatorID == GRIDDES) ? 1 : 0;
+      const auto opt = (operatorID == GRIDDES) ? 1 : 0;
+      const auto ngrids = vlistNgrids(vlistID);
       for (int index = 0; index < ngrids; index++)
         {
-          printf("#\n"
-                 "# gridID %d\n"
-                 "#\n",
-                 index + 1);
-          cdo_print_grid(vlistGrid(vlistID, index), opt);
+          printf("#\n# gridID %d\n#\n", index + 1);
+          cdoPrintGrid(vlistGrid(vlistID, index), opt);
+          const auto nsubtypes = vlistNsubtypes(vlistID);
+          for (int index = 0; index < nsubtypes; index++) subtypePrint(vlistSubtype(vlistID, index));
         }
     }
   else if (operatorID == ZAXISDES)
     {
+      const auto nzaxis = vlistNzaxis(vlistID);
       for (int index = 0; index < nzaxis; index++)
         {
-          printf("#\n"
-                 "# zaxisID %d\n"
-                 "#\n",
-                 index + 1);
-          cdo_print_zaxis(vlistZaxis(vlistID, index));
+          printf("#\n# zaxisID %d\n#\n", index + 1);
+          cdoPrintZaxis(vlistZaxis(vlistID, index));
         }
     }
   else if (operatorID == VCT || operatorID == VCT2)
     {
-      for (int index = 0; index < nzaxis; index++)
-        {
-          int zaxisID = vlistZaxis(vlistID, index);
-          int type = zaxisInqType(zaxisID);
-          if (type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF)
-            {
-              int vctsize = zaxisInqVctSize(zaxisID);
-              const double *vct = zaxisInqVctPtr(zaxisID);
-
-              if (vctsize % 2 == 0)
-                {
-                  if (operatorID == VCT)
-                    {
-                      fprintf(stdout, "#   k         vct_a(k) [Pa]             "
-                                      "vct_b(k) []\n");
-                      for (int i = 0; i < vctsize / 2; i++)
-                        fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize / 2 + i]);
-                    }
-                  else
-                    {
-                      fprintf(stdout, "vctsize   = %d\n", vctsize);
-                      int nbyte0 = fprintf(stdout, "vct       = ");
-                      int nbyte = nbyte0;
-                      for (int i = 0; i < vctsize; i++)
-                        {
-                          if (nbyte > 70 || i == vctsize / 2)
-                            {
-                              fprintf(stdout, "\n%*s", nbyte0, "");
-                              nbyte = nbyte0;
-                            }
-                          nbyte += fprintf(stdout, "%.9g ", vct[i]);
-                        }
-                      fprintf(stdout, "\n");
-                    }
-                }
-              else
-                for (int i = 0; i < vctsize; i++) fprintf(stdout, "%5d %25.17f\n", i, vct[i]);
-
-              break;
-            }
-        }
+      printVCT(vlistID, operatorID == VCT);
     }
   else if (operatorID == VLIST)
     {
@@ -384,32 +413,11 @@ Filedes(void *process)
     }
   else if (operatorID == CODETAB)
     {
-      char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
-
-      for (int varID = 0; varID < nvars; varID++)
-        {
-          varname[0] = 0;
-          varlongname[0] = 0;
-          varunits[0] = 0;
-          int code = vlistInqVarCode(vlistID, varID);
-          vlistInqVarName(vlistID, varID, varname);
-          vlistInqVarLongname(vlistID, varID, varlongname);
-          vlistInqVarUnits(vlistID, varID, varunits);
-          fprintf(stdout, "%4d  %-12s", code, varname);
-          if (varlongname[0])
-            {
-              fprintf(stdout, "  %s", varlongname);
-              if (varunits[0]) fprintf(stdout, " [%s]", varunits);
-            }
-          fprintf(stdout, "\n");
-        }
+      printCodeTable(vlistID);
     }
   else if (operatorID == PARTAB || operatorID == SPARTAB || operatorID == PARTAB2)
     {
-      int option = 1;
-      if (operatorID == SPARTAB) option = 4;
-      if (operatorID == PARTAB2) option = 2;
-
+      const auto option = (operatorID == SPARTAB) ? 4 : ((operatorID == PARTAB2) ? 2 : 1);
       partab(stdout, streamID, option);
     }
   else if (operatorID == FILEDES)
@@ -417,9 +425,9 @@ Filedes(void *process)
       filedes(streamID);
     }
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
