@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,15 +22,19 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h" /* VERSION */
+#include "config.h" /* _FILE_OFFSET_BITS influence off_t */
 #endif
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
 #include "util_fileextensions.h"
+#include "util_files.h"
+#include "cdo_zaxis.h"
 
 /*
   Values output into the grib map file:
@@ -159,7 +163,7 @@ flt2ibm(float x, unsigned char *ibm)
   int sign, exp, i;
   double mant;
 
-  if (!(fabs((double) x) > 0))
+  if (std::fabs((double) x) <= 0)
     {
       ibm[0] = ibm[1] = ibm[2] = ibm[3] = 0;
       return 0;
@@ -211,16 +215,16 @@ flt2ibm(float x, unsigned char *ibm)
   ibm[0] = sign | exp;
 
   mant = mant * 256.0;
-  i = (int) floor(mant);
+  i = (int) std::floor(mant);
   mant = mant - i;
   ibm[1] = i;
 
   mant = mant * 256.0;
-  i = (int) floor(mant);
+  i = (int) std::floor(mant);
   mant = mant - i;
   ibm[2] = i;
 
-  ibm[3] = (int) floor(mant * 256.0);
+  ibm[3] = (int) std::floor(mant * 256.0);
 
   return 0;
 }
@@ -247,15 +251,15 @@ dumpmap()
   size_t nbytes;
   FILE *mapfp;
 
-  indx.hipnt = NULL;
-  indx.hfpnt = NULL;
-  indx.intpnt = NULL;
-  indx.fltpnt = NULL;
-  indxb.bigpnt = NULL;
+  indx.hipnt = nullptr;
+  indx.hfpnt = nullptr;
+  indx.intpnt = nullptr;
+  indx.fltpnt = nullptr;
+  indxb.bigpnt = nullptr;
   indxb.bignum = 0;
 
-  mapfp = fopen(cdoGetStreamName(0).c_str(), "r");
-  if (mapfp == NULL) cdoAbort("Open failed on %s", cdoGetStreamName(0).c_str());
+  mapfp = fopen(cdoGetStreamName(0), "r");
+  if (mapfp == nullptr) cdoAbort("Open failed on %s", cdoGetStreamName(0));
 
   /* check the version number */
 
@@ -365,7 +369,7 @@ dumpmap()
         }
     }
 
-  UNUSED(nbytes);
+  (void)(nbytes); //CDO_UNUSED
 
   fclose(mapfp);
 
@@ -429,7 +433,7 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
     {
       double xmin = 1.e10, xmax = -1.e10, ymin = 1.e10, ymax = -1.e10;
       double xrange, yrange;
-      int nx, ny, ni;
+      int nx = 0, ny = 0, ni;
       double inc[] = { 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001 };
 
       double xinc = gridInqXinc(gridID);
@@ -437,14 +441,14 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 
       double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
 
-      gridInqParamLCC(gridID, grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
+      gridInqParamLCC(gridID, cdiInqGridMissval(), &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
       fprintf(gdp, "PDEF %d %d LCCR %g %g 1 1 %g %g %g %g %g\n", xsize, ysize, xval_0, yval_0, lat_1, lat_2, lon_0, xinc, yinc);
 
       gridID = gridToCurvilinear(gridID, 0);
-      double *xvals = (double *) Malloc(xsize * ysize * sizeof(double));
-      double *yvals = (double *) Malloc(xsize * ysize * sizeof(double));
-      gridInqXvals(gridID, xvals);
-      gridInqYvals(gridID, yvals);
+      Varray<double> xvals(xsize * ysize);
+      Varray<double> yvals(xsize * ysize);
+      gridInqXvals(gridID, xvals.data());
+      gridInqYvals(gridID, yvals.data());
       for (i = 0; i < xsize * ysize; ++i)
         {
           if (xvals[i] > 180) xvals[i] -= 360;
@@ -453,8 +457,6 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
           if (yvals[i] < ymin) ymin = yvals[i];
           if (yvals[i] > ymax) ymax = yvals[i];
         }
-      Free(xvals);
-      Free(yvals);
 
       xfirst = ((int) (xmin - 0.0));
       yfirst = ((int) (ymin - 0.0));
@@ -481,10 +483,10 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
     {
       xfirst = gridInqXval(gridID, 0);
       xinc = gridInqXinc(gridID);
-      if (IS_EQUAL(xinc, 0) && gridInqXvals(gridID, NULL))
+      if (IS_EQUAL(xinc, 0) && gridInqXvals(gridID, nullptr))
         {
-          double *xvals = (double *) Malloc(xsize * sizeof(double));
-          gridInqXvals(gridID, xvals);
+          Varray<double> xvals(xsize);
+          gridInqXvals(gridID, xvals.data());
           fprintf(gdp, "XDEF %d LEVELS ", xsize);
           j = 0;
           for (i = 0; i < xsize; i++)
@@ -499,8 +501,6 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
                 }
             }
           if (j) fprintf(gdp, "\n");
-
-          Free(xvals);
         }
       else
         {
@@ -517,10 +517,10 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
       yinc = gridInqYinc(gridID);
       if (gridtype == GRID_GAUSSIAN) yinc = 0;
 
-      if (IS_EQUAL(yinc, 0) && gridInqYvals(gridID, NULL))
+      if (IS_EQUAL(yinc, 0) && gridInqYvals(gridID, nullptr))
         {
-          double *yvals = (double *) Malloc(ysize * sizeof(double));
-          gridInqYvals(gridID, yvals);
+          Varray<double> yvals(ysize);
+          gridInqYvals(gridID, yvals.data());
           fprintf(gdp, "YDEF %d LEVELS ", ysize);
           j = 0;
           if (yvals[0] > yvals[ysize - 1])
@@ -554,8 +554,6 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
             }
 
           if (j) fprintf(gdp, "\n");
-
-          Free(yvals);
         }
       else
         {
@@ -594,8 +592,8 @@ ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
         }
     }
 
-  double *levels = (double *) Malloc(nlevmax * sizeof(double));
-  cdoZaxisInqLevels(zaxisIDmax, levels);
+  Varray<double> levels(nlevmax);
+  cdoZaxisInqLevels(zaxisIDmax, levels.data());
   bool lplev = (zaxisInqType(zaxisIDmax) == ZAXIS_PRESSURE);
   double level0 = levels[0];
   if (nlevmax > 1)
@@ -658,8 +656,6 @@ ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
       }
       if (j) fprintf(gdp, "\n");
     }
-
-  Free(levels);
 }
 
 static void
@@ -703,7 +699,7 @@ ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
 
   for (int varID = 0; varID < nvars; varID++)
     {
-      if (vars[varID] == TRUE)
+      if (vars[varID] == true)
         {
           zaxisID = vlistInqVarZaxis(vlistID, varID);
           ltype = zaxisInqLtype(zaxisID);
@@ -799,7 +795,7 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
   memset(&indx, 0, sizeof(struct gaindx));
 
   mapfp = fopen(ctlfile, "w");
-  if (mapfp == NULL) cdoAbort("Open failed on %s", ctlfile);
+  if (mapfp == nullptr) cdoAbort("Open failed on %s", ctlfile);
 
   indx.type = map_version;
   indx.hfnum = 0;
@@ -817,11 +813,11 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
     }
   indx.fltnum = 3 * nrecords;
 
-  indx.hipnt = NULL;
-  indx.hfpnt = NULL;
-  indx.intpnt = NULL;
-  indx.fltpnt = NULL;
-  indxb.bigpnt = NULL;
+  indx.hipnt = nullptr;
+  indx.hfpnt = nullptr;
+  indx.intpnt = nullptr;
+  indx.fltpnt = nullptr;
+  indxb.bigpnt = nullptr;
 
   hinum[0] = map_version;
   hinum[1] = 1;
@@ -833,7 +829,6 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
     {
       int nb, bcnt, rc, j;
       float fdum;
-      unsigned char *map;
       unsigned char ibmfloat[4];
 
       /* calculate the size of the ver==1 index file */
@@ -843,11 +838,10 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
 
       /* add additional info */
 
-      nb += 7;     /* base time (+ sec)  for compatibility with earlier version 2
-                      maps */
+      nb += 7;     /* base time (+ sec)  for compatibility with earlier version 2 maps */
       nb += 8 * 4; /* grvals for time <-> grid conversion */
 
-      map = (unsigned char *) Malloc(nb);
+      unsigned char *map = (unsigned char *) Malloc(nb);
 
       bcnt = 0;
       Put1Byte(map, bcnt, 0);
@@ -908,16 +902,14 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
       if (indx.hinum > 0) fwrite(hinum, sizeof(int), indx.hinum, mapfp);
       if (map_version == 1)
         {
-          int *intnumbuf;
-          intnumbuf = (int *) Malloc(indx.intnum * sizeof(int));
+          std::vector<int> intnumbuf(indx.intnum);
           for (i = 0; i < nrecords; i++)
             {
               intnumbuf[i * 3 + 0] = (int) bignum[i * 2];
               intnumbuf[i * 3 + 1] = (int) bignum[i * 2 + 1];
               intnumbuf[i * 3 + 2] = intnum[i];
             }
-          if (indx.intnum > 0) fwrite(intnumbuf, sizeof(int), indx.intnum, mapfp);
-          Free(intnumbuf);
+          if (indx.intnum > 0) fwrite(intnumbuf.data(), sizeof(int), indx.intnum, mapfp);
           if (indx.fltnum > 0) fwrite(fltnum, sizeof(float), indx.fltnum, mapfp);
         }
       else
@@ -948,7 +940,7 @@ Gradsdes(void *process)
   int varID;
   int levelID;
   int nrecs;
-  char *idxfile = NULL;
+  char *idxfile = nullptr;
   char varname[CDI_MAX_NAME];
   bool yrev = false;
   bool zrev = false;
@@ -970,30 +962,28 @@ Gradsdes(void *process)
   int map_version = 2;
   int maxrecs = 0;
   int monavg = -1;
-  int *intnum = NULL;
-  float *fltnum = NULL;
-  off_t *bignum = NULL;
-  double *array = NULL;
+  std::vector<int> intnum;
+  std::vector<float> fltnum;
+  std::vector<off_t> bignum;
+  Varray<double> array;
   const char *cmons[] = { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };
 
   cdoInitialize(process);
 
   // clang-format off
-  int GRADSDES = cdoOperatorAdd("gradsdes",  0, 0, NULL);
-  int DUMPMAP  = cdoOperatorAdd("dumpmap",   0, 0, NULL);
+  const auto GRADSDES = cdoOperatorAdd("gradsdes",  0, 0, nullptr);
+  const auto DUMPMAP  = cdoOperatorAdd("dumpmap",   0, 0, nullptr);
   // clang-format on
 
-  UNUSED(GRADSDES);
+  (void)(GRADSDES); //CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  const char *datfile = cdoGetStreamName(0).c_str();
-  size_t len = strlen(datfile);
+  auto datfile = cdoGetStreamName(0);
+  const auto len = strlen(datfile);
   char *ctlfile = (char *) Malloc(len + 10);
   strcpy(ctlfile, datfile);
 
-  if (cdoGetStreamName(0).c_str()[0] == '-') cdoAbort("This operator does not work with pipes!");
-
   if (operatorID == DUMPMAP)
     {
       dumpmap();
@@ -1005,38 +995,34 @@ Gradsdes(void *process)
 
   if (operatorArgc() == 1)
     {
-      map_version = parameter2int(operatorArgv()[0]);
+      map_version = parameter2int(cdoOperatorArgv(0));
       if (map_version != 1 && map_version != 2 && map_version != 4) cdoAbort("map_version=%d unsupported!", map_version);
     }
   else
     {
-      if (fileSize(cdoGetStreamName(0).c_str()) > 2147483647L) map_version = 4;
+      if (fileSize(cdoGetStreamName(0)) > 2147483647L) map_version = 4;
     }
 
-  if (cdoVerbose) cdoPrint("GrADS GRIB map version: %d", map_version);
+  if (Options::cdoVerbose) cdoPrint("GrADS GRIB map version: %d", map_version);
 
   if (map_version == 4 && sizeof(off_t) != 8)
-    cdoAbort("GrADS GRIB map version %d requires size of off_t to be 8! The "
-             "size of off_t is %ld.",
-             map_version, sizeof(off_t));
+    cdoAbort("GrADS GRIB map version %d requires size of off_t to be 8! The size of off_t is %ld.", map_version, sizeof(off_t));
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID = cdoOpenRead(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
-  int nvars = vlistNvars(vlistID);
-  int ntsteps = vlistNtsteps(vlistID);
-  int ngrids = vlistNgrids(vlistID);
+  const auto nvars = vlistNvars(vlistID);
+  const auto ntsteps = vlistNtsteps(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
 
-  int filetype = pstreamInqFiletype(streamID);
-  int byteorder = pstreamInqByteorder(streamID);
+  auto filetype = cdoInqFiletype(streamID);
+  auto byteorder = cdoInqByteorder(streamID);
 
-  if (filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C
-      || filetype == CDI_FILETYPE_NC5)
+  if (filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C || filetype == CDI_FILETYPE_NC5)
     filetype = CDI_FILETYPE_NC;
 
-  if (filetype != CDI_FILETYPE_SRV && filetype != CDI_FILETYPE_EXT && filetype != CDI_FILETYPE_IEG
-      && filetype != CDI_FILETYPE_GRB)
+  if (filetype != CDI_FILETYPE_SRV && filetype != CDI_FILETYPE_EXT && filetype != CDI_FILETYPE_IEG && filetype != CDI_FILETYPE_GRB)
     {
       if (filetype == CDI_FILETYPE_NC)
         //        cdoAbort("Unsupported file format: NetCDF");
@@ -1053,16 +1039,17 @@ Gradsdes(void *process)
     {
       gridID = vlistGrid(vlistID, index);
       gridtype = gridInqType(gridID);
-      int projtype = gridInqProjType(gridID);
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC))
+      const auto projtype = gridInqProjType(gridID);
+      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC)
+          || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL))
         break;
     }
 
   if (index == ngrids) cdoAbort("No Lon/Lat, Gaussian or Lambert grid found (%s data unsupported)!", gridNamePtr(gridtype));
 
   /* select all variables with used gridID */
-  int *vars = (int *) Malloc(nvars * sizeof(int));
-  int *recoffset = (int *) Malloc(nvars * sizeof(int));
+  std::vector<int> vars(nvars);
+  std::vector<int> recoffset(nvars);
   int nvarsout = 0;
   int nrecsout = 0;
   for (varID = 0; varID < nvars; varID++)
@@ -1074,21 +1061,19 @@ Gradsdes(void *process)
               prec = vlistInqVarDatatype(vlistID, varID);
               if (prec == CDI_DATATYPE_FLT64) flt64 = true;
             }
-          vars[varID] = TRUE;
+          vars[varID] = true;
           recoffset[varID] = nrecsout;
           nvarsout++;
           nrecsout += zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
           if (ntsteps != 1 && ntsteps != 0 && vlistInqVarTimetype(vlistID, varID) == TIME_CONSTANT)
-            cdoAbort("Unsupported GrADS record structure! Variable %d has only "
-                     "1 time step.",
-                     vlistInqVarCode(vlistID, varID));
+            cdoAbort("Unsupported GrADS record structure! Variable %d has only 1 time step.", vlistInqVarCode(vlistID, varID));
         }
       else
         {
           vlistInqVarName(vlistID, varID, varname);
-          cdoPrint("Unsupported grid type >%s<, skipped variable %s!",
-                   gridNamePtr(gridInqType(vlistInqVarGrid(vlistID, varID))), varname);
-          vars[varID] = FALSE;
+          cdoPrint("Unsupported grid type >%s<, skipped variable %s!", gridNamePtr(gridInqType(vlistInqVarGrid(vlistID, varID))),
+                   varname);
+          vars[varID] = false;
         }
     }
 
@@ -1125,18 +1110,13 @@ Gradsdes(void *process)
   repl_filetypeext(ctlfile, filetypeext(filetype), ".ctl");
 
   /* open ctl file*/
-  FILE *gdp = fopen(ctlfile, "w");
-  if (gdp == NULL) cdoAbort("Open failed on %s", ctlfile);
+  auto gdp = fopen(ctlfile, "w");
+  if (gdp == nullptr) cdoAbort("Open failed on %s", ctlfile);
 
   /* VERSION */
   fprintf(gdp, "* Generated by CDO operator gradsdes\n");
   fprintf(gdp, "*\n");
-  /*
-#if defined(VERSION)
-  fprintf(gdp, "* Generated by CDO version %s\n", VERSION);
-  fprintf(gdp, "*\n");
-#endif
-  */
+
   /* DSET */
   if (datfile[0] == '/')
     fprintf(gdp, "DSET  %s\n", datfile);
@@ -1144,7 +1124,7 @@ Gradsdes(void *process)
     {
       datfile = strrchr(datfile, '/');
       if (datfile == 0)
-        datfile = cdoGetStreamName(0).c_str();
+        datfile = cdoGetStreamName(0);
       else
         datfile++;
       fprintf(gdp, "DSET  ^%s\n", datfile);
@@ -1186,7 +1166,7 @@ Gradsdes(void *process)
         }
 
       gridsize = vlistGridsizeMax(vlistID);
-      array = (double *) Malloc(gridsize * sizeof(double));
+      array.resize(gridsize);
     }
   else if (filetype == CDI_FILETYPE_NC)
     {
@@ -1198,15 +1178,15 @@ Gradsdes(void *process)
 
   /* TIME */
 
-  int taxisID = vlistInqTaxis(vlistID);
+  const auto taxisID = vlistInqTaxis(vlistID);
 
   bool cal365day = (taxisInqCalendar(taxisID) == CALENDAR_365DAYS);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-      int vdate = taxisInqVdate(taxisID);
-      int vtime = taxisInqVtime(taxisID);
+      const auto vdate = taxisInqVdate(taxisID);
+      const auto vtime = taxisInqVtime(taxisID);
 
       if (tsID == 0)
         {
@@ -1261,7 +1241,7 @@ Gradsdes(void *process)
           if (dt <= 0) dt = 1;
         }
 
-      if (tsID > 0 && tsID < 6 && iik != 3 && (monavg == TRUE || monavg == -1))
+      if (tsID > 0 && tsID < 6 && iik != 3 && (monavg == true || monavg == -1))
         {
           cdiDecodeDate(vdate, &iyy, &imm, &idd);
           cdiDecodeTime(vtime, &ihh, &imn, &isd);
@@ -1278,16 +1258,15 @@ Gradsdes(void *process)
           if (/*idmn == 0 && idhh == 0 &&*/ (iddd == 0 || iddd == 1 || idd > 27) && idmm > 0 && (mdt == 0 || idmm == mdt))
             {
               mdt = idmm;
-              monavg = TRUE;
+              monavg = true;
             }
           else
             {
-              monavg = FALSE;
+              monavg = false;
             }
           /*
           printf("monavg %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
-                 tsID, monavg, mdt, imm , imms, idmm, iyy, iyys, idyy, idd,
-          idds, iddd);
+                 tsID, monavg, mdt, imm , imms, idmm, iyy, iyys, idyy, idd, idds, iddd);
           */
           imns = imn;
           ihhs = ihh;
@@ -1302,21 +1281,21 @@ Gradsdes(void *process)
           if (nrecords >= maxrecs)
             {
               maxrecs = nrecords;
-              intnum = (int *) Realloc(intnum, 1 * maxrecs * sizeof(int));
-              fltnum = (float *) Realloc(fltnum, 3 * maxrecs * sizeof(float));
-              bignum = (off_t *) Realloc(bignum, 2 * maxrecs * sizeof(off_t));
+              intnum.resize(1 * maxrecs);
+              fltnum.resize(3 * maxrecs);
+              bignum.resize(2 * maxrecs);
             }
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID, &varID, &levelID);
-              if (vars[varID] == TRUE)
+              cdoInqRecord(streamID, &varID, &levelID);
+              if (vars[varID] == true)
                 {
-                  pstreamReadRecord(streamID, array, &nmiss);
+                  cdoReadRecord(streamID, array.data(), &nmiss);
 
                   index = (tsID * nrecsout + recoffset[varID] + levelID);
 
-                  pstreamInqGRIBinfo(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
+                  cdoInqGRIBinfo(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
 
                   if (map_version != 4)
                     {
@@ -1324,8 +1303,7 @@ Gradsdes(void *process)
                       if (checksize < 0L || checksize > 2147483647L)
                         {
                           nrecords -= nrecsout;
-                          cdoWarning("File size limit reached for GrADS GRIB "
-                                     "map_version=%d! Only the first %d time "
+                          cdoWarning("File size limit reached for GrADS GRIB map_version=%d! Only the first %d time "
                                      "steps (2GB) are processed.",
                                      map_version, tsID);
                           goto LABEL_STOP;
@@ -1348,7 +1326,7 @@ LABEL_STOP:
 
   /* TDEF */
 
-  if (monavg == TRUE)
+  if (monavg == true)
     {
       dt = mdt;
       iik = 3;
@@ -1385,36 +1363,27 @@ LABEL_STOP:
   ctl_undef(gdp, vlistID);
 
   /* VARS */
-  ctl_vars(gdp, filetype, vlistID, nvarsout, vars);
+  ctl_vars(gdp, filetype, vlistID, nvarsout, vars.data());
 
   /* INDEX file */
   if (filetype == CDI_FILETYPE_GRB)
     {
-      write_map_grib1(idxfile, map_version, nrecords, intnum, fltnum, bignum);
+      write_map_grib1(idxfile, map_version, nrecords, intnum.data(), fltnum.data(), bignum.data());
     }
   if (filetype == CDI_FILETYPE_GRB2)
     {
-      cdoAbort("The fileformat GRIB2 is not fully supported yet for the "
-               "gradsdes operator.\n"
-               "The .ctl file %s was generated. You can add the necessary .idx "
-               "file by running\n\tgribmap -i %s",
+      cdoAbort("The fileformat GRIB2 is not fully supported yet for the gradsdes operator.\n"
+               "The .ctl file %s was generated. You can add the necessary .idx file by running\n\tgribmap -i %s",
                ctlfile, ctlfile);
-      // write_map_grib2(idxfile, map_version, nrecords, intnum, fltnum,
-      // bignum);
+      // write_map_grib2(idxfile, map_version, nrecords, intnum, fltnum, bignum);
     }
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   if (ctlfile) Free(ctlfile);
   if (idxfile) Free(idxfile);
 
-  if (vars) Free(vars);
-  if (recoffset) Free(recoffset);
-  if (array) Free(array);
-  if (intnum) Free(intnum);
-  if (fltnum) Free(fltnum);
-
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
