@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,28 +32,32 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
-#include "cdoOptions.h"
+#include "functs.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
+#include "cdo_options.h"
+#include "progress.h"
+#include "cimdOmp.h"
+
 
 static void
 genBoxGridReg2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 {
   size_t i, j, i1;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
 
-  bool gridHasBounds = (gridInqXbounds(gridID1, NULL) && gridInqYbounds(gridID1, NULL));
+  const bool gridHasBounds = (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr));
 
   {
-    std::vector<double> xvals1(nlon1);
-    std::vector<double> yvals1(nlat1);
-    std::vector<double> xvals2(nlon2);
-    std::vector<double> yvals2(nlat2);
+    Varray<double> xvals1(nlon1);
+    Varray<double> yvals1(nlat1);
+    Varray<double> xvals2(nlon2);
+    Varray<double> yvals2(nlat2);
     gridInqXvals(gridID1, xvals1.data());
     gridInqYvals(gridID1, yvals1.data());
 
@@ -81,10 +85,10 @@ genBoxGridReg2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 
   if (gridHasBounds)
     {
-      std::vector<double> grid1_corner_lon(2 * nlon1);
-      std::vector<double> grid1_corner_lat(2 * nlat1);
-      std::vector<double> grid2_corner_lon(2 * nlon2);
-      std::vector<double> grid2_corner_lat(2 * nlat2);
+      Varray<double> grid1_corner_lon(2 * nlon1);
+      Varray<double> grid1_corner_lat(2 * nlat1);
+      Varray<double> grid2_corner_lon(2 * nlon2);
+      Varray<double> grid2_corner_lat(2 * nlat2);
       gridInqXbounds(gridID1, grid1_corner_lon.data());
       gridInqYbounds(gridID1, grid1_corner_lat.data());
 
@@ -117,28 +121,28 @@ genBoxGridReg2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 static void
 genBoxGridCurv2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 {
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
-  size_t gridsize1 = gridInqSize(gridID1);
-  size_t gridsize2 = gridInqSize(gridID2);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
+  const auto gridsize1 = gridInqSize(gridID1);
+  const auto gridsize2 = gridInqSize(gridID2);
 
   size_t x1, y1, x2, y2;
   size_t use_x1, use_y1;
   size_t g1_add, g2_add;
   int corner, add;
-  int circular = gridIsCircular(gridID1);
+  const auto circular = gridIsCircular(gridID1);
   double on_up, on_lo, ox_up, ox_lo, an_le, an_ri, ax_le, ax_ri;
   double xvals2_0 = 0;
   double area_norm;
 
-  bool gridHasBounds = (gridInqXbounds(gridID1, NULL) && gridInqYbounds(gridID1, NULL));
+  bool gridHasBounds = (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr));
 
-  std::vector<double> xvals1(gridsize1);
-  std::vector<double> yvals1(gridsize1);
-  std::vector<double> xvals2(gridsize2);
-  std::vector<double> yvals2(gridsize2);
+  Varray<double> xvals1(gridsize1);
+  Varray<double> yvals1(gridsize1);
+  Varray<double> xvals2(gridsize2);
+  Varray<double> yvals2(gridsize2);
   gridInqXvals(gridID1, xvals1.data());
   gridInqYvals(gridID1, yvals1.data());
 
@@ -151,8 +155,8 @@ genBoxGridCurv2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
     grid_to_degree(units, gridsize1, yvals1.data(), "grid center lat");
   }
 
-  std::vector<double> grid1_corner_lon, grid1_corner_lat;
-  std::vector<double> grid2_corner_lon, grid2_corner_lat;
+  Varray<double> grid1_corner_lon, grid1_corner_lat;
+  Varray<double> grid2_corner_lon, grid2_corner_lat;
   if (gridHasBounds)
     {
       grid1_corner_lon.resize(4 * gridsize1);
@@ -206,7 +210,7 @@ genBoxGridCurv2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
                       xvals2[g2_add] = xvals1[g1_add] / area_norm;
                       yvals2[g2_add] = yvals1[g1_add] / area_norm;
                     }
-                  else if (fabs(xvals1[g1_add] - xvals2_0) > 270.)
+                  else if (std::fabs(xvals1[g1_add] - xvals2_0) > 270.)
                     {
                       if ((xvals1[g1_add] - xvals2_0) > 270.)
                         xvals2[g2_add] += (xvals1[g1_add] - 360.) / area_norm;
@@ -397,18 +401,18 @@ genBoxGrid(int gridID1, size_t xinc, size_t yinc)
   if (xinc < 1 || yinc < 1) cdoAbort("xinc and yinc must not be smaller than 1!");
 
   int gridID2 = -1;
-  int gridtype = gridInqType(gridID1);
+  const auto gridtype = gridInqType(gridID1);
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR || gridtype == GRID_GENERIC)
     {
-      size_t nlon1 = gridInqXsize(gridID1);
-      size_t nlat1 = gridInqYsize(gridID1);
+      const auto nlon1 = gridInqXsize(gridID1);
+      const auto nlat1 = gridInqYsize(gridID1);
       if (xinc > nlon1 || yinc > nlat1) cdoAbort("xinc and/or yinc exceeds gridsize!");
 
-      size_t nlon2 = nlon1 / xinc;
-      size_t nlat2 = nlat1 / yinc;
+      auto nlon2 = nlon1 / xinc;
+      auto nlat2 = nlat1 / yinc;
       if (nlon1 % xinc) nlon2++;
       if (nlat1 % yinc) nlat2++;
-      size_t gridsize2 = nlon2 * nlat2;
+      const auto gridsize2 = nlon2 * nlat2;
 
       gridID2 = gridCreate(gridtype, gridsize2);
       gridDefXsize(gridID2, nlon2);
@@ -439,87 +443,78 @@ genBoxGrid(int gridID1, size_t xinc, size_t yinc)
 }
 
 static void
-gridboxstat(Field *field1, Field *field2, size_t xinc, size_t yinc, int statfunc)
+gridboxstat(const Field &field1, Field &field2, const size_t xinc, const size_t yinc, const int statfunc)
 {
-  bool useWeight = (field1->weight != NULL);
-  /*
-  double findex = 0;
+  const auto useWeight = !field1.weightv.empty();
 
-  progressInit();
-  */
-
-  size_t gridsize = xinc * yinc;
-  Field *field = (Field *) Malloc(Threading::ompNumThreads * sizeof(Field));
+  const auto gridsize = xinc * yinc;
+  FieldVector field(Threading::ompNumThreads);
   for (int i = 0; i < Threading::ompNumThreads; i++)
     {
-      field[i].size = gridsize;
-      field[i].ptr = (double *) Malloc(gridsize * sizeof(double));
-      field[i].weight = (useWeight) ? (double *) Malloc(gridsize * sizeof(double)) : NULL;
-      field[i].missval = field1->missval;
-      field[i].nmiss = 0;
+      field[i].resize(gridsize);
+      if (useWeight) field[i].weightv.resize(gridsize);
+      field[i].missval = field1.missval;
     }
 
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
-  double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
-  double missval = field1->missval;
+  const auto gridID1 = field1.grid;
+  const auto gridID2 = field2.grid;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
   for (size_t ig = 0; ig < nlat2 * nlon2; ++ig)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
-      /*
-      int lprogress = 1;
-#ifdef  _OPENMP
-      if ( ompthID != 0 ) lprogress = 0;
-#endif
-      findex++;
-      if ( lprogress ) progressStatus(0, 1, findex/nlat2*nlon2);
-      */
-      size_t ilat = ig / nlon2;
-      size_t ilon = ig - ilat * nlon2;
+      const auto ilat = ig / nlon2;
+      const auto ilon = ig - ilat * nlon2;
 
       size_t isize = 0;
       field[ompthID].nmiss = 0;
       for (size_t j = 0; j < yinc; ++j)
         {
-          size_t jj = ilat * yinc + j;
+          const auto jj = ilat * yinc + j;
           if (jj >= nlat1) break;
           for (size_t i = 0; i < xinc; ++i)
             {
-              size_t ii = ilon * xinc + i;
-              size_t index = jj * nlon1 + ii;
+              const auto ii = ilon * xinc + i;
+              const auto index = jj * nlon1 + ii;
               if (ii >= nlon1) break;
-              field[ompthID].ptr[isize] = array1[index];
-              if (useWeight) field[ompthID].weight[isize] = field1->weight[index];
-              if (DBL_IS_EQUAL(field[ompthID].ptr[isize], field[ompthID].missval)) field[ompthID].nmiss++;
+              field[ompthID].vec[isize] = field1.vec[index];
+              if (useWeight) field[ompthID].weightv[isize] = field1.weightv[index];
+              if (DBL_IS_EQUAL(field[ompthID].vec[isize], field[ompthID].missval)) field[ompthID].nmiss++;
               isize++;
             }
         }
 
       field[ompthID].size = isize;
-      field2->ptr[ig] = fldfun(field[ompthID], statfunc);
+      field2.vec[ig] = vfldfun(field[ompthID], statfunc);
     }
 
-  field2->nmiss = arrayNumMV(nlat2 * nlon2, array2, missval);
+  field2.nmiss = fieldNumMiss(field2);
+}
 
-  for (int i = 0; i < Threading::ompNumThreads; i++)
-    {
-      if (field[i].ptr) Free(field[i].ptr);
-      if (field[i].weight) Free(field[i].weight);
-    }
-
-  if (field) Free(field);
+static void
+addOperators(void)
+{
+  // clang-format off
+  cdoOperatorAdd("gridboxrange", func_range, 0, nullptr);
+  cdoOperatorAdd("gridboxmin",   func_min,   0, nullptr);
+  cdoOperatorAdd("gridboxmax",   func_max,   0, nullptr);
+  cdoOperatorAdd("gridboxsum",   func_sum,   0, nullptr);
+  cdoOperatorAdd("gridboxmean",  func_meanw, 1, nullptr);
+  cdoOperatorAdd("gridboxavg",   func_avgw,  1, nullptr);
+  cdoOperatorAdd("gridboxvar",   func_varw,  1, nullptr);
+  cdoOperatorAdd("gridboxvar1",  func_var1w, 1, nullptr);
+  cdoOperatorAdd("gridboxstd",   func_stdw,  1, nullptr);
+  cdoOperatorAdd("gridboxstd1",  func_std1w, 1, nullptr);
+  // clang-format on
 }
 
 void *
@@ -535,72 +530,56 @@ Gridboxstat(void *process)
 
   operatorInputArg("xinc, yinc");
   operatorCheckArgc(2);
-  int xinc = parameter2int(operatorArgv()[0]);
-  int yinc = parameter2int(operatorArgv()[1]);
+  const auto xinc = parameter2int(cdoOperatorArgv(0));
+  const auto yinc = parameter2int(cdoOperatorArgv(1));
 
-  // clang-format off
-  cdoOperatorAdd("gridboxrange", func_range, 0, NULL);
-  cdoOperatorAdd("gridboxmin",   func_min,   0, NULL);
-  cdoOperatorAdd("gridboxmax",   func_max,   0, NULL);
-  cdoOperatorAdd("gridboxsum",   func_sum,   0, NULL);
-  cdoOperatorAdd("gridboxmean",  func_meanw, 1, NULL);
-  cdoOperatorAdd("gridboxavg",   func_avgw,  1, NULL);
-  cdoOperatorAdd("gridboxvar",   func_varw,  1, NULL);
-  cdoOperatorAdd("gridboxvar1",  func_var1w, 1, NULL);
-  cdoOperatorAdd("gridboxstd",   func_stdw,  1, NULL);
-  cdoOperatorAdd("gridboxstd1",  func_std1w, 1, NULL);
-  // clang-format on
+  addOperators();
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
-  bool needWeights = cdoOperatorF2(operatorID) != 0;
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto needWeights = cdoOperatorF2(operatorID) != 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   if (ngrids > 1) cdoAbort("Too many different grids!");
 
-  int gridID1 = vlistGrid(vlistID1, 0);
+  const auto gridID1 = vlistGrid(vlistID1, 0);
   if (gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED)
     cdoAbort("Gaussian reduced grid found. Use option -R to convert it to a regular grid!");
 
-  int gridID2 = genBoxGrid(gridID1, xinc, yinc);
+  const auto gridID2 = genBoxGrid(gridID1, xinc, yinc);
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
+
+  const auto gridsize1 = gridInqSize(gridID1);
 
   Field field1, field2;
-  field_init(&field1);
-  field_init(&field2);
+  field1.resize(gridsize1);
+  if (needWeights) field1.weightv.resize(gridsize1);
 
-  size_t gridsize1 = gridInqSize(gridID1);
-  field1.ptr = (double *) Malloc(gridsize1 * sizeof(double));
-  field1.weight = needWeights ? (double *) Malloc(gridsize1 * sizeof(double)) : NULL;
-
-  size_t gridsize2 = gridInqSize(gridID2);
-  field2.ptr = (double *) Malloc(gridsize2 * sizeof(double));
-  field2.weight = NULL;
+  const auto gridsize2 = gridInqSize(gridID2);
+  field2.resize(gridsize2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
           field1.grid = vlistInqVarGrid(vlistID1, varID);
           field1.size = gridInqSize(field1.grid);
@@ -613,32 +592,26 @@ Gridboxstat(void *process)
           if (needWeights && field1.grid != lastgrid)
             {
               lastgrid = field1.grid;
-              wstatus = gridWeights(field1.grid, field1.weight);
+              wstatus = gridWeights(field1.grid, field1.weightv.data());
             }
           if (wstatus != 0 && tsID == 0 && levelID == 0)
             {
               vlistInqVarName(vlistID1, varID, varname);
-              cdoWarning("Using constant grid cell area weights for variable %s!", varname);
+              cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
             }
 
-          gridboxstat(&field1, &field2, xinc, yinc, operfunc);
+          gridboxstat(field1, field2, xinc, yinc, operfunc);
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, field2.ptr, field2.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
         }
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  if (field1.ptr) Free(field1.ptr);
-  if (field1.weight) Free(field1.weight);
-
-  if (field2.ptr) Free(field2.ptr);
-  if (field2.weight) Free(field2.weight);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
