@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,42 +25,72 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
+#include "cdo_zaxis.h"
+#include <mpim_grid.h>
 #include "constants.h"
 
+
 static inline double
 orthodrome(double px1, double py1, double px2, double py2)
 {
-  return acos(sin(py1) * sin(py2) + cos(py1) * cos(py2) * cos(px2 - px1));
+  return std::acos(std::sin(py1) * std::sin(py2) + std::cos(py1) * std::cos(py2) * std::cos(px2 - px1));
+}
+
+double gridGetPlanetRadius(int gridID);
+
+double
+getPlanetRadius(int gridID)
+{
+  static auto radiusFromEnv = true;
+  if (radiusFromEnv)
+    {
+      radiusFromEnv = false;
+      auto envstr = getenv("PLANET_RADIUS");
+      if (envstr)
+        {
+          const auto fval = atof(envstr);
+          if (fval > 0) PlanetRadius = fval;
+        }
+    }
+
+  auto planetRadius = PlanetRadius;
+
+  const auto gridRadius = gridGetPlanetRadius(gridID);
+  if (gridRadius > 1) planetRadius = gridRadius;
+
+  return planetRadius;
 }
 
 void
 grid_cell_area(int gridID, double *array)
 {
-  int gridtype = gridInqType(gridID);
-  int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  const auto gridtype = gridInqType(gridID);
+  const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
 
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA
       || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR
-      || gridtype == GRID_UNSTRUCTURED)
+      || gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       if (gridHasArea(gridID))
         {
-          if (cdoVerbose) cdoPrint("Using existing grid cell area!");
+          if (Options::cdoVerbose) cdoPrint("Using existing grid cell area!");
           gridInqArea(gridID, array);
         }
       else
         {
-          int status = gridGenArea(gridID, array);
+          const int status = gridGenArea(gridID, array);
           if (status == 1)
-            cdoAbort("%s: Grid corner missing!", __func__);
+            cdoAbort("%s: Cell corner coordinates missing!", __func__);
           else if (status == 2)
             cdoAbort("%s: Can't compute grid cell area for this grid!", __func__);
 
-          size_t ngp = gridInqSize(gridID);
-          for (size_t i = 0; i < ngp; ++i) array[i] *= PlanetRadius * PlanetRadius;
+          const auto planetRadius = getPlanetRadius(gridID);
+          if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
+          const auto ngp = gridInqSize(gridID);
+          for (size_t i = 0; i < ngp; ++i) array[i] *= planetRadius * planetRadius;
         }
     }
   else
@@ -78,42 +108,30 @@ Gridcell(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int GRIDAREA    = cdoOperatorAdd("gridarea",     1,  0, NULL);
-  int GRIDWGTS    = cdoOperatorAdd("gridweights",  1,  0, NULL);
-  int GRIDMASK    = cdoOperatorAdd("gridmask",     0,  0, NULL);
-  int GRIDDX      = cdoOperatorAdd("griddx",       1,  0, NULL);
-  int GRIDDY      = cdoOperatorAdd("griddy",       1,  0, NULL);
-  int GRIDCELLIDX = cdoOperatorAdd("gridcellidx",  0,  0, NULL);
+  const auto GRIDAREA    = cdoOperatorAdd("gridarea",     1,  0, nullptr);
+  const auto GRIDWGTS    = cdoOperatorAdd("gridweights",  1,  0, nullptr);
+  const auto GRIDMASK    = cdoOperatorAdd("gridmask",     0,  0, nullptr);
+  const auto GRIDDX      = cdoOperatorAdd("griddx",       1,  0, nullptr);
+  const auto GRIDDY      = cdoOperatorAdd("griddy",       1,  0, nullptr);
+  const auto GRIDCELLIDX = cdoOperatorAdd("gridcellidx",  0,  0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  bool needRadius = cdoOperatorF1(operatorID) > 0;
-  if (needRadius)
-    {
-      char *envstr = getenv("PLANET_RADIUS");
-      if (envstr)
-        {
-          double fval = atof(envstr);
-          if (fval > 0) PlanetRadius = fval;
-        }
-    }
+  // const bool needRadius = cdoOperatorF1(operatorID) > 0;
 
-  if (cdoVerbose) cdoPrint("PlanetRadius: %g", PlanetRadius);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
 
   if (ngrids > 1) cdoWarning("Found more than 1 grid, using the first one!");
 
-  int gridID = vlistGrid(vlistID1, 0);
-  int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  auto gridID = vlistGrid(vlistID1, 0);
+  const auto zaxisID = zaxisFromName("surface");
 
-  int vlistID2 = vlistCreate();
-  int varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_CONSTANT);
+  const auto vlistID2 = vlistCreate();
+  auto varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_CONSTANT);
   vlistDefNtsteps(vlistID2, 0);
 
   if (operatorID == GRIDAREA)
@@ -150,13 +168,14 @@ Gridcell(void *process)
     {
       vlistDefVarName(vlistID2, varID, "gridcellidx");
       vlistDefVarLongname(vlistID2, varID, "grid cell index");
+      vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_INT32);
     }
 
-  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  const int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID);
 
-  size_t gridsize = gridInqSize(gridID);
-  std::vector<double> array(gridsize);
+  auto gridsize = gridInqSize(gridID);
+  Varray<double> array(gridsize);
 
   if (operatorID == GRIDAREA)
     {
@@ -164,13 +183,13 @@ Gridcell(void *process)
     }
   else if (operatorID == GRIDWGTS)
     {
-      int status = gridWeights(gridID, &array[0]);
-      if (status != 0) cdoWarning("Using constant grid cell area weights!");
+      auto status = gridWeights(gridID, &array[0]);
+      if (status != 0) cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
     }
   else if (operatorID == GRIDMASK)
     {
       std::vector<int> mask(gridsize);
-      if (gridInqMask(gridID, NULL))
+      if (gridInqMask(gridID, nullptr))
         {
           gridInqMask(gridID, &mask[0]);
         }
@@ -187,9 +206,10 @@ Gridcell(void *process)
     }
   else if (operatorID == GRIDDX || operatorID == GRIDDY)
     {
-      int gridtype = gridInqType(gridID);
-      int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC || gridtype == GRID_CURVILINEAR)
+      const auto gridtype = gridInqType(gridID);
+      const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC || projtype == CDI_PROJ_STERE
+          || gridtype == GRID_CURVILINEAR)
         {
           double len1 = 0, len2 = 0;
           char units[CDI_MAX_NAME];
@@ -197,20 +217,24 @@ Gridcell(void *process)
           if (gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 1);
 
           gridsize = gridInqSize(gridID);
-          size_t xsize = gridInqXsize(gridID);
-          size_t ysize = gridInqYsize(gridID);
+          const auto xsize = gridInqXsize(gridID);
+          const auto ysize = gridInqYsize(gridID);
 
-          std::vector<double> xv(gridsize);
-          std::vector<double> yv(gridsize);
+          if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+            cdoAbort("Cell center coordinates missing!");
 
-          gridInqXvals(gridID, &xv[0]);
-          gridInqYvals(gridID, &yv[0]);
+          Varray<double> xv(gridsize), yv(gridsize);
+          gridInqXvals(gridID, xv.data());
+          gridInqYvals(gridID, yv.data());
 
           // Convert lat/lon units if required
           gridInqXunits(gridID, units);
           grid_to_radian(units, gridsize, &xv[0], "grid longitudes");
           grid_to_radian(units, gridsize, &yv[0], "grid latitudes");
 
+          const auto planetRadius = getPlanetRadius(gridID);
+          if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
+
           if (operatorID == GRIDDX)
             {
               for (size_t j = 0; j < ysize; ++j)
@@ -232,7 +256,7 @@ Gridcell(void *process)
                         len2 = orthodrome(xv[j * xsize + i], yv[j * xsize + i], xv[j * xsize + i + 1], yv[j * xsize + i + 1]);
                       }
 
-                    array[j * xsize + i] = 0.5 * (len1 + len2) * PlanetRadius;
+                    array[j * xsize + i] = 0.5 * (len1 + len2) * planetRadius;
                   }
             }
           else
@@ -256,7 +280,7 @@ Gridcell(void *process)
                         len2 = orthodrome(xv[j * xsize + i], yv[j * xsize + i], xv[(j + 1) * xsize + i], yv[(j + 1) * xsize + i]);
                       }
 
-                    array[j * xsize + i] = 0.5 * (len1 + len2) * PlanetRadius;
+                    array[j * xsize + i] = 0.5 * (len1 + len2) * planetRadius;
                   }
             }
         }
@@ -269,16 +293,16 @@ Gridcell(void *process)
         }
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefTimestep(streamID2, 0);
-  pstreamDefRecord(streamID2, 0, 0);
-  pstreamWriteRecord(streamID2, &array[0], 0);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefTimestep(streamID2, 0);
+  cdoDefRecord(streamID2, 0, 0);
+  cdoWriteRecord(streamID2, &array[0], 0);
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
