@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,17 +15,20 @@
   GNU General Public License for more details.
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <cassert>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
+#include "printinfo.h"
 
-#include "gradsdeslib.h"
+extern "C"
+{
+#include "lib/gradsdes/gradsdes.h"
+}
 
 static void
 get_dim_vals(dsets_t *pfi, double *vals, int dimlen, int dim)
@@ -73,36 +76,32 @@ rev_vals(double *vals, int n)
     }
 }
 
-static int
-y_is_gauss(double *gridyvals, int ysize)
+static bool
+y_is_gauss(Varray<double> &gridyvals, int ysize)
 {
-  int lgauss = FALSE;
+  bool lgauss = false;
   int i;
 
   if (ysize > 2)
     {
-      double *yvals, *yw;
-      yvals = (double *) Malloc(ysize * sizeof(double));
-      yw = (double *) Malloc(ysize * sizeof(double));
-      gaussaw(yvals, yw, (size_t) ysize);
-      Free(yw);
-      for (i = 0; i < (int) ysize; i++) yvals[i] = asin(yvals[i]) / M_PI * 180.0;
+      Varray<double> yvals(ysize);
+      Varray<double> yw(ysize);
+      gaussianLatitudes(yvals.data(), yw.data(), (size_t) ysize);
+      for (i = 0; i < ysize; i++) yvals[i] = std::asin(yvals[i]) / M_PI * 180.0;
 
-      for (i = 0; i < (int) ysize; i++)
-        if (fabs(yvals[i] - gridyvals[i]) > ((yvals[0] - yvals[1]) / 500)) break;
+      for (i = 0; i < ysize; i++)
+        if (std::fabs(yvals[i] - gridyvals[i]) > ((yvals[0] - yvals[1]) / 500)) break;
 
-      if (i == (int) ysize) lgauss = TRUE;
+      if (i == ysize) lgauss = true;
 
-      /* check S->N */
-      if (lgauss == FALSE)
+      // check S->N
+      if (lgauss == false)
         {
-          for (i = 0; i < (int) ysize; i++)
-            if (fabs(yvals[i] - gridyvals[ysize - i - 1]) > ((yvals[0] - yvals[1]) / 500)) break;
+          for (i = 0; i < ysize; i++)
+            if (std::fabs(yvals[i] - gridyvals[ysize - i - 1]) > ((yvals[0] - yvals[1]) / 500)) break;
 
-          if (i == (int) ysize) lgauss = TRUE;
+          if (i == ysize) lgauss = true;
         }
-
-      Free(yvals);
     }
 
   return lgauss;
@@ -111,36 +110,27 @@ y_is_gauss(double *gridyvals, int ysize)
 static int
 define_grid(dsets_t *pfi)
 {
-  int gridID, gridtype;
-  int lgauss = FALSE;
+  const int nx = pfi->dnum[0];
+  const int ny = pfi->dnum[1];
 
-  int nx = pfi->dnum[0];
-  int ny = pfi->dnum[1];
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
 
-  double *xvals = (double *) Malloc(nx * sizeof(double));
-  double *yvals = (double *) Malloc(ny * sizeof(double));
+  get_dim_vals(pfi, xvals.data(), nx, 0);
+  get_dim_vals(pfi, yvals.data(), ny, 1);
 
-  get_dim_vals(pfi, xvals, nx, 0);
-  get_dim_vals(pfi, yvals, ny, 1);
+  if (pfi->yrflg) rev_vals(yvals.data(), ny);
 
-  if (pfi->yrflg) rev_vals(yvals, ny);
+  const bool lgauss = (pfi->linear[1] == 0) ? y_is_gauss(yvals, (size_t) ny) : false;
 
-  if (pfi->linear[1] == 0) lgauss = y_is_gauss(yvals, (size_t) ny);
+  const int gridtype = lgauss ? GRID_GAUSSIAN : GRID_LONLAT;
 
-  if (lgauss)
-    gridtype = GRID_GAUSSIAN;
-  else
-    gridtype = GRID_LONLAT;
-
-  gridID = gridCreate(gridtype, nx * ny);
+  const int gridID = gridCreate(gridtype, nx * ny);
   gridDefXsize(gridID, nx);
   gridDefYsize(gridID, ny);
 
-  gridDefXvals(gridID, xvals);
-  gridDefYvals(gridID, yvals);
-
-  Free(xvals);
-  Free(yvals);
+  gridDefXvals(gridID, xvals.data());
+  gridDefYvals(gridID, yvals.data());
 
   return gridID;
 }
@@ -195,25 +185,24 @@ Importbinary(void *process)
   int vtime;
   int tcur, told, fnum;
   int tmin = 0, tmax = 0;
-  char *ch = NULL;
+  char *ch = nullptr;
   int nlevels;
   int e, flag;
   size_t rc, recsize;
   int recoffset;
-  char *rec = NULL;
+  char *rec = nullptr;
   struct dt dtim, dtimi;
   double missval;
   double fmin, fmax;
   double *array;
   double sfclevel = 0;
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
   dsets_init(&pfi);
 
-  int status = read_gradsdes((char *) cdoGetStreamName(0).c_str(), &pfi);
-  if (cdoVerbose) fprintf(stderr, "status %d\n", status);
+  int status = read_gradsdes((char *) cdoGetStreamName(0), &pfi);
+  if (Options::cdoVerbose) fprintf(stderr, "status %d\n", status);
   // if ( status ) cdoAbort("Open failed on %s!", pfi.name);
   if (status) cdoAbort("Open failed!");
 
@@ -223,23 +212,23 @@ Importbinary(void *process)
 
   if (nvars == 0) cdoAbort("No variables found!");
 
-  int gridID = define_grid(&pfi);
-  int zaxisID = define_level(&pfi, 0);
+  const auto gridID = define_grid(&pfi);
+  const auto zaxisID = define_level(&pfi, 0);
 
-  int zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
+  const auto zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(zaxisIDsfc, &sfclevel);
 
-  int vlistID = vlistCreate();
+  const auto vlistID = vlistCreate();
 
   int *var_zaxisID = (int *) Malloc(nvars * sizeof(int));
   int *var_dfrm = (int *) Malloc(nrecs * sizeof(int));
-  std::vector<RecordInfo> recinfo(nrecs);
+  std::vector<RecordInfo> recList(nrecs);
 
   int recID = 0;
   for (ivar = 0; ivar < nvars; ++ivar)
     {
       /*
-      if ( cdoVerbose )
+      if ( Options::cdoVerbose )
         fprintf(stderr, "1:%s 2:%s %d %d %d %d 3:%s %d \n",
                 pvar->abbrv, pvar->longnm, pvar->offset, pvar->recoff,
       pvar->levels, pvar->nvardims, pvar->varnm, pvar->var_t);
@@ -277,7 +266,7 @@ Importbinary(void *process)
       vlistDefVarName(vlistID, varID, pvar->abbrv);
       {
         char *longname = pvar->varnm;
-        size_t len = strlen(longname);
+        const auto len = strlen(longname);
         if (longname[0] == '\'' && longname[len - 1] == '\'')
           {
             longname[len - 1] = 0;
@@ -319,8 +308,8 @@ Importbinary(void *process)
       for (levelID = 0; levelID < nlevels; ++levelID)
         {
           if (recID >= nrecs) cdoAbort("Internal problem with number of records!");
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
           var_dfrm[recID] = pvar->dfrm;
           recID++;
         }
@@ -328,17 +317,13 @@ Importbinary(void *process)
       pvar++;
     }
 
-  int calendar = CALENDAR_STANDARD;
-
-  int taxisID = taxisCreate(TAXIS_RELATIVE);
-
+  const auto calendar = CALENDAR_STANDARD;
+  const auto taxisID = cdoTaxisCreate(TAXIS_RELATIVE);
   taxisDefCalendar(taxisID, calendar);
-
   vlistDefTaxis(vlistID, taxisID);
 
-  int streamID = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-
-  pstreamDefVlist(streamID, vlistID);
+  const auto streamID = cdoOpenWrite(1);
+  cdoDefVlist(streamID, vlistID);
 
   gridsize = pfi.dnum[0] * pfi.dnum[1];
   if (pfi.flt64)
@@ -360,7 +345,7 @@ Importbinary(void *process)
       printf("%d %d\n", i, pfi.fnums[i]);
   */
 
-  pfi.infile = NULL;
+  pfi.infile = nullptr;
   tcur = 0;
   e = 1;
   while (1)
@@ -368,10 +353,10 @@ Importbinary(void *process)
       if (pfi.tmplat)
         {
           /* make sure no file is open */
-          if (pfi.infile != NULL)
+          if (pfi.infile != nullptr)
             {
               fclose(pfi.infile);
-              pfi.infile = NULL;
+              pfi.infile = nullptr;
             }
           /* advance to first valid time step for this ensemble */
           if (tcur == 0)
@@ -411,8 +396,8 @@ Importbinary(void *process)
                 }
               gr2t(pfi.grvals[3], (gadouble) tcur, &dtim);
               gr2t(pfi.grvals[3], (gadouble) 1, &dtimi);
-              ch = gafndt(pfi.name, &dtim, &dtimi, pfi.abvals[3], pfi.pchsub1, NULL, tcur, e, &flag);
-              if (ch == NULL) cdoAbort("Couldn't determine data file name for e=%d t=%d!", e, tcur);
+              ch = gafndt(pfi.name, &dtim, &dtimi, pfi.abvals[3], pfi.pchsub1, nullptr, tcur, e, &flag);
+              if (ch == nullptr) cdoAbort("Couldn't determine data file name for e=%d t=%d!", e, tcur);
             }
         }
       else
@@ -424,9 +409,9 @@ Importbinary(void *process)
         }
 
       /* Open this file and position to start of first record */
-      if (cdoVerbose) cdoPrint("Opening file: %s", ch);
+      if (Options::cdoVerbose) cdoPrint("Opening file: %s", ch);
       pfi.infile = fopen(ch, "rb");
-      if (pfi.infile == NULL)
+      if (pfi.infile == nullptr)
         {
           if (pfi.tmplat)
             {
@@ -438,7 +423,7 @@ Importbinary(void *process)
               cdoAbort("Could not open file: %s", ch);
             }
         }
-      if (pfi.tmplat) gree(ch, "312");
+      // if (pfi.tmplat) gree(ch, "312"); //US: ch is a local stack variable, which is not memory allocated by malloc()
 
       /* file header */
       if (pfi.fhdr > 0) fseeko(pfi.infile, pfi.fhdr, SEEK_SET);
@@ -457,15 +442,11 @@ Importbinary(void *process)
           gr2t(pfi.grvals[3], (gadouble)(tsID + 1), &dtim);
           vdate = cdiEncodeDate(dtim.yr, dtim.mo, dtim.dy);
           vtime = cdiEncodeTime(dtim.hr, dtim.mn, 0);
-
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
-
-          if (cdoVerbose) cdoPrint(" Reading timestep: %3d %s %s", tsID + 1, vdatestr, vtimestr);
+          if (Options::cdoVerbose) cdoPrint(" Reading timestep: %3d %s %s", tsID + 1, dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
-          pstreamDefTimestep(streamID, tsID);
+          cdoDefTimestep(streamID, tsID);
 
           for (int recID = 0; recID < nrecs; ++recID)
             {
@@ -537,7 +518,7 @@ Importbinary(void *process)
                       array[i] = pfi.undef;
                       nmiss++;
                     }
-                  else if (DBL_IS_NAN(array[i]))
+                  else if (std::isnan(array[i]))
                     {
                       array[i] = pfi.undef;
                       nmiss++;
@@ -550,14 +531,14 @@ Importbinary(void *process)
                     }
                 }
               /*
-              if ( cdoVerbose )
+              if ( Options::cdoVerbose )
                 printf("%3d %4d %3d %6zu %6zu %12.5g %12.5g\n", tsID, recID,
               recoffset, nmiss, n_nan, fmin, fmax);
               */
-              varID = recinfo[recID].varID;
-              levelID = recinfo[recID].levelID;
-              pstreamDefRecord(streamID, varID, levelID);
-              pstreamWriteRecord(streamID, array, nmiss);
+              varID = recList[recID].varID;
+              levelID = recList[recID].levelID;
+              cdoDefRecord(streamID, varID, levelID);
+              cdoWriteRecord(streamID, array, nmiss);
             }
         }
 
@@ -568,7 +549,7 @@ Importbinary(void *process)
 
   processDefVarNum(vlistNvars(vlistID));
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   vlistDestroy(vlistID);
   gridDestroy(gridID);
@@ -583,5 +564,5 @@ Importbinary(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
