@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
+#include "cdo_defaultValues.h"
+#include <mpim_grid.h>
 
 #define MAX_DSETS 1024
 
@@ -100,20 +103,20 @@ get_grid_info(double c0, double re, int *nrxp, int *nryp, double *r0p, double *s
   double r0, s0, c;
   int nrx, nry;
 
-  git = 2. * pi * re * cos(pi / 6.) / c0;
+  git = 2. * pi * re * std::cos(pi / 6.) / c0;
   /* number of longitude pixels */
-  nrx = 2 * (int) lround(0.5 * git);
+  nrx = 2 * (int) std::lround(0.5 * git);
 
   /* central index in longitude */
   r0 = nrx / 2 + 0.5;
 
   /* resolution in km */
-  c = 2. * pi * re * cos(30. * pi / 180.) / nrx;
+  c = 2. * pi * re * std::cos(30. * pi / 180.) / nrx;
 
   phi = pi / 2.;
-  s90 = re / c * sin(phi) / cos(30. * pi / 180.);
+  s90 = re / c * std::sin(phi) / std::cos(30. * pi / 180.);
 
-  nry = (int) floor(s90);
+  nry = (int) std::floor(s90);
   /* central index in latitude */
   s0 = nry + 0.5;
   /* number of latitude pixels */
@@ -131,7 +134,7 @@ det_lon_atovs(double r, double r0, double lts, double c, double re)
 {
   const double pi = M_PI;
 
-  double xla = (r - r0) * c / re / cos(lts * pi / 180.); /* longitude */
+  double xla = (r - r0) * c / re / std::cos(lts * pi / 180.); /* longitude */
   xla = 180. * xla / pi;
 
   return xla;
@@ -142,8 +145,8 @@ det_lat_atovs(double s, double s0, double lts, double c, double re)
 {
   const double pi = M_PI;
 
-  double siphi = (s - s0) * c * cos(lts * pi / 180.) / re;
-  double phi = 180. * asin(siphi) / pi; /* latitude */
+  double siphi = (s - s0) * c * std::cos(lts * pi / 180.) / re;
+  double phi = 180. * std::asin(siphi) / pi; /* latitude */
 
   return phi;
 }
@@ -166,10 +169,10 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
       return -1;
     }
 
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
-  std::vector<double> xbounds(nx * 2);
-  std::vector<double> ybounds(ny * 2);
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
+  Varray<double> xbounds(nx * 2);
+  Varray<double> ybounds(ny * 2);
 
   for (i = 0; i < nx; ++i)
     {
@@ -209,17 +212,15 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
 }
 
 static int
-defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, double dx, double dy, double p1, double p2,
+defSinusoidalGrid(int nx, int ny, double xmin, double ymax, double dx, double dy, double p1, double p2,
                   double p3, double p4)
 {
-  UNUSED(p1);
-  UNUSED(p2);
-  UNUSED(p3);
-  UNUSED(p4);
-  UNUSED(xmax);
-  UNUSED(ymin);
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  (void)(p1); //CDO_UNUSED
+  (void)(p2); //CDO_UNUSED
+  (void)(p3); //CDO_UNUSED
+  (void)(p4); //CDO_UNUSED
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
 
   for (int i = 0; i < nx; ++i) xvals[i] = xmin + i * dx + dx / 2;
 
@@ -238,16 +239,12 @@ defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double
 }
 
 static int
-defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, double dx, double dy, double a, double lon0,
-            double lat0)
+defLaeaGrid(int nx, int ny, double xmin, double ymax, double dx, double dy, double a, double lon0, double lat0)
 {
-  UNUSED(xmax);
-  UNUSED(ymin);
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
 
   for (int i = 0; i < nx; ++i) xvals[i] = xmin + i * dx + dx / 2;
-
   for (int i = 0; i < ny; ++i) yvals[i] = ymax - i * dy - dy / 2;
 
   int gridID = gridCreate(GRID_PROJECTION, nx * ny);
@@ -345,9 +342,9 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
 
   proj_t proj;
   region_t region;
-  char *projection_name = NULL;
+  char *projection_name = nullptr;
 
-  if (cdoVerbose) cdoPrint("Read geolocation:");
+  if (Options::cdoVerbose) cdoPrint("Read geolocation:");
 
   if (lprojtype)
     {
@@ -362,7 +359,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   str_tid = H5Tcopy(H5T_C_S1);
   H5Tset_size(str_tid, 64);
   dims = 10;
-  fltarr_tid = H5Tarray_create(H5T_NATIVE_FLOAT, 1, &dims, NULL);
+  fltarr_tid = H5Tarray_create(H5T_NATIVE_FLOAT, 1, &dims, nullptr);
 
   proj_tid = H5Tcreate(H5T_COMPOUND, sizeof(proj_t));
   if (projection_name)
@@ -402,7 +399,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   H5Tclose(str_tid);
   H5Tclose(fltarr_tid);
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     cdoPrint("  Projection: name=%s\n\t\t\tellipsoid=%s\n\t\t\tparameter=%g %g %g %g %g %g", proj.name, proj.ellipsoid,
              proj.parameter[0], proj.parameter[1], proj.parameter[2], proj.parameter[3], proj.parameter[4], proj.parameter[5]);
 
@@ -430,20 +427,20 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       double xmin = region.xmin;
       region.xmin = region.xmax;
       region.xmax = xmin;
-      if (cdoVerbose) cdoPrint("  Swap xmin/xmax");
+      if (Options::cdoVerbose) cdoPrint("  Swap xmin/xmax");
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     cdoPrint("  Region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g", region.xmin, region.xmax, region.ymin, region.ymax, region.dx,
              region.dy);
 
   H5Gclose(grp_id);
 
   /* check region */
-  xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-  ysize = (int) lround((region.ymax - region.ymin) / region.dy);
+  xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+  ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
 
-  if (cdoVerbose) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
+  if (Options::cdoVerbose) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
 
   /* some CM-SAF files have incorrect entries for some metadata. */
   /* these are corrected in the following sections. */
@@ -455,13 +452,13 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
     {
       region.xmax = 8887500.0;
       region.ymin = -8887500.0;
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g", region.xmin, region.xmax, region.ymin,
                  region.ymax, region.dx, region.dy);
 
-      xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-      ysize = (int) lround((region.ymax - region.ymin) / region.dy);
-      if (cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
+      xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+      ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
+      if (Options::cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
   if (nx == 298 && ny == 371 && (int) region.xmin == -6709222 && (int) region.xmax == 6709222 && (int) region.ymin == -6664078
@@ -474,9 +471,9 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g", region.xmin, region.xmax, region.ymin,
                region.ymax, region.dx, region.dy);
 
-      xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-      ysize = (int) lround((region.ymax - region.ymin) / region.dy);
-      if (cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
+      xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+      ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
+      if (Options::cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
   if (strcmp(proj.name, "sinusoidal") != 0
@@ -497,7 +494,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
               && (int) region.ymin == -6705000 && (int) region.ymax == 9990000 && (int) region.dx == 45000
               && (int) region.dy == 45000)))
     {
-      if (cdoVerbose) cdoPrint("Replacing incorrect projection parameters for sinusoidal products:");
+      if (Options::cdoVerbose) cdoPrint("Replacing incorrect projection parameters for sinusoidal products:");
       strcpy(proj.ellipsoid, "WGS-84");
       strcpy(proj.name, "sinusoidal");
       proj.parameter[0] = 0.0;
@@ -506,14 +503,14 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       proj.parameter[3] = 0.0;
       proj.parameter[4] = -99.99;
       proj.parameter[5] = -99.99;
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         cdoPrint("proj1 = %g, proj2 = %g, proj3 = %g, proj4 = %g,", proj.parameter[0], proj.parameter[1], proj.parameter[2],
                  proj.parameter[3]);
     }
 
   if (nx == xsize && ny == ysize && strcmp(proj.name, "sinusoidal") == 0 && strcmp(proj.ellipsoid, "WGS-84") == 0)
     {
-      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy,
+      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.ymax, region.dx, region.dy,
                                  proj.parameter[0], proj.parameter[1], proj.parameter[2], proj.parameter[3]);
     }
   /* modification by Frank Kaspar */
@@ -521,15 +518,15 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
            && memcmp(proj.ellipsoid, "Sphere", 6) == 0)
     {
       double a = (proj.parameter[4] < 0) ? 6370997.0 : proj.parameter[4];
-      gridID = defLaeaGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy, a, proj.parameter[2],
+      gridID = defLaeaGrid(nx, ny, region.xmin, region.ymax, region.dx, region.dy, a, proj.parameter[2],
                            proj.parameter[3]);
     }
   else if (memcmp(proj.name, "Cylindrical Equal Area", 22) == 0 && memcmp(proj.ellipsoid, "Sphere", 6) == 0)
     {
-      double c0 = 0.001 * sqrt(proj.parameter[5]); /* nominal spatial resolution */
+      double c0 = 0.001 * std::sqrt(proj.parameter[5]); /* nominal spatial resolution */
       double lts = proj.parameter[3];
       double re = proj.parameter[4] / 1000; /* Earth radius [km]*/
-      if (cdoVerbose) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
+      if (Options::cdoVerbose) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
       gridID = defLonLatGrid(nx, ny, c0, lts, re);
     }
   else if (nx == 386 && ny == 162)
@@ -537,7 +534,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       double c0 = 90; /* nominal spatial resolution */
       double lts = 30;
       double re = 6371.228; /* Earth radius [km]*/
-      if (cdoVerbose) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
+      if (Options::cdoVerbose) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
       gridID = defLonLatGrid(nx, ny, c0, lts, re);
     }
 
@@ -567,14 +564,14 @@ read_region(hid_t loc_id, int nx, int ny)
   char proj[128];
   double a, lon0, lat0;
 
-  if (cdoVerbose) cdoPrint("Read region:");
+  if (Options::cdoVerbose) cdoPrint("Read region:");
 
   /*
    * Create a data type for region
    */
   hid_t region_tid = H5Tcreate(H5T_COMPOUND, sizeof(region_t));
   hsize_t dims = 4;
-  hid_t fltarr_tid = H5Tarray_create(H5T_NATIVE_DOUBLE, 1, &dims, NULL);
+  hid_t fltarr_tid = H5Tarray_create(H5T_NATIVE_DOUBLE, 1, &dims, nullptr);
   hid_t str64_tid = H5Tcopy(H5T_C_S1);
   H5Tset_size(str64_tid, 128);
   hid_t str128_tid = H5Tcopy(H5T_C_S1);
@@ -611,9 +608,9 @@ read_region(hid_t loc_id, int nx, int ny)
   }
   */
   herr_t status = H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
-  UNUSED(status);
+  (void)(status); //CDO_UNUSED
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       printf("area_extent[0] = %g\n", region.area_extent[0]);
       printf("area_extent[1] = %g\n", region.area_extent[1]);
@@ -644,7 +641,7 @@ read_region(hid_t loc_id, int nx, int ny)
 
   int nfound = scan_pcs_def(region.pcs_def, proj, &a, &lon0, &lat0);
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       printf("proj = %s\n", proj);
       printf("a    = %g\n", a);
@@ -663,12 +660,12 @@ read_region(hid_t loc_id, int nx, int ny)
   xsize = (int)lround((region.xmax-region.xmin)/region.dx);
   ysize = (int)lround((region.ymax-region.ymin)/region.dy);
 
-  if ( cdoVerbose ) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
+  if ( Options::cdoVerbose ) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
   */
 
   if (nfound == 4 && nx == region.xsize && ny == region.ysize && strcmp(proj, "laea") == 0)
     {
-      gridID = defLaeaGrid(nx, ny, xmin, xmax, ymin, ymax, dx, dy, a, lon0, lat0);
+      gridID = defLaeaGrid(nx, ny, xmin, ymax, dx, dy, a, lon0, lat0);
     }
 
   return gridID;
@@ -700,7 +697,6 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
   int dtype = CDI_DATATYPE_FLT32;
   char attstring[4096]; /* Buffer to read string attribute back */
   char varname[CDI_MAX_NAME];
-  double minval, maxval;
   size_t nmiss;
   int num_attrs;
 
@@ -782,7 +778,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
   dataspace = H5Dget_space(dset_id); /* dataspace handle */
   rank = H5Sget_simple_extent_ndims(dataspace);
-  status = H5Sget_simple_extent_dims(dataspace, dims_out, NULL);
+  status = H5Sget_simple_extent_dims(dataspace, dims_out, nullptr);
 
   if (rank == 2)
     {
@@ -830,7 +826,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
   if (nset < MAX_DSETS)
     {
-      if (cdoVerbose) print_filter(dset_id, varname);
+      if (Options::cdoVerbose) print_filter(dset_id, varname);
 
       num_attrs = H5Aget_num_attrs(dset_id);
       for (i = 0; i < num_attrs; i++)
@@ -1050,23 +1046,21 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
       nmiss = 0;
 
-      minval = 1e35;
-      maxval = -1e35;
-      arrayMinMax(gridsize * nt, array, &minval, &maxval);
+      const auto minmax = varrayMinMax(gridsize * nt, array);
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         cdoPrint("Dataset %s: missval = %g  addoffset = %g  scalefactor = %g", varname, missval, addoffset, scalefactor);
 
-      if (cdoVerbose)
-        cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minval, maxval, missval);
+      if (Options::cdoVerbose)
+        cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minmax.first, minmax.second, missval);
 
       if (dtype == CDI_DATATYPE_UINT8)
         {
-          if (minval >= 0 && maxval <= 127) dtype = CDI_DATATYPE_INT8;
+          if (minmax.first >= 0 && minmax.second <= 127) dtype = CDI_DATATYPE_INT8;
         }
       else if (dtype == CDI_DATATYPE_UINT16)
         {
-          if (minval >= 0 && maxval <= 32767) dtype = CDI_DATATYPE_INT16;
+          if (minmax.first >= 0 && minmax.second <= 32767) dtype = CDI_DATATYPE_INT16;
         }
 
       laddoffset = IS_NOT_EQUAL(addoffset, 0);
@@ -1089,8 +1083,8 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
               }
         }
 
-      minval = 1e35;
-      maxval = -1e35;
+      double minval = 1e35;
+      double maxval = -1e35;
       for (size_t i = 0; i < gridsize * nt; i++)
         if (mask[i] == false)
           {
@@ -1098,10 +1092,10 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
             if (array[i] > maxval) maxval = array[i];
           }
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minval, maxval, missval);
 
-      if (nmiss > 0)
+      if (nmiss)
         {
           if (!(missval < minval || missval > maxval))
             {
@@ -1147,50 +1141,50 @@ static herr_t
 obj_info(hid_t loc_id, const char *name, void *opdata)
 {
   H5G_stat_t statbuf;
-  H5Gget_objinfo(loc_id, name, FALSE, &statbuf);
+  H5Gget_objinfo(loc_id, name, false, &statbuf);
 
   H5G_obj_t obj_type = statbuf.type;
 
   switch (obj_type)
     {
     case H5G_GROUP:
-      if (cdoVerbose) cdoPrint(" Object with name %s is a group", name);
+      if (Options::cdoVerbose) cdoPrint(" Object with name %s is a group", name);
       if (strcmp(name, "Data") == 0)
         {
-          ((datasets_t *) opdata)->mergelevel = TRUE;
-          H5Giterate(loc_id, name, NULL, obj_info, opdata);
+          ((datasets_t *) opdata)->mergelevel = true;
+          H5Giterate(loc_id, name, nullptr, obj_info, opdata);
         }
       else if (strcmp(name, "Geolocation") == 0)
         {
-          ((datasets_t *) opdata)->lgeoloc = TRUE;
+          ((datasets_t *) opdata)->lgeoloc = true;
         }
       else if (strcmp(name, "Metadata") == 0)
         {
-          ((datasets_t *) opdata)->lmetadata = TRUE;
+          ((datasets_t *) opdata)->lmetadata = true;
         }
       break;
     case H5G_DATASET:
-      if (cdoVerbose) cdoPrint(" Object with name %s is a dataset", name);
+      if (Options::cdoVerbose) cdoPrint(" Object with name %s is a dataset", name);
       if (strstr(name, "PALETTE"))
         {
-          if (cdoVerbose) cdoPrint("   Skip dataset: %s", name);
+          if (Options::cdoVerbose) cdoPrint("   Skip dataset: %s", name);
         }
       /*else if ( strstr(name, "egion") ) */
       else if (strcmp(name, "region") == 0)
         {
-          ((datasets_t *) opdata)->lregion = TRUE;
+          ((datasets_t *) opdata)->lregion = true;
         }
       else
         {
-          if (cdoVerbose) cdoPrint("   Read dataset: %s", name);
+          if (Options::cdoVerbose) cdoPrint("   Read dataset: %s", name);
           read_dataset(loc_id, name, opdata);
         }
       break;
     case H5G_TYPE:
-      if (cdoVerbose) cdoPrint(" Object with name %s is a named datatype", name);
+      if (Options::cdoVerbose) cdoPrint(" Object with name %s is a named datatype", name);
       if (strcmp(name, "ProjType") == 0)
         {
-          ((datasets_t *) opdata)->lprojtype = TRUE;
+          ((datasets_t *) opdata)->lprojtype = true;
         }
       break;
     default: cdoAbort(" Unable to identify an object %s", name); break;
@@ -1297,17 +1291,17 @@ dsets_init(datasets_t *dsets)
       dsets->obj[i].nx = 0;
       dsets->obj[i].ny = 0;
       dsets->obj[i].nz = 0;
-      dsets->obj[i].name = NULL;
-      dsets->obj[i].description = NULL;
-      dsets->obj[i].units = NULL;
-      dsets->obj[i].title = NULL;
-      dsets->obj[i].time = NULL;
-      dsets->obj[i].dtype = cdoDefaultDataType;
+      dsets->obj[i].name = nullptr;
+      dsets->obj[i].description = nullptr;
+      dsets->obj[i].units = nullptr;
+      dsets->obj[i].title = nullptr;
+      dsets->obj[i].time = nullptr;
+      dsets->obj[i].dtype = CdoDefault::DataType;
       dsets->obj[i].lscale = 0;
       dsets->obj[i].loffset = 0;
       dsets->obj[i].lmissval = 0;
       dsets->obj[i].missval = cdiInqMissval();
-      dsets->obj[i].array = NULL;
+      dsets->obj[i].array = nullptr;
     }
 }
 #endif
@@ -1316,36 +1310,35 @@ void *
 Importcmsaf(void *process)
 {
 #ifdef HAVE_LIBHDF5
-  int streamID;
-  int gridID = -1, zaxisID, taxisID, vlistID;
+  int gridID = -1, zaxisID;
   int i, offset;
   size_t nmiss;
   int ivar;
   int varID, levelID, tsID;
   double *array;
-  double missval, minval, maxval;
+  double missval;
   hid_t file_id; /* HDF5 File ID	        	*/
   herr_t status; /* Generic return value		*/
   datasets_t dsets;
   int64_t vdate;
   int vtime;
-  int *vtimes = NULL;
+  int *vtimes = nullptr;
 #endif
 
   cdoInitialize(process);
 
-  if (cdoDefaultFileType == CDI_UNDEFID) cdoDefaultFileType = CDI_FILETYPE_NC;
+  if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
 #ifdef HAVE_LIBHDF5
   dsets_init(&dsets);
 
   /* Open an existing file. */
-  file_id = H5Fopen(cdoGetStreamName(0).c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
-  if (file_id < 0) cdoAbort("H5Fopen failed on %s", cdoGetStreamName(0).c_str());
+  file_id = H5Fopen(cdoGetStreamName(0), H5F_ACC_RDONLY, H5P_DEFAULT);
+  if (file_id < 0) cdoAbort("H5Fopen failed on %s", cdoGetStreamName(0));
 
   /* cmsaf_type = get_cmsaf_type(file_id); */
 
-  H5Giterate(file_id, "/", NULL, obj_info, (void *) &dsets);
+  H5Giterate(file_id, "/", nullptr, obj_info, (void *) &dsets);
 
   if (dsets.nsets == 0) cdoAbort("No dataset found!");
 
@@ -1386,7 +1379,7 @@ Importcmsaf(void *process)
         }
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     for (ivar = 0; ivar < dsets.nsets; ++ivar)
       cdoPrint(" Var %d %-20s %dx%d nlev = %d nts = %d", ivar, dsets.obj[ivar].name, nx, ny, nz, dsets.obj[ivar].nt);
 
@@ -1416,30 +1409,21 @@ Importcmsaf(void *process)
     zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
   else
     {
-      std::vector<double> levels(nz);
+      Varray<double> levels(nz);
       for (i = 0; i < nz; ++i) levels[i] = i + 1;
       zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
       zaxisDefLevels(zaxisID, levels.data());
     }
 
-  vlistID = vlistCreate();
-
-  if (nt > 1)
-    taxisID = taxisCreate(TAXIS_RELATIVE);
-  else
-    taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  const auto vlistID = vlistCreate();
 
+  const auto taxisID = cdoTaxisCreate(nt > 1 ? TAXIS_RELATIVE : TAXIS_ABSOLUTE);
   taxisDefCalendar(taxisID, CALENDAR_STANDARD);
-
   vlistDefTaxis(vlistID, taxisID);
 
   for (ivar = 0; ivar < dsets.nsets; ++ivar)
     {
-      if (dsets.obj[ivar].nt > 1)
-        varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
-      else
-        varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
-
+      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
       vlistDefVarName(vlistID, varID, dsets.obj[ivar].name);
       if (dsets.obj[ivar].description) vlistDefVarLongname(vlistID, varID, dsets.obj[ivar].description);
       if (dsets.obj[ivar].units) vlistDefVarUnits(vlistID, varID, dsets.obj[ivar].units);
@@ -1460,9 +1444,9 @@ Importcmsaf(void *process)
   vdate = get_vdate(vlistID);
   if (vdate == 0) vdate = 10101;
 
-  streamID = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const auto streamID = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID, vlistID);
+  cdoDefVlist(streamID, vlistID);
 
   for (tsID = 0; tsID < nt; ++tsID)
     {
@@ -1470,7 +1454,7 @@ Importcmsaf(void *process)
       vtime = 0;
       if (vtimes) vtime = vtimes[tsID];
       taxisDefVtime(taxisID, vtime);
-      pstreamDefTimestep(streamID, tsID);
+      cdoDefTimestep(streamID, tsID);
 
       for (ivar = 0; ivar < dsets.nsets; ++ivar)
         {
@@ -1487,33 +1471,27 @@ Importcmsaf(void *process)
               if (nz == 1) offset = gridsize * tsID;
               array = dsets.obj[ivar].array + offset;
 
-              minval = 1e35;
-              maxval = -1e35;
-              arrayMinMaxMV(gridsize, array, missval, &minval, &maxval);
+              size_t nvals;
+              const auto minmax = varrayMinMaxMV(gridsize, array, missval, nvals);
+              nmiss = gridsize - nvals;
 
-              nmiss = arrayNumMV(gridsize, array, missval);
-
-              if (cdoVerbose)
+              if (Options::cdoVerbose)
                 cdoPrint(" Write var %d,  level %d, nmiss %zu, missval %g, minval %g, maxval %g", varID, levelID, nmiss, missval,
-                         minval, maxval);
-              /*
-                if ( ! (missval < minval || missval > maxval) )
-                cdoWarning(" Missval is inside of valid values! Name: %s  Range:
-                %g - %g  Missval: %g\n", dsets.obj[ivar].name, minval, maxval, missval);
-              */
-              pstreamDefRecord(streamID, varID, levelID);
-              pstreamWriteRecord(streamID, array, nmiss);
+                         minmax.first, minmax.second);
+
+              cdoDefRecord(streamID, varID, levelID);
+              cdoWriteRecord(streamID, array, nmiss);
             }
         }
     }
 
   /* Close file */
   status = H5Fclose(file_id);
-  UNUSED(status);
+  (void)(status); //CDO_UNUSED
 
   processDefVarNum(vlistNvars(vlistID));
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   vlistDestroy(vlistID);
   gridDestroy(gridID);
@@ -1529,5 +1507,5 @@ Importcmsaf(void *process)
   cdoAbort("HDF5 support not compiled in!");
 #endif
 
-  return 0;
+  return nullptr;
 }
