@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,18 +22,37 @@
       Info       map             Dataset information and simple map
 */
 
+#include <cfloat>
+
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "text.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "mpmo_color.h"
 #include "array.h"
 #include "datetime.h"
+#include "printinfo.h"
+#include "cdo_zaxis.h"
+
 
 static void
-printMap(int nlon, int nlat, double *array, double missval, double min, double max)
+printGridIndex(int nlon, int nlat, int ilon, int i)
 {
-  /* source code from PINGO */
+  const int index = nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < i ? 4 : 5;
+
+  std::stringstream s;
+  s << std::string(index, ' ');
+  for (ilon = 0; ilon < nlon; ilon++) s << ((ilon + 1) / i) % 10;
+
+  printf("%s", s.str().c_str());
+  putchar('\n');
+  fflush(stdout);
+}
+
+static void
+printMap(const int nlon, const int nlat, const double *array, const double missval, const double min, const double max)
+{
+  // source code from PINGO
   int ilon, ilat, i;
   double x, a, b;
   double level[10];
@@ -45,15 +64,15 @@ printMap(int nlon, int nlat, double *array, double missval, double min, double m
 
   if (IS_NOT_EQUAL(step, 0))
     {
-      a = pow(10, floor(log(step) / M_LN10));
+      a = std::pow(10, std::floor(std::log(step) / M_LN10));
       b = step / a;
 
       if (b > 5)
-        b = 0.5 * ceil(b / 0.5);
+        b = 0.5 * std::ceil(b / 0.5);
       else if (b > 2)
-        b = 0.2 * ceil(b / 0.2);
+        b = 0.2 * std::ceil(b / 0.2);
       else if (b > 1)
-        b = 0.1 * ceil(b / 0.1);
+        b = 0.1 * std::ceil(b / 0.1);
       else
         b = 1;
 
@@ -61,15 +80,15 @@ printMap(int nlon, int nlat, double *array, double missval, double min, double m
 
       if (min < 0 && max > 0)
         {
-          min_n = (int) floor(10 * (-min) / (max - min) - 0.5);
-          max_n = (int) ceil(10 * (-min) / (max - min) - 0.5);
+          min_n = (int) std::floor(10 * (-min) / (max - min) - 0.5);
+          max_n = (int) std::ceil(10 * (-min) / (max - min) - 0.5);
           level[min_n] = 0;
           for (i = min_n - 1; i >= 0; i--) level[i] = level[i + 1] - step;
           for (i = max_n; i < 9; i++) level[i] = level[i - 1] + step;
         }
       else
         {
-          level[0] = step * ceil(min / step + 0.5);
+          level[0] = step * std::ceil(min / step + 0.5);
           for (i = 1; i < 9; i++) level[i] = level[i - 1] + step;
         }
     }
@@ -146,70 +165,33 @@ printMap(int nlon, int nlat, double *array, double missval, double min, double m
                   }
             }
 
-          if (c == '0')
+          TextMode mode(MODELESS);
+          TextColor color(BLACK);
+          switch (c)
             {
-              set_text_color(stdout, BRIGHT, BLUE);
-            }
-          else if (c == '1')
-            {
-              set_text_color(stdout, RESET, BLUE);
-            }
-          else if (c == '2')
-            {
-              set_text_color(stdout, BRIGHT, CYAN);
-            }
-          else if (c == '3')
-            {
-              set_text_color(stdout, RESET, CYAN);
-            }
-          else if (c == '4')
-            {
-              set_text_color(stdout, RESET, GREEN);
-            }
-          else if (c == '5')
-            {
-              set_text_color(stdout, RESET, YELLOW);
-            }
-          else if (c == '6')
-            {
-              set_text_color(stdout, RESET, RED);
-            }
-          else if (c == '7')
-            {
-              set_text_color(stdout, BRIGHT, RED);
-            }
-          else if (c == '8')
-            {
-              set_text_color(stdout, RESET, MAGENTA);
-            }
-          else if (c == '9')
-            {
-              set_text_color(stdout, BRIGHT, MAGENTA);
-            }
-          else if (c == 'M')
-            {
-              if (bmax)
-                {
-                  bmax = 0;
-                  set_text_color(stdout, BLINK, BLACK);
-                }
-              else
-                {
-                  set_text_color(stdout, RESET, BLACK);
-                }
-            }
-          else if (c == 'm')
-            {
-              if (bmin)
-                {
-                  bmin = 0;
-                  set_text_color(stdout, BLINK, BLACK);
-                }
-              else
-                {
-                  set_text_color(stdout, RESET, BLACK);
-                }
+            /* clang-format off */
+            case '0': mode = BRIGHT   ; color = BLUE    ; break ;
+            case '1': mode = MODELESS ; color = BLUE    ; break ;
+            case '2': mode = BRIGHT   ; color = CYAN    ; break ;
+            case '3': mode = MODELESS ; color = CYAN    ; break ;
+            case '4': mode = MODELESS ; color = GREEN   ; break ;
+            case '5': mode = MODELESS ; color = YELLOW  ; break ;
+            case '6': mode = MODELESS ; color = RED     ; break ;
+            case '7': mode = BRIGHT   ; color = RED     ; break ;
+            case '8': mode = MODELESS ; color = MAGENTA ; break ;
+            case '9': mode = BRIGHT   ; color = MAGENTA ; break ;
+            /* clang-format on */
+            case 'm':
+              bmax == 1 ? mode = BLINK : mode = MODELESS, color = BLACK;
+              if (bmax) bmax = 0;
+              break;
+            case 'M':
+              bmin == 1 ? mode = BLINK : mode = MODELESS, color = BLACK;
+              if (bmin) bmin = 0;
+              break;
             }
+
+          set_text_color(stdout, mode, color);
           putchar(c);
           reset_text_color(stdout);
         }
@@ -219,41 +201,22 @@ printMap(int nlon, int nlat, double *array, double missval, double min, double m
   putchar('\n');
   fflush(stdout);
 
-  if (nlon >= 1000)
+  for (int i = 1; i <= 4; i++)
     {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 1000) % 10);
-      putchar('\n');
-      fflush(stdout);
+      int current = 10000 / std::pow(10, i);
+      if (nlon >= current)
+        {
+          printGridIndex(nlon, nlat, ilon, current);
+        }
     }
-
-  if (nlon >= 100)
-    {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 100) % 10);
-      putchar('\n');
-      fflush(stdout);
-    }
-
-  if (nlon >= 10)
-    {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 10) % 10);
-      putchar('\n');
-      fflush(stdout);
-    }
-
-  printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-  for (ilon = 0; ilon < nlon; ilon++) printf("%d", (ilon + 1) % 10);
-  putchar('\n');
   fflush(stdout);
   putchar('\n');
   fflush(stdout);
 
   for (i = 0; i < 10; i++)
     {
-      printf("%d=%c%+9.3e,%+9.3e%c%s", (int) i, i == 0 || level[i - 1] >= 0 ? '[' : '[', i == 0 ? min : level[i - 1],
-             i == 9 ? max : level[i], i == 9 || level[i] <= 0 ? ']' : ']', i != 2 && i != 5 && i != 8 ? "  " : "");
+      printf("%d=%c%+9.3e,%+9.3e%c%s", i, '[', i == 0 ? min : level[i - 1], i == 9 ? max : level[i],
+             i == 9 || level[i] <= 0 ? ']' : ']', i != 2 && i != 5 && i != 8 ? "  " : "");
 
       if (i == 2 || i == 5 || i == 8)
         {
@@ -302,58 +265,72 @@ Info(void *process)
   size_t imiss = 0;
   char varname[CDI_MAX_NAME];
   char paramstr[32];
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
   // clang-format off
-  int INFO   = cdoOperatorAdd("info",   E_PARAM,  0, NULL);
-  int INFOP  = cdoOperatorAdd("infop",  E_PARAM,  0, NULL);
-  int INFON  = cdoOperatorAdd("infon",  E_NAME,   0, NULL);
-  int INFOC  = cdoOperatorAdd("infoc",  E_CODE,   0, NULL);
-  int XINFON = cdoOperatorAdd("xinfon", E_NAME,   0, NULL);
-  int MAP    = cdoOperatorAdd("map",    E_PARAM,  0, NULL);
+  const auto INFO   = cdoOperatorAdd("info",   E_PARAM,  0, nullptr);
+  const auto INFOP  = cdoOperatorAdd("infop",  E_PARAM,  0, nullptr);
+  const auto INFON  = cdoOperatorAdd("infon",  E_NAME,   0, nullptr);
+  const auto INFOC  = cdoOperatorAdd("infoc",  E_CODE,   0, nullptr);
+  const auto VINFON = cdoOperatorAdd("vinfon", E_NAME,   0, nullptr);
+  const auto XINFON = cdoOperatorAdd("xinfon", E_NAME,   0, nullptr);
+  const auto MAP    = cdoOperatorAdd("map",    E_PARAM,  0, nullptr);
   // clang-format on
 
-  UNUSED(INFO);
-  UNUSED(INFOP);
-  UNUSED(INFON);
-  UNUSED(INFOC);
-  UNUSED(XINFON);
+  (void)(INFO); //CDO_UNUSED
+  (void)(INFOP); //CDO_UNUSED
+  (void)(INFON); //CDO_UNUSED
+  (void)(INFOC); //CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  int operfunc = cdoOperatorF1(operatorID);
+  const bool lvinfo = (operatorID == VINFON || operatorID == XINFON);
 
   DateTimeList dtlist;
 
+  std::string e = "";
+  if (operfunc == E_NAME)
+    e = "Parameter name";
+  else if (operfunc == E_CODE)
+    e = "Code number";
+  else
+    e = "Parameter ID";
+
+  std::string v = (Options::cdoVerbose) ? " : Extra" : "";
+  int indg = 0;
+
   for (int indf = 0; indf < cdoStreamCnt(); indf++)
     {
-      int streamID = cdoStreamOpenRead(indf);
+      const auto streamID = cdoOpenRead(indf);
+      const auto vlistID = cdoStreamInqVlist(streamID);
+      const auto taxisID = vlistInqTaxis(vlistID);
 
-      int vlistID = cdoStreamInqVlist(streamID);
-      int taxisID = vlistInqTaxis(vlistID);
-
-      int nvars = vlistNvars(vlistID);
+      const auto nvars = vlistNvars(vlistID);
       if (nvars == 0) continue;
 
       std::vector<Infostat> infostat(nvars);
 
-      size_t gridsizemax = vlistGridsizeMax(vlistID);
+      auto gridsizemax = vlistGridsizeMax(vlistID);
       if (vlistNumber(vlistID) != CDI_REAL) gridsizemax *= 2;
 
-      std::vector<double> array(gridsizemax);
+      std::vector<float> arrayf;
+      Varray<double> array;
+      if (Options::CDO_Memtype == MEMTYPE_FLOAT)
+        arrayf.resize(gridsizemax);
+      else
+        array.resize(gridsizemax);
 
-      int indg = 0;
+      indg = 0;
       int tsID = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
         {
           dtlist.taxisInqTimestep(taxisID, 0);
-          int64_t vdate = dtlist.getVdate(0);
-          int vtime = dtlist.getVtime(0);
-
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
+          const auto vdate = dtlist.getVdate(0);
+          const auto vtime = dtlist.getVtime(0);
+          const auto vdateString = dateToString(vdate);
+          const auto vtimeString = timeToString(vtime);
 
           for (varID = 0; varID < nvars; ++varID) infostatInit(infostat[varID]);
 
@@ -361,47 +338,39 @@ Info(void *process)
             {
               if ((tsID == 0 && recID == 0) || operatorID == MAP)
                 {
-                  set_text_color(stdout, BRIGHT, BLACK);
-                  fprintf(stdout,
-                          "%6d :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : ", -(indf + 1),
-                          operatorID == XINFON ? "Nlevs" : "Level");
-
-                  if (operfunc == E_NAME)
-                    fprintf(stdout, "Parameter name");
-                  else if (operfunc == E_CODE)
-                    fprintf(stdout, "Code number");
-                  else
-                    fprintf(stdout, "Parameter ID");
-
-                  if (cdoVerbose) fprintf(stdout, " : Extra");
+                  set_text_color(stdout, BRIGHT);
+                  MpMO::Print("%6d :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : %s%s",
+                              -(indf + 1), lvinfo ? "Nlevs" : "Level", e, v);
                   reset_text_color(stdout);
-                  fprintf(stdout, "\n");
                 }
 
-              pstreamInqRecord(streamID, &varID, &levelID);
-              pstreamReadRecord(streamID, array.data(), &nmiss);
+              cdoInqRecord(streamID, &varID, &levelID);
+              if (Options::CDO_Memtype == MEMTYPE_FLOAT)
+                cdoReadRecordF(streamID, arrayf.data(), &nmiss);
+              else
+                cdoReadRecord(streamID, array.data(), &nmiss);
 
-              indg = (operatorID == XINFON) ? varID + 1 : indg + 1;
+              indg = lvinfo ? varID + 1 : indg + 1;
 
-              int param = vlistInqVarParam(vlistID, varID);
-              int code = vlistInqVarCode(vlistID, varID);
-              int gridID = vlistInqVarGrid(vlistID, varID);
-              int zaxisID = vlistInqVarZaxis(vlistID, varID);
-              int number = vlistInqVarNumber(vlistID, varID);
-              size_t gridsize = gridInqSize(gridID);
-              size_t nlevs = zaxisInqSize(zaxisID);
-              double level = cdoZaxisInqLevel(zaxisID, levelID);
-              double missval = vlistInqVarMissval(vlistID, varID);
+              const auto param = vlistInqVarParam(vlistID, varID);
+              const auto code = vlistInqVarCode(vlistID, varID);
+              const auto gridID = vlistInqVarGrid(vlistID, varID);
+              const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+              const auto number = vlistInqVarNumber(vlistID, varID);
+              const auto gridsize = gridInqSize(gridID);
+              const size_t nlevs = zaxisInqSize(zaxisID);
+              const auto level = cdoZaxisInqLevel(zaxisID, levelID);
+              const auto missval = vlistInqVarMissval(vlistID, varID);
 
-              bool loutput = (operatorID != XINFON);
+              bool loutput = !lvinfo;
 
               if (loutput) infostatInit(infostat[varID]);
 
-              Infostat *infostatp = &infostat[varID];
-              infostatp->nlevs += 1;
-              infostatp->nmiss += nmiss;
+              auto &infostatr = infostat[varID];
+              infostatr.nlevs += 1;
+              infostatr.nmiss += nmiss;
 
-              if (nlevs == infostatp->nlevs) loutput = true;
+              if (nlevs == infostatr.nlevs) loutput = true;
 
               if (loutput)
                 {
@@ -409,69 +378,71 @@ Info(void *process)
 
                   if (operfunc == E_NAME) vlistInqVarName(vlistID, varID, varname);
 
-                  set_text_color(stdout, BRIGHT, BLACK);
                   fprintf(stdout, "%6d ", indg);
-                  reset_text_color(stdout);
-                  set_text_color(stdout, RESET, BLACK);
                   fprintf(stdout, ":");
+
+                  set_text_color(stdout, MAGENTA);
+                  fprintf(stdout, "%s %s ", vdateString.c_str(), vtimeString.c_str());
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, MAGENTA);
-                  fprintf(stdout, "%s %s ", vdatestr, vtimestr);
-                  reset_text_color(stdout);
-
-                  set_text_color(stdout, RESET, GREEN);
-                  if (operatorID == XINFON)
+                  set_text_color(stdout, GREEN);
+                  if (lvinfo)
                     fprintf(stdout, "%7zu ", nlevs);
                   else
                     fprintf(stdout, "%7g ", level);
 
-                  fprintf(stdout, "%8zu %7zu ", gridsize, infostatp->nmiss);
-
-                  set_text_color(stdout, RESET, BLACK);
-                  fprintf(stdout, ":");
+                  fprintf(stdout, "%8zu %7zu ", gridsize, infostatr.nmiss);
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, BLUE);
+                  fprintf(stdout, ":");
+
+                  set_text_color(stdout, BLUE);
                 }
 
               if (number == CDI_REAL)
                 {
                   fpeRaised = 0;
 
-                  if (infostatp->nmiss > 0)
+                  if (infostatr.nmiss)
                     {
-                      size_t nvals
-                          = arrayMinMaxSumMV(gridsize, array.data(), missval, &infostatp->min, &infostatp->max, &infostatp->sum);
+                      size_t nvals;
+                      if (Options::CDO_Memtype == MEMTYPE_FLOAT)
+                        nvals = varrayMinMaxSumMV_f(gridsize, arrayf, missval, infostatr.min, infostatr.max, infostatr.sum);
+                      else
+                        nvals = varrayMinMaxSumMV(gridsize, array, missval, infostatr.min, infostatr.max, infostatr.sum);
                       imiss = gridsize - nvals;
-                      infostatp->nvals += nvals;
+                      infostatr.nvals += nvals;
                     }
                   else if (gridsize == 1)
                     {
-                      if (infostatp->nvals == 0)
-                        infostatp->sum = array[0];
+                      const double val = (Options::CDO_Memtype == MEMTYPE_FLOAT) ? arrayf[0] : array[0];
+                      if (infostatr.nvals == 0)
+                        infostatr.sum = val;
                       else
-                        infostatp->sum += array[0];
-                      infostatp->nvals += 1;
+                        infostatr.sum += val;
+                      infostatr.nvals += 1;
                     }
                   else
                     {
-                      arrayMinMaxSum(gridsize, array.data(), &infostatp->min, &infostatp->max, &infostatp->sum);
-                      infostatp->nvals += gridsize;
+                      if (Options::CDO_Memtype == MEMTYPE_FLOAT)
+                        varrayMinMaxSum_f(gridsize, arrayf, infostatr.min, infostatr.max, infostatr.sum);
+                      else
+                        varrayMinMaxSum(gridsize, array, infostatr.min, infostatr.max, infostatr.sum);
+                      infostatr.nvals += gridsize;
                     }
 
                   if (loutput)
                     {
-                      if (infostatp->nvals)
+                      if (infostatr.nvals)
                         {
-                          if (infostatp->nvals == 1)
+                          if (infostatr.nvals == 1)
                             {
-                              fprintf(stdout, "            %#12.5g            ", infostatp->sum);
+                              fprintf(stdout, "            %#12.5g            ", infostatr.sum);
                             }
                           else
                             {
-                              double mean = infostatp->sum / (double) infostatp->nvals;
-                              fprintf(stdout, "%#12.5g%#12.5g%#12.5g", infostatp->min, mean, infostatp->max);
+                              const double mean = infostatr.sum / (double) infostatr.nvals;
+                              fprintf(stdout, "%#12.5g%#12.5g%#12.5g", infostatr.min, mean, infostatr.max);
                             }
                         }
                       else
@@ -483,25 +454,40 @@ Info(void *process)
               else
                 {
                   size_t nvals = 0;
-                  for (size_t i = 0; i < gridsize; i++)
+                  if (Options::CDO_Memtype == MEMTYPE_FLOAT)
                     {
-                      if (!DBL_IS_EQUAL(array[i * 2], missval) && !DBL_IS_EQUAL(array[i * 2 + 1], missval))
+                      for (size_t i = 0; i < gridsize; i++)
                         {
-                          infostatp->sum += array[i * 2];
-                          infostatp->sumi += array[i * 2 + 1];
-                          nvals++;
+                          if (!DBL_IS_EQUAL(arrayf[i * 2], missval) && !DBL_IS_EQUAL(arrayf[i * 2 + 1], missval))
+                            {
+                              infostatr.sum += arrayf[i * 2];
+                              infostatr.sumi += arrayf[i * 2 + 1];
+                              nvals++;
+                            }
                         }
                     }
+                  else
+                    {
+                      for (size_t i = 0; i < gridsize; i++)
+                        {
+                          if (!DBL_IS_EQUAL(array[i * 2], missval) && !DBL_IS_EQUAL(array[i * 2 + 1], missval))
+                            {
+                              infostatr.sum += array[i * 2];
+                              infostatr.sumi += array[i * 2 + 1];
+                              nvals++;
+                            }
+                        }
+                    }
+
                   fpeRaised = 0;
 
                   imiss = gridsize - nvals;
-                  infostatp->nvals += nvals;
+                  infostatr.nvals += nvals;
 
                   if (loutput)
                     {
-                      double arrmean_r = 0, arrmean_i = 0;
-                      if (infostatp->nvals > 0) arrmean_r = infostatp->sum / infostatp->nvals;
-                      if (infostatp->nvals > 0) arrmean_i = infostatp->sumi / infostatp->nvals;
+                      const double arrmean_r = (infostatr.nvals > 0) ? infostatr.sum / infostatr.nvals : 0;
+                      const double arrmean_i = (infostatr.nvals > 0) ? infostatr.sumi / infostatr.nvals : 0;
                       fprintf(stdout, "   -  (%#12.5g,%#12.5g)  -", arrmean_r, arrmean_i);
                     }
                 }
@@ -510,20 +496,18 @@ Info(void *process)
                 {
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, BLACK);
                   fprintf(stdout, " : ");
-                  reset_text_color(stdout);
 
-                  set_text_color(stdout, BRIGHT, GREEN);
+                  // set_text_color(stdout, GREEN);
                   if (operfunc == E_NAME)
                     fprintf(stdout, "%-14s", varname);
                   else if (operfunc == E_CODE)
                     fprintf(stdout, "%4d   ", code);
                   else
                     fprintf(stdout, "%-14s", paramstr);
-                  reset_text_color(stdout);
+                  // reset_text_color(stdout);
 
-                  if (cdoVerbose)
+                  if (Options::cdoVerbose)
                     {
                       char varextra[CDI_MAX_NAME];
                       vlistInqVarExtra(vlistID, varID, varextra);
@@ -533,20 +517,20 @@ Info(void *process)
                   fprintf(stdout, "\n");
                 }
 
-              if (imiss != nmiss && nmiss > 0) cdoPrint("Found %zu of %zu missing values!", imiss, nmiss);
+              if (imiss != nmiss && nmiss) cdoPrint("Found %zu of %zu missing values!", imiss, nmiss);
 
               if (fpeRaised > 0) cdoWarning("floating-point exception reported: %s!", fpe_errstr(fpeRaised));
 
               if (operatorID == MAP)
                 {
-                  size_t nlon = gridInqXsize(gridID);
-                  size_t nlat = gridInqYsize(gridID);
+                  const auto nlon = gridInqXsize(gridID);
+                  const auto nlat = gridInqYsize(gridID);
 
                   if (gridInqType(gridID) == GRID_GAUSSIAN || gridInqType(gridID) == GRID_LONLAT
                       || gridInqType(gridID) == GRID_CURVILINEAR
                       || (gridInqType(gridID) == GRID_GENERIC && nlon * nlat == gridInqSize(gridID) && nlon < 1024))
                     {
-                      printMap(nlon, nlat, array.data(), missval, infostatp->min, infostatp->max);
+                      printMap(nlon, nlat, array.data(), missval, infostatr.min, infostatr.max);
                     }
                 }
             }
@@ -554,10 +538,18 @@ Info(void *process)
           tsID++;
         }
 
-      pstreamClose(streamID);
+      cdoStreamClose(streamID);
+    }
+
+  if (indg > 36 && operatorID != MAP)
+    {
+      set_text_color(stdout, BRIGHT);
+      MpMO::Print("       :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : %s%s",
+                  lvinfo ? "Nlevs" : "Level", e, v);
+      reset_text_color(stdout);
     }
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
