@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,12 +32,17 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include "array.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "color.h"
+#include "printinfo.h"
+#include "interpol.h"
+#include "cdo_zaxis.h"
 
-double intlin(double x, double y1, double x1, double y2, double x2);
 
 static int
 check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
@@ -74,10 +79,9 @@ make_cyclic(double *array1, double *array2, int nlon, int nlat)
 }
 
 static void
-output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval, CPT *cpt)
+output_vrml(int nlon, int nlat, int ngp, const Varray<double> &array, double missval, CPT *cpt)
 {
-  double minval, maxval, meanval;
-  arrayMinMaxMeanMV(ngp, array, missval, &minval, &maxval, &meanval);
+  const auto minmax = varrayMinMaxMV(ngp, array, missval);
 
   double dx = 10. / nlon;
 
@@ -110,8 +114,8 @@ output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval,
   printf("        material Material {}\n");
   printf("      }\n");
   printf("      geometry ElevationGrid {\n");
-  printf("        colorPerVertex TRUE\n");
-  printf("        solid FALSE\n");
+  printf("        colorPerVertex true\n");
+  printf("        solid false\n");
   printf("        xDimension %d\n", nlon);
   printf("        zDimension %d\n", nlat);
   printf("        xSpacing %g\n", dx);
@@ -166,7 +170,7 @@ output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval,
   printf("  ]\n");
   printf("  translation -5 0 %g\n", -5. * nlat / nlon);
   printf("  rotation 0.0 0.0 0.0 0.0\n");
-  printf("  scale 1.0 %g 1.0\n", 0.5 / (maxval - minval));
+  printf("  scale 1.0 %g 1.0\n", 0.5 / (minmax.second - minmax.first));
   printf("}\n");
 }
 
@@ -180,35 +184,34 @@ Outputgmt(void *process)
   size_t nmiss;
   int ninc = 1;
   bool lzon = false, lmer = false, lhov = false;
-  bool lgrid_gen_bounds = false, luse_grid_corner = false;
+  bool luse_grid_corner = false;
   char varname[CDI_MAX_NAME];
-  std::vector<double> grid_center_lat2, grid_center_lon2;
-  std::vector<double> grid_corner_lat, grid_corner_lon;
+  Varray<double> grid_center_lat2, grid_center_lon2;
+  Varray<double> grid_corner_lat, grid_corner_lon;
   std::vector<int> grid_mask;
   CPT cpt;
   char units[CDI_MAX_NAME];
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
   // clang-format off
-  int OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, NULL);
-  int OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, NULL);
-  int OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, NULL);
-  int OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, NULL);
-  int OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, NULL);
-  int OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, NULL);
-  int OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, NULL);
-  int OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, NULL);
+  const auto OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
+  const auto OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
+  const auto OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
+  const auto OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
+  const auto OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
+  const auto OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
+  const auto OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
+  const auto OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == OUTPUTVECTOR)
     {
       operatorInputArg("increment");
       operatorCheckArgc(1);
-      ninc = parameter2int(operatorArgv()[0]);
+      ninc = parameter2int(cdoOperatorArgv(0));
       if (ninc < 1) cdoAbort("Increment must be greater than 0!");
     }
 
@@ -217,43 +220,63 @@ Outputgmt(void *process)
   if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTVRML)
     {
       operatorCheckArgc(1);
-      char *cpt_file = operatorArgv()[0];
+      auto cpt_file = cdoOperatorArgv(0).c_str();
 
-      FILE *cpt_fp = fopen(cpt_file, "r");
-      if (cpt_fp == NULL) cdoAbort("Open failed on color palette table %s", cpt_file);
+      auto cpt_fp = fopen(cpt_file, "r");
+      if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
 
-      int status = cptRead(cpt_fp, &cpt);
+      const auto status = cptRead(cpt_fp, &cpt);
       if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
 
-      if (cdoVerbose) cptWrite(stderr, cpt);
+      if (Options::cdoVerbose) cptWrite(stderr, cpt);
     }
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID = cdoOpenRead(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
-  int taxisID = vlistInqTaxis(vlistID);
+  const auto vlistID = cdoStreamInqVlist(streamID);
+  const auto taxisID = vlistInqTaxis(vlistID);
 
   int varID = 0;
   vlistInqVarName(vlistID, varID, varname);
-  int code = vlistInqVarCode(vlistID, varID);
-  int gridID = vlistInqVarGrid(vlistID, varID);
-  int zaxisID = vlistInqVarZaxis(vlistID, varID);
-  double missval = vlistInqVarMissval(vlistID, varID);
+  const auto code = vlistInqVarCode(vlistID, varID);
+  auto gridID = vlistInqVarGrid(vlistID, varID);
+  const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+  const auto missval = vlistInqVarMissval(vlistID, varID);
 
-  if (gridInqType(gridID) == GRID_GME) gridID = gridToUnstructured(gridID, 1);
+  const auto gridsize = gridInqSize(gridID);
 
-  if (gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR)
+  int gridtype = gridInqType(gridID);
+  if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
     {
-      gridID = gridToCurvilinear(gridID, 1);
-      lgrid_gen_bounds = true;
+      gridID = gridToUnstructured(gridID, 1);
+      gridtype = gridInqType(gridID);
     }
 
-  size_t gridsize = gridInqSize(gridID);
+  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
+    {
+      gridID = gridToCurvilinear(gridID, 1);
+      gridtype = gridInqType(gridID);
+    }
+
+  if (gridtype == GRID_UNSTRUCTURED)
+    {
+      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+        {
+          if (gridInqNumber(gridID) > 0)
+            {
+              gridID = referenceToGrid(gridID);
+              if (gridID == -1) cdoAbort("Reference to source grid not found!");
+            }
+        }
+    }
+
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
+
   int nlon = gridInqXsize(gridID);
   int nlat = gridInqYsize(gridID);
   int nlev = zaxisInqSize(zaxisID);
 
-  if (gridInqMaskGME(gridID, NULL))
+  if (gridInqMaskGME(gridID, nullptr))
     {
       grid_mask.resize(gridsize);
       gridInqMaskGME(gridID, grid_mask.data());
@@ -270,9 +293,9 @@ Outputgmt(void *process)
       nlat = 1;
     }
 
-  if (cdoVerbose && lhov) cdoPrint("Process hovmoeller data");
-  if (cdoVerbose && lzon) cdoPrint("Process zonal data");
-  if (cdoVerbose && lmer) cdoPrint("Process meridional data");
+  if (Options::cdoVerbose && lhov) cdoPrint("Process hovmoeller data");
+  if (Options::cdoVerbose && lzon) cdoPrint("Process zonal data");
+  if (Options::cdoVerbose && lmer) cdoPrint("Process meridional data");
   /*
   if ( lzon || lmer )
     {
@@ -285,13 +308,11 @@ Outputgmt(void *process)
       if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdoAbort("Bounds not available hovmoeller data!");
     }
 
-  int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
+  const int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-  bool grid_is_circular = gridIsCircular(gridID);
-
-  std::vector<double> grid_center_lat(gridsize);
-  std::vector<double> grid_center_lon(gridsize);
+  const bool grid_is_circular = gridIsCircular(gridID);
 
+  Varray<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
   gridInqYvals(gridID, grid_center_lat.data());
   gridInqXvals(gridID, grid_center_lon.data());
 
@@ -317,7 +338,7 @@ Outputgmt(void *process)
 
       for (int j = 0; j < nlat; ++j)
         {
-          int ij2 = j * (nlon + 1);
+          const int ij2 = j * (nlon + 1);
           grid_center_lon2[ij2 + nlon] += 360;
         }
 
@@ -326,44 +347,34 @@ Outputgmt(void *process)
       plat = grid_center_lat2.data();
     }
 
-  std::vector<double> zaxis_center_lev(nlev);
-  std::vector<double> zaxis_lower_lev(nlev);
-  std::vector<double> zaxis_upper_lev(nlev);
+  Varray<double> zaxis_center_lev(nlev);
+  Varray<double> zaxis_lower_lev(nlev);
+  Varray<double> zaxis_upper_lev(nlev);
 
   cdoZaxisInqLevels(zaxisID, zaxis_center_lev.data());
 
   if (luse_grid_corner)
     {
-      if (ncorner == 0) cdoAbort("grid corner missing!");
-      size_t nalloc = ncorner * gridsize;
+      if (ncorner == 0) cdoAbort("Number of cell corners undefined!");
+      const size_t nalloc = ncorner * gridsize;
       grid_corner_lat.resize(nalloc);
       grid_corner_lon.resize(nalloc);
 
-      if (gridInqYbounds(gridID, NULL) && gridInqXbounds(gridID, NULL))
+      if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
         {
           gridInqYbounds(gridID, grid_corner_lat.data());
           gridInqXbounds(gridID, grid_corner_lon.data());
         }
       else
         {
-          if (lgrid_gen_bounds)
-            {
-              char xunitstr[CDI_MAX_NAME];
-              char yunitstr[CDI_MAX_NAME];
-              gridInqXunits(gridID, xunitstr);
-              gridInqYunits(gridID, yunitstr);
-              if (!lzon) grid_cell_center_to_bounds_X2D(xunitstr, nlon, nlat, grid_center_lon.data(), grid_corner_lon.data(), 0);
-              if (!lmer) grid_cell_center_to_bounds_Y2D(yunitstr, nlon, nlat, grid_center_lat.data(), grid_corner_lat.data());
-            }
-          else
-            cdoAbort("Grid corner missing!");
+          cdoAbort("Cell corner coordinates missing!");
         }
 
       /* Note: using units from latitude instead from bounds */
       grid_to_degree(units, ncorner * gridsize, grid_corner_lon.data(), "grid corner lon");
       grid_to_degree(units, ncorner * gridsize, grid_corner_lat.data(), "grid corner lat");
 
-      if (zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL))
+      if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
         {
           zaxisInqLbounds(zaxisID, zaxis_lower_lev.data());
           zaxisInqUbounds(zaxisID, zaxis_upper_lev.data());
@@ -376,23 +387,23 @@ Outputgmt(void *process)
           zaxis_upper_lev[nlev - 1] = zaxis_center_lev[nlev - 1];
           for (int i = 0; i < nlev - 1; ++i) zaxis_upper_lev[i] = zaxis_lower_lev[i + 1];
 
-          if (cdoVerbose)
+          if (Options::cdoVerbose)
             for (int i = 0; i < nlev; ++i)
               fprintf(stderr, "level: %d %g %g %g\n", i + 1, zaxis_lower_lev[i], zaxis_center_lev[i], zaxis_upper_lev[i]);
         }
     }
 
-  std::vector<double> array(gridsize);
+  Varray<double> array(gridsize);
   double *parray = array.data();
 
-  std::vector<double> array2;
+  Varray<double> array2;
   if (operatorID == OUTPUTCENTER2 && grid_is_circular)
     {
       array2.resize(nlat * (nlon + 1));
       parray = array2.data();
     }
 
-  std::vector<double> uf, vf, alpha, auv;
+  Varray<double> uf, vf, alpha, auv;
   if (operatorID == OUTPUTVECTOR)
     {
       uf.resize(gridsize);
@@ -404,11 +415,10 @@ Outputgmt(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-      int vdate = taxisInqVdate(taxisID);
-      int vtime = taxisInqVtime(taxisID);
-
-      date2str(vdate, vdatestr, sizeof(vdatestr));
-      time2str(vtime, vtimestr, sizeof(vtimestr));
+      const auto vdate = taxisInqVdate(taxisID);
+      const auto vtime = taxisInqVtime(taxisID);
+      const auto vdateString = dateToString(vdate);
+      const auto vtimeString = timeToString(vtime);
 
       if (tsID == 0 && operatorID != OUTPUTTRI)
         {
@@ -429,43 +439,43 @@ Outputgmt(void *process)
 
           if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
           fprintf(stdout, "#\n");
-          fprintf(stdout, "# File  = %s\n", cdoGetStreamName(0).c_str());
-          fprintf(stdout, "# Date  = %s\n", vdatestr);
-          fprintf(stdout, "# Time  = %s\n", vtimestr);
-          fprintf(stdout, "# Name  = %s\n", varname);
-          fprintf(stdout, "# Code  = %d\n", code);
+          fprintf(stdout, "# Stream = %s\n", cdoGetStreamName(0));
+          fprintf(stdout, "# Date   = %s\n", vdateString.c_str());
+          fprintf(stdout, "# Time   = %s\n", vtimeString.c_str());
+          fprintf(stdout, "# Name   = %s\n", varname);
+          fprintf(stdout, "# Code   = %d\n", code);
         }
 
       varID0 = varID;
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID, &varID, &levelID);
+          cdoInqRecord(streamID, &varID, &levelID);
 
           if (varID != varID0) continue;
           if (recID > 0 && !lzon && !lmer) continue;
 
-          pstreamReadRecord(streamID, array.data(), &nmiss);
+          cdoReadRecord(streamID, array.data(), &nmiss);
 
           if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
-          double level = zaxis_center_lev[levelID];
+          const double level = zaxis_center_lev[levelID];
 
-          if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level = %g\n", level);
+          if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level  = %g\n", level);
           if (lhov) fprintf(stdout, "# Timestep = %d\n", tsID + 1);
 
           if (operatorID != OUTPUTTRI) fprintf(stdout, "#\n");
 
           if (operatorID == OUTPUTCENTER || operatorID == OUTPUTCENTER2 || operatorID == OUTPUTCENTERCPT)
             {
-              if (cdoVerbose)
+              if (Options::cdoVerbose)
                 {
-                  double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, &minval, &maxval, &meanval);
-                  double range = maxval - minval;
+                  const auto minmax = varrayMinMaxMV(gridsize, array, missval);
+                  const double minval = minmax.first;
+                  const double maxval = minmax.second;
+                  const double range = maxval - minval;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
-                  fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt "
-                                  "data.gmt > gmtplot.ps\n");
+                  fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt data.gmt > gmtplot.ps\n");
                   fprintf(stderr, "pscoast -O -J -R -Dc -W -B40g20 >> gmtplot.ps\n");
                 }
 
@@ -502,20 +512,18 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTTRI)
             {
-              int c1, c2, c3;
-              int ip1;
               if (gridInqType(gridID) != GRID_CURVILINEAR) cdoAbort("Unsupported grid!");
 
-              int mlon = nlon - 1;
-              /* if ( gridIsCircular(gridID) ) mlon = nlon; */
+              const int mlon = nlon - 1;
+              // if ( gridIsCircular(gridID) ) mlon = nlon;
               for (int j = 0; j < nlat - 1; ++j)
                 for (int i = 0; i < mlon; ++i)
                   {
-                    ip1 = i + 1;
+                    int ip1 = i + 1;
                     if (i == nlon - 1) ip1 = 0;
-                    c1 = (j) *nlon + ip1;
-                    c2 = (j) *nlon + i;
-                    c3 = (j + 1) * nlon + i;
+                    int c1 = (j) *nlon + ip1;
+                    int c2 = (j) *nlon + i;
+                    int c3 = (j + 1) * nlon + i;
                     fprintf(stdout, "%d   %d   %d\n", c1, c2, c3);
                     c1 = (j) *nlon + i + 1;
                     c2 = (j + 1) * nlon + i;
@@ -527,16 +535,16 @@ Outputgmt(void *process)
             {
               if (nrecs < 2) cdoAbort("Too few fields!");
 
-              arrayCopy(gridsize, array.data(), uf.data());
-              pstreamInqRecord(streamID, &varID, &levelID);
-              pstreamReadRecord(streamID, vf.data(), &nmiss);
+              varrayCopy(gridsize, array, uf);
+              cdoInqRecord(streamID, &varID, &levelID);
+              cdoReadRecord(streamID, vf.data(), &nmiss);
 
               for (int j = 0; j < nlat; j += ninc)
                 for (int i = 0; i < nlon; i += ninc)
                   {
-                    /* compute length of velocity vector */
+                    // compute length of velocity vector
                     auv[IX2D(j, i, nlon)]
-                        = sqrt(uf[IX2D(j, i, nlon)] * uf[IX2D(j, i, nlon)] + vf[IX2D(j, i, nlon)] * vf[IX2D(j, i, nlon)]);
+                        = std::sqrt(uf[IX2D(j, i, nlon)] * uf[IX2D(j, i, nlon)] + vf[IX2D(j, i, nlon)] * vf[IX2D(j, i, nlon)]);
 
                     alpha[IX2D(j, i, nlon)] = atan2(vf[IX2D(j, i, nlon)], uf[IX2D(j, i, nlon)]);
                     alpha[IX2D(j, i, nlon)] = 90. - alpha[IX2D(j, i, nlon)] * RAD2DEG;
@@ -544,7 +552,7 @@ Outputgmt(void *process)
                     if (alpha[IX2D(j, i, nlon)] < 0) alpha[IX2D(j, i, nlon)] += 360;
                     if (alpha[IX2D(j, i, nlon)] > 360) alpha[IX2D(j, i, nlon)] -= 360;
 
-                    if (fabs(auv[IX2D(j, i, nlon)]) > 0)
+                    if (std::fabs(auv[IX2D(j, i, nlon)]) > 0)
                       fprintf(stdout, " %g  %g  %g  %g\n", grid_center_lon[IX2D(j, i, nlon)], grid_center_lat[IX2D(j, i, nlon)],
                               alpha[IX2D(j, i, nlon)], auv[IX2D(j, i, nlon)]);
                   }
@@ -554,18 +562,18 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTVRML)
             {
-              output_vrml(nlon, nlat, gridsize, array.data(), missval, &cpt);
+              output_vrml(nlon, nlat, gridsize, array, missval, &cpt);
             }
           else if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT)
             {
-              if (cdoVerbose)
+              if (Options::cdoVerbose)
                 {
-                  double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, &minval, &maxval, &meanval);
-                  double range = maxval - minval;
+                  const auto minmax = varrayMinMaxMV(gridsize, array, missval);
+                  const double minval = minmax.first;
+                  const double maxval = minmax.second;
+                  const double range = maxval - minval;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
-                  fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m "
-                                  "data.gmt > gmtplot.ps\n");
+                  fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
                   // fprintf(stderr, "psxy -K -Jx0.028id -Rd -L -Cgmt.cpt -m
                   // data.gmt > gmtplot.ps\n"); fprintf(stderr, "psxy -K
                   // -JN0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
@@ -670,7 +678,7 @@ Outputgmt(void *process)
                     {
                       const double *lon_bounds = grid_corner_lon.data() + i * ncorner;
                       const double *lat_bounds = grid_corner_lat.data() + i * ncorner;
-                      int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
+                      const int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
 
                       for (int ic = 0; ic < ncorner_new; ic++) fprintf(stdout, "   %g  %g\n", lon_bounds[ic], lat_bounds[ic]);
                       fprintf(stdout, "   %g  %g\n", lon_bounds[0], lat_bounds[0]);
@@ -685,9 +693,9 @@ Outputgmt(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
