@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,25 +20,29 @@
       Remapeta     remapeta          Model to model level interpolation
 */
 
-#include "hetaeta.h"
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "readline.h"
+#include "hetaeta.h"
 #include "after_vertint.h"
-#include "listarray.h"
 #include "stdnametable.h"
 #include "util_string.h"
 #include "timer.h"
+#include "const.h"
+#include "cdo_options.h"
+#include "cdo_zaxis.h"
+#include "cdi_lockedIO.h"
+
 
 static void
-setmissval(long nvals, int *imiss, double missval, double *array)
+setmissval(long nvals, const Varray<int> &imiss, double missval, double *array)
 {
-  if (imiss)
-    {
-      for (long i = 0; i < nvals; ++i)
-        if (imiss[i]) array[i] = missval;
-    }
+  if (!imiss.empty())
+    for (long i = 0; i < nvals; ++i)
+      if (imiss[i]) array[i] = missval;
 }
 
 static void
@@ -51,7 +55,7 @@ corr_hum(long gridsize, double *q, double q_min)
 }
 
 static long
-ncctop(double cptop, long nlev, long nlevp1, double *vct_a, double *vct_b)
+ncctop(double cptop, long nlev, long nlevp1, const double *vct_a, const double *vct_b)
 {
   /*
     Description:
@@ -61,18 +65,16 @@ ncctop(double cptop, long nlev, long nlevp1, double *vct_a, double *vct_b)
 
     E. Roeckner, MPI, October 2001
   */
-  /* local variables */
   long nctop = 0;
-  double za, zb;
-  std::vector<double> zph(nlevp1), zp(nlev);
+  Varray<double> zph(nlevp1), zp(nlev);
   // double    cptop  =  1000.;   /* min. pressure level for cond. */
 
   /* half level pressure values, assuming 101320. Pa surface pressure */
 
   for (long jk = 0; jk < nlevp1; ++jk)
     {
-      za = vct_a[jk];
-      zb = vct_b[jk];
+      const double za = vct_a[jk];
+      const double zb = vct_b[jk];
       zph[jk] = za + zb * 101320.;
     }
 
@@ -91,35 +93,35 @@ ncctop(double cptop, long nlev, long nlevp1, double *vct_a, double *vct_b)
   return nctop;
 }
 
-double *
-vctFromFile(const char *filename, int *nvct)
+static void
+vctFromFile(const char *filename, int *nvct, Varray<double> &vct2)
 {
   char line[1024], *pline;
-  int num, i = 0;
-  int maxvct = 8192;
+  int i = 0;
+  constexpr int maxvct = 8192;
 
-  FILE *fp = fopen(filename, "r");
-  if (fp == NULL)
+  auto fp = fopen(filename, "r");
+  if (fp == nullptr)
     {
       perror(filename);
       exit(EXIT_FAILURE);
     }
 
-  double *vct2 = (double *) Malloc(maxvct * sizeof(double));
+  vct2.resize(maxvct);
 
   while (readline(fp, line, 1024))
     {
       if (line[0] == '#' || line[0] == '\0') continue;
 
       pline = line;
-      num = (int) strtod(pline, &pline);
-      if (pline == NULL) cdoAbort("Format error in VCT file %s!", filename);
+      int num = (int) strtod(pline, &pline);
+      if (pline == nullptr) cdoAbort("Format error in VCT file %s!", filename);
       if (num != i) cdoWarning("Inconsistent VCT file, entry %d is %d.", i, num);
 
       if (i + maxvct / 2 >= maxvct - 1) cdoAbort("Too many values in VCT file!");
 
       vct2[i] = strtod(pline, &pline);
-      if (pline == NULL) cdoAbort("Format error in VCT file %s!", filename);
+      if (pline == nullptr) cdoAbort("Format error in VCT file %s!", filename);
 
       vct2[i + maxvct / 2] = strtod(pline, &pline);
 
@@ -128,82 +130,72 @@ vctFromFile(const char *filename, int *nvct)
 
   fclose(fp);
 
-  int nvct2 = 2 * i;
-  int nlevh2 = i - 1;
+  const int nvct2 = 2 * i;
+  const int nlevh2 = i - 1;
 
   for (i = 0; i < nlevh2 + 1; ++i) vct2[i + nvct2 / 2] = vct2[i + maxvct / 2];
 
-  vct2 = (double *) Realloc(vct2, nvct2 * sizeof(double));
+  vct2.resize(nvct2);
 
   *nvct = nvct2;
-
-  return vct2;
 }
 
 static void
-vert_sum(double *sum, double *var3d, size_t gridsize, long nlevel)
+vertSum(Varray<double> &sum, const double *var3d, size_t gridsize, size_t nlevel)
 {
-  size_t i;
-  int k;
+  for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
-  for (i = 0; i < gridsize; ++i) sum[i] = 0;
-
-  for (k = 0; k < nlevel; ++k)
-    for (i = 0; i < gridsize; ++i)
+  for (size_t k = 0; k < nlevel; ++k)
+    for (size_t i = 0; i < gridsize; ++i)
       {
         sum[i] += var3d[k * gridsize + i];
       }
 }
 
 static void
-vert_sumw(double *sum, double *var3d, size_t gridsize, long nlevel, double *deltap)
+vertSumw(Varray<double> &sum, const double *var3d, size_t gridsize, size_t nlevel, const Varray<double> &deltap)
 {
-  size_t i;
-  int k;
+  for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
-  for (i = 0; i < gridsize; ++i) sum[i] = 0;
-
-  for (k = 0; k < nlevel; ++k)
-    for (i = 0; i < gridsize; ++i)
+  for (size_t k = 0; k < nlevel; ++k)
+    for (size_t i = 0; i < gridsize; ++i)
       {
         sum[i] += var3d[k * gridsize + i] * deltap[k * gridsize + i];
       }
 }
 
-double *
-vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlevf)
+void
+vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, Varray<double> &vct, int *rnhlevf)
 {
   int zaxisIDh = -1;
   int nhlevf = 0;
   int nvct = 0;
-  double *vct = NULL;
 
   bool lhavevct = false;
-  int nzaxis = vlistNzaxis(vlistID);
+  const int nzaxis = vlistNzaxis(vlistID);
   for (int i = 0; i < nzaxis; i++)
     {
-      int zaxisID = vlistZaxis(vlistID, i);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
       if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel > 1)
         {
           nvct = zaxisInqVctSize(zaxisID);
-
           if (nlevel == (nvct / 2 - 1))
             {
-              if (lhavevct == false)
+              if (!lhavevct)
                 {
                   lhavevct = true;
                   zaxisIDh = zaxisID;
                   nhlevf = nlevel;
 
-                  vct = (double *) Malloc(nvct * sizeof(double));
-                  zaxisInqVct(zaxisID, vct);
+                  vct.resize(nvct);
+                  zaxisInqVct(zaxisID, vct.data());
                 }
             }
           else
             {
-              if (cdoVerbose) cdoPrint("nlevel = (nvct1/2 - 1): nlevel = %d", nlevel);
+              if (Options::cdoVerbose) cdoPrint("nlevel = (nvct1/2 - 1): nlevel = %d", nlevel);
               if (nlevel < (nvct / 2 - 1))
                 cdoPrint("z-axis %d has only %d of %d hybrid sigma pressure levels!", i + 1, nlevel, (nvct / 2 - 1));
             }
@@ -213,8 +205,6 @@ vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlevf)
   *rzaxisIDh = zaxisIDh;
   *rnvct = nvct;
   *rnhlevf = nhlevf;
-
-  return vct;
 }
 
 #define MAX_VARS3D 1024
@@ -231,95 +221,88 @@ Remapeta(void *process)
   int sgeopotID = -1, tempID = -1, sqID = -1, psID = -1, lnpsID = -1;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
   double *single2;
-  double *fis2 = NULL;
-  double *t1 = NULL, *q1 = NULL;
-  double *t2 = NULL, *q2 = NULL;
-  double *tscor = NULL, *pscor = NULL, *secor = NULL;
+  Varray<double> fis2;
+  double *t1 = nullptr, *q1 = nullptr;
+  double *t2 = nullptr, *q2 = nullptr;
+  double *tscor = nullptr, *pscor = nullptr, *secor = nullptr;
   size_t nmiss, nmissout = 0;
   bool ltq = false;
   bool lfis2 = false;
   int varids[MAX_VARS3D];
-  int *imiss = NULL;
+  Varray<int> imiss;
   int timer_hetaeta = 0;
-  long nctop = 0;
-  double *deltap1 = NULL, *deltap2 = NULL;
-  double *half_press1 = NULL, *half_press2 = NULL;
-  double *sum1 = NULL, *sum2 = NULL;
-  double **vars1 = NULL, **vars2 = NULL;
+  double **vars1 = nullptr, **vars2 = nullptr;
   double minval, maxval;
   double missval = 0;
   double cconst = 1.E-6;
   double cptop = 0; /* min. pressure level for cond. */
 
-  if (cdoTimer) timer_hetaeta = timer_new("Remapeta_hetaeta");
+  if (Options::Timer) timer_hetaeta = timer_new("Remapeta_hetaeta");
 
   cdoInitialize(process);
 
   // clang-format off
-  int REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
-  int REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
-  int REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
+  const auto REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
+  const auto REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
+  const auto REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  char *envstr = getenv("REMAPETA_PTOP");
+  const char *envstr = getenv("REMAPETA_PTOP");
   if (envstr)
     {
       double fval = atof(envstr);
       if (fval > 0)
         {
           cptop = fval;
-          //	  if ( cdoVerbose )
+          //	  if ( Options::cdoVerbose )
           cdoPrint("Set REMAPETA_PTOP to %g", cptop);
         }
     }
 
   int nvct2 = 0;
-  double *vct2 = vctFromFile(operatorArgv()[0], &nvct2);
-  int nhlevf2 = nvct2 / 2 - 1;
+  Varray<double> vct2;
+  vctFromFile(cdoOperatorArgv(0).c_str(), &nvct2, vct2);
+  const int nhlevf2 = nvct2 / 2 - 1;
 
-  double *a2 = vct2;
-  double *b2 = vct2 + nvct2 / 2;
+  const double *a2 = vct2.data();
+  const double *b2 = vct2.data() + nvct2 / 2;
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     for (i = 0; i < nhlevf2 + 1; ++i) cdoPrint("vct2: %5d %25.17f %25.17f", i, vct2[i], vct2[nvct2 / 2 + i]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
   if (operatorArgc() == 2)
     {
       lfis2 = true;
 
-      const char *fname = operatorArgv()[1];
-      int streamID = streamOpenRead(fname);
-
-      int vlistID1 = streamInqVlist(streamID);
+      const char *fname = cdoOperatorArgv(1).c_str();
+      int streamID = streamOpenReadLocked(fname);
+      const auto vlistID1 = streamInqVlist(streamID);
 
       streamInqRecord(streamID, &varID, &levelID);
-      int gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
       nfis2gp = gridInqSize(gridID);
 
-      fis2 = (double *) Malloc(nfis2gp * sizeof(double));
+      fis2.resize(nfis2gp);
 
-      streamReadRecord(streamID, fis2, &nmiss);
+      streamReadRecord(streamID, fis2.data(), &nmiss);
 
       if (nmiss)
         {
           missval = vlistInqVarMissval(vlistID1, varID);
-          imiss = (int *) Malloc(nfis2gp * sizeof(int));
-          for (size_t i = 0; i < nfis2gp; ++i)
-            {
-              imiss[i] = (DBL_IS_EQUAL(fis2[i], missval));
-            }
+          imiss.resize(nfis2gp);
+          for (size_t i = 0; i < nfis2gp; ++i) imiss[i] = DBL_IS_EQUAL(fis2[i], missval);
 
           nmissout = nmiss;
         }
 
-      /* check range of surface_geopotential */
-      arrayMinMaxMask(nfis2gp, fis2, imiss, &minval, &maxval);
+      // check range of surface_geopotential
+      arrayMinMaxMask(nfis2gp, fis2.data(), imiss, minval, maxval);
       if (minval < MIN_FIS || maxval > MAX_FIS)
         cdoWarning("%s out of range (min=%g max=%g)!", var_stdname(surface_geopotential), minval, maxval);
 
@@ -328,100 +311,97 @@ Remapeta(void *process)
       streamClose(streamID);
     }
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int gridID = vlistGrid(vlistID1, 0);
+  const auto gridID = vlistGrid(vlistID1, 0);
   if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
-  double *lev2 = (double *) Malloc(nhlevf2 * sizeof(double));
-  for (i = 0; i < nhlevf2; ++i) lev2[i] = i + 1;
-  zaxisDefLevels(zaxisID2, lev2);
-  Free(lev2);
+  const auto zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
+
+  {
+    Varray<double> lev2(nhlevf2);
+    for (int i = 0; i < nhlevf2; ++i) lev2[i] = i + 1;
+    zaxisDefLevels(zaxisID2, lev2.data());
+  }
 
   if (nvct2 == 0) cdoAbort("Internal problem, vct2 undefined!");
-  zaxisDefVct(zaxisID2, nvct2, vct2);
+  zaxisDefVct(zaxisID2, nvct2, vct2.data());
 
-  int surfaceID = zaxisFromName("surface");
+  auto surfaceID = zaxisFromName("surface");
 
   int zaxisIDh = -1;
   int nvct1 = 0;
   int nhlevf1 = 0;
-  double *vct1 = vlist_hybrid_vct(vlistID1, &zaxisIDh, &nvct1, &nhlevf1);
+  Varray<double> vct1;
+  vlist_hybrid_vct(vlistID1, &zaxisIDh, &nvct1, vct1, &nhlevf1);
 
   vlist_change_hybrid_zaxis(vlistID1, vlistID2, zaxisIDh, zaxisID2);
 
-  int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int i = 0; i < nzaxis; i++)
     {
-      int zaxisID = vlistZaxis(vlistID1, i);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
       if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel == 1) vlistChangeZaxisIndex(vlistID2, i, surfaceID);
     }
 
-  double *a1 = vct1;
-  double *b1 = vct1 + nvct1 / 2;
-  if (cdoVerbose)
+  const double *a1 = vct1.data();
+  const double *b1 = vct1.data() + nvct1 / 2;
+  if (Options::cdoVerbose)
     for (i = 0; i < nvct1 / 2; ++i) cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1 / 2 + i]);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   if (zaxisIDh == -1) cdoWarning("No 3D variable with hybrid sigma pressure coordinate found!");
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
-      int code = vlistInqVarCode(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
       /* code = -1; */
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(stdname);
+          cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
 
           if (code == -1)
             {
-              /*                                  ECHAM ECMWF       */
-              if (sgeopotID == -1 && (strcmp(varname, "geosp") == 0 || strcmp(varname, "z") == 0))
-                code = 129;
-              else if (tempID == -1 && (strcmp(varname, "st") == 0 || strcmp(varname, "t") == 0))
-                code = 130;
-              else if (psID == -1 && (strcmp(varname, "aps") == 0 || strcmp(varname, "ps") == 0))
-                code = 134;
-              else if (lnpsID == -1 && (strcmp(varname, "lsp") == 0 || strcmp(varname, "lnsp") == 0))
-                code = 152;
-              else if (sqID == -1 && (strcmp(varname, "q") == 0))
-                code = 133;
+              //                                            ECHAM                         ECMWF
+              // clang-format off
+              if (sgeopotID == -1 && (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z"))) code = 129;
+              else if (tempID == -1 && (cstrIsEqual(varname, "st") || cstrIsEqual(varname, "t"))) code = 130;
+              else if (psID == -1 && (cstrIsEqual(varname, "aps") || cstrIsEqual(varname, "ps"))) code = 134;
+              else if (lnpsID == -1 && (cstrIsEqual(varname, "lsp") || cstrIsEqual(varname, "lnsp"))) code = 152;
+              else if (sqID == -1 && (cstrIsEqual(varname, "q"))) code = 133;
+              // clang-format on
             }
         }
 
-      if (code == 129 && nlevel == 1)
-        sgeopotID = varID;
-      else if (code == 130 && nlevel == nhlevf1)
-        tempID = varID;
-      else if (code == 133 && nlevel == nhlevf1)
-        sqID = varID;
-      else if (code == 134 && nlevel == 1)
-        psID = varID;
-      else if (code == 152 && nlevel == 1)
-        lnpsID = varID;
+      // clang-format off
+      if      (code == 129 && nlevel == 1) sgeopotID = varID;
+      else if (code == 134 && nlevel == 1) psID = varID;
+      else if (code == 152 && nlevel == 1) lnpsID = varID;
+      else if (code == 130 && nlevel == nhlevf1) tempID = varID;
+      else if (code == 133 && nlevel == nhlevf1) sqID = varID;
+      // clang-format on
 
       if (gridInqType(gridID) == GRID_SPECTRAL && zaxisInqType(zaxisID) == ZAXIS_HYBRID)
         cdoAbort("Spectral data on model level unsupported!");
@@ -439,7 +419,7 @@ Remapeta(void *process)
         }
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       cdoPrint("Found:");
       if (tempID != -1) cdoPrint("  %s", var_stdname(air_temperature));
@@ -468,29 +448,32 @@ Remapeta(void *process)
     {
     }
 
+  Varray<double> sum1, sum2;
   if (operatorID == REMAPETAS || operatorID == REMAPETAZ)
     {
-      sum1 = (double *) Malloc(gridsize * sizeof(double));
-      sum2 = (double *) Malloc(gridsize * sizeof(double));
+      sum1.resize(gridsize);
+      sum2.resize(gridsize);
     }
 
+  Varray<double> deltap1, deltap2;
+  Varray<double> half_press1, half_press2;
   if (operatorID == REMAPETAZ)
     {
-      deltap1 = (double *) Malloc(gridsize * nhlevf1 * sizeof(double));
-      deltap2 = (double *) Malloc(gridsize * nhlevf2 * sizeof(double));
-      half_press1 = (double *) Malloc(gridsize * (nhlevf1 + 1) * sizeof(double));
-      half_press2 = (double *) Malloc(gridsize * (nhlevf2 + 1) * sizeof(double));
+      deltap1.resize(gridsize * nhlevf1);
+      deltap2.resize(gridsize * nhlevf2);
+      half_press1.resize(gridsize * (nhlevf1 + 1));
+      half_press2.resize(gridsize * (nhlevf2 + 1));
     }
 
-  double *array = (double *) Malloc(gridsize * sizeof(double));
+  Varray<double> array(gridsize);
 
-  double *fis1 = (double *) Malloc(gridsize * sizeof(double));
-  double *ps1 = (double *) Malloc(gridsize * sizeof(double));
+  Varray<double> fis1(gridsize);
+  Varray<double> ps1(gridsize);
 
-  if (lfis2 == false) fis2 = (double *) Malloc(gridsize * sizeof(double));
-  if (lfis2 == true && gridsize != nfis2gp) cdoAbort("Orographies have different grid size!");
+  if (!lfis2) fis2.resize(gridsize);
+  if (lfis2 && gridsize != nfis2gp) cdoAbort("Orographies have different grid size!");
 
-  double *ps2 = (double *) Malloc(gridsize * sizeof(double));
+  Varray<double> ps2(gridsize);
 
   if (ltq)
     {
@@ -521,7 +504,7 @@ Remapeta(void *process)
     {
       if (ltq) cdoWarning("%s not found - set to zero!", var_stdname(surface_geopotential));
 
-      arrayFill(gridsize, fis1, 0.0);
+      varrayFill(fis1, 0.0);
     }
 
   int presID = lnpsID;
@@ -533,7 +516,7 @@ Remapeta(void *process)
         presID = psID;
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       if (presID == lnpsID)
         cdoPrint("using LOG(%s)", var_stdname(surface_air_pressure));
@@ -541,37 +524,37 @@ Remapeta(void *process)
         cdoPrint("using %s", var_stdname(surface_air_pressure));
     }
 
-  if (cdoVerbose) cdoPrint("nvars3D = %d   ltq = %d", nvars3D, (int) ltq);
+  if (Options::cdoVerbose) cdoPrint("nvars3D = %d   ltq = %d", nvars3D, (int) ltq);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          int nlevel = zaxisInqSize(zaxisID);
-          size_t offset = gridsize * levelID;
-          pstreamReadRecord(streamID1, array, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+          const auto nlevel = zaxisInqSize(zaxisID);
+          const auto offset = gridsize * levelID;
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           if (zaxisIDh != -1)
             {
               if (varID == sgeopotID)
-                arrayCopy(gridsize, array, fis1);
+                varrayCopy(gridsize, array, fis1);
               else if (varID == presID)
                 {
                   if (lnpsID != -1)
-                    for (size_t i = 0; i < gridsize; ++i) ps1[i] = exp(array[i]);
+                    for (size_t i = 0; i < gridsize; ++i) ps1[i] = std::exp(array[i]);
                   else if (psID != -1)
-                    arrayCopy(gridsize, array, ps1);
+                    varrayCopy(gridsize, array, ps1);
                 }
               else if (ltq && varID == tempID)
-                arrayCopy(gridsize, array, t1 + offset);
+                arrayCopy(gridsize, array.data(), t1 + offset);
               else if (ltq && varID == sqID)
-                arrayCopy(gridsize, array, q1 + offset);
+                arrayCopy(gridsize, array.data(), q1 + offset);
               /* else if ( zaxisID == zaxisIDh ) */
               else if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel == nhlevf1)
                 {
@@ -580,33 +563,33 @@ Remapeta(void *process)
 
                   if (i == nvars3D) cdoAbort("Internal error, 3D variable not found!");
 
-                  arrayCopy(gridsize, array, vars1[i] + offset);
+                  arrayCopy(gridsize, array.data(), vars1[i] + offset);
                 }
               else
                 {
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, array, nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array, nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
       if (zaxisIDh != -1)
         {
-          /* check range of ps_prog */
-          arrayMinMaxMask(gridsize, ps1, imiss, &minval, &maxval);
+          // check range of ps_prog
+          arrayMinMaxMask(gridsize, ps1.data(), imiss, minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 
-          /* check range of geop */
-          arrayMinMaxMask(gridsize, fis1, imiss, &minval, &maxval);
+          // check range of geop
+          arrayMinMaxMask(gridsize, fis1.data(), imiss, minval, maxval);
           if (minval < MIN_FIS || maxval > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
         }
 
-      if (lfis2 == false)
+      if (!lfis2)
         for (size_t i = 0; i < gridsize; i++) fis2[i] = fis1[i];
 
       if (ltq)
@@ -615,26 +598,24 @@ Remapeta(void *process)
           int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              size_t offset = gridsize * levelID;
+              const auto offset = gridsize * levelID;
               single2 = t1 + offset;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_T || maxval > MAX_T)
-                cdoWarning("Input temperature at level %d out of range (min=%g "
-                           "max=%g)!",
-                           levelID + 1, minval, maxval);
+                cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
             }
 
           varID = sqID;
           nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              size_t offset = gridsize * levelID;
+              const auto offset = gridsize * levelID;
               single2 = q1 + offset;
 
               corr_hum(gridsize, single2, MIN_Q);
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_Q || maxval > MAX_Q)
                 cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
             }
@@ -642,33 +623,33 @@ Remapeta(void *process)
 
       if (nvars3D || ltq)
         {
-          if (cdoTimer) timer_start(timer_hetaeta);
-          hetaeta(ltq, gridsize, imiss, nhlevf1, a1, b1, fis1, ps1, t1, q1, nhlevf2, a2, b2, fis2, ps2, t2, q2, nvars3D, vars1,
-                  vars2, tscor, pscor, secor);
-          if (cdoTimer) timer_stop(timer_hetaeta);
+          if (Options::Timer) timer_start(timer_hetaeta);
+          hetaeta(ltq, gridsize, imiss.data(), nhlevf1, a1, b1, fis1.data(), ps1.data(), t1, q1, nhlevf2, a2, b2, fis2.data(), ps2.data(),
+                  t2, q2, nvars3D, vars1, vars2, tscor, pscor, secor);
+          if (Options::Timer) timer_stop(timer_hetaeta);
         }
 
-      if (cptop > 0) nctop = ncctop(cptop, (long) nhlevf2, (long) nhlevf2 + 1, a2, b2);
+      const long nctop = (cptop > 0) ? ncctop(cptop, (long) nhlevf2, (long) nhlevf2 + 1, a2, b2) : 0;
 
       if (zaxisIDh != -1 && sgeopotID != -1)
         {
           varID = sgeopotID;
           levelID = 0;
-          setmissval(gridsize, imiss, missval, fis2);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, fis2, nmissout);
+          setmissval(gridsize, imiss, missval, fis2.data());
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, fis2.data(), nmissout);
         }
 
       if (zaxisIDh != -1 && lnpsID != -1)
-        for (size_t i = 0; i < gridsize; ++i) ps2[i] = log(ps2[i]);
+        for (size_t i = 0; i < gridsize; ++i) ps2[i] = std::log(ps2[i]);
 
       if (zaxisIDh != -1 && presID != -1)
         {
           varID = presID;
           levelID = 0;
-          setmissval(gridsize, imiss, missval, ps2);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, ps2, nmissout);
+          setmissval(gridsize, imiss, missval, ps2.data());
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, ps2.data(), nmissout);
         }
 
       if (ltq)
@@ -677,25 +658,23 @@ Remapeta(void *process)
           int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              size_t offset = gridsize * levelID;
+              const auto offset = gridsize * levelID;
               single2 = t2 + offset;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_T || maxval > MAX_T)
-                cdoWarning("Output temperature at level %d out of range "
-                           "(min=%g max=%g)!",
-                           levelID + 1, minval, maxval);
+                cdoWarning("Output temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
 
           varID = sqID;
           nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              size_t offset = gridsize * levelID;
+              const auto offset = gridsize * levelID;
               single2 = q2 + offset;
 
               corr_hum(gridsize, single2, MIN_Q);
@@ -703,13 +682,13 @@ Remapeta(void *process)
               if (levelID < nctop)
                 for (size_t i = 0; i < gridsize; ++i) single2[i] = cconst;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_Q || maxval > MAX_Q)
                 cdoWarning("Output humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
         }
 
@@ -717,39 +696,37 @@ Remapeta(void *process)
         {
           varID = varids[iv];
 
-          int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+          const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 
           if (operatorID == REMAPETAS)
             {
-              vert_sum(sum1, vars1[iv], gridsize, nhlevf1);
-              vert_sum(sum2, vars2[iv], gridsize, nhlevf2);
+              vertSum(sum1, vars1[iv], gridsize, nhlevf1);
+              vertSum(sum2, vars2[iv], gridsize, nhlevf2);
             }
           else if (operatorID == REMAPETAZ)
             {
-              int k;
-
-              presh(NULL, half_press1, vct1, ps1, nhlevf1, gridsize);
-              for (k = 0; k < nhlevf1; ++k)
+              presh(nullptr, half_press1.data(), vct1.data(), ps1.data(), nhlevf1, gridsize);
+              for (int k = 0; k < nhlevf1; ++k)
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     deltap1[k * gridsize + i] = half_press1[(k + 1) * gridsize + i] - half_press1[k * gridsize + i];
-                    deltap1[k * gridsize + i] = log(deltap1[k * gridsize + i]);
+                    deltap1[k * gridsize + i] = std::log(deltap1[k * gridsize + i]);
                   }
-              vert_sumw(sum1, vars1[iv], gridsize, nhlevf1, deltap1);
+              vertSumw(sum1, vars1[iv], gridsize, nhlevf1, deltap1);
 
-              presh(NULL, half_press2, vct2, ps1, nhlevf2, gridsize);
-              for (k = 0; k < nhlevf2; ++k)
+              presh(nullptr, half_press2.data(), vct2.data(), ps1.data(), nhlevf2, gridsize);
+              for (int k = 0; k < nhlevf2; ++k)
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     deltap2[k * gridsize + i] = half_press2[(k + 1) * gridsize + i] - half_press2[k * gridsize + i];
-                    deltap2[k * gridsize + i] = log(deltap2[k * gridsize + i]);
+                    deltap2[k * gridsize + i] = std::log(deltap2[k * gridsize + i]);
                   }
-              vert_sumw(sum2, vars2[iv], gridsize, nhlevf2, deltap2);
+              vertSumw(sum2, vars2[iv], gridsize, nhlevf2, deltap2);
             }
 
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              size_t offset = gridsize * levelID;
+              const size_t offset = gridsize * levelID;
               single2 = vars2[iv] + offset;
 
               if (operatorID == REMAPETAS || operatorID == REMAPETAZ)
@@ -764,16 +741,16 @@ Remapeta(void *process)
                 }
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   if (nvars3D)
     {
@@ -797,27 +774,7 @@ Remapeta(void *process)
       Free(tscor);
     }
 
-  if (imiss) Free(imiss);
-
-  Free(ps2);
-  Free(fis2);
-  Free(ps1);
-  Free(fis1);
-
-  if (sum1) Free(sum1);
-  if (sum2) Free(sum2);
-
-  if (deltap1) Free(deltap1);
-  if (deltap2) Free(deltap2);
-
-  if (half_press1) Free(half_press1);
-  if (half_press2) Free(half_press2);
-
-  Free(array);
-  Free(vct2);
-  if (vct1) Free(vct1);
-
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
