@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,55 +23,49 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
+
 
 static void
-rot_uv_back(int gridID, double *us, double *vs)
+rot_uv_back(int gridID, Varray<double> &us, Varray<double> &vs)
 {
   double xpole = 0, ypole = 0, angle = 0;
   if (gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL)
     gridInqParamRLL(gridID, &xpole, &ypole, &angle);
 
-  size_t nlon = gridInqXsize(gridID);
-  size_t nlat = gridInqYsize(gridID);
+  const auto nlon = gridInqXsize(gridID);
+  const auto nlat = gridInqYsize(gridID);
 
-  double *xvals = (double *) Malloc(nlon * sizeof(double));
-  double *yvals = (double *) Malloc(nlat * sizeof(double));
-  gridInqXvals(gridID, xvals);
-  gridInqYvals(gridID, yvals);
+  Varray<double> xvals(nlon);
+  Varray<double> yvals(nlat);
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
 
   /* Convert lat/lon units if required */
   char units[CDI_MAX_NAME];
   gridInqXunits(gridID, units);
   grid_to_degree(units, 1, &angle, "angle");
   grid_to_degree(units, 1, &xpole, "xpole");
-  grid_to_degree(units, nlon, xvals, "grid center lon");
+  grid_to_degree(units, nlon, xvals.data(), "grid center lon");
   gridInqYunits(gridID, units);
   grid_to_degree(units, 1, &ypole, "ypole");
-  grid_to_degree(units, nlat, yvals, "grid center lat");
+  grid_to_degree(units, nlat, yvals.data(), "grid center lat");
 
   if (xpole > 180) xpole -= 360;
   if (angle > 180) angle -= 360;
 
-  double u, v;
   for (size_t ilat = 0; ilat < nlat; ilat++)
     for (size_t ilon = 0; ilon < nlon; ilon++)
       {
-        size_t i = ilat * nlon + ilon;
-
-        double xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
-        double yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
-
-        usvs_to_uv(us[i], vs[i], yval, xval, ypole, xpole, &u, &v);
-
-        us[i] = u;
-        vs[i] = v;
+        const auto i = ilat * nlon + ilon;
+        const auto xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
+        const auto yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
+        usvs_to_uv(us[i], vs[i], yval, xval, ypole, xpole, &us[i], &vs[i]);
       }
-
-  Free(xvals);
-  Free(yvals);
 }
 
 #define MAXARG 16384
@@ -80,16 +74,11 @@ void *
 Rotuv(void *process)
 {
   int varID, levelID;
-  int varID1, varID2, nlevel1, nlevel2;
-  size_t gridsize;
-  int code, gridID;
-  int nlevel;
   int nrecs;
   int chcodes[MAXARG];
-  char *chvars[MAXARG];
+  const char *chvars[MAXARG];
   char varname[CDI_MAX_NAME];
   char varname2[CDI_MAX_NAME];
-  double *single, *usvar = NULL, *vsvar = NULL;
 
   cdoInitialize(process);
 
@@ -99,10 +88,10 @@ Rotuv(void *process)
   if (nch % 2) cdoAbort("Odd number of input arguments!");
 
   bool lvar = false;  // We have a list of codes
-  int len = (int) strlen(operatorArgv()[0]);
-  int ix = (operatorArgv()[0][0] == '-') ? 1 : 0;
+  int len = (int) cdoOperatorArgv(0).size();
+  int ix = (cdoOperatorArgv(0)[0] == '-') ? 1 : 0;
   for (int i = ix; i < len; ++i)
-    if (!isdigit(operatorArgv()[0][i]))
+    if (!isdigit(cdoOperatorArgv(0)[i]))
       {
         lvar = true;  // We have a list of variables
         break;
@@ -110,25 +99,25 @@ Rotuv(void *process)
 
   if (lvar)
     {
-      for (int i = 0; i < nch; i++) chvars[i] = operatorArgv()[i];
+      for (int i = 0; i < nch; i++) chvars[i] = cdoOperatorArgv(i).c_str();
     }
   else
     {
-      for (int i = 0; i < nch; i++) chcodes[i] = parameter2int(operatorArgv()[i]);
+      for (int i = 0; i < nch; i++) chcodes[i] = parameter2int(cdoOperatorArgv(i));
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const auto maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t **varnmiss = (size_t **) Malloc(nvars * sizeof(size_t *));
-  double **vardata = (double **) Malloc(nvars * sizeof(double *));
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  Varray3D<double> vardata(nvars);
 
   bool lfound[MAXARG];
   for (int i = 0; i < nch; i++) lfound[i] = false;
@@ -148,7 +137,7 @@ Rotuv(void *process)
     {
       for (varID = 0; varID < nvars; varID++)
         {
-          code = vlistInqVarCode(vlistID2, varID);
+          const auto code = vlistInqVarCode(vlistID2, varID);
           for (int i = 0; i < nch; i++)
             if (code == chcodes[i]) lfound[i] = true;
         }
@@ -158,41 +147,39 @@ Rotuv(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
       if (!(gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL))
         cdoAbort("Only rotated lon/lat grids supported!");
 
-      gridsize = gridInqSize(gridID);
-      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
-      vardata[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      const auto gridsize = gridInqSize(gridID);
+      const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      varnmiss[varID].resize(nlevel);
+      vardata[varID].resize(nlevel);
+      for (levelID = 0; levelID < nlevel; levelID++) vardata[varID][levelID].resize(gridsize);
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
-          recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
+          recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          size_t offset = gridsize * levelID;
-          single = vardata[varID] + offset;
-          pstreamReadRecord(streamID1, single, &varnmiss[varID][levelID]);
+          cdoReadRecord(streamID1, vardata[varID][levelID].data(), &varnmiss[varID][levelID]);
           if (varnmiss[varID][levelID]) cdoAbort("Missing values unsupported for this operator!");
         }
 
@@ -207,17 +194,14 @@ Rotuv(void *process)
                 }
               else
                 {
-                  code = vlistInqVarCode(vlistID2, varID);
+                  const auto code = vlistInqVarCode(vlistID2, varID);
                   if (code == chcodes[i]) break;
                 }
             }
 
-          if (varID == nvars)
-            cdoAbort("u-wind not found!");
-          else
-            usvar = vardata[varID];
+          if (varID == nvars) cdoAbort("u-wind not found!");
 
-          varID1 = varID;
+          const auto usvarID = varID;
 
           for (varID = 0; varID < nvars; varID++)
             {
@@ -228,70 +212,52 @@ Rotuv(void *process)
                 }
               else
                 {
-                  code = vlistInqVarCode(vlistID2, varID);
+                  const auto code = vlistInqVarCode(vlistID2, varID);
                   if (code == chcodes[i + 1]) break;
                 }
             }
 
-          if (varID == nvars)
-            cdoAbort("v-wind not found!");
-          else
-            vsvar = vardata[varID];
+          if (varID == nvars) cdoAbort("v-wind not found!");
 
-          varID2 = varID;
+          const auto vsvarID = varID;
 
-          if (cdoVerbose)
+          if (Options::cdoVerbose)
             {
               if (lvar)
                 {
-                  vlistInqVarName(vlistID2, varID1, varname);
-                  vlistInqVarName(vlistID2, varID2, varname2);
+                  vlistInqVarName(vlistID2, usvarID, varname);
+                  vlistInqVarName(vlistID2, vsvarID, varname2);
                   cdoPrint("Using var %s [%s](u) and var %s [%s](v)", varname, chvars[i], varname2, chvars[i + 1]);
                 }
               else
-                cdoPrint("Using code %d [%d](u) and code %d [%d](v)", vlistInqVarCode(vlistID1, varID1), chcodes[i],
-                         vlistInqVarCode(vlistID1, varID2), chcodes[i + 1]);
+                cdoPrint("Using code %d [%d](u) and code %d [%d](v)", vlistInqVarCode(vlistID1, usvarID), chcodes[i],
+                         vlistInqVarCode(vlistID1, vsvarID), chcodes[i + 1]);
             }
 
-          gridID = vlistInqVarGrid(vlistID1, varID);
-          gridsize = gridInqSize(gridID);
-          nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
-          nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID2));
-
+          const auto gridID = vlistInqVarGrid(vlistID1, varID);
+          const auto nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, usvarID));
+          const auto nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID1, vsvarID));
           if (nlevel1 != nlevel2) cdoAbort("u-wind and v-wind have different number of levels!");
 
           for (levelID = 0; levelID < nlevel1; levelID++)
-            {
-              size_t offset = gridsize * levelID;
-              rot_uv_back(gridID, usvar + offset, vsvar + offset);
-            }
+            rot_uv_back(gridID, vardata[usvarID][levelID], vardata[vsvarID][levelID]);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          varID = recinfo[recID].varID;
-          levelID = recinfo[recID].levelID;
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          size_t offset = gridsize * levelID;
-          single = vardata[varID] + offset;
-
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, single, varnmiss[varID][levelID]);
+          varID = recList[recID].varID;
+          levelID = recList[recID].levelID;
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, vardata[varID][levelID].data(), varnmiss[varID][levelID]);
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata[varID]);
-    }
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
