@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,9 +16,14 @@
 */
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdi_lockedIO.h"
+#include <mpim_grid.h>
 #include "grid_point_search.h"
+#include "verifygrid.h"
+
 
 constexpr int MAX_CHILDS = 9;
 
@@ -34,7 +39,7 @@ struct CellIndex
 static void
 copy_data_to_index(long ncells, const double *restrict data, long *restrict cellindex)
 {
-  for (long i = 0; i < ncells; ++i) cellindex[i] = lround(data[i]);
+  for (long i = 0; i < ncells; ++i) cellindex[i] = std::lround(data[i]);
 }
 
 static void
@@ -47,14 +52,9 @@ free_cellindex(CellIndex *cellindex)
 }
 
 static CellIndex *
-read_cellindex(const char *filename)
+read_cellindex(const std::string &filename)
 {
-  openLock();
-  int streamID = streamOpenRead(filename);
-  openUnlock();
-
-  if (streamID < 0) cdiOpenError(streamID, "Open failed on >%s<", filename);
-
+  int streamID = streamOpenReadLocked(filename.c_str());
   int vlistID = streamInqVlist(streamID);
   int ngrids = vlistNgrids(vlistID);
   int gridID = -1;
@@ -91,12 +91,12 @@ read_cellindex(const char *filename)
   CellIndex *cellindex = (CellIndex *) Malloc(sizeof(CellIndex));
   cellindex->ncells = ncells;
 
-  cellindex->neighbor = NULL;
+  cellindex->neighbor = nullptr;
   // cellindex->neighbor = (long*) Malloc(3*ncells*sizeof(long));
   cellindex->parent = (long *) Malloc(ncells * sizeof(long));
-  cellindex->child = NULL;
-  // cellindex->child    = (cid != CDI_UNDEFID) ? (int*) Malloc(MAX_CHILDS*ncells*sizeof(int)) : NULL;
-  std::vector<double> data(ncells);
+  cellindex->child = nullptr;
+  // cellindex->child    = (cid != CDI_UNDEFID) ? (int*) Malloc(MAX_CHILDS*ncells*sizeof(int)) : nullptr;
+  Varray<double> data(ncells);
 
   int nrecs = streamInqTimestep(streamID, 0);
   for (int recID = 0; recID < nrecs; recID++)
@@ -126,12 +126,7 @@ read_cellindex(const char *filename)
 static int
 read_grid(const char *filename)
 {
-  openLock();
-  int streamID = streamOpenRead(filename);
-  openUnlock();
-
-  if (streamID < 0) cdiOpenError(streamID, "Open failed on >%s<", filename);
-
+  int streamID = streamOpenReadLocked(filename);
   int vlistID = streamInqVlist(streamID);
   int ngrids = vlistNgrids(vlistID);
   int gridID = -1;
@@ -214,7 +209,7 @@ compute_child_from_parent(CellIndex *cellindex1, CellIndex *cellindex2)
   for (long i = 1; i < ncells1; ++i)
     if (parent1[i] < parent1[i - 1])
       {
-        if (cdoVerbose) cdoPrint("Sort parent index of %s!", cellindex1->filename);
+        if (Options::cdoVerbose) cdoPrint("Sort parent index of %s!", cellindex1->filename);
         sinfo_t *sinfo = (sinfo_t *) Malloc(ncells1 * sizeof(sinfo_t));
         for (long j = 0; j < ncells1; ++j)
           {
@@ -254,12 +249,7 @@ compute_child_from_parent(CellIndex *cellindex1, CellIndex *cellindex2)
 static void
 read_coordinates(const char *filename, long n, double *lon, double *lat, int nv, double *lon_bnds, double *lat_bnds)
 {
-  openLock();
-  int streamID = streamOpenRead(filename);
-  openUnlock();
-
-  if (streamID < 0) cdiOpenError(streamID, "Open failed on >%s<", filename);
-
+  CdiStreamID streamID = streamOpenRead(filename);
   int vlistID = streamInqVlist(streamID);
   int ngrids = vlistNgrids(vlistID);
   int gridID = -1;
@@ -295,11 +285,6 @@ read_coordinates(const char *filename, long n, double *lon, double *lat, int nv,
   streamClose(streamID);
 }
 
-int find_coordinate_to_ignore(double *cell_corners_xyz);
-double calculate_the_polygon_area(double cell_corners[], int number_corners);
-bool are_polygon_vertices_arranged_in_clockwise_order(double cell_area);
-int winding_numbers_algorithm(double cell_corners[], int number_corners, double point[]);
-
 #define MAX_SEARCH 128  // the triangles are distorted!
 
 static void
@@ -330,15 +315,11 @@ compute_child_from_bounds(CellIndex *cellindex2, long ncells2, double *grid_cent
 
       for (int corner_no = 0; corner_no < ncorner; corner_no++)
         {
-          /* Conversion of corner spherical coordinates to Cartesian
-           * coordinates. */
-
-          cdoLLtoXYZ(grid_corner_lon2[cell_no2 * ncorner + corner_no], grid_corner_lat2[cell_no2 * ncorner + corner_no],
-                     corner_coordinates);
-
-          /* The components of the result vector are appended to the list of
-           * cell corner coordinates. */
+          // Conversion of corner spherical coordinates to Cartesian coordinates.
+          gcLLtoXYZ(grid_corner_lon2[cell_no2 * ncorner + corner_no], grid_corner_lat2[cell_no2 * ncorner + corner_no],
+                    corner_coordinates);
 
+          // The components of the result vector are appended to the list of cell corner coordinates.
           int off = corner_no * 3;
           cell_corners_xyz[off + 0] = corner_coordinates[0];
           cell_corners_xyz[off + 1] = corner_coordinates[1];
@@ -392,7 +373,7 @@ compute_child_from_bounds(CellIndex *cellindex2, long ncells2, double *grid_cent
           size_t cell_no1 = nbr_addr[i];
           if (cell_no1 < SIZE_MAX)
             {
-              cdoLLtoXYZ(grid_center_lon1[cell_no1], grid_center_lat1[cell_no1], center_point_xyz);
+              gcLLtoXYZ(grid_center_lon1[cell_no1], grid_center_lat1[cell_no1], center_point_xyz);
 
               switch (coordinate_to_ignore)
                 {
@@ -431,34 +412,24 @@ compute_child_from_bounds(CellIndex *cellindex2, long ncells2, double *grid_cent
 static void
 compute_child_from_coordinates(CellIndex *cellindex1, CellIndex *cellindex2)
 {
-  long ncells1 = cellindex1->ncells;
-  long ncells2 = cellindex2->ncells;
+  const long ncells1 = cellindex1->ncells;
+  const long ncells2 = cellindex2->ncells;
 
-  double *lon1 = (double *) Malloc(ncells1 * sizeof(double));
-  double *lat1 = (double *) Malloc(ncells1 * sizeof(double));
-  double *lon2 = (double *) Malloc(ncells2 * sizeof(double));
-  double *lat2 = (double *) Malloc(ncells2 * sizeof(double));
-  double *lon2_bnds = (double *) Malloc(3 * ncells2 * sizeof(double));
-  double *lat2_bnds = (double *) Malloc(3 * ncells2 * sizeof(double));
+  Varray<double> lon1(ncells1), lat1(ncells1), lon2(ncells2), lat2(ncells2);
+  Varray<double> lon2_bnds(3 * ncells2), lat2_bnds(3 * ncells2);
 
-  read_coordinates(cellindex1->filename, ncells1, lon1, lat1, 0, NULL, NULL);
-  read_coordinates(cellindex2->filename, ncells2, lon2, lat2, 3, lon2_bnds, lat2_bnds);
+  read_coordinates(cellindex1->filename, ncells1, lon1.data(), lat1.data(), 0, nullptr, nullptr);
+  read_coordinates(cellindex2->filename, ncells2, lon2.data(), lat2.data(), 3, lon2_bnds.data(), lat2_bnds.data());
 
-  compute_child_from_bounds(cellindex2, ncells2, lon2, lat2, lon2_bnds, lat2_bnds, ncells1, lon1, lat1);
-
-  Free(lon1);
-  Free(lat1);
-  Free(lon2);
-  Free(lat2);
-  Free(lon2_bnds);
-  Free(lat2_bnds);
+  compute_child_from_bounds(cellindex2, ncells2, lon2.data(), lat2.data(), lon2_bnds.data(), lat2_bnds.data(), ncells1, lon1.data(),
+                            lat1.data());
 }
 
 static void
 compute_child(CellIndex *cellindex1, CellIndex *cellindex2)
 {
   bool lparent = true;
-  long ncells1 = cellindex1->ncells;
+  const long ncells1 = cellindex1->ncells;
   long *parent1 = cellindex1->parent;
   {
     long i;
@@ -477,7 +448,7 @@ compute_child(CellIndex *cellindex1, CellIndex *cellindex2)
 }
 
 static void
-compute_sum(long i, long *n, double *sum, double *sumq, long kci, CellIndex **cellindex, double *array)
+compute_sum(long i, long *n, double *sum, double *sumq, long kci, CellIndex **cellindex, const Varray<double> &array)
 {
   // printf("compute: i, kci %d %d\n", i, kci);
   long ncells2 = cellindex[kci]->ncells;
@@ -499,7 +470,7 @@ compute_sum(long i, long *n, double *sum, double *sumq, long kci, CellIndex **ce
 }
 
 static void
-samplegrid(double missval, long nci, CellIndex **cellindex, double *array1, double *array2, double *array3)
+samplegrid(double missval, long nci, CellIndex **cellindex, const Varray<double> &array1, Varray<double> &array2, Varray<double> &array3)
 {
   static bool lstat = true;
   long kci = nci - 1;
@@ -525,15 +496,13 @@ samplegrid(double missval, long nci, CellIndex **cellindex, double *array1, doub
           x += n;
         }
     }
-  if (cdoVerbose && lstat)
+  if (Options::cdoVerbose && lstat)
     {
       lstat = false;
       cdoPrint("Mean number of childs %g", nx ? x / nx : 0);
     }
 }
 
-#include "pstream_int.h"
-
 void *
 Samplegridicon(void *process)
 {
@@ -551,40 +520,40 @@ Samplegridicon(void *process)
 
   for (int i = 0; i < nsamplegrids; ++i)
     {
-      cellindex[i] = read_cellindex(operatorArgv()[i]);
-      cellindex[i]->filename = operatorArgv()[i];
-      if (cdoVerbose) cdoPrint("Found %ld grid cells in %s", cellindex[i]->ncells, cellindex[i]->filename);
+      cellindex[i] = read_cellindex(cdoOperatorArgv(i));
+      cellindex[i]->filename = cdoOperatorArgv(i).c_str();
+      if (Options::cdoVerbose) cdoPrint("Found %ld grid cells in %s", cellindex[i]->ncells, cellindex[i]->filename);
     }
 
   for (int i = 0; i < nsamplegrids - 1; ++i) compute_child(cellindex[i], cellindex[i + 1]);
 
-  int gridID2 = read_grid(operatorArgv()[nsamplegrids - 1]);
+  const auto gridID2 = read_grid(cdoOperatorArgv(nsamplegrids - 1).c_str());
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   long gridsize = vlistGridsizeMax(vlistID1);
-  if (cdoVerbose) cdoPrint("Source gridsize = %zu", gridsize);
+  if (Options::cdoVerbose) cdoPrint("Source gridsize = %zu", gridsize);
   if (gridsize != cellindex[0]->ncells)
     cdoAbort("Gridsize (%ld) of input stream and first grid (%ld) differ!", gridsize, cellindex[0]->ncells);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  double *array1 = (double *) Malloc(gridsize * sizeof(double));
+  Varray<double> array1(gridsize);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; ++index)
     {
-      int gridID = vlistGrid(vlistID1, index);
-      int gridtype = gridInqType(gridID);
+      const auto vlistID2 = vlistDuplicate(vlistID1);
+      const auto gridID = vlistGrid(vlistID1, index);
+      const auto gridtype = gridInqType(gridID);
       if (!(gridtype == GRID_UNSTRUCTURED && gridInqNvertex(gridID) == 3))
         cdoAbort("Unsupported gridtype: %s with %d corners", gridNamePtr(gridtype), gridInqNvertex(gridID));
 
@@ -592,62 +561,58 @@ Samplegridicon(void *process)
       vlistChangeGridIndex(vlistID3, index, gridID2);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
-  pstreamDefVlist(streamID3, vlistID3);
+  const auto streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   long gridsize2 = gridInqSize(gridID2);
-  if (cdoVerbose) cdoPrint("Target gridsize = %ld", gridsize2);
+  if (Options::cdoVerbose) cdoPrint("Target gridsize = %ld", gridsize2);
   if (vlistNumber(vlistID2) != CDI_REAL) gridsize2 *= 2;
-  double *array2 = (double *) Malloc(gridsize2 * sizeof(double));
-  double *array3 = (double *) Malloc(gridsize2 * sizeof(double));
+  Varray<double> array2(gridsize2);
+  Varray<double> array3(gridsize2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
 
           samplegrid(missval, nsamplegrids, &cellindex[0], array1, array2, array3);
 
-          nmiss = arrayNumMV(gridsize2, array2, missval);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2, nmiss);
+          nmiss = varrayNumMV(gridsize2, array2, missval);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
 
-          nmiss = arrayNumMV(gridsize2, array3, missval);
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, array3, nmiss);
+          nmiss = varrayNumMV(gridsize2, array3, missval);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, array3.data(), nmiss);
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   vlistDestroy(vlistID2);
   gridDestroy(gridID2);
 
-  if (array3) Free(array3);
-  if (array2) Free(array2);
-  if (array1) Free(array1);
-
   for (int i = 0; i < nsamplegrids; ++i) free_cellindex(cellindex[i]);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
