@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,11 +28,13 @@
       Seltime    seldate         Select dates
       Seltime    selsmon         Select single month
 */
-
+#include <cassert>
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "param_conversion.h"
 #include "listarray.h"
 #include "util_string.h"
 
@@ -43,7 +45,7 @@ season_to_months(const char *season, int *imonths)
   const int imons[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };
   assert(strlen(smons) == (sizeof(imons) / sizeof(int)));
 
-  size_t len = strlen(season);
+  const auto len = strlen(season);
   if (len == 3 && strcmp(season, "ANN") == 0)
     {
       for (size_t k = 0; k < 12; ++k) imonths[k + 1] = 1;
@@ -52,12 +54,12 @@ season_to_months(const char *season, int *imonths)
     {
       if (len > 12) cdoAbort("Too many months %d (limit=12)!", (int) len);
       char *season_u = strdup(season);
-      strtoupper(season_u);
+      cstrToUpperCase(season_u);
       const char *sstr = strstr(smons, season_u);
       free(season_u);
-      if (sstr == NULL) cdoAbort("Season %s not available!", season);
-      size_t ks = (size_t)(sstr - smons);
-      size_t ke = ks + len;
+      if (sstr == nullptr) cdoAbort("Season %s not available!", season);
+      const size_t ks = (size_t)(sstr - smons);
+      const size_t ke = ks + len;
       for (size_t k = ks; k < ke; ++k) imonths[imons[k]]++;
     }
 }
@@ -68,12 +70,12 @@ seaslist(ListArray<int> &listArrayInt)
   int imon[13]; /* 1-12 ! */
   for (int i = 0; i < 13; ++i) imon[i] = 0;
 
-  int nsel = operatorArgc();
-  if (isdigit(*operatorArgv()[0]))
+  auto nsel = operatorArgc();
+  if (isdigit(cdoOperatorArgv(0)[0]))
     {
       for (int i = 0; i < nsel; i++)
         {
-          int ival = parameter2int(operatorArgv()[i]);
+          const auto ival = parameter2int(cdoOperatorArgv(i));
           // clang-format off
           if      ( ival == 1 ) { imon[12]++; imon[ 1]++; imon[ 2]++; }
           else if ( ival == 2 ) { imon[ 3]++; imon[ 4]++; imon[ 5]++; }
@@ -85,7 +87,7 @@ seaslist(ListArray<int> &listArrayInt)
     }
   else
     {
-      for (int i = 0; i < nsel; i++) season_to_months(operatorArgv()[i], imon);
+      for (int i = 0; i < nsel; i++) season_to_months(cdoOperatorArgv(i).c_str(), imon);
     }
 
   nsel = 0;
@@ -96,12 +98,12 @@ seaslist(ListArray<int> &listArrayInt)
 }
 
 double
-datestr_to_double(const char *datestr, int opt)
+datestrToDouble(const char *datestr, int opt)
 {
   int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
   double fval = 0;
 
-  size_t len = strlen(datestr);
+  const auto len = strlen(datestr);
 
   for (size_t i = 0; i < len; ++i)
     {
@@ -117,7 +119,7 @@ datestr_to_double(const char *datestr, int opt)
       second = 59;
     }
 
-  if (strchr(datestr, '-') == NULL)
+  if (strchr(datestr, '-') == nullptr)
     {
       fval = parameter2double(datestr);
     }
@@ -126,7 +128,7 @@ datestr_to_double(const char *datestr, int opt)
       int status = sscanf(datestr, "%d-%d-%dT%d:%d:%d", &year, &month, &day, &hour, &minute, &second);
       if (status != 6) cdoAbort("Invalid date string >%s<!", datestr);
       fval = cdiEncodeTime(hour, minute, second);
-      if (fabs(fval) > 0) fval /= 1000000;
+      if (std::fabs(fval) > 0) fval /= 1000000;
       fval += cdiEncodeDate(year, month, day);
     }
   else
@@ -134,7 +136,7 @@ datestr_to_double(const char *datestr, int opt)
       int status = sscanf(datestr, "%d-%d-%d", &year, &month, &day);
       if (status != 3) cdoAbort("Invalid date string >%s<!", datestr);
       fval = cdiEncodeTime(hour, minute, second);
-      if (fabs(fval) > 0) fval /= 1000000;
+      if (std::fabs(fval) > 0) fval /= 1000000;
       fval += cdiEncodeDate(year, month, day);
     }
 
@@ -153,15 +155,15 @@ datelist(ListArray<double> &listArrayFlt)
 
   for (int i = 0; i < nsel; i++)
     {
-      if (operatorArgv()[i][0] == '-' && operatorArgv()[i][1] == 0)
+      if (cdoOperatorArgv(i)[0] == '-' && cdoOperatorArgv(i)[1] == 0)
         {
           fval = (i == 0) ? -99999999999. : 99999999999.;
           listArrayFlt.setValue(i, fval);
         }
       else
         {
-          fval = datestr_to_double(operatorArgv()[i], 0);
-          if (strchr(operatorArgv()[i], 'T'))
+          fval = datestrToDouble(cdoOperatorArgv(i).c_str(), 0);
+          if (strchr(cdoOperatorArgv(i).c_str(), 'T'))
             set2 = false;
           else if (nsel > 1 && i > 0)
             fval += 0.999;
@@ -183,7 +185,7 @@ datelist(ListArray<double> &listArrayFlt)
 void *
 Seltime(void *process)
 {
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int nrecs;
   int varID, levelID;
   int nsel = 0;
@@ -200,27 +202,25 @@ Seltime(void *process)
   std::vector<int> vtime_list;
   ListArray<int> listArrayInt;
   ListArray<double> listArrayFlt;
-  Field ***vars = NULL;
 
   cdoInitialize(process);
 
-  bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
-  int SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     0, "timesteps");
-  int SELDATE     = cdoOperatorAdd("seldate",     func_datetime, 0, "start date and end date (format YYYY-MM-DDThh:mm:ss)");
-  int SELTIME     = cdoOperatorAdd("seltime",     func_time,     0, "times (format hh:mm:ss)");
-  int SELHOUR     = cdoOperatorAdd("selhour",     func_time,     0, "hours");
-  int SELDAY      = cdoOperatorAdd("selday",      func_date,     0, "days");
-  int SELMONTH    = cdoOperatorAdd("selmonth",    func_date,     0, "months");
-  int SELYEAR     = cdoOperatorAdd("selyear",     func_date,     0, "years");
-  int SELSEASON   = cdoOperatorAdd("selseason",   func_date,     0, "seasons");
-  int SELSMON     = cdoOperatorAdd("selsmon",     func_date,     0, "month[,nts1[,nts2]]");
+  const auto SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     0, "timesteps");
+  const auto SELDATE     = cdoOperatorAdd("seldate",     func_datetime, 0, "start date and end date (format YYYY-MM-DDThh:mm:ss)");
+  const auto SELTIME     = cdoOperatorAdd("seltime",     func_time,     0, "times (format hh:mm:ss)");
+  const auto SELHOUR     = cdoOperatorAdd("selhour",     func_time,     0, "hours");
+  const auto SELDAY      = cdoOperatorAdd("selday",      func_date,     0, "days");
+  const auto SELMONTH    = cdoOperatorAdd("selmonth",    func_date,     0, "months");
+  const auto SELYEAR     = cdoOperatorAdd("selyear",     func_date,     0, "years");
+  const auto SELSEASON   = cdoOperatorAdd("selseason",   func_date,     0, "seasons");
+  const auto SELSMON     = cdoOperatorAdd("selsmon",     func_date,     0, "month[,nts1[,nts2]]");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
@@ -238,20 +238,21 @@ Seltime(void *process)
       if (nsel < 1) cdoAbort("Too few arguments!");
       for (i = 0; i < nsel; i++)
         {
-          if (strchr(operatorArgv()[i], ':'))
+            const char * currentArgument = cdoOperatorArgv(i).c_str();
+          if (strchr(currentArgument, ':'))
             {
-              sscanf(operatorArgv()[i], "%d:%d:%d", &hour, &minute, &second);
+              sscanf(currentArgument, "%d:%d:%d", &hour, &minute, &second);
               listArrayInt.setValue(i, cdiEncodeTime(hour, minute, second));
             }
           else
             {
-              listArrayInt.setValue(i, parameter2int(operatorArgv()[i]));
+              listArrayInt.setValue(i, parameter2int(currentArgument));
             }
         }
     }
   else
     {
-      nsel = listArrayInt.argvToInt(operatorArgc(), operatorArgv());
+      nsel = listArrayInt.argvToInt(cdoGetOperArgv());
     }
 
   if (nsel < 1) cdoAbort("No timestep selected!");
@@ -266,7 +267,7 @@ Seltime(void *process)
 
       if (nsel > 3) cdoAbort("Too many parameters");
 
-      if (cdoVerbose) cdoPrint("mon=%d  nts1=%d  nts2=%d", intarr[0], nts1, nts2);
+      if (Options::cdoVerbose) cdoPrint("mon=%d  nts1=%d  nts2=%d", intarr[0], nts1, nts2);
 
       nsel = 1;
     }
@@ -278,41 +279,51 @@ Seltime(void *process)
       for (i = 0; i < nsel; i++) selfound[i] = false;
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
   vlistDefNtsteps(vlistID2, (nsel == 1 && operfunc == func_step) ? 1 : -1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      auto gridsizemax = vlistGridsizeMax(vlistID1);
       if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
       array.resize(gridsizemax);
     }
 
-  int ntsteps = vlistNtsteps(vlistID1);
+  auto ntsteps = vlistNtsteps(vlistID1);
 
   // add support for negative timestep values
-  if (operatorID == SELTIMESTEP && ntsteps > 0)
+  if (operatorID == SELTIMESTEP)
     {
       for (i = 0; i < nsel; i++)
         {
           if (intarr[i] < 0)
             {
-              if (cdoVerbose) cdoPrint("timestep %d changed to %d", intarr[i], ntsteps + 1 + intarr[i]);
-              intarr[i] = ntsteps + 1 + intarr[i];
+              if (ntsteps < 0 && !cdoAssertFilesOnly() == false)
+                {
+                  int tsID = 0;
+                  while (cdoStreamInqTimestep(streamID1, tsID)) tsID++;
+                  ntsteps = tsID;
+                  if (Options::cdoVerbose) cdoPrint("Found %d timesteps", ntsteps);
+                }
+              if (ntsteps > 0)
+                {
+                  if (Options::cdoVerbose) cdoPrint("timestep %d changed to %d", intarr[i], ntsteps + 1 + intarr[i]);
+                  intarr[i] = ntsteps + 1 + intarr[i];
+                }
             }
         }
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       for (int i = 0; i < nsel; i++)
         if (operatorID == SELDATE)
@@ -321,12 +332,14 @@ Seltime(void *process)
           cdoPrint("intarr entry: %d %d", i + 1, intarr[i]);
     }
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   int nconst = 0;
   for (varID = 0; varID < nvars; varID++)
     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
 
-  bool lnts1 = (operatorID == SELSMON) && (nts1 > 0);
+  FieldVector3D vars;
+
+  const bool lnts1 = (operatorID == SELSMON) && (nts1 > 0);
 
   if (lnts1 || nconst)
     {
@@ -340,23 +353,23 @@ Seltime(void *process)
           nts1 = 1;
         }
 
-      vars = (Field ***) Malloc(nts1 * sizeof(Field **));
+      vars.resize(nts1);
 
       for (int tsID = 0; tsID < nts1; tsID++)
         {
-          vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
+          fieldsFromVlist(vlistID1, vars[tsID], FIELD_NONE);
 
           for (varID = 0; varID < nvars; varID++)
             {
               if (lnts1 || (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT))
                 {
-                  int gridID = vlistInqVarGrid(vlistID1, varID);
-                  int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  size_t gridsize = gridInqSize(gridID);
+                  const auto gridID = vlistInqVarGrid(vlistID1, varID);
+                  const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                  const auto gridsize = gridInqSize(gridID);
 
                   for (levelID = 0; levelID < nlevel; levelID++)
                     {
-                      vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                      vars[tsID][varID][levelID].resize(gridsize);
                     }
                 }
             }
@@ -367,8 +380,8 @@ Seltime(void *process)
   int tsID2 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       bool copytimestep = false;
       int selival = -1;
@@ -428,7 +441,7 @@ Seltime(void *process)
         }
 
       bool copy_nts2 = false;
-      if (operatorID == SELSMON && copytimestep == false)
+      if (operatorID == SELSMON && !copytimestep)
         {
           copy_nts2 = false;
 
@@ -453,8 +466,8 @@ Seltime(void *process)
           taxisCopyTimestep(taxisID2, taxisID1);
           if (tsID2 == 0)
             {
-              streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-              pstreamDefVlist(streamID2, vlistID2);
+              streamID2 = cdoOpenWrite(1);
+              cdoDefVlist(streamID2, vlistID2);
             }
 
           if (lnts1 && ncts == 0)
@@ -470,18 +483,18 @@ Seltime(void *process)
                 {
                   taxisDefVdate(taxisID2, vdate_list[it]);
                   taxisDefVtime(taxisID2, vtime_list[it]);
-                  pstreamDefTimestep(streamID2, tsID2++);
+                  cdoDefTimestep(streamID2, tsID2++);
 
                   for (varID = 0; varID < nvars; varID++)
                     {
                       if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT && tsID2 > 1) continue;
-                      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                      for (levelID = 0; levelID < nlevel; levelID++)
+                      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                      for (levelID = 0; levelID < nlevels; levelID++)
                         {
-                          pstreamDefRecord(streamID2, varID, levelID);
-                          double *single = vars[it][varID][levelID].ptr;
-                          size_t nmiss = vars[it][varID][levelID].nmiss;
-                          pstreamWriteRecord(streamID2, single, nmiss);
+                          cdoDefRecord(streamID2, varID, levelID);
+                          double *single = vars[it][varID][levelID].vec.data();
+                          const size_t nmiss = vars[it][varID][levelID].nmiss;
+                          cdoWriteRecord(streamID2, single, nmiss);
                         }
                     }
                 }
@@ -496,7 +509,7 @@ Seltime(void *process)
               process_nts1 = true;
             }
 
-          pstreamDefTimestep(streamID2, tsID2++);
+          cdoDefTimestep(streamID2, tsID2++);
 
           if (tsID > 0 && lconstout)
             {
@@ -506,13 +519,13 @@ Seltime(void *process)
                 {
                   if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                     {
-                      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                      const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
                       for (levelID = 0; levelID < nlevel; levelID++)
                         {
-                          pstreamDefRecord(streamID2, varID, levelID);
-                          double *single = vars[nts][varID][levelID].ptr;
-                          size_t nmiss = vars[nts][varID][levelID].nmiss;
-                          pstreamWriteRecord(streamID2, single, nmiss);
+                          cdoDefRecord(streamID2, varID, levelID);
+                          double *single = vars[nts][varID][levelID].vec.data();
+                          const size_t nmiss = vars[nts][varID][levelID].nmiss;
+                          cdoWriteRecord(streamID2, single, nmiss);
                         }
                     }
                 }
@@ -520,16 +533,16 @@ Seltime(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
@@ -554,12 +567,10 @@ Seltime(void *process)
                         for (varID = 0; varID < nvars; varID++)
                           {
                             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-                            int gridID = vlistInqVarGrid(vlistID1, varID);
-                            size_t gridsize = gridInqSize(gridID);
-                            int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                            for (levelID = 0; levelID < nlevel; levelID++)
+                            const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                            for (levelID = 0; levelID < nlevels; levelID++)
                               {
-                                memcpy(vars[it][varID][levelID].ptr, vars[it + 1][varID][levelID].ptr, gridsize * sizeof(double));
+                                vars[it][varID][levelID].vec = vars[it + 1][varID][levelID].vec;
                                 vars[it][varID][levelID].nmiss = vars[it + 1][varID][levelID].nmiss;
                               }
                           }
@@ -573,11 +584,11 @@ Seltime(void *process)
 
               for (int recID = 0; recID < nrecs; recID++)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
+                  cdoInqRecord(streamID1, &varID, &levelID);
                   if (lnts1 || (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT))
                     {
-                      double *single = vars[nts][varID][levelID].ptr;
-                      pstreamReadRecord(streamID1, single, &nmiss);
+                      double *single = vars[nts][varID][levelID].vec.data();
+                      cdoReadRecord(streamID1, single, &nmiss);
                       vars[nts][varID][levelID].nmiss = nmiss;
                     }
                 }
@@ -587,8 +598,8 @@ Seltime(void *process)
       tsID++;
     }
 
-  if (streamID2 != -1) pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   if (operatorID == SELSMON)
     if (its2 < nts2) cdoWarning("%d timesteps missing after the last month!", nts2 - its2);
@@ -599,19 +610,19 @@ Seltime(void *process)
         {
 
           int isel2;
-          int lcont = FALSE;
+          bool lcont = false;
           for (isel2 = isel + 1; isel2 < nsel; isel2++)
             if (selfound[isel2]) break;
-          if (isel2 == nsel && (nsel - isel) > 1) lcont = TRUE;
+          if (isel2 == nsel && (nsel - isel) > 1) lcont = true;
 
           if (operatorID == SELTIMESTEP)
             {
-              int lcont2 = FALSE;
+              bool lcont2 = false;
               if (lcont)
                 {
                   for (isel2 = isel + 1; isel2 < nsel; isel2++)
                     if (intarr[isel2 - 1] != intarr[isel2] - 1) break;
-                  if (isel2 == nsel) lcont2 = TRUE;
+                  if (isel2 == nsel) lcont2 = true;
                 }
 
               if (lcont2)
@@ -653,17 +664,11 @@ Seltime(void *process)
         }
     }
 
-  if (lnts1 || nconst)
-    {
-      for (tsID = 0; tsID < nts1; tsID++) field_free(vars[tsID], vlistID2);
-      if (vars) Free(vars);
-    }
-
   vlistDestroy(vlistID2);
 
   if (tsID2 == 0) cdoAbort("No timesteps selected!");
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
