@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,138 +32,70 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
+#include "param_conversion.h"
 #include "calendar.h"
-#include "pstream_int.h"
 #include "util_string.h"
 #include "datetime.h"
 
+
 int
-get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit)
+get_tunits(const char *unit, int &incperiod, int &incunit, int &tunit)
 {
-  size_t len = strlen(unit);
+  const size_t len = strlen(unit);
 
-  if (memcmp(unit, "seconds", len) == 0)
-    {
-      *incunit = 1;
-      *tunit = TUNIT_SECOND;
-    }
-  else if (memcmp(unit, "minutes", len) == 0)
-    {
-      *incunit = 60;
-      *tunit = TUNIT_MINUTE;
-    }
-  else if (memcmp(unit, "hours", len) == 0)
-    {
-      *incunit = 3600;
-      *tunit = TUNIT_HOUR;
-    }
-  else if (memcmp(unit, "3hours", len) == 0)
-    {
-      *incunit = 10800;
-      *tunit = TUNIT_3HOURS;
-    }
-  else if (memcmp(unit, "6hours", len) == 0)
-    {
-      *incunit = 21600;
-      *tunit = TUNIT_6HOURS;
-    }
-  else if (memcmp(unit, "12hours", len) == 0)
-    {
-      *incunit = 43200;
-      *tunit = TUNIT_12HOURS;
-    }
-  else if (memcmp(unit, "days", len) == 0)
-    {
-      *incunit = 86400;
-      *tunit = TUNIT_DAY;
-    }
-  else if (memcmp(unit, "months", len) == 0)
-    {
-      *incunit = 1;
-      *tunit = TUNIT_MONTH;
-    }
-  else if (memcmp(unit, "years", len) == 0)
-    {
-      *incunit = 12;
-      *tunit = TUNIT_YEAR;
-    }
-  else
-    cdoAbort("Time unit >%s< unsupported!", unit);
+  // clang-format off
+  if      (memcmp(unit, "seconds", len) == 0) { incunit = 1;     tunit = TUNIT_SECOND; }
+  else if (memcmp(unit, "minutes", len) == 0) { incunit = 60;    tunit = TUNIT_MINUTE; }
+  else if (memcmp(unit, "hours", len) == 0)   { incunit = 3600;  tunit = TUNIT_HOUR; }
+  else if (memcmp(unit, "3hours", len) == 0)  { incunit = 10800; tunit = TUNIT_3HOURS; }
+  else if (memcmp(unit, "6hours", len) == 0)  { incunit = 21600; tunit = TUNIT_6HOURS; }
+  else if (memcmp(unit, "12hours", len) == 0) { incunit = 43200; tunit = TUNIT_12HOURS; }
+  else if (memcmp(unit, "days", len) == 0)    { incunit = 86400; tunit = TUNIT_DAY; }
+  else if (memcmp(unit, "months", len) == 0)  { incunit = 1;     tunit = TUNIT_MONTH; }
+  else if (memcmp(unit, "years", len) == 0)   { incunit = 12;    tunit = TUNIT_YEAR; }
+  else cdoAbort("Time unit >%s< unsupported!", unit);
 
-  if (*tunit == TUNIT_HOUR)
+  if (tunit == TUNIT_HOUR)
     {
-      if (*incperiod == 3)
-        {
-          *incperiod = 1;
-          *incunit = 10800;
-          *tunit = TUNIT_3HOURS;
-        }
-      else if (*incperiod == 6)
-        {
-          *incperiod = 1;
-          *incunit = 21600;
-          *tunit = TUNIT_6HOURS;
-        }
-      else if (*incperiod == 12)
-        {
-          *incperiod = 1;
-          *incunit = 43200;
-          *tunit = TUNIT_12HOURS;
-        }
+      if      (incperiod ==  3) { incperiod = 1; incunit = 10800; tunit = TUNIT_3HOURS;  }
+      else if (incperiod ==  6) { incperiod = 1; incunit = 21600; tunit = TUNIT_6HOURS;  }
+      else if (incperiod == 12) { incperiod = 1; incunit = 43200; tunit = TUNIT_12HOURS; }
     }
+  // clang-format on
 
   return 0;
 }
 
 static void
-shifttime(int calendar, int tunit, int64_t ijulinc, int64_t *pdate, int *ptime)
+shifttime(int calendar, int tunit, int64_t ijulinc, int64_t &vdate, int &vtime)
 {
-  int64_t vdate = *pdate;
-  int vtime = *ptime;
-
   if (tunit == TUNIT_MONTH || tunit == TUNIT_YEAR)
     {
       int year, month, day;
       cdiDecodeDate(vdate, &year, &month, &day);
 
       month += (int) ijulinc;
-
-      while (month > 12)
-        {
-          month -= 12;
-          year++;
-        }
-      while (month < 1)
-        {
-          month += 12;
-          year--;
-        }
+      adjustMonthAndYear(month, year);
 
       vdate = cdiEncodeDate(year, month, day);
-
-      *pdate = vdate;
     }
   else
     {
-      juldate_t juldate;
-      juldate = juldate_encode(calendar, vdate, vtime);
-      juldate = juldate_add_seconds(ijulinc, juldate);
-      juldate_decode(calendar, juldate, &vdate, &vtime);
+      auto juldate = julianDateEncode(calendar, vdate, vtime);
+      juldate = julianDateAddSeconds(ijulinc, juldate);
+      julianDateDecode(calendar, juldate, vdate, vtime);
 
-      *pdate = vdate;
-      *ptime = vtime;
-
-      if (cdoVerbose)
-        cdoPrint("juldate, ijulinc, vdate, vtime: %g %lld %lld %d", juldate_to_seconds(juldate), ijulinc, vdate, vtime);
+      if (Options::cdoVerbose)
+        cdoPrint("juldate, ijulinc, vdate, vtime: %g %lld %lld %d", julianDateToSeconds(juldate), ijulinc, vdate, vtime);
     }
 }
 
 static void
 time_gen_bounds(int calendar, int tunit, int incperiod, int64_t vdate, int vtime, int64_t *vdateb, int *vtimeb)
 {
-  juldate_t juldate;
-
   vdateb[0] = vdate;
   vdateb[1] = vdate;
   vtimeb[0] = 0;
@@ -191,21 +123,20 @@ time_gen_bounds(int calendar, int tunit, int incperiod, int64_t vdate, int vtime
   else if (tunit == TUNIT_DAY)
     {
       vdateb[0] = vdate;
-      juldate = juldate_encode(calendar, vdateb[0], vtimeb[0]);
-      juldate = juldate_add_seconds(86400, juldate);
-      juldate_decode(calendar, juldate, &vdateb[1], &vtimeb[1]);
+      auto juldate = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
+      juldate = julianDateAddSeconds(86400, juldate);
+      julianDateDecode(calendar, juldate, vdateb[1], vtimeb[1]);
     }
   else if (tunit == TUNIT_HOUR || tunit == TUNIT_3HOURS || tunit == TUNIT_6HOURS || tunit == TUNIT_12HOURS)
     {
       if (incperiod == 0) incperiod = 1;
       if (incperiod > 24) cdoAbort("Time period must be less equal 24!");
 
-      if (tunit == TUNIT_3HOURS)
-        incperiod = 3;
-      else if (tunit == TUNIT_6HOURS)
-        incperiod = 6;
-      else if (tunit == TUNIT_12HOURS)
-        incperiod = 12;
+      // clang-format off
+      if      (tunit == TUNIT_3HOURS)  incperiod = 3;
+      else if (tunit == TUNIT_6HOURS)  incperiod = 6;
+      else if (tunit == TUNIT_12HOURS) incperiod = 12;
+      // clang-format on
 
       int hour, minute, second;
       cdiDecodeTime(vtime, &hour, &minute, &second);
@@ -215,15 +146,37 @@ time_gen_bounds(int calendar, int tunit, int incperiod, int64_t vdate, int vtime
       if (h1 >= 24)
         {
           vdateb[1] = cdiEncodeDate(year, month, day + 1);
-          juldate = juldate_encode(calendar, vdateb[0], vtimeb[0]);
-          juldate = juldate_add_seconds(incperiod * 3600, juldate);
-          juldate_decode(calendar, juldate, &vdateb[1], &vtimeb[1]);
+          auto juldate = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
+          juldate = julianDateAddSeconds(incperiod * 3600, juldate);
+          julianDateDecode(calendar, juldate, vdateb[1], vtimeb[1]);
         }
       else
         vtimeb[1] = cdiEncodeTime(h1, 0, 0);
     }
 }
 
+int
+evaluateCalendarString(int operatorID, const std::string &calendarName)
+{
+  int calendar = CALENDAR_STANDARD;
+  const auto calendarString = stringToLower(calendarName);
+  // clang-format off
+  if      (calendarString == "standard")  calendar = CALENDAR_STANDARD;
+  else if (calendarString == "gregorian") calendar = CALENDAR_GREGORIAN;
+  else if (calendarString == "proleptic") calendar = CALENDAR_PROLEPTIC;
+  else if (calendarString == "proleptic_gregorian") calendar = CALENDAR_PROLEPTIC;
+  else if (calendarString == "360days") calendar = CALENDAR_360DAYS;
+  else if (calendarString == "360_day") calendar = CALENDAR_360DAYS;
+  else if (calendarString == "365days") calendar = CALENDAR_365DAYS;
+  else if (calendarString == "365_day") calendar = CALENDAR_365DAYS;
+  else if (calendarString == "366days") calendar = CALENDAR_366DAYS;
+  else if (calendarString == "366_day") calendar = CALENDAR_366DAYS;
+  else cdoAbort("Calendar >%s< unsupported! Available %s", calendarName.c_str(), cdoOperatorEnter(operatorID));
+  // clang-format on
+
+  return calendar;
+}
+
 void *
 Settime(void *process)
 {
@@ -240,29 +193,29 @@ Settime(void *process)
   int64_t ijulinc = 0;
   int incperiod = 1, incunit = 86400;
   int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
-  int day0;
+  int day0 = 0;
   bool copy_timestep = false;
   int newcalendar = CALENDAR_STANDARD;
   // int nargs;
-  juldate_t juldate;
+  JulianDate juldate;
 
   cdoInitialize(process);
 
   // clang-format off
-  int SETYEAR     = cdoOperatorAdd("setyear",      0,  1, "year");
-  int SETMON      = cdoOperatorAdd("setmon",       0,  1, "month");
-  int SETDAY      = cdoOperatorAdd("setday",       0,  1, "day");
-  int SETDATE     = cdoOperatorAdd("setdate",      0,  1, "date (format: YYYY-MM-DD)");
-  int SETTIME     = cdoOperatorAdd("settime",      0,  1, "time (format: hh:mm:ss)");
-  int SETTUNITS   = cdoOperatorAdd("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
-  int SETTAXIS    = cdoOperatorAdd("settaxis",     0, -2, "date<,time<,increment>> (format YYYY-MM-DD,hh:mm:ss)");
-  int SETTBOUNDS  = cdoOperatorAdd("settbounds",   0,  1, "frequency (day, month, year)");
-  int SETREFTIME  = cdoOperatorAdd("setreftime",   0, -2, "date<,time<,units>> (format YYYY-MM-DD,hh:mm:ss)");
-  int SETCALENDAR = cdoOperatorAdd("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
-  int SHIFTTIME   = cdoOperatorAdd("shifttime",    0,  1, "shift value");
+  const auto SETYEAR     = cdoOperatorAdd("setyear",      0,  1, "year");
+  const auto SETMON      = cdoOperatorAdd("setmon",       0,  1, "month");
+  const auto SETDAY      = cdoOperatorAdd("setday",       0,  1, "day");
+  const auto SETDATE     = cdoOperatorAdd("setdate",      0,  1, "date (format: YYYY-MM-DD)");
+  const auto SETTIME     = cdoOperatorAdd("settime",      0,  1, "time (format: hh:mm:ss)");
+  const auto SETTUNITS   = cdoOperatorAdd("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
+  const auto SETTAXIS    = cdoOperatorAdd("settaxis",     0, -2, "date<,time<,increment>> (format YYYY-MM-DD,hh:mm:ss)");
+  const auto SETTBOUNDS  = cdoOperatorAdd("settbounds",   0,  1, "frequency (day, month, year)");
+  const auto SETREFTIME  = cdoOperatorAdd("setreftime",   0, -2, "date<,time<,units>> (format YYYY-MM-DD,hh:mm:ss)");
+  const auto SETCALENDAR = cdoOperatorAdd("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
+  const auto SHIFTTIME   = cdoOperatorAdd("shifttime",    0,  1, "shift value");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
   // nargs = cdoOperatorF2(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
@@ -274,8 +227,8 @@ Settime(void *process)
       if (operatorArgc() < 1) cdoAbort("Too few arguments!");
       if (operatorArgc() > 3) cdoAbort("Too many arguments!");
 
-      const char *datestr = operatorArgv()[0];
-      const char *timestr = operatorArgv()[1];
+      const auto datestr = cdoOperatorArgv(0).c_str();
+      const auto timestr = cdoOperatorArgv(1).c_str();
 
       if (strchr(datestr + 1, '-'))
         {
@@ -301,25 +254,25 @@ Settime(void *process)
 
           if (operatorArgc() == 3)
             {
-              const char *timeunits = operatorArgv()[2];
+              auto timeunits = cdoOperatorArgv(2).c_str();
               int ich = timeunits[0];
               if (ich == '-' || ich == '+' || isdigit(ich))
                 {
-                  incperiod = (int) strtol(timeunits, NULL, 10);
+                  incperiod = (int) strtol(timeunits, nullptr, 10);
                   if (ich == '-' || ich == '+') timeunits++;
                   while (isdigit((int) *timeunits)) timeunits++;
                 }
-              get_tunits(timeunits, &incperiod, &incunit, &tunit);
+              get_tunits(timeunits, incperiod, incunit, tunit);
             }
         }
 
-      /* increment in seconds */
+      // increment in seconds
       ijulinc = (int64_t) incperiod * incunit;
     }
   else if (operatorID == SETDATE)
     {
       operatorCheckArgc(1);
-      const char *datestr = operatorArgv()[0];
+      const auto datestr = cdoOperatorArgv(0).c_str();
       if (strchr(datestr, '-'))
         {
           sscanf(datestr, "%d-%d-%d", &year, &month, &day);
@@ -333,7 +286,7 @@ Settime(void *process)
   else if (operatorID == SETTIME)
     {
       operatorCheckArgc(1);
-      const char *timestr = operatorArgv()[0];
+      const auto timestr = cdoOperatorArgv(0).c_str();
 
       if (strchr(timestr, ':'))
         {
@@ -348,25 +301,25 @@ Settime(void *process)
   else if (operatorID == SHIFTTIME)
     {
       operatorCheckArgc(1);
-      const char *timeunits = operatorArgv()[0];
-      incperiod = (int) strtol(timeunits, NULL, 10);
+      auto timeunits = cdoOperatorArgv(0).c_str();
+      incperiod = (int) strtol(timeunits, nullptr, 10);
       if (timeunits[0] == '-' || timeunits[0] == '+') timeunits++;
       while (isdigit((int) *timeunits)) timeunits++;
 
-      get_tunits(timeunits, &incperiod, &incunit, &tunit);
+      get_tunits(timeunits, incperiod, incunit, tunit);
 
-      /* increment in seconds */
+      // increment in seconds
       ijulinc = (int64_t) incperiod * incunit;
     }
   else if (operatorID == SETTUNITS || operatorID == SETTBOUNDS)
     {
       operatorCheckArgc(1);
-      const char *timeunits = operatorArgv()[0];
-      incperiod = (int) strtol(timeunits, NULL, 10);
+      auto timeunits = cdoOperatorArgv(0).c_str();
+      incperiod = (int) strtol(timeunits, nullptr, 10);
       if (timeunits[0] == '-' || timeunits[0] == '+') timeunits++;
       while (isdigit((int) *timeunits)) timeunits++;
 
-      get_tunits(timeunits, &incperiod, &incunit, &tunit);
+      get_tunits(timeunits, incperiod, incunit, tunit);
 
       if (operatorID == SETTBOUNDS
           && !(tunit == TUNIT_HOUR || tunit == TUNIT_3HOURS || tunit == TUNIT_6HOURS || tunit == TUNIT_12HOURS || tunit == TUNIT_DAY
@@ -376,39 +329,24 @@ Settime(void *process)
   else if (operatorID == SETCALENDAR)
     {
       operatorCheckArgc(1);
-      char *cname = operatorArgv()[0];
-      strtolower(cname);
-      size_t len = strlen(cname);
-      if (len < 3) len = 7;
-      // clang-format off
-      if      (strcmp(cname, "standard") == 0) newcalendar = CALENDAR_STANDARD;
-      else if (strcmp(cname, "gregorian") == 0) newcalendar = CALENDAR_GREGORIAN;
-      else if (strcmp(cname, "proleptic") == 0) newcalendar = CALENDAR_PROLEPTIC;
-      else if (strcmp(cname, "proleptic_gregorian") == 0) newcalendar = CALENDAR_PROLEPTIC;
-      else if (strncmp(cname, "360days", len) == 0) newcalendar = CALENDAR_360DAYS;
-      else if (strncmp(cname, "360_day", len) == 0) newcalendar = CALENDAR_360DAYS;
-      else if (strncmp(cname, "365days", len) == 0) newcalendar = CALENDAR_365DAYS;
-      else if (strncmp(cname, "365_day", len) == 0) newcalendar = CALENDAR_365DAYS;
-      else if (strncmp(cname, "366days", len) == 0) newcalendar = CALENDAR_366DAYS;
-      else if (strncmp(cname, "366_day", len) == 0) newcalendar = CALENDAR_366DAYS;
-      else cdoAbort("Calendar >%s< unsupported! Available %s", cname, cdoOperatorEnter(operatorID));
-      // clang-format on
+      auto cname = cdoOperatorArgv(0);
+      newcalendar = evaluateCalendarString(operatorID, cname);
     }
   else
     {
       operatorCheckArgc(1);
-      newval = parameter2int(operatorArgv()[0]);
+      newval = parameter2int(cdoOperatorArgv(0));
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
   bool taxis_has_bounds = taxisHasBounds(taxisID1) > 0;
-  int ntsteps = vlistNtsteps(vlistID1);
-  int nvars = vlistNvars(vlistID1);
+  auto ntsteps = vlistNtsteps(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   if (ntsteps == 1)
     {
@@ -423,9 +361,9 @@ Settime(void *process)
       for (varID = 0; varID < nvars; ++varID) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
     }
 
-  int calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
 
-  if (cdoVerbose) cdoPrint("calendar = %d", calendar);
+  if (Options::cdoVerbose) cdoPrint("calendar = %d", calendar);
 
   if (operatorID == SETREFTIME)
     {
@@ -435,7 +373,7 @@ Settime(void *process)
         {
           cdoPrint("Changing absolute to relative time axis!");
 
-          taxisID2 = taxisCreate(TAXIS_RELATIVE);
+          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
         }
       else
         taxisID2 = taxisDuplicate(taxisID1);
@@ -451,7 +389,7 @@ Settime(void *process)
         {
           cdoPrint("Changing absolute to relative time axis!");
 
-          taxisID2 = taxisCreate(TAXIS_RELATIVE);
+          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
           taxisDefTunit(taxisID2, tunit);
         }
       else
@@ -466,11 +404,11 @@ Settime(void *process)
       */
       if (taxisInqType(taxisID1) == TAXIS_ABSOLUTE)
         { /*
-           if ( cdoFiletype() != CDI_FILETYPE_NC )
-             cdoAbort("This operator does not work on an absolute time axis!");
-          */
+            if ( CdoDefault::FileType != CDI_FILETYPE_NC )
+              cdoAbort("This operator does not work on an absolute time axis!");
+           */
           cdoPrint("Changing absolute to relative time axis!");
-          taxisID2 = taxisCreate(TAXIS_RELATIVE);
+          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
         }
       else
         taxisID2 = taxisDuplicate(taxisID1);
@@ -483,7 +421,7 @@ Settime(void *process)
       taxisDefTunit(taxisID2, tunit);
       taxisDefRdate(taxisID2, sdate);
       taxisDefRtime(taxisID2, stime);
-      juldate = juldate_encode(calendar, sdate, stime);
+      juldate = julianDateEncode(calendar, sdate, stime);
     }
   else if (operatorID == SETTUNITS)
     {
@@ -508,17 +446,17 @@ Settime(void *process)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-  std::vector<double> array(gridsizemax);
+  Varray<double> array(gridsizemax);
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      auto vdate = taxisInqVdate(taxisID1);
+      auto vtime = taxisInqVtime(taxisID1);
 
       if (operatorID == SETTAXIS)
         {
@@ -533,17 +471,7 @@ Settime(void *process)
               else
                 {
                   month += (int) ijulinc;
-
-                  while (month > 12)
-                    {
-                      month -= 12;
-                      year++;
-                    }
-                  while (month < 1)
-                    {
-                      month += 12;
-                      year--;
-                    }
+                  adjustMonthAndYear(month, year);
 
                   day = (day0 == 31) ? days_per_month(calendar, year, month) : day0;
 
@@ -552,32 +480,32 @@ Settime(void *process)
             }
           else
             {
-              juldate_decode(calendar, juldate, &vdate, &vtime);
-              juldate = juldate_add_seconds(ijulinc, juldate);
+              julianDateDecode(calendar, juldate, vdate, vtime);
+              juldate = julianDateAddSeconds(ijulinc, juldate);
             }
         }
       else if (operatorID == SETTBOUNDS)
         {
           time_gen_bounds(calendar, tunit, incperiod, vdate, vtime, vdateb, vtimeb);
 
-          if (CDO_CMOR_Mode)
+          if (Options::CMOR_Mode)
             {
-              juldate_t juldate1 = juldate_encode(calendar, vdateb[0], vtimeb[0]);
-              juldate_t juldate2 = juldate_encode(calendar, vdateb[1], vtimeb[1]);
-              double seconds = juldate_to_seconds(juldate_sub(juldate2, juldate1)) / 2;
-              juldate_t juldatem = juldate_add_seconds(lround(seconds), juldate1);
-              juldate_decode(calendar, juldatem, &vdate, &vtime);
+              const auto juldate1 = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
+              const auto juldate2 = julianDateEncode(calendar, vdateb[1], vtimeb[1]);
+              const auto seconds = julianDateToSeconds(julianDateSub(juldate2, juldate1)) / 2;
+              const auto juldatem = julianDateAddSeconds(lround(seconds), juldate1);
+              julianDateDecode(calendar, juldatem, vdate, vtime);
             }
         }
       else if (operatorID == SHIFTTIME)
         {
-          shifttime(calendar, tunit, ijulinc, &vdate, &vtime);
+          shifttime(calendar, tunit, ijulinc, vdate, vtime);
           if (taxis_has_bounds)
             {
               taxisInqVdateBounds(taxisID1, &vdateb[0], &vdateb[1]);
               taxisInqVtimeBounds(taxisID1, &vtimeb[0], &vtimeb[1]);
-              shifttime(calendar, tunit, ijulinc, &vdateb[0], &vtimeb[0]);
-              shifttime(calendar, tunit, ijulinc, &vdateb[1], &vtimeb[1]);
+              shifttime(calendar, tunit, ijulinc, vdateb[0], vtimeb[0]);
+              shifttime(calendar, tunit, ijulinc, vdateb[1], vtimeb[1]);
             }
         }
       else if (operatorID == SETREFTIME || operatorID == SETCALENDAR || operatorID == SETTUNITS)
@@ -611,7 +539,7 @@ Settime(void *process)
         }
       else
         {
-          int numavg = taxisInqNumavg(taxisID1);
+          const auto numavg = taxisInqNumavg(taxisID1);
           taxisDefNumavg(taxisID2, numavg);
 
           taxisDefVdate(taxisID2, vdate);
@@ -624,30 +552,30 @@ Settime(void *process)
             }
         }
 
-      if (streamID2 == -1)
+      if (streamID2 == CDO_STREAM_UNDEF)
         {
-          streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-          pstreamDefVlist(streamID2, vlistID2);
+          streamID2 = cdoOpenWrite(1);
+          cdoDefVlist(streamID2, vlistID2);
         }
 
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, &array[0], &nmiss);
-          pstreamWriteRecord(streamID2, &array[0], nmiss);
+          cdoReadRecord(streamID1, &array[0], &nmiss);
+          cdoWriteRecord(streamID2, &array[0], nmiss);
         }
 
       tsID1++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
