@@ -23,19 +23,19 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "constants.h"  // planet radius
 #include "pmlist.h"
 #include "cdo_options.h"
 #include "progress.h"
 #include "cimdOmp.h"
-
 #include "grid_point_search.h"
 
+
 enum
 {
   FORM_LINEAR
@@ -55,16 +55,15 @@ struct smoothpoint_t
 static void
 smooth(int gridID, double missval, const double *restrict array1, double *restrict array2, size_t *nmiss, smoothpoint_t spoint)
 {
-  int gridID0 = gridID;
-  size_t gridsize = gridInqSize(gridID);
-  size_t numNeighbors = spoint.maxpoints;
+  auto gridID0 = gridID;
+  auto gridsize = gridInqSize(gridID);
+  auto numNeighbors = spoint.maxpoints;
   if (numNeighbors > gridsize) numNeighbors = gridsize;
 
-  std::vector<uint8_t> vmask(gridsize);
-  uint8_t *mask = vmask.data();
+  std::vector<uint8_t> mask(gridsize);
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(array1[i], missval);
 
-  int gridtype = gridInqType(gridID);
+  auto gridtype = gridInqType(gridID);
 
   if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
 
@@ -82,10 +81,9 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
         }
     }
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-    cdoAbort("Cell center coordinates missing!");
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
 
-  std::vector<double> xvals(gridsize), yvals(gridsize);
+  Varray<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, &xvals[0]);
   gridInqYvals(gridID, &yvals[0]);
 
@@ -103,7 +101,7 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 
   bool xIsCyclic = false;
   size_t dims[2] = { gridsize, 0 };
-  GridPointSearch *gps = gridPointSearchCreate(xIsCyclic, dims, gridsize, &xvals[0], &yvals[0]);
+  auto gps = gridPointSearchCreate(xIsCyclic, dims, gridsize, &xvals[0], &yvals[0]);
 
   gps->searchRadius = spoint.radius;
 
@@ -123,7 +121,7 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
@@ -131,10 +129,10 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
       findex++;
       if (Options::cdoVerbose && cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
 
-      gridSearchPoint(gps, xvals[i], yvals[i], knnWeights[ompthID]);
+      gridSearchPointSmooth(gps, xvals[i], yvals[i], knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(mask, spoint.radius, spoint.weight0, spoint.weightR);
+      const auto nadds = knnWeights[ompthID].computeWeights(mask, spoint.radius, spoint.weight0, spoint.weightR);
       if (nadds > naddsMax) naddsMax = nadds;
       if (nadds)
         {
@@ -172,13 +170,13 @@ smooth9_sum(size_t ij, uint8_t *mask, double sfac, const double *restrict array,
 static void
 smooth9(int gridID, double missval, const double *restrict array1, double *restrict array2, size_t *nmiss)
 {
-  size_t gridsize = gridInqSize(gridID);
-  size_t nlon = gridInqXsize(gridID);
-  size_t nlat = gridInqYsize(gridID);
-  int grid_is_cyclic = gridIsCircular(gridID);
+  auto gridsize = gridInqSize(gridID);
+  auto nlon = gridInqXsize(gridID);
+  auto nlat = gridInqYsize(gridID);
+  auto grid_is_cyclic = gridIsCircular(gridID);
 
   std::vector<uint8_t> vmask(gridsize);
-  uint8_t *mask = vmask.data();
+  auto mask = vmask.data();
 
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(missval, array1[i]);
 
@@ -268,40 +266,16 @@ smooth9(int gridID, double missval, const double *restrict array1, double *restr
     }
 }
 
-double
-radius_str_to_deg(const char *string)
-{
-  char *endptr = nullptr;
-  double radius = strtod(string, &endptr);
-
-  if (*endptr != 0)
-    {
-      if (strncmp(endptr, "km", 2) == 0)
-        radius = 360 * ((radius * 1000) / (2 * PlanetRadius * M_PI));
-      else if (strncmp(endptr, "m", 1) == 0)
-        radius = 360 * ((radius) / (2 * PlanetRadius * M_PI));
-      else if (strncmp(endptr, "deg", 3) == 0)
-        ;
-      else if (strncmp(endptr, "rad", 3) == 0)
-        radius *= RAD2DEG;
-      else
-        cdoAbort("Float parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-    }
-
-  if (radius > 180.) radius = 180.;
-
-  return radius;
-}
 
 static int
-convert_form(const char *formstr)
+convert_form(const std::string &formstr)
 {
   int form = FORM_LINEAR;
 
-  if (strcmp(formstr, "linear") == 0)
+  if (formstr == "linear")
     form = FORM_LINEAR;
   else
-    cdoAbort("form=%s unsupported!", formstr);
+    cdoAbort("form=%s unsupported!", formstr.c_str());
 
   return form;
 }
@@ -309,10 +283,10 @@ convert_form(const char *formstr)
 static void
 smoothGetParameter(int *xnsmooth, smoothpoint_t *spoint)
 {
-  const int pargc = operatorArgc();
+  const auto pargc = operatorArgc();
   if (pargc)
     {
-      const char **pargv = (const char **) operatorArgv();
+      const auto pargv = cdoGetOperArgv();
 
       KVList kvlist;
       kvlist.name = "SMOOTH";
@@ -321,19 +295,19 @@ smoothGetParameter(int *xnsmooth, smoothpoint_t *spoint)
 
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv.values[0].c_str();
+          const auto &key = kv.key;
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          const auto &value = kv.values[0];
 
           // clang-format off
-          if      (cstrIsEqual(key, "nsmooth")) *xnsmooth = parameter2int(value);
-          else if (cstrIsEqual(key, "maxpoints")) spoint->maxpoints = parameter2sizet(value);
-          else if (cstrIsEqual(key, "weight0")) spoint->weight0 = parameter2double(value);
-          else if (cstrIsEqual(key, "weightR")) spoint->weightR = parameter2double(value);
-          else if (cstrIsEqual(key, "radius")) spoint->radius = radius_str_to_deg(value);
-          else if (cstrIsEqual(key, "form")) spoint->form = convert_form(value);
-          else cdoAbort("Invalid parameter key >%s<!", key);
+          if      (key == "nsmooth")   *xnsmooth = parameter2int(value);
+          else if (key == "maxpoints") spoint->maxpoints = parameter2sizet(value);
+          else if (key == "weight0")   spoint->weight0 = parameter2double(value);
+          else if (key == "weightR")   spoint->weightR = parameter2double(value);
+          else if (key == "radius")    spoint->radius = radius_str_to_deg(value.c_str());
+          else if (key == "form")      spoint->form = convert_form(value);
+          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
           // clang-format on
         }
     }
@@ -354,33 +328,33 @@ Smooth(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int SMOOTH  = cdoOperatorAdd("smooth",   0,   0, nullptr);
-  const int SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, nullptr);
+  const auto SMOOTH  = cdoOperatorAdd("smooth",   0,   0, nullptr);
+  const auto SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == SMOOTH) smoothGetParameter(&xnsmooth, &spoint);
 
   if (spoint.radius < 0 || spoint.radius > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "radius", spoint.radius);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> varIDs(nvars);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int gridtype = gridInqType(gridID);
-      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto gridtype = gridInqType(gridID);
+      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR || gridtype == GRID_PROJECTION
           || (operatorID == SMOOTH9 && gridtype == GRID_GENERIC && gridInqXsize(gridID) > 0 && gridInqYsize(gridID) > 0))
         {
           varIDs[varID] = true;
@@ -398,38 +372,38 @@ Smooth(void *process)
         }
     }
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (gridsizemax < spoint.maxpoints) spoint.maxpoints = gridsizemax;
-  if (Options::cdoVerbose)
+  if (Options::cdoVerbose && operatorID == SMOOTH)
     cdoPrint("nsmooth = %d, maxpoints = %zu, radius = %gdeg, form = %s, weight0 = %g, weightR = %g", xnsmooth, spoint.maxpoints,
              spoint.radius, Form[spoint.form], spoint.weight0, spoint.weightR);
 
   spoint.radius *= DEG2RAD;
 
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
+  Varray<double> array1(gridsizemax);
+  Varray<double> array2(gridsizemax);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
           size_t nmiss;
 
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           if (varIDs[varID])
             {
-              double missval = vlistInqVarMissval(vlistID1, varID);
-              int gridID = vlistInqVarGrid(vlistID1, varID);
+              const auto missval = vlistInqVarMissval(vlistID1, varID);
+              const auto gridID = vlistInqVarGrid(vlistID1, varID);
 
               for (int i = 0; i < xnsmooth; ++i)
                 {
@@ -438,16 +412,16 @@ Smooth(void *process)
                   else if (operatorID == SMOOTH9)
                     smooth9(gridID, missval, array1.data(), array2.data(), &nmiss);
 
-                  arrayCopy(gridsizemax, array2.data(), array1.data());
+                  varrayCopy(gridsizemax, array2, array1);
                 }
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array2.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array1.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array1.data(), nmiss);
             }
         }
 
