@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "cdoOptions.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_cdiWrapper.h"
+#include "cdo_options.h"
+#include "cimdOmp.h"
 
 #define NALLOC_INC 1024
 
@@ -41,27 +43,31 @@ void *
 Timsort(void *process)
 {
   int nrecs;
-  int gridID, varID, levelID;
+  int varID, levelID;
   int nalloc = 0;
   size_t nmiss;
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisCreate(TAXIS_ABSOLUTE);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int nvars = vlistNvars(vlistID1);
-  std::vector<field_type **> vars;
-  std::vector<int> vdate, vtime;
+  VarList varList;
+  varListInit(varList, vlistID1);
+
+  const auto nvars = vlistNvars(vlistID1);
+  FieldVector3D vars;
+  std::vector<int64_t> vdate;
+  std::vector<int> vtime;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -77,15 +83,13 @@ Timsort(void *process)
       vdate[tsID] = taxisInqVdate(taxisID1);
       vtime[tsID] = taxisInqVtime(taxisID1);
 
-      vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
+      fieldsFromVlist(vlistID1, vars[tsID], FIELD_NONE);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          vars[tsID][varID][levelID].resize(varList[varID].gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
@@ -94,66 +98,58 @@ Timsort(void *process)
 
   int nts = tsID;
 
-  double **sarray = (double **) Malloc(Threading::ompNumThreads * sizeof(double *));
-  for (int i = 0; i < Threading::ompNumThreads; i++) sarray[i] = (double *) Malloc(nts * sizeof(double));
+  Varray2D<double> sarray(Threading::ompNumThreads);
+  for (int i = 0; i < Threading::ompNumThreads; i++) sarray[i].resize(nts);
 
   for (varID = 0; varID < nvars; varID++)
     {
       if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-      gridID = vlistInqVarGrid(vlistID1, varID);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
+      size_t gridsize = varList[varID].gridsize;
+      const int nlevels = varList[varID].nlevels;
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(gridsize, nts, sarray, vars, varID, levelID)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
-              int ompthID = cdo_omp_get_thread_num();
+              const auto ompthID = cdo_omp_get_thread_num();
 
-              for (int tsID = 0; tsID < nts; tsID++) sarray[ompthID][tsID] = vars[tsID][varID][levelID].ptr[i];
+              for (int tsID = 0; tsID < nts; tsID++) sarray[ompthID][tsID] = vars[tsID][varID][levelID].vec[i];
 
-              qsort(sarray[ompthID], nts, sizeof(double), compareDouble);
+              qsort(sarray[ompthID].data(), nts, sizeof(double), compareDouble);
 
-              for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].ptr[i] = sarray[ompthID][tsID];
+              for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].vec[i] = sarray[ompthID][tsID];
             }
         }
     }
 
-  for (int i = 0; i < Threading::ompNumThreads; i++)
-    if (sarray[i]) Free(sarray[i]);
-
-  if (sarray) Free(sarray);
-
   for (tsID = 0; tsID < nts; tsID++)
     {
       taxisDefVdate(taxisID2, vdate[tsID]);
       taxisDefVtime(taxisID2, vtime[tsID]);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
-          int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          const auto nlevels = varList[varID].nlevels;
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
-              if (vars[tsID][varID][levelID].ptr)
+              if (!vars[tsID][varID][levelID].empty())
                 {
                   nmiss = vars[tsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
                 }
             }
         }
-
-      field_free(vars[tsID], vlistID1);
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
