@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+
 
 // correlation in time
 static void
@@ -133,61 +135,60 @@ covariance(size_t gridsize, double missval1, double missval2, size_t *nofvals, d
 void *
 Timstat2(void *process)
 {
-  int vdate = 0, vtime = 0;
+  int64_t vdate = 0;
+  int vtime = 0;
   int varID, levelID;
-  size_t nmiss;
+  size_t nmiss = 0;
 
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("timcor",   func_cor,   0, NULL);
-  cdoOperatorAdd("timcovar", func_covar, 0, NULL);
+  cdoOperatorAdd("timcor",   func_cor,   5, nullptr);
+  cdoOperatorAdd("timcovar", func_covar, 3, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto nwork = cdoOperatorF2(operatorID);
 
-  int nwork = 0;
-  if (operfunc == func_cor)
-    nwork = 5;
-  else if (operfunc == func_covar)
-    nwork = 3;
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  int nvars = vlistNvars(vlistID1);
-  int nrecs = vlistNrecs(vlistID1);
-  int nrecs3 = nrecs;
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  const auto nvars = vlistNvars(vlistID1);
+  auto nrecs = vlistNrecs(vlistID1);
+  const int nrecs3 = nrecs;
   std::vector<int> recVarID(nrecs);
   std::vector<int> recLevelID(nrecs);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  // int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  // const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
 
   vlistDefTaxis(vlistID3, taxisID3);
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
-  pstreamDefVlist(streamID3, vlistID3);
+  const auto streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax);
+  Varray<double> array2(gridsizemax);
 
-  std::vector<std::vector<std::vector<std::vector<double>>>> work(nvars);
-  std::vector<std::vector<std::vector<size_t>>> nofvals(nvars);
+  Varray4D<double> work(nvars);
+  Varray3D<size_t> nofvals(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, 0);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevs = varList1[varID].nlevels;
 
       work[varID].resize(nlevs);
       nofvals[varID].resize(nlevs);
@@ -206,13 +207,13 @@ Timstat2(void *process)
       vdate = taxisInqVdate(taxisID1);
       vtime = taxisInqVtime(taxisID1);
 
-      int nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
+      auto nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
       if (nrecs != nrecs2) cdoWarning("Input streams have different number of records!");
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamInqRecord(streamID2, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID2, &varID, &levelID);
 
           if (tsID == 0)
             {
@@ -220,13 +221,12 @@ Timstat2(void *process)
               recLevelID[recID] = levelID;
             }
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const auto gridsize = varList1[varID].gridsize;
+          const auto missval1 = varList1[varID].missval;
+          const auto missval2 = varList2[varID].missval;
 
-          double missval1 = vlistInqVarMissval(vlistID1, varID);
-          double missval2 = vlistInqVarMissval(vlistID2, varID);
-
-          pstreamReadRecord(streamID1, &array1[0], &nmiss);
-          pstreamReadRecord(streamID2, &array2[0], &nmiss);
+          cdoReadRecord(streamID1, &array1[0], &nmiss);
+          cdoReadRecord(streamID2, &array2[0], &nmiss);
 
           if (operfunc == func_cor)
             {
@@ -247,17 +247,16 @@ Timstat2(void *process)
   tsID = 0;
   taxisDefVdate(taxisID3, vdate);
   taxisDefVtime(taxisID3, vtime);
-  pstreamDefTimestep(streamID3, tsID);
+  cdoDefTimestep(streamID3, tsID);
 
   for (int recID = 0; recID < nrecs3; recID++)
     {
       varID = recVarID[recID];
       levelID = recLevelID[recID];
 
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-
-      double missval1 = vlistInqVarMissval(vlistID1, varID);
-      double missval2 = vlistInqVarMissval(vlistID2, varID);
+      const auto gridsize = varList1[varID].gridsize;
+      const auto missval1 = varList1[varID].missval;
+      const auto missval2 = varList2[varID].missval;
 
       if (operfunc == func_cor)
         {
@@ -271,15 +270,15 @@ Timstat2(void *process)
                              &work[varID][levelID][1][0], &work[varID][levelID][2][0]);
         }
 
-      pstreamDefRecord(streamID3, varID, levelID);
-      pstreamWriteRecord(streamID3, &work[varID][levelID][0][0], nmiss);
+      cdoDefRecord(streamID3, varID, levelID);
+      cdoWriteRecord(streamID3, &work[varID][levelID][0][0], nmiss);
     }
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
