@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,18 +18,21 @@
 /*
    This module contains the following operators:
 
-      Vertint    ap2pl           Model air pressure level to pressure level
-   interpolation
+      Vertint    ap2pl           Model air pressure level to pressure level interpolation
 */
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "after_vertint.h"
 #include "listarray.h"
 #include "stdnametable.h"
 #include "util_string.h"
+#include "const.h"
+#include "cdo_zaxis.h"
+
 
 static bool
 is_height_axis(int zaxisID, int nlevel)
@@ -43,7 +46,7 @@ is_height_axis(int zaxisID, int nlevel)
           char stdname[CDI_MAX_NAME];
           zaxisInqUnits(zaxisID, units);
           zaxisInqStdname(zaxisID, stdname);
-          if (strcmp(stdname, "height") == 0 && *units == 0) isheight = true;
+          if (cstrIsEqual(stdname, "height") && *units == 0) isheight = true;
         }
     }
   return isheight;
@@ -52,11 +55,11 @@ is_height_axis(int zaxisID, int nlevel)
 static void
 change_height_zaxis(int vlistID1, int vlistID2, int zaxisID2)
 {
-  int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int iz = 0; iz < nzaxis; ++iz)
     {
-      int zaxisID = vlistZaxis(vlistID1, iz);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, iz);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
       if (is_height_axis(zaxisID, nlevel) && nlevel > 1)
         {
@@ -80,11 +83,10 @@ Vertintap(void *process)
   };
   int nrecs;
   int varID, levelID;
-  int zaxisIDh = -1;
   int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel;
   int apressID = -1, dpressID = -1;
   int psID = -1, tempID = -1;
-  // int sortlevels = TRUE;
+  // bool sortlevels = true;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
   bool extrapolate = false;
   ListArray<double> listArrayFlt;
@@ -92,22 +94,21 @@ Vertintap(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int AP2PL     = cdoOperatorAdd("ap2pl",     func_pl, type_lin, "pressure levels in pascal");
-  int AP2PLX    = cdoOperatorAdd("ap2plx",    func_pl, type_lin, "pressure levels in pascal");
-  int AP2HL     = cdoOperatorAdd("ap2hl",     func_hl, type_lin, "height levels in meter");
-  int AP2HLX    = cdoOperatorAdd("ap2hlx",    func_hl, type_lin, "height levels in meter");
-  int AP2PL_LP  = cdoOperatorAdd("ap2pl_lp",  func_pl, type_log, "pressure levels in pascal");
-  int AP2PLX_LP = cdoOperatorAdd("ap2plx_lp", func_pl, type_log, "pressure levels in pascal");
+  const auto AP2PL     = cdoOperatorAdd("ap2pl",     func_pl, type_lin, "pressure levels in pascal");
+  const auto AP2PLX    = cdoOperatorAdd("ap2plx",    func_pl, type_lin, "pressure levels in pascal");
+  const auto AP2HL     = cdoOperatorAdd("ap2hl",     func_hl, type_lin, "height levels in meter");
+  const auto AP2HLX    = cdoOperatorAdd("ap2hlx",    func_hl, type_lin, "height levels in meter");
+  const auto AP2PL_LP  = cdoOperatorAdd("ap2pl_lp",  func_pl, type_log, "pressure levels in pascal");
+  const auto AP2PLX_LP = cdoOperatorAdd("ap2plx_lp", func_pl, type_log, "pressure levels in pascal");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
-  bool useLogType = cdoOperatorF2(operatorID) == type_log;
+  const auto operatorID = cdoOperatorID();
+  const bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
+  const bool useLogType = cdoOperatorF2(operatorID) == type_log;
 
   if (operatorID == AP2PL || operatorID == AP2HL || operatorID == AP2PL_LP)
     {
-      char *envstr = getenv("EXTRAPOLATE");
-
+      const char *envstr = getenv("EXTRAPOLATE");
       if (envstr && isdigit((int) envstr[0]))
         {
           if (atoi(envstr) == 1) extrapolate = true;
@@ -122,64 +123,67 @@ Vertintap(void *process)
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   int nplev = 0;
-  double *plev = NULL;
-  if (operatorArgc() == 1 && strcmp(operatorArgv()[0], "default") == 0)
+  Varray<double> plev;
+  if (operatorArgc() == 1 && cdoOperatorArgv(0).compare("default") == 0)
     {
       if (useHightLevel)
         {
-          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
+          const double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
           nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev = (double *) Malloc(nplev * sizeof(double));
+          plev.resize(nplev);
           for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
         }
       else
         {
-          double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
-                              20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
+          const double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
+                                    20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
           nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev = (double *) Malloc(nplev * sizeof(double));
+          plev.resize(nplev);
           for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
         }
     }
   else
     {
-      nplev = listArrayFlt.argvToFlt(operatorArgc(), operatorArgv());
-      plev = listArrayFlt.data();
+      nplev = listArrayFlt.argvToFlt(cdoGetOperArgv());
+      plev.resize(nplev);
+      double *larr = listArrayFlt.data();
+      for (int i = 0; i < nplev; ++i) plev[i] = larr[i];
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
-  int zaxisIDp = zaxisCreate(zaxistype, nplev);
-  zaxisDefLevels(zaxisIDp, plev);
+  const int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
+  const auto zaxisIDp = zaxisCreate(zaxistype, nplev);
+  zaxisDefLevels(zaxisIDp, plev.data());
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+
+  const auto nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
       vlistInqVarStdname(vlistID1, varID, stdname);
-      strtolower(stdname);
+      cstrToLowerCase(stdname);
 
-      if (strcmp(stdname, var_stdname(surface_air_pressure)) == 0)
-        psID = varID;
-      else if (strcmp(stdname, var_stdname(air_pressure)) == 0)
-        apressID = varID;
-      else if (strcmp(stdname, var_stdname(pressure_thickness)) == 0)
-        dpressID = varID;
-      else if (strcmp(stdname, var_stdname(air_temperature)) == 0)
-        tempID = varID;
+      // clang-format off
+      if      (cstrIsEqual(stdname, var_stdname(surface_air_pressure))) psID = varID;
+      else if (cstrIsEqual(stdname, var_stdname(air_pressure)))         apressID = varID;
+      else if (cstrIsEqual(stdname, var_stdname(pressure_thickness)))   dpressID = varID;
+      else if (cstrIsEqual(stdname, var_stdname(air_temperature)))      tempID = varID;
+      // clang-format on
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       cdoPrint("Found:");
       if (psID != -1) cdoPrint("  %s -> %s", var_stdname(surface_air_pressure), cdoVlistInqVarName(vlistID1, psID, varname));
@@ -190,18 +194,19 @@ Vertintap(void *process)
 
   if (apressID == -1) cdoAbort("%s not found!", var_stdname(air_pressure));
 
-  int nzaxis = vlistNzaxis(vlistID1);
+  int zaxisIDh = -1;
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int i = 0; i < nzaxis; i++)
     {
-      int zaxisID = vlistZaxis(vlistID1, i);
-      if (zaxisID == vlistInqVarZaxis(vlistID1, apressID))
+      const auto zaxisID = vlistZaxis(vlistID1, i);
+      if (zaxisID == varList1[apressID].zaxisID)
         {
           bool mono_level = true;
           nlevel = zaxisInqSize(zaxisID);
 
           if (is_height_axis(zaxisID, nlevel))
             {
-              std::vector<double> level(nlevel);
+              Varray<double> level(nlevel);
               cdoZaxisInqLevels(zaxisID, &level[0]);
               int l;
               for (l = 0; l < nlevel; l++)
@@ -224,14 +229,15 @@ Vertintap(void *process)
     }
 
   change_height_zaxis(vlistID1, vlistID2, zaxisIDp);
+  varListInit(varList2, vlistID2);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
-  std::vector<size_t *> varnmiss(nvars);
-  std::vector<double *> vardata1(nvars);
-  std::vector<double *> vardata2(nvars);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  Varray2D<double> vardata1(nvars);
+  Varray2D<double> vardata2(nvars);
 
-  int maxlev = nhlevh > nplev ? nhlevh : nplev;
+  const int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
   std::vector<size_t> pnmiss;
   if (!extrapolate) pnmiss.resize(nplev);
@@ -239,23 +245,23 @@ Vertintap(void *process)
   // check levels
   if (zaxisIDh != -1)
     {
-      int nlev = zaxisInqSize(zaxisIDh);
+      const int nlev = zaxisInqSize(zaxisIDh);
       if (nlev != nhlev) cdoAbort("Internal error, wrong number of height level!");
-      std::vector<double> levels(nlev);
+      Varray<double> levels(nlev);
       cdoZaxisInqLevels(zaxisIDh, &levels[0]);
 
       for (int ilev = 0; ilev < nlev; ++ilev)
         {
           if ((ilev + 1) != (int) levels[ilev])
             {
-              // sortlevels = FALSE;
+              // sortlevels = false;
               break;
             }
         }
     }
 
   std::vector<int> vert_index;
-  std::vector<double> ps_prog, full_press, half_press;
+  Varray<double> ps_prog, full_press, half_press;
   if (zaxisIDh != -1 && gridsize > 0)
     {
       vert_index.resize(gridsize * nplev);
@@ -268,33 +274,35 @@ Vertintap(void *process)
 
   if (useHightLevel)
     {
-      std::vector<double> phlev(nplev);
-      height2pressure(&phlev[0], plev, nplev);
+      Varray<double> phlev(nplev);
+      height2pressure(&phlev[0], plev.data(), nplev);
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         for (int i = 0; i < nplev; ++i) cdoPrint("level = %d   height = %g   pressure = %g", i + 1, plev[i], phlev[i]);
 
-      arrayCopy(nplev, &phlev[0], plev);
+      varrayCopy(nplev, phlev, plev);
     }
 
   if (useLogType)
-    for (int k = 0; k < nplev; k++) plev[k] = log(plev[k]);
+    for (int k = 0; k < nplev; k++) plev[k] = std::log(plev[k]);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto gridID = varList1[varID].gridID;
+      const auto zaxisID = varList1[varID].zaxisID;
+      const auto nlevel = varList1[varID].nlevels;
 
       if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-      vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      vardata1[varID].resize(gridsize * nlevel);
 
-      if (zaxisID == zaxisIDh || (is_height_axis(zaxisID, nlevel) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)))
+      varinterp[varID]
+          = (zaxisID == zaxisIDh || (is_height_axis(zaxisID, nlevel) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)));
+
+      if (varinterp[varID])
         {
-          varinterp[varID] = true;
-          vardata2[varID] = (double *) Malloc(gridsize * nplev * sizeof(double));
-          varnmiss[varID] = (size_t *) Calloc(maxlev, sizeof(size_t));
+          vardata2[varID].resize(gridsize * nplev);
+          varnmiss[varID].resize(maxlev, 0);
         }
       else
         {
@@ -304,9 +312,7 @@ Vertintap(void *process)
               cdoWarning("Parameter %d has wrong number of levels, skipped! (param=%s nlevel=%d)", varID + 1, varname, nlevel);
             }
 
-          varinterp[varID] = false;
-          vardata2[varID] = vardata1[varID];
-          varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
+          varnmiss[varID].resize(nlevel);
         }
     }
 
@@ -320,13 +326,12 @@ Vertintap(void *process)
 
   for (varID = 0; varID < nvars; ++varID)
     {
-      if (varinterp[varID] && vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
-        vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
+      if (varinterp[varID] && varList1[varID].timetype == TIME_CONSTANT) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const auto streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -334,30 +339,28 @@ Vertintap(void *process)
       for (varID = 0; varID < nvars; ++varID)
         {
           vars[varID] = false;
-          nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+          nlevel = varList1[varID].nlevels;
           for (levelID = 0; levelID < nlevel; levelID++) varnmiss[varID][levelID] = 0;
         }
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           /*
-          zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-          nlevel   = zaxisInqSize(zaxisID);
-          if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel ==
-          nhlev )
+          zaxisID  = varList1[varID].zaxisID;
+          nlevel   = varList1[varID].nlevels;
+          if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel == nhlev )
             {
               levelID = (int) (zaxisInqLevel(zaxisIDh, levelID)-1);
               printf("levelID %d\n", levelID);
             }
           */
-          size_t offset = gridsize * levelID;
-          double *single1 = vardata1[varID] + offset;
-          pstreamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
-
+          const size_t offset = gridsize * levelID;
+          double *single1 = &vardata1[varID][offset];
+          cdoReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -368,7 +371,7 @@ Vertintap(void *process)
         {
           if (psID != -1)
             {
-              arrayCopy(gridsize, vardata1[psID], &ps_prog[0]);
+              varrayCopy(gridsize, vardata1[psID], ps_prog);
             }
           else if (dpressID != -1)
             {
@@ -378,16 +381,16 @@ Vertintap(void *process)
             }
           else
             {
-              arrayCopy(gridsize, vardata1[apressID] + gridsize * (nhlevf - 1), &ps_prog[0]);
+              arrayCopy(gridsize, &vardata1[apressID][gridsize * (nhlevf - 1)], &ps_prog[0]);
               // for ( size_t i = 0; i < gridsize; i++ )  ps_prog[i] = 110000;
             }
 
           /* check range of ps_prog */
-          double minval, maxval;
-          arrayMinMaxMask(gridsize, &ps_prog[0], NULL, &minval, &maxval);
-          if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
+          const auto minmax = varrayMinMax(ps_prog);
+          if (minmax.first < MIN_PS || minmax.second > MAX_PS)
+            cdoWarning("Surface pressure out of range (min=%g max=%g)!", minmax.first, minmax.second);
 
-          arrayCopy(gridsize * nhlevf, vardata1[apressID], &full_press[0]);
+          varrayCopy(gridsize * nhlevf, vardata1[apressID], full_press);
 
           for (size_t i = 0; i < gridsize; i++) half_press[i] = 0;
           for (int k = 1; k < nhlevf; k++)
@@ -397,31 +400,26 @@ Vertintap(void *process)
 
           if (useLogType)
             {
-              for (size_t i = 0; i < gridsize; i++) ps_prog[i] = log(ps_prog[i]);
-              for (size_t ki = 0; ki < nhlevh * gridsize; ki++) half_press[ki] = log(half_press[ki]);
-              for (size_t ki = 0; ki < nhlevf * gridsize; ki++) full_press[ki] = log(full_press[ki]);
+              for (size_t i = 0; i < gridsize; i++) ps_prog[i] = std::log(ps_prog[i]);
+              for (size_t ki = 0; ki < nhlevh * gridsize; ki++) half_press[ki] = std::log(half_press[ki]);
+              for (size_t ki = 0; ki < nhlevf * gridsize; ki++) full_press[ki] = std::log(full_press[ki]);
             }
 
-          genind(&vert_index[0], plev, &full_press[0], gridsize, nplev, nhlevf);
+          genind(&vert_index[0], plev.data(), &full_press[0], gridsize, nplev, nhlevf);
 
-          if (!extrapolate) genindmiss(&vert_index[0], plev, gridsize, nplev, &ps_prog[0], &pnmiss[0]);
+          if (!extrapolate) genindmiss(&vert_index[0], plev.data(), gridsize, nplev, &ps_prog[0], &pnmiss[0]);
         }
 
       for (varID = 0; varID < nvars; varID++)
         {
           if (vars[varID])
             {
-              int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              int nlevel = zaxisInqSize(zaxisID);
-              double missval = vlistInqVarMissval(vlistID1, varID);
+              const auto nlevel = varList1[varID].nlevels;
+              const auto missval = varList1[varID].missval;
               if (varinterp[varID])
                 {
-                  double *hyb_press = NULL;
-                  if (nlevel == nhlevf)
-                    hyb_press = &full_press[0];
-                  else if (nlevel == nhlevh)
-                    hyb_press = &half_press[0];
-                  else
+                  double *hyb_press = (nlevel == nhlevf) ? &full_press[0] : (nlevel == nhlevh) ? &half_press[0] : nullptr;
+                  if (hyb_press == nullptr)
                     {
                       vlistInqVarName(vlistID1, varID, varname);
                       cdoAbort("Number of generalized height level differ from full/half level (param=%s)!", varname);
@@ -432,9 +430,10 @@ Vertintap(void *process)
                       if (varnmiss[varID][levelID]) cdoAbort("Missing values unsupported for this operator!");
                     }
 
-                  interp_X(vardata1[varID], vardata2[varID], hyb_press, &vert_index[0], plev, nplev, gridsize, nlevel, missval);
+                  interp_X(&vardata1[varID][0], &vardata2[varID][0], hyb_press, &vert_index[0], plev.data(), nplev, gridsize,
+                           nlevel, missval);
 
-                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID]);
+                  if (!extrapolate) varrayCopy(nplev, pnmiss, varnmiss[varID]);
                 }
             }
         }
@@ -443,13 +442,13 @@ Vertintap(void *process)
         {
           if (vars[varID])
             {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              const int nlevel = varList2[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single2, varnmiss[varID][levelID]);
+                  const size_t offset = gridsize * levelID;
+                  double *single = varinterp[varID] ? &vardata2[varID][offset] : &vardata1[varID][offset];
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -457,17 +456,10 @@ Vertintap(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata1[varID]);
-      if (varinterp[varID]) Free(vardata2[varID]);
-    }
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
