@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,13 +24,18 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "after_vertint.h"
 #include "listarray.h"
 #include "stdnametable.h"
 #include "constants.h"
 #include "util_string.h"
+#include "const.h"
+#include "cdo_zaxis.h"
+
 
 static bool
 zaxis_is_hybrid(int zaxistype)
@@ -41,16 +46,16 @@ zaxis_is_hybrid(int zaxistype)
 static void
 change_hybrid_zaxis(int vlistID1, int vlistID2, int nvct, double *vct, int zaxisID2, int nhlevf, int nhlevh)
 {
-  int nzaxis = vlistNzaxis(vlistID1);
+  const int nzaxis = vlistNzaxis(vlistID1);
   for (int iz = 0; iz < nzaxis; ++iz)
     {
-      int zaxisID = vlistZaxis(vlistID1, iz);
-      int nlevel = zaxisInqSize(zaxisID);
-      int zaxistype = zaxisInqType(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, iz);
+      const auto nlevel = zaxisInqSize(zaxisID);
+      const auto zaxistype = zaxisInqType(zaxisID);
 
       if (zaxis_is_hybrid(zaxistype) && (nlevel == nhlevh || nlevel == nhlevf))
         {
-          int nvct2 = zaxisInqVctSize(zaxisID);
+          const auto nvct2 = zaxisInqVctSize(zaxisID);
           if (nvct2 == nvct && memcmp(vct, zaxisInqVctPtr(zaxisID), nvct * sizeof(double)) == 0)
             vlistChangeZaxisIndex(vlistID2, iz, zaxisID2);
         }
@@ -79,7 +84,6 @@ Vertintml(void *process)
   // bool sortlevels = true;
   char paramstr[32];
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
-  double minval, maxval;
   gribcode_t gribcodes;
   memset(&gribcodes, 0, sizeof(gribcode_t));
   ListArray<double> listArrayFlt;
@@ -87,23 +91,23 @@ Vertintml(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int ML2PL     = cdoOperatorAdd("ml2pl",     func_pl, type_lin, "pressure levels in pascal");
-  int ML2PLX    = cdoOperatorAdd("ml2plx",    func_pl, type_lin, "pressure levels in pascal");
-  int ML2HL     = cdoOperatorAdd("ml2hl",     func_hl, type_lin, "height levels in meter");
-  int ML2HLX    = cdoOperatorAdd("ml2hlx",    func_hl, type_lin, "height levels in meter");
-  int ML2PL_LP  = cdoOperatorAdd("ml2pl_lp",  func_pl, type_log, "pressure levels in pascal");
-  int ML2PLX_LP = cdoOperatorAdd("ml2plx_lp", func_pl, type_log, "pressure levels in pascal");
-  int ML2HL_LP  = cdoOperatorAdd("ml2hl_lp",  func_hl, type_log, "height levels in meter");
-  int ML2HLX_LP = cdoOperatorAdd("ml2hlx_lp", func_hl, type_log, "height levels in meter");
+  const auto ML2PL     = cdoOperatorAdd("ml2pl",     func_pl, type_lin, "pressure levels in pascal");
+  const auto ML2PLX    = cdoOperatorAdd("ml2plx",    func_pl, type_lin, "pressure levels in pascal");
+  const auto ML2HL     = cdoOperatorAdd("ml2hl",     func_hl, type_lin, "height levels in meter");
+  const auto ML2HLX    = cdoOperatorAdd("ml2hlx",    func_hl, type_lin, "height levels in meter");
+  const auto ML2PL_LP  = cdoOperatorAdd("ml2pl_lp",  func_pl, type_log, "pressure levels in pascal");
+  const auto ML2PLX_LP = cdoOperatorAdd("ml2plx_lp", func_pl, type_log, "pressure levels in pascal");
+  const auto ML2HL_LP  = cdoOperatorAdd("ml2hl_lp",  func_hl, type_log, "height levels in meter");
+  const auto ML2HLX_LP = cdoOperatorAdd("ml2hlx_lp", func_hl, type_log, "height levels in meter");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
-  bool useLogType = cdoOperatorF2(operatorID) == type_log;
+  const auto operatorID = cdoOperatorID();
+  const bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
+  const bool useLogType = cdoOperatorF2(operatorID) == type_log;
 
   if (operatorID == ML2PL || operatorID == ML2HL || operatorID == ML2PL_LP || operatorID == ML2HL_LP)
     {
-      char *envstr = getenv("EXTRAPOLATE");
+      const char *envstr = getenv("EXTRAPOLATE");
       if (envstr && isdigit((int) envstr[0]))
         {
           if (atoi(envstr) == 1) extrapolate = true;
@@ -118,45 +122,40 @@ Vertintml(void *process)
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   int nplev = 0;
-  double *plev = NULL;
-  if (operatorArgc() == 1 && strcmp(operatorArgv()[0], "default") == 0)
+  Varray<double> plev;
+  if (operatorArgc() == 1 && cdoOperatorArgv(0).compare("default") == 0)
     {
+      Varray<double> stdlev;
       if (useHightLevel)
-        {
-          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
-          nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev = (double *) Malloc(nplev * sizeof(double));
-          for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
-        }
+        stdlev = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
       else
-        {
-          double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
-                              20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
-          nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev = (double *) Malloc(nplev * sizeof(double));
-          for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
-        }
+        stdlev
+            = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000, 10000, 7000, 5000, 3000, 2000, 1000 };
+      plev = stdlev;
+      nplev = plev.size();
     }
   else
     {
-      nplev = listArrayFlt.argvToFlt(operatorArgc(), operatorArgv());
-      plev = listArrayFlt.data();
+      nplev = listArrayFlt.argvToFlt(cdoGetOperArgv());
+      plev.resize(nplev);
+      double *larr = listArrayFlt.data();
+      for (int i = 0; i < nplev; ++i) plev[i] = larr[i];
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
-  int zaxisIDp = zaxisCreate(zaxistype, nplev);
-  zaxisDefLevels(zaxisIDp, plev);
+  const int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
+  const auto zaxisIDp = zaxisCreate(zaxistype, nplev);
+  zaxisDefLevels(zaxisIDp, plev.data());
 
   int nvct = 0;
   int zaxisIDh = -1;
@@ -177,11 +176,11 @@ Vertintml(void *process)
       if (i == nvct) linvertvct = true;
     }
 
-  if (cdoVerbose) cdoPrint("linvertvct = %d", (int) linvertvct);
+  if (Options::cdoVerbose) cdoPrint("linvertvct = %d", (int) linvertvct);
 
   if (linvertvct)
     {
-      std::vector<double> vctbuf(nvct);
+      Varray<double> vctbuf(nvct);
       for (int i = 0; i < nvct; ++i) vctbuf[i] = vct[i];
       for (int i = 0; i < nvct / 2; i++)
         {
@@ -190,15 +189,19 @@ Vertintml(void *process)
         }
     }
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  const int nvars = vlistNvars(vlistID1);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
-  std::vector<size_t *> varnmiss(nvars);
-  std::vector<double *> vardata1(nvars);
-  std::vector<double *> vardata2(nvars);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  Varray2D<double> vardata1(nvars);
+  Varray2D<double> vardata2(nvars);
 
-  int maxlev = nhlevh > nplev ? nhlevh : nplev;
+  const int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
   std::vector<size_t> pnmiss;
   if (!extrapolate) pnmiss.resize(nplev);
@@ -206,9 +209,9 @@ Vertintml(void *process)
   // check levels
   if (zaxisIDh != -1)
     {
-      int nlev = zaxisInqSize(zaxisIDh);
+      const auto nlev = zaxisInqSize(zaxisIDh);
       if (nlev != nhlev) cdoAbort("Internal error, wrong number of hybrid level!");
-      std::vector<double> levels(nlev);
+      Varray<double> levels(nlev);
       cdoZaxisInqLevels(zaxisIDh, &levels[0]);
 
       for (int ilev = 0; ilev < nlev; ++ilev)
@@ -222,7 +225,7 @@ Vertintml(void *process)
     }
 
   std::vector<int> vert_index;
-  std::vector<double> ps_prog, full_press, half_press;
+  Varray<double> ps_prog, full_press, half_press;
   if (zaxisIDh != -1 && gridsize > 0)
     {
       vert_index.resize(gridsize * nplev);
@@ -235,22 +238,22 @@ Vertintml(void *process)
 
   if (useHightLevel)
     {
-      std::vector<double> phlev(nplev);
-      height2pressure(&phlev[0], plev, nplev);
+      Varray<double> phlev(nplev);
+      height2pressure(&phlev[0], plev.data(), nplev);
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         for (int i = 0; i < nplev; ++i) cdoPrint("level = %d   height = %g   pressure = %g", i + 1, plev[i], phlev[i]);
 
-      arrayCopy(nplev, &phlev[0], plev);
+      varrayCopy(nplev, phlev, plev);
     }
 
   if (useLogType)
-    for (int k = 0; k < nplev; k++) plev[k] = log(plev[k]);
+    for (int k = 0; k < nplev; k++) plev[k] = std::log(plev[k]);
 
   bool useTable = false;
   for (varID = 0; varID < nvars; varID++)
     {
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       if (tableNum > 0 && tableNum != 255)
         {
           useTable = true;
@@ -258,21 +261,21 @@ Vertintml(void *process)
         }
     }
 
-  if (cdoVerbose && useTable) cdoPrint("Using code tables!");
+  if (Options::cdoVerbose && useTable) cdoPrint("Using code tables!");
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int zaxistype = zaxisInqType(zaxisID);
+      const auto gridID = varList1[varID].gridID;
+      const auto zaxisID = varList1[varID].zaxisID;
+      const auto zaxistype = zaxisInqType(zaxisID);
       // gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(zaxisID);
-      int instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto nlevel = zaxisInqSize(zaxisID);
+      const auto instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
+      const auto tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
 
-      int code = vlistInqVarCode(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
 
-      int param = vlistInqVarParam(vlistID1, varID);
+      const auto param = vlistInqVarParam(vlistID1, varID);
       cdiParamToString(param, paramstr, sizeof(paramstr));
       int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
@@ -306,32 +309,32 @@ Vertintml(void *process)
           echam_gribcodes(&gribcodes);
         }
 
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          cdoPrint("Mode = %d  Center = %d TableNum =%d Code = %d Param = %s Varname = %s varID = %d", mode, instNum, tableNum,
-                   code, paramstr, varname, varID);
+          cdoPrint("Mode=%d  Center=%d  TableNum=%d  Code=%d  Param=%s  Varname=%s  varID=%d", mode, instNum, tableNum, code,
+                   paramstr, varname, varID);
         }
 
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(stdname);
+          cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
           if (code == -1)
             {
               //                                       ECHAM                         ECMWF
               // clang-format off
-              if      (sgeopotID == -1 && (strcmp(varname, "geosp") == 0 || strcmp(varname, "z") == 0)) code = gribcodes.geopot;
-              else if (tempID == -1 && (strcmp(varname, "st") == 0 || strcmp(varname, "t") == 0)) code = gribcodes.temp;
-              else if (psID == -1 && (strcmp(varname, "aps") == 0 || strcmp(varname, "sp") == 0)) code = gribcodes.ps;
-              else if (lnpsID == -1 && (strcmp(varname, "lsp") == 0 || strcmp(varname, "lnsp") == 0)) code = gribcodes.lsp;
-              else if (geopotID == -1 && strcmp(stdname, "geopotential_full") == 0) code = gribcodes.geopot;
-              // else if ( strcmp(varname, "geopoth") == 0 ) code = 156;
+              if      (sgeopotID == -1 && (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z"))) code = gribcodes.geopot;
+              else if (tempID == -1 && (cstrIsEqual(varname, "st") || cstrIsEqual(varname, "t"))) code = gribcodes.temp;
+              else if (psID == -1 && (cstrIsEqual(varname, "aps") || cstrIsEqual(varname, "sp"))) code = gribcodes.ps;
+              else if (lnpsID == -1 && (cstrIsEqual(varname, "lsp") || cstrIsEqual(varname, "lnsp"))) code = gribcodes.lsp;
+              else if (geopotID == -1 && cstrIsEqual(stdname, "geopotential_full")) code = gribcodes.geopot;
+              // else if (cstrIsEqual(varname, "geopoth")) code = 156;
               // clang-format on
             }
         }
@@ -361,17 +364,17 @@ Vertintml(void *process)
 
       if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-      if (varID == gheightID)
-        vardata1[varID] = (double *) Malloc(gridsize * (nlevel + 1) * sizeof(double));
-      else
-        vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      const size_t maxlevel = (varID == gheightID) ? (nlevel + 1) : nlevel;
+      vardata1[varID].resize(gridsize * maxlevel);
 
-      // if ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel == nhlev )
-      if (zaxisID == zaxisIDh || (zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)))
+      // varinterp[varID] = ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel == nhlev );
+      varinterp[varID]
+          = (zaxisID == zaxisIDh || (zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)));
+
+      if (varinterp[varID])
         {
-          varinterp[varID] = true;
-          vardata2[varID] = (double *) Malloc(gridsize * nplev * sizeof(double));
-          varnmiss[varID] = (size_t *) Calloc(maxlev, sizeof(size_t));
+          vardata2[varID].resize(gridsize * nplev);
+          varnmiss[varID].resize(maxlev, 0);
         }
       else
         {
@@ -381,13 +384,11 @@ Vertintml(void *process)
               cdoWarning("Parameter %d has wrong number of levels, skipped! (param=%s nlevel=%d)", varID + 1, varname, nlevel);
             }
 
-          varinterp[varID] = false;
-          vardata2[varID] = vardata1[varID];
-          varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
+          varnmiss[varID].resize(nlevel);
         }
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       cdoPrint("Found:");
       if (tempID != -1) cdoPrint("  %s -> %s", var_stdname(air_temperature), cdoVlistInqVarName(vlistID1, tempID, varname));
@@ -403,7 +404,7 @@ Vertintml(void *process)
 
   if (tempID != -1 || gheightID != -1) sgeopot_needed = true;
 
-  std::vector<double> sgeopot;
+  Varray<double> sgeopot;
   if (zaxisIDh != -1 && sgeopot_needed)
     {
       sgeopot.resize(gridsize);
@@ -416,7 +417,7 @@ Vertintml(void *process)
               else
                 cdoPrint("%s not found - using bottom layer of %s!", var_stdname(surface_geopotential), var_stdname(geopotential));
             }
-          arrayFill(gridsize, &sgeopot[0], 0.0);
+          varrayFill(sgeopot, 0.0);
         }
     }
 
@@ -434,7 +435,7 @@ Vertintml(void *process)
         presID = psID;
     }
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       vlistInqVarName(vlistID1, presID, varname);
       if (presID == lnpsID)
@@ -447,16 +448,13 @@ Vertintml(void *process)
   if (zaxisIDh != -1)
     {
       double suma = 0, sumb = 0;
-      for (int i = 0; i < nhlevh; i++)
-        {
-          suma += vct[i];
-          sumb += vct[i + nhlevh];
-        }
+      for (int i = 0; i < nhlevh; ++i) suma += vct[i];
+      for (int i = 0; i < nhlevh; ++i) sumb += vct[i + nhlevh];
       if (!(suma > 0 || sumb > 0)) cdoWarning("VCT is empty!");
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -464,14 +462,14 @@ Vertintml(void *process)
       for (varID = 0; varID < nvars; ++varID) vars[varID] = false;
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          // gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          int nlevel = zaxisInqSize(zaxisID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          // gridsize = varList1[varID].gridsize;
+          const auto zaxisID = varList1[varID].zaxisID;
+          const auto nlevel = varList1[varID].nlevels;
           /*
           if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel == nhlev )
             {
@@ -481,9 +479,9 @@ Vertintml(void *process)
           */
           if (linvertvct && zaxisIDh != -1 && zaxisID == zaxisIDh) levelID = nlevel - 1 - levelID;
 
-          size_t offset = gridsize * levelID;
-          double *single = vardata1[varID] + offset;
-          pstreamReadRecord(streamID1, single, &varnmiss[varID][levelID]);
+          const size_t offset = gridsize * levelID;
+          double *single = &vardata1[varID][offset];
+          cdoReadRecord(streamID1, single, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -492,65 +490,61 @@ Vertintml(void *process)
           if (sgeopot_needed)
             {
               if (sgeopotID != -1)
-                arrayCopy(gridsize, vardata1[sgeopotID], &sgeopot[0]);
+                varrayCopy(gridsize, vardata1[sgeopotID], sgeopot);
               else if (geopotID != -1)
-                arrayCopy(gridsize, vardata1[geopotID] + gridsize * (nhlevf - 1), &sgeopot[0]);
+                arrayCopy(gridsize, &vardata1[geopotID][gridsize * (nhlevf - 1)], &sgeopot[0]);
 
               // check range of surface geopot
               if (extrapolate && (sgeopotID != -1 || geopotID != -1))
                 {
-                  arrayMinMaxMask(gridsize, &sgeopot[0], NULL, &minval, &maxval);
-                  if (minval < MIN_FIS || maxval > MAX_FIS)
-                    cdoWarning("Surface geopotential out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID + 1);
-                  if (gridsize > 1 && minval >= 0 && maxval <= 9000 && IS_NOT_EQUAL(minval, maxval))
-                    cdoWarning("Surface geopotential has an unexpected range (min=%g max=%g) [timestep:%d]!", minval, maxval,
+                  const auto minmax = varrayMinMax(sgeopot);
+                  if (minmax.first < MIN_FIS || minmax.second > MAX_FIS)
+                    cdoWarning("Surface geopotential out of range (min=%g max=%g) [timestep:%d]!", minmax.first, minmax.second, tsID + 1);
+                  if (gridsize > 1 && minmax.first >= 0 && minmax.second <= 9000 && IS_NOT_EQUAL(minmax.first, minmax.second))
+                    cdoWarning("Surface geopotential has an unexpected range (min=%g max=%g) [timestep:%d]!", minmax.first, minmax.second,
                                tsID + 1);
                 }
             }
 
           if (presID == lnpsID)
-            for (size_t i = 0; i < gridsize; i++) ps_prog[i] = exp(vardata1[lnpsID][i]);
+            for (size_t i = 0; i < gridsize; i++) ps_prog[i] = std::exp(vardata1[lnpsID][i]);
           else if (presID != -1)
-            arrayCopy(gridsize, vardata1[presID], &ps_prog[0]);
+            varrayCopy(gridsize, vardata1[presID], ps_prog);
 
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, &ps_prog[0], NULL, &minval, &maxval);
-          if (minval < MIN_PS || maxval > MAX_PS)
-            cdoWarning("Surface pressure out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID + 1);
+          const auto minmax = varrayMinMax(ps_prog);
+          if (minmax.first < MIN_PS || minmax.second > MAX_PS)
+            cdoWarning("Surface pressure out of range (min=%g max=%g) [timestep:%d]!", minmax.first, minmax.second, tsID + 1);
 
           presh(&full_press[0], &half_press[0], vct, &ps_prog[0], nhlevf, gridsize);
 
           if (useLogType)
             {
-              for (size_t i = 0; i < gridsize; i++) ps_prog[i] = log(ps_prog[i]);
-              for (size_t ki = 0; ki < nhlevh * gridsize; ki++) half_press[ki] = log(half_press[ki]);
-              for (size_t ki = 0; ki < nhlevf * gridsize; ki++) full_press[ki] = log(full_press[ki]);
+              for (size_t i = 0; i < gridsize; i++) ps_prog[i] = std::log(ps_prog[i]);
+              for (size_t ki = 0; ki < nhlevh * gridsize; ki++) half_press[ki] = std::log(half_press[ki]);
+              for (size_t ki = 0; ki < nhlevf * gridsize; ki++) full_press[ki] = std::log(full_press[ki]);
             }
 
-          genind(&vert_index[0], plev, &full_press[0], gridsize, nplev, nhlevf);
+          genind(&vert_index[0], plev.data(), &full_press[0], gridsize, nplev, nhlevf);
 
-          if (!extrapolate) genindmiss(&vert_index[0], plev, gridsize, nplev, &ps_prog[0], &pnmiss[0]);
+          if (!extrapolate) genindmiss(&vert_index[0], plev.data(), gridsize, nplev, &ps_prog[0], &pnmiss[0]);
         }
 
       for (varID = 0; varID < nvars; varID++)
         {
           if (vars[varID])
             {
-              int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              int nlevel = zaxisInqSize(zaxisID);
-              double missval = vlistInqVarMissval(vlistID1, varID);
+              const auto nlevel = varList1[varID].nlevels;
+              const auto missval = varList1[varID].missval;
               if (varinterp[varID])
                 {
                   /*
                   if ( nlevel == nhlevh )
                     {
-                      int i, k;
-                      double *vl1, *vl2;
-
                       for ( int k = 1; k < nlevel; k++ )
                         {
-                          vl1  = vardata1[varID] + gridsize*(k-1);
-                          vl2  = vardata1[varID] + gridsize*(k);
+                          double *vl1  = &vardata1[varID][gridsize*(k-1)];
+                          double *vl2  = &vardata1[varID][gridsize*(k)];
                           for ( size_t i = 0; i < gridsize; i++ )
                             vl1[i] = 0.5*(vl1[i] + vl2[i]);
                         }
@@ -558,12 +552,8 @@ Vertintml(void *process)
                       nlevel = nhlevf;
                     }
                   */
-                  double *hyb_press = NULL;
-                  if (nlevel == nhlevf)
-                    hyb_press = &full_press[0];
-                  else if (nlevel == nhlevh)
-                    hyb_press = &half_press[0];
-                  else
+                  double *hyb_press = (nlevel == nhlevf) ? &full_press[0] : (nlevel == nhlevh) ? &half_press[0] : nullptr;
+                  if (hyb_press == nullptr)
                     {
                       vlistInqVarName(vlistID1, varID, varname);
                       cdoAbort("Number of hybrid level differ from full/half level (param=%s)!", varname);
@@ -580,22 +570,23 @@ Vertintml(void *process)
 
                       if (useLogType && extrapolate) cdoAbort("Log. extrapolation of temperature unsupported!");
 
-                      interp_T(&sgeopot[0], vardata1[varID], vardata2[varID], &full_press[0], &half_press[0], &vert_index[0], plev,
-                               nplev, gridsize, nlevel, missval);
+                      interp_T(&sgeopot[0], &vardata1[varID][0], &vardata2[varID][0], &full_press[0], &half_press[0],
+                               &vert_index[0], plev.data(), nplev, gridsize, nlevel, missval);
                     }
                   else if (varID == gheightID)
                     {
                       for (size_t i = 0; i < gridsize; ++i) vardata1[varID][gridsize * nlevel + i] = sgeopot[i] / PlanetGrav;
 
-                      interp_Z(&sgeopot[0], vardata1[varID], vardata2[varID], &full_press[0], &half_press[0], &vert_index[0],
-                               vardata1[tempID], plev, nplev, gridsize, nlevel, missval);
+                      interp_Z(&sgeopot[0], &vardata1[varID][0], &vardata2[varID][0], &full_press[0], &half_press[0],
+                               &vert_index[0], &vardata1[tempID][0], plev.data(), nplev, gridsize, nlevel, missval);
                     }
                   else
                     {
-                      interp_X(vardata1[varID], vardata2[varID], hyb_press, &vert_index[0], plev, nplev, gridsize, nlevel, missval);
+                      interp_X(&vardata1[varID][0], &vardata2[varID][0], hyb_press, &vert_index[0], plev.data(), nplev, gridsize,
+                               nlevel, missval);
                     }
 
-                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID]);
+                  if (!extrapolate) varrayCopy(nplev, pnmiss, varnmiss[varID]);
                 }
             }
         }
@@ -604,14 +595,14 @@ Vertintml(void *process)
         {
           if (vars[varID])
             {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              const auto gridsize = varList2[varID].gridsize;
+              const auto nlevel = varList2[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  // gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-                  size_t offset = gridsize * levelID;
-                  double *single = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single, varnmiss[varID][levelID]);
+                  const size_t offset = gridsize * levelID;
+                  double *single = varinterp[varID] ? &vardata2[varID][offset] : &vardata1[varID][offset];
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -619,19 +610,12 @@ Vertintml(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata1[varID]);
-      if (varinterp[varID]) Free(vardata2[varID]);
-    }
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   if (vct) Free(vct);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
