@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -20,139 +20,136 @@
 
       Wind       uv2dv           U and V wind to divergence and vorticity
       Wind       dv2uv           Divergence and vorticity to U and V wind
-      Wind       dv2ps           Divergence and vorticity to
-                                 velocity potential and stream function
+      Wind       dv2ps           Divergence and vorticity to velocity potential and stream function
 */
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 #include "specspace.h"
 #include "listarray.h"
 #include "util_string.h"
 
+
 void *
 Wind(void *process)
 {
   int nrecs;
   int varID, levelID;
-  int nlev = 0;
-  int index, ngrids;
+  size_t nlev = 0;
   int gridIDsp = -1, gridIDgp = -1;
   int gridID1 = -1, gridID2 = -1;
-  int gridID;
   size_t nmiss;
-  int nlon, nlat, ntr = -1;
-  int code, param;
-  int pnum, pcat, pdis;
+  long ntr = -1;
   int varID1 = -1, varID2 = -1;
-  size_t gridsize;
-  size_t offset;
-  SPTRANS *sptrans = NULL;
-  DVTRANS *dvtrans = NULL;
+  SP_Transformation spTrans;
+  DV_Transformation dvTrans;
   char varname[CDI_MAX_NAME];
 
   cdoInitialize(process);
 
-  bool lcopy = UNCHANGED_RECORD;
+  const auto lcopy = unchangedRecord();
 
   // clang-format off
-  int UV2DV  = cdoOperatorAdd("uv2dv",  0, 0, NULL);
-  int UV2DVL = cdoOperatorAdd("uv2dvl", 0, 0, NULL);
-  int DV2UV  = cdoOperatorAdd("dv2uv",  0, 0, NULL);
-  int DV2UVL = cdoOperatorAdd("dv2uvl", 0, 0, NULL);
-  int DV2PS  = cdoOperatorAdd("dv2ps",  0, 0, NULL);
+  const auto UV2DV  = cdoOperatorAdd("uv2dv",  0, 0, nullptr);
+  const auto UV2DVL = cdoOperatorAdd("uv2dvl", 0, 0, nullptr);
+  const auto DV2UV  = cdoOperatorAdd("dv2uv",  0, 0, nullptr);
+  const auto DV2UVL = cdoOperatorAdd("dv2uvl", 0, 0, nullptr);
+  const auto DV2PS  = cdoOperatorAdd("dv2ps",  0, 0, nullptr);
+
+  const auto operatorID = cdoOperatorID();
+
+  const bool luv2dv = (operatorID == UV2DV || operatorID == UV2DVL);
+  const bool ldv2uv = (operatorID == DV2UV || operatorID == DV2UVL);
+  const bool linear = (operatorID == UV2DVL || operatorID == DV2UVL);
+
+  int (*nlat2ntr)(int) = linear ? nlat_to_ntr_linear : nlat_to_ntr;
+  const char *ctype = linear ? "l" : "";
+
+  if ((luv2dv || ldv2uv) && operatorArgc() == 1)
+    {
+      std::string type = parameter2word(cdoOperatorArgv(0));
+      if      (type == "linear")    { nlat2ntr = nlat_to_ntr_linear; ctype = "l"; }
+      else if (type == "cubic")     { nlat2ntr = nlat_to_ntr_cubic; ctype = "c"; }
+      else if (type == "quadratic") { nlat2ntr = nlat_to_ntr; }
+      else cdoAbort("Unsupported type: %s\n", type.c_str());
+    }
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto streamID1 = cdoOpenRead(0);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
-
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  /* find variables */
-  int nvars = vlistNvars(vlistID2);
+  // find variables
+  const auto nvars = vlistNvars(vlistID2);
   for (varID = 0; varID < nvars; varID++)
     {
-      param = vlistInqVarParam(vlistID2, varID);
+      const auto param = vlistInqVarParam(vlistID2, varID);
+      int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
-      code = pnum;
+      int code = pnum;
       if (operatorID == UV2DV || operatorID == UV2DVL)
         {
-          /* search for u and v wind */
+          // search for u and v wind
           if (pdis != 255 || code <= 0)
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
-
-              if (strcmp(varname, "u") == 0)
-                code = 131;
-              else if (strcmp(varname, "v") == 0)
-                code = 132;
+              cstrToLowerCase(varname);
+              if (cstrIsEqual(varname, "u")) code = 131;
+              if (cstrIsEqual(varname, "v")) code = 132;
             }
 
-          if (code == 131)
-            varID1 = varID;
-          else if (code == 132)
-            varID2 = varID;
+          if (code == 131) varID1 = varID;
+          if (code == 132) varID2 = varID;
         }
       else if (operatorID == DV2UV || operatorID == DV2UVL || operatorID == DV2PS)
         {
-          /* search for divergence and vorticity */
+          // search for divergence and vorticity
           if (pdis != 255)  // GRIB2
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
-
-              if (strcmp(varname, "d") == 0)
-                code = 155;
-              else if (strcmp(varname, "vo") == 0)
-                code = 138;
+              cstrToLowerCase(varname);
+              if (cstrIsEqual(varname, "d")) code = 155;
+              if (cstrIsEqual(varname, "vo")) code = 138;
             }
           else if (code <= 0)
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
-
-              if (strcmp(varname, "sd") == 0)
-                code = 155;
-              else if (strcmp(varname, "svo") == 0)
-                code = 138;
+              cstrToLowerCase(varname);
+              if (cstrIsEqual(varname, "sd")) code = 155;
+              if (cstrIsEqual(varname, "svo")) code = 138;
             }
 
-          if (code == 155)
-            varID1 = varID;
-          else if (code == 138)
-            varID2 = varID;
+          if (code == 155) varID1 = varID;
+          if (code == 138) varID2 = varID;
         }
       else
         cdoAbort("Unexpected operatorID %d", operatorID);
     }
 
-  ngrids = vlistNgrids(vlistID1);
-
-  /* find first spectral grid */
-  for (index = 0; index < ngrids; index++)
+  const auto ngrids = vlistNgrids(vlistID1);
+  // find first spectral grid
+  for (int index = 0; index < ngrids; index++)
     {
-      gridID = vlistGrid(vlistID1, index);
+      const auto gridID = vlistGrid(vlistID1, index);
       if (gridInqType(gridID) == GRID_SPECTRAL)
         {
           gridIDsp = gridID;
           break;
         }
     }
-
-  /* find first gaussian grid */
-  for (index = 0; index < ngrids; index++)
+  // find first gaussian grid
+  for (int index = 0; index < ngrids; index++)
     {
-      gridID = vlistGrid(vlistID1, index);
+      const auto gridID = vlistGrid(vlistID1, index);
       if (gridInqType(gridID) == GRID_GAUSSIAN)
         {
           gridIDgp = gridID;
@@ -160,8 +157,8 @@ Wind(void *process)
         }
     }
 
-  /* define output grid */
-  if (operatorID == UV2DV || operatorID == UV2DVL)
+  // define output grid
+  if (luv2dv)
     {
       if (varID1 == -1) cdoWarning("U-wind not found!");
       if (varID2 == -1) cdoWarning("V-wind not found!");
@@ -176,10 +173,7 @@ Wind(void *process)
 
           if (gridID1 != -1)
             {
-              if (operatorID == UV2DV)
-                ntr = nlat_to_ntr(gridInqYsize(gridID1));
-              else
-                ntr = nlat_to_ntr_linear(gridInqYsize(gridID1));
+              const long ntr = nlat2ntr(gridInqYsize(gridID1));
 
               if (gridIDsp != -1)
                 if (ntr != gridInqTrunc(gridIDsp)) gridIDsp = -1;
@@ -193,8 +187,7 @@ Wind(void *process)
             }
 
           if (gridIDsp == -1 && gridInqType(vlistGrid(vlistID1, 0)) == GRID_GAUSSIAN_REDUCED)
-            cdoAbort("Gaussian reduced grid found. Use option -R to convert it "
-                     "to a regular grid!");
+            cdoAbort("Gaussian reduced grid found. Use option -R to convert it to a regular grid!");
 
           if (gridIDsp == -1) cdoAbort("No Gaussian grid data found!");
 
@@ -211,14 +204,14 @@ Wind(void *process)
           vlistDefVarUnits(vlistID2, varID1, "1/s");
           vlistDefVarUnits(vlistID2, varID2, "1/s");
 
-          nlon = gridInqXsize(gridID1);
-          nlat = gridInqYsize(gridID1);
+          const long nlon = gridInqXsize(gridID1);
+          const long nlat = gridInqYsize(gridID1);
           ntr = gridInqTrunc(gridID2);
 
-          sptrans = sptrans_new(nlon, nlat, ntr, 1);
+          spTrans.init(nlon, nlat, ntr, 1);
         }
     }
-  else if (operatorID == DV2UV || operatorID == DV2UVL)
+  else if (ldv2uv)
     {
       if (varID1 == -1) cdoWarning("Divergence not found!");
       if (varID2 == -1) cdoWarning("Vorticity not found!");
@@ -230,29 +223,20 @@ Wind(void *process)
           if (gridInqType(gridID1) != GRID_SPECTRAL) cdoAbort("Vorticity is not on spectral grid!");
 
           if (gridID1 != vlistInqVarGrid(vlistID1, varID1))
-            cdoAbort("Divergence and vorticity must have the same grid "
-                     "represention!");
+            cdoAbort("Divergence and vorticity must have the same grid represention!");
 
           if (gridIDgp != -1)
             {
-              if (operatorID == DV2UV)
-                ntr = nlat_to_ntr(gridInqYsize(gridIDgp));
-              else
-                ntr = nlat_to_ntr_linear(gridInqYsize(gridIDgp));
-
+              const long nlat = gridInqYsize(gridIDgp);
+              const long ntr = nlat2ntr(nlat);
               if (gridInqTrunc(gridIDsp) != ntr) gridIDgp = -1;
             }
 
           if (gridIDgp == -1)
             {
               char gridname[20];
-
-              if (operatorID == DV2UV)
-                snprintf(gridname, sizeof(gridname), "t%dgrid", gridInqTrunc(gridIDsp));
-              else
-                snprintf(gridname, sizeof(gridname), "tl%dgrid", gridInqTrunc(gridIDsp));
-
-              gridIDgp = grid_from_name(gridname);
+              snprintf(gridname, sizeof(gridname), "t%s%dgrid", ctype, gridInqTrunc(gridIDsp));
+              gridIDgp = gridFromName(gridname);
             }
 
           gridID2 = gridIDgp;
@@ -268,12 +252,12 @@ Wind(void *process)
           vlistDefVarUnits(vlistID2, varID1, "m/s");
           vlistDefVarUnits(vlistID2, varID2, "m/s");
 
+          const long nlon = gridInqXsize(gridID2);
+          const long nlat = gridInqYsize(gridID2);
           ntr = gridInqTrunc(gridID1);
-          nlon = gridInqXsize(gridID2);
-          nlat = gridInqYsize(gridID2);
 
-          sptrans = sptrans_new(nlon, nlat, ntr, 0);
-          dvtrans = dvtrans_new(ntr);
+          spTrans.init(nlon, nlat, ntr, 0);
+          dvTrans.init(ntr);
         }
     }
   else if (operatorID == DV2PS)
@@ -304,19 +288,18 @@ Wind(void *process)
         }
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsizemax);
-
-  std::vector<double> ivar1, ivar2, ovar1, ovar2;
+  Varray<double> ivar1, ivar2, ovar1, ovar2;
   if (varID1 != -1 && varID2 != -1)
     {
       nlev = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
 
-      gridsize = gridInqSize(gridID1);
+      auto gridsize = gridInqSize(gridID1);
       ivar1.resize(nlev * gridsize);
       ivar2.resize(nlev * gridsize);
 
@@ -329,19 +312,19 @@ Wind(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if ((varID1 != -1 && varID2 != -1) && (varID == varID1 || varID == varID2))
             {
-              pstreamReadRecord(streamID1, array1.data(), &nmiss);
+              cdoReadRecord(streamID1, array1.data(), &nmiss);
               if (nmiss) cdoAbort("Missing values unsupported for spectral data!");
 
-              gridsize = gridInqSize(gridID1);
-              offset = gridsize * levelID;
+              const auto gridsize = gridInqSize(gridID1);
+              const auto offset = gridsize * levelID;
 
               if (varID == varID1)
                 arrayCopy(gridsize, array1.data(), &ivar1[offset]);
@@ -350,60 +333,60 @@ Wind(void *process)
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array1.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array1.data(), nmiss);
+                  cdoReadRecord(streamID1, array1.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array1.data(), nmiss);
                 }
             }
         }
 
       if (varID1 != -1 && varID2 != -1)
         {
-          if (operatorID == UV2DV || operatorID == UV2DVL)
-            trans_uv2dv(sptrans, nlev, gridID1, ivar1.data(), ivar2.data(), gridID2, ovar1.data(), ovar2.data());
-          else if (operatorID == DV2UV || operatorID == DV2UVL)
-            trans_dv2uv(sptrans, dvtrans, nlev, gridID1, ivar1.data(), ivar2.data(), gridID2, ovar1.data(), ovar2.data());
+          if (luv2dv)
+            trans_uv2dv(spTrans, nlev, gridID1, ivar1.data(), ivar2.data(), gridID2, ovar1.data(), ovar2.data());
+          else if (ldv2uv)
+            trans_dv2uv(spTrans, dvTrans, nlev, gridID1, ivar1.data(), ivar2.data(), gridID2, ovar1.data(), ovar2.data());
           else if (operatorID == DV2PS)
             {
               dv2ps(ivar1.data(), ovar1.data(), nlev, ntr);
               dv2ps(ivar2.data(), ovar2.data(), nlev, ntr);
             }
 
-          gridsize = gridInqSize(gridID2);
-          if (operatorID == UV2DV || operatorID == UV2DVL || operatorID == DV2PS)
+          const auto gridsize = gridInqSize(gridID2);
+          if (luv2dv || operatorID == DV2PS)
             {
-              for (levelID = 0; levelID < nlev; levelID++)
+              for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
-                  offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID2, levelID);
-                  pstreamWriteRecord(streamID2, &ovar2[offset], 0);
+                  const auto offset = gridsize * levelID;
+                  cdoDefRecord(streamID2, varID2, levelID);
+                  cdoWriteRecord(streamID2, &ovar2[offset], 0);
                 }
-              for (levelID = 0; levelID < nlev; levelID++)
+              for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
-                  offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID1, levelID);
-                  pstreamWriteRecord(streamID2, &ovar1[offset], 0);
+                  const auto offset = gridsize * levelID;
+                  cdoDefRecord(streamID2, varID1, levelID);
+                  cdoWriteRecord(streamID2, &ovar1[offset], 0);
                 }
             }
-          else if (operatorID == DV2UV || operatorID == DV2UVL)
+          else if (ldv2uv)
             {
-              for (levelID = 0; levelID < nlev; levelID++)
+              for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
-                  offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID1, levelID);
-                  pstreamWriteRecord(streamID2, &ovar1[offset], 0);
+                  const auto offset = gridsize * levelID;
+                  cdoDefRecord(streamID2, varID1, levelID);
+                  cdoWriteRecord(streamID2, &ovar1[offset], 0);
                 }
-              for (levelID = 0; levelID < nlev; levelID++)
+              for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
-                  offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID2, levelID);
-                  pstreamWriteRecord(streamID2, &ovar2[offset], 0);
+                  const auto offset = gridsize * levelID;
+                  cdoDefRecord(streamID2, varID2, levelID);
+                  cdoWriteRecord(streamID2, &ovar2[offset], 0);
                 }
             }
         }
@@ -411,13 +394,10 @@ Wind(void *process)
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  sptrans_delete(sptrans);
-  if (dvtrans) dvtrans_delete(dvtrans);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
