@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.1
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,62 +33,70 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "param_conversion.h"
+#include <mpim_grid.h>
 #include "percentiles.h"
 
+
+static void
+addOperators(void)
+{
+  // clang-format off
+  cdoOperatorAdd("zonmin",   func_min,   0, nullptr);
+  cdoOperatorAdd("zonmax",   func_max,   0, nullptr);
+  cdoOperatorAdd("zonrange", func_range, 0, nullptr);
+  cdoOperatorAdd("zonsum",   func_sum,   0, nullptr);
+  cdoOperatorAdd("zonmean",  func_mean,  0, nullptr);
+  cdoOperatorAdd("zonavg",   func_avg,   0, nullptr);
+  cdoOperatorAdd("zonvar",   func_var,   0, nullptr);
+  cdoOperatorAdd("zonvar1",  func_var1,  0, nullptr);
+  cdoOperatorAdd("zonstd",   func_std,   0, nullptr);
+  cdoOperatorAdd("zonstd1",  func_std1,  0, nullptr);
+  cdoOperatorAdd("zonpctl",  func_pctl,  0, nullptr);
+  // clang-format on
+}
+
 void *
 Zonstat(void *process)
 {
   int gridID1 = -1, gridID2 = -1;
   int zongridID = -1;
   int index;
-  size_t nmiss;
   int nrecs;
   int varID, levelID;
 
   cdoInitialize(process);
 
-  // clang-format off
-  cdoOperatorAdd("zonmin",   func_min,   0, NULL);
-  cdoOperatorAdd("zonmax",   func_max,   0, NULL);
-  cdoOperatorAdd("zonrange", func_range, 0, NULL);
-  cdoOperatorAdd("zonsum",   func_sum,   0, NULL);
-  cdoOperatorAdd("zonmean",  func_mean,  0, NULL);
-  cdoOperatorAdd("zonavg",   func_avg,   0, NULL);
-  cdoOperatorAdd("zonvar",   func_var,   0, NULL);
-  cdoOperatorAdd("zonvar1",  func_var1,  0, NULL);
-  cdoOperatorAdd("zonstd",   func_std,   0, NULL);
-  cdoOperatorAdd("zonstd1",  func_std1,  0, NULL);
-  cdoOperatorAdd("zonpctl",  func_pctl,  0, NULL);
-  // clang-format on
+  addOperators();
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   double pn = 0;
   if (operfunc == func_pctl)
     {
       operatorInputArg("percentile number");
-      pn = parameter2double(operatorArgv()[0]);
+      pn = parameter2double(cdoOperatorArgv(0));
       percentile_check_number(pn);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (index = 0; index < ngrids; index++)
     {
-      int gridID = vlistGrid(vlistID1, index);
-      if (gridInqXsize(gridID) > 1)
+      const auto gridID = vlistGrid(vlistID1, index);
+      if (gridInqXsize(gridID) > 1 || gridInqType(gridID) == GRID_GAUSSIAN_REDUCED)
         {
           if (gridID1 == -1) gridID1 = gridID;
         }
@@ -104,12 +112,12 @@ Zonstat(void *process)
       if (zongridID != -1 && zongridID == vlistGrid(vlistID1, index)) continue;
       if (gridID1 != vlistGrid(vlistID1, index)) ndiffgrids++;
     }
-  if (ndiffgrids > 0) cdoAbort("Too many different grids!");
+  if (ndiffgrids) cdoAbort("Too many different grids!");
 
   if (gridID1 != -1)
     {
-      int gridtype = gridInqType(gridID1);
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GENERIC)
+      const auto gridtype = gridInqType(gridID1);
+      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED || gridtype == GRID_GENERIC)
         {
           if (zongridID != -1 && gridInqYsize(zongridID) == gridInqYsize(gridID1))
             gridID2 = zongridID;
@@ -129,40 +137,43 @@ Zonstat(void *process)
 
   for (index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  if (Options::cdoChunkType == CDI_UNDEFID)
+    {
+      const auto nvars = vlistNvars(vlistID2);
+      for (varID = 0; varID < nvars; ++varID) vlistDefVarChunkType(vlistID2, varID, CDI_CHUNK_AUTO);
+    }
+
+  const auto streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   gridID1 = vlistInqVarGrid(vlistID1, 0);
-  int nlatmax = gridInqYsize(gridID1); /* max nlat ? */
+  const auto nlatmax = gridInqYsize(gridID1); /* max nlat ? */
 
-  size_t lim = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
-  field_init(&field2);
-  field_init(&field2);
-  field1.ptr = (double *) Malloc(lim * sizeof(double));
-  field2.ptr = (double *) Malloc(nlatmax * sizeof(double));
+  field1.resize(gridsizemax);
+  field2.resize(nlatmax);
   field2.grid = gridID2;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
-          field1.nmiss = nmiss;
           field1.grid = vlistInqVarGrid(vlistID1, varID);
           field1.missval = vlistInqVarMissval(vlistID1, varID);
           field2.missval = vlistInqVarMissval(vlistID1, varID);
 
           if (zongridID != -1 && zongridID == field1.grid)
             {
-              arrayCopy(nlatmax, field1.ptr, field2.ptr);
+              varrayCopy(nlatmax, field1.vec, field2.vec);
               field2.nmiss = field1.nmiss;
             }
           else
@@ -173,20 +184,17 @@ Zonstat(void *process)
                 zonfun(field1, field2, operfunc);
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, field2.ptr, field2.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  if (field1.ptr) Free(field1.ptr);
-  if (field2.ptr) Free(field2.ptr);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
