@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,10 @@
 #include "dmemory.h"
 #endif
 
-#include "cdo_int.h"
-#include "cdoOptions.h"
+#include "array.h"
+#include "process_int.h"
+#include "cdo_options.h"
+#include "cimdOmp.h"
 
 #ifdef SX
 #define NFFT 1024
@@ -46,61 +48,39 @@
 
 #define D60 (S60 + S60)
 
-long
-get_nfft(void)
-{
-  return (long) NFFT;
-}
-
-void
+int
 fft_set(double *trigs, long *ifax, long n)
 {
-  long j, len = n;
-  long nhl = n / 2;
-  double del = 4.0 * asin(1.0) / n;
-  double angle;
-  for (long k = 0; k < nhl; k++)
+  if (n == 0 || trigs == nullptr || ifax == nullptr)
     {
-      angle = k * del;
-      trigs[2 * k] = cos(angle);
-      trigs[2 * k + 1] = sin(angle);
+      fprintf(stderr, "fft_set: parameter not initialized!\n");
+      return -2;
+    }
+
+  const long len = n;
+  const long nhl = n / 2;
+  const double del = 4.0 * std::asin(1.0) / n;
+
+  for (long k = 0; k < nhl; k++)
+    {
+      const double angle = k * del;
+      trigs[2 * k] = std::cos(angle);
+      trigs[2 * k + 1] = std::sin(angle);
     }
 
-  long nfax = 0;
   for (long k = 0; k < 9; ++k) ifax[k] = 0;
 
   ifax[9] = n;
 
-  if (n % 8 == 0)
-    {
-      ifax[++nfax] = 8;
-      n /= 8;
-    }
-  while (n % 6 == 0)
-    {
-      ifax[++nfax] = 6;
-      n /= 6;
-    }
-  while (n % 5 == 0)
-    {
-      ifax[++nfax] = 5;
-      n /= 5;
-    }
-  while (n % 4 == 0)
-    {
-      ifax[++nfax] = 4;
-      n /= 4;
-    }
-  while (n % 3 == 0)
-    {
-      ifax[++nfax] = 3;
-      n /= 3;
-    }
-  if (n % 2 == 0)
-    {
-      ifax[++nfax] = 2;
-      n /= 2;
-    }
+  long nfax = 0;
+  // clang-format off
+  if    (n % 8 == 0) { ifax[++nfax] = 8; n /= 8; }
+  while (n % 6 == 0) { ifax[++nfax] = 6; n /= 6; }
+  while (n % 5 == 0) { ifax[++nfax] = 5; n /= 5; }
+  while (n % 4 == 0) { ifax[++nfax] = 4; n /= 4; }
+  while (n % 3 == 0) { ifax[++nfax] = 3; n /= 3; }
+  if    (n % 2 == 0) { ifax[++nfax] = 2; n /= 2; }
+  // clang-format on
 
   ifax[0] = nfax;
 
@@ -115,43 +95,50 @@ fft_set(double *trigs, long *ifax, long n)
 #endif
   for (long k = 0; k < nfax / 2; k++)
     {
-      j = ifax[k + 1];
+      const long j = ifax[k + 1];
       ifax[k + 1] = ifax[nfax - k];
       ifax[nfax - k] = j;
     }
-
-  if (n > 8)
+  /*
+  long prod = ifax[1];
+  for (long k = 2; k <= nfax; k++) prod *= ifax[k];
+  printf("nfax = %ld  len %ld  prod %ld  n %ld\n", nfax, len, prod, n);
+  for (long k = 1; k <= nfax; k++) printf(" %ld", ifax[k]);
+  printf("\n");
+  */
+  if (n > 8 || n == 7)
     {
       fprintf(stderr, "FFT does not work with len=%ld (n=%ld)!\n", len, n);
-      exit(1);
+      return -1;
     }
+
+  return 0;
 }
 
 static int
-rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, long inc2, long inc3, long inc4, long lot, long n,
-       long ifac, long la)
+rpassc(const double *a, const double *b, double *c, double *d, const double *trigs, long inc1, long inc2, long inc3, long inc4,
+       long lot, long n, long ifac, long la)
 {
   /*
-     rpassc' - performs one pass through data as part;
-     of multiple real fft (fourier synthesis) routine;
+     rpassc - performs one pass through data as part of multiple real fft (fourier synthesis) routine
 
-     a is first real input vector
-     b is equivalent to a + la * inc1
-     c is first real output vector
-     d is equivalent to c + ifac * la * inc2
+     a      is first real input vector
+     b      is equivalent to a + la * inc1
+     c      is first real output vector
+     d      is equivalent to c + ifac * la * inc2
      trigs  is a precalculated list of sines & cosines
-     inc1 is the addressing increment for a;
-     inc2 is the addressing increment for c;
-     inc3 is the increment between input vectors a;
-     inc4 is the increment between output vectors c;
-     lot is the number of vectors;
-     n is the length of the vectors;
-     ifac is the current factor of n;
-     la is the product of previous factors;
-     ierr is an error indicator:;
-     0 - pass completed without error;
-     2 - ifac not catered for;
-     3 - ifac only catered for if la=n/ifac;
+     inc1   is the addressing increment for a
+     inc2   is the addressing increment for c
+     inc3   is the increment between input vectors a
+     inc4   is the increment between output vectors c
+     lot    is the number of vectors
+     n      is the length of the vectors
+     ifac   is the current factor of n
+     la     is the product of previous factors
+     ierr   is an error indicator:
+               0 - pass completed without error
+               2 - ifac not catered for
+               3 - ifac only catered for if la=n/ifac
    */
 
   long i0, i1, i2, i3, i4, i5, i6, i7;
@@ -197,8 +184,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -229,8 +216,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -258,8 +245,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -278,8 +265,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -294,7 +281,6 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 3:
       {
         double afa1, a1p2, a1m2, a0mm, a0mp;
@@ -312,8 +298,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -351,8 +337,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -390,8 +376,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -414,8 +400,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -434,7 +420,6 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 4:
       {
         double a0m1, a0p2, a1p3, a0m2, a1m3, a0p2ma1p3, a0m2pb1p3, a0m2mb1p3;
@@ -453,8 +438,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -497,8 +482,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -545,8 +530,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -571,8 +556,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -592,7 +577,6 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 5:
       {
         double a1p2, a1m2, a0mm, a0mp, b136, b172, b236, b272;
@@ -611,8 +595,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -668,8 +652,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -717,8 +701,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -747,8 +731,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -770,7 +754,6 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 6:
       {
         ia = 0;
@@ -792,8 +775,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -843,8 +826,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -896,8 +879,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -920,8 +903,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -940,7 +923,6 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 8:
       {
         double a0p7, a1p5, a2p6, p073, p074, p152;
@@ -968,8 +950,8 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           {
             i = ibase;
             j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
             for (ijk = 0; ijk < lot; ++ijk)
               {
@@ -1010,12 +992,11 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
 }
 
 static int
-qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, long inc2, long inc3, long inc4, long lot, long n,
-       long ifac, long la)
+qpassc(const double *a, const double *b, double *c, double *d, const double *trigs, long inc1, long inc2, long inc3, long inc4,
+       long lot, long n, long ifac, long la)
 {
   /*
-     qpassc - performs one pass through data as part
-     of multiple real fft (fourier analysis) routine.
+     qpassc - performs one pass through data as part of multiple real fft (fourier analysis) routine
 
      a      is first real input vector
      b      is equivalent to a + ifac * la * inc1
@@ -1070,8 +1051,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1101,8 +1082,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -1127,8 +1108,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1148,8 +1129,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1164,7 +1145,6 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 3:
       {
         ia = 0;
@@ -1181,8 +1161,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1217,8 +1197,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -1253,8 +1233,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1277,8 +1257,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1294,7 +1274,6 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 4:
       {
         double a0p2, a1p3;
@@ -1314,8 +1293,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1359,8 +1338,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -1401,8 +1380,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1424,8 +1403,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1445,7 +1424,6 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 5:
       {
         double a1p4, a2p3, b1p4, b2p3, a025, b025, asps, bsps, a0pq, b0pq;
@@ -1468,8 +1446,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1522,8 +1500,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -1581,8 +1559,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1618,8 +1596,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1645,7 +1623,6 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 6:
       {
         double ab1a, ab2a, ab3a, ab4a, ab5a;
@@ -1676,8 +1653,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1736,8 +1713,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
                       {
                         i = ibase;
                         j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                         for (ijk = 0; ijk < lot; ++ijk)
                           {
@@ -1805,8 +1782,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1836,8 +1813,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
               {
                 i = ibase;
                 j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
                 for (ijk = 0; ijk < lot; ++ijk)
                   {
@@ -1865,7 +1842,6 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           }
         return 0;
       }
-
     case 8:
       {
         double a0p4, a1p5, a2p6, a3p7;
@@ -1895,8 +1871,8 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
           {
             i = ibase;
             j = jbase;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
             for (ijk = 0; ijk < lot; ++ijk)
               {
@@ -1934,7 +1910,7 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
 /* Fast Fourier Transform */
 /* ====================== */
 void
-fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *restrict gp, long nlat, long nlon, long nlev,
+fc2gp(const double *restrict trig, const long *restrict ifax, double *restrict fc, double *restrict gp, long nlat, long nlon, long nlev,
       long nfc)
 {
   /* fc2gp performs fourier to gridpoint transforms using           */
@@ -1952,21 +1928,21 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 
   if (ifax[9] != nlon) fprintf(stderr, "fc2gp: wrong initialization!\n");
 
-  long nfax = ifax[0];
+  const long nfax = ifax[0];
 
-  long jump = (nlon + 2);
-  long lot = nlev * nlat;
+  const long jump = (nlon + 2);
+  const long lot = nlev * nlat;
 
-  long nx = (nlon % 2 == 1) ? nlon : nlon + 1;
-  long nblox = 1 + (lot - 1) / NFFT;
+  const long nx = (nlon % 2 == 1) ? nlon : nlon + 1;
+  const long nblox = 1 + (lot - 1) / NFFT;
   long nvex = lot - (nblox - 1) * NFFT;
-  long nvex0 = nvex;
+  const long nvex0 = nvex;
 
-  long nthmax = (nblox < Threading::ompNumThreads) ? nblox : Threading::ompNumThreads;
-  long nvals = lot * jump;
+  const long nthmax = (nblox < Threading::ompNumThreads) ? nblox : Threading::ompNumThreads;
+  const long nvals = lot * jump;
 
   double *restrict wfc = (double *) Malloc(nvals * sizeof(double));
-  VECTOR_2D(double, wgp2d, nthmax, nvals);
+  VARRAY_2D(double, wgp2d, nthmax, nvals);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
@@ -1998,15 +1974,15 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 #endif
   for (long nb = 0; nb < nblox; nb++)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
       double *restrict wgp = &wgp2d[ompthID][0];
 
-      long istart = istartv[nb];
-      long nvex = (nb == 0) ? nvex0 : NFFT;
+      const long istart = istartv[nb];
+      const long nvex = (nb == 0) ? nvex0 : NFFT;
 
       long i = istart;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
       for (long j = 0; j < nvex; j++)
         {
@@ -2027,7 +2003,7 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
       long la = 1;
       for (long k = 0; k < nfax; ++k)
         {
-          long ifac = ifax[k + 1];
+          const long ifac = ifax[k + 1];
 
           if (k & 1)
             rpassc(wgp, wgp + la, wfc + ia, wfc + ia + ifac * la, trig, 1, 1, nx, jump, nvex, nlon, ifac, la);
@@ -2057,8 +2033,8 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
       /* Fill in zeros at end */
 
       long ix = istart + nlon;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
       for (long j = 0; j < nvex; j++)
         {
@@ -2081,7 +2057,7 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 }
 
 void
-gp2fc(double *trig, long *ifax, const double *restrict gp, double *restrict fc, long nlat, long nlon, long nlev, long nfc)
+gp2fc(const double *trig, const long *ifax, const double *restrict gp, double *restrict fc, long nlat, long nlon, long nlev, long nfc)
 {
   long lot, fou, ia, ifac, jump, k, la;
   long lat, lev, lon, nfax, rix, wix;
@@ -2166,8 +2142,8 @@ gp2fc(double *trig, long *ifax, const double *restrict gp, double *restrict fc,
       /* Shift a(0) & fill in zero imag parts */
 
       ix = istart;
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
       for (j = 0; j < nvex; j++)
         {
