@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,21 +15,22 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
-#include "grid.h"
+
+#include "process_int.h"
+#include <mpim_grid.h>
 
 void
-fld_field_init(field_type *field, size_t nmiss, double missval, size_t ngp, double *array, double *w)
+fld_field_init(Field &field, size_t nmiss, double missval, size_t ngp, double *array, double *w)
 {
-  field_init(field);
-
-  field->size = ngp;
-  field->nmiss = nmiss;
-  field->missval = missval;
-  field->ptr = array;
-  field->weight = w;
+  field.size = ngp;
+  field.nmiss = nmiss;
+  field.missval = missval;
+  if (array != nullptr)
+    for (size_t i = 0; i < ngp; ++i) field.vec[i] = array[i];
+  if (w != nullptr)
+    for (size_t i = 0; i < ngp; ++i) field.weightv[i] = w[i];
 }
-
+/*
 double *
 fld_weights(int gridID, size_t ngp)
 {
@@ -43,35 +44,30 @@ fld_weights(int gridID, size_t ngp)
       if (wstatus != 0 && lwarn)
         {
           lwarn = false;
-          cdoWarning("Grid cell bounds not available, using constant grid cell "
-                     "area weights!");
+          cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
         }
     }
 
   return weights;
 }
-
+*/
 int getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int nlev, double *thickness, double *weights);
 
-double *
-vert_weights(int zaxisID, size_t nlev)
+void
+vert_weights(int zaxisID, size_t nlev, std::vector<double> &weights)
 {
-  static bool lwarn = true;
-  double *weights = (double *) Malloc(nlev * sizeof(double));
-  double *thickness = (double *) Malloc(nlev * sizeof(double));
+  weights.resize(nlev);
+  std::vector<double> thickness(nlev);
   for (size_t i = 0; i < nlev; ++i) weights[i] = 1;
 
   if (nlev > 1)
     {
-      int wstatus = getLayerThickness(1, 0, 0, zaxisID, nlev, thickness, weights);
-      if (wstatus != 0 && lwarn && nlev > 1)
+      static bool lwarn = true;
+      int wstatus = getLayerThickness(1, 0, 0, zaxisID, nlev, thickness.data(), weights.data());
+      if (wstatus == 0 && lwarn && nlev > 1)
         {
           lwarn = false;
           cdoWarning("Layer bounds not available, using constant vertical weights!");
         }
     }
-
-  Free(thickness);
-
-  return weights;
 }
