@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -13,55 +13,29 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
+#include <vector>
 
 #include <cdi.h>
-#include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
+#include "cdo_output.h"
+#include "compare.h"
+#include "cimdOmp.h"
+#include "progress.h"
+#include "cimdOmp.h"
+
 
 static void
 lonlat_to_xyz(double lon, double lat, double *xyz)
 {
-  double coslat = cos(lat);
-  xyz[0] = coslat * cos(lon);
-  xyz[1] = coslat * sin(lon);
-  xyz[2] = sin(lat);
+  double coslat = std::cos(lat);
+  xyz[0] = coslat * std::cos(lon);
+  xyz[1] = coslat * std::sin(lon);
+  xyz[2] = std::sin(lat);
 }
 
-/*
-extern "C" {
-#include "lib/yac/grid.h"
-#include "lib/yac/grid_cell.h"
-#include "lib/yac/area.h"
-}
-
-static
-double yac_huiliers_area(int num_corners, double *cell_corner_lon, double
-*cell_corner_lat)
-{
-  if ( num_corners < 3 ) return 0;
-
-  double coordinates_xyz[num_corners*3];
-  enum edge_type edge_types[num_corners];
-  struct grid_cell cell =
-    {.coordinates_x   = cell_corner_lon,
-     .coordinates_y   = cell_corner_lat,
-     .coordinates_xyz = coordinates_xyz,
-     .edge_type       = edge_types,
-     .num_corners     = num_corners};
-
-  for ( int i = 0; i < num_corners; ++i ) edge_types[i] = GREAT_CIRCLE;
-  for ( int i = 0; i < num_corners; ++i )
-    lonlat_to_xyz(cell_corner_lon[i], cell_corner_lat[i], coordinates_xyz+i*3);
-
-  double area = huiliers_area(cell);
-  area /= (EarthRadius*EarthRadius);
-
-  return area;
-}
-*/
-
 static void
-cross_product(const double *restrict a, const double *restrict b, double *restrict c)
+cross_product(const double *a, const double *b, double *c)
 {
   c[0] = a[1] * b[2] - a[2] * b[1];
   c[1] = a[2] * b[0] - a[0] * b[2];
@@ -70,14 +44,14 @@ cross_product(const double *restrict a, const double *restrict b, double *restri
 
 /*
 static
-double scalar_product(const double *restrict a, const double *restrict b)
+double scalar_product(const double *a, const double *b)
 {
   return (a[0]*b[0] + a[1]*b[1] + a[2]*b[2]);
 }
 */
 
 static double
-norm(const double *restrict a)
+norm(const double *a)
 {
   return (a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
 }
@@ -120,10 +94,10 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
 
   // Test for a degenerated cells associated with collinear vertices.
 
-  if ( fabs(area) > 0.0 )
+  if ( std::fabs(area) > 0.0 )
     {
       for ( m = 0; m < M; m++ )
-        s[m] = sqrt(s[m]);
+        s[m] = std::sqrt(s[m]);
 
       for ( m = 0; m < M; m++ )
         for ( i = 0; i < 3; i++ )
@@ -147,7 +121,7 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
           ca[m] = - scalar_product(u[m], u[(m+1)%M]);
           if ( ca[m] < -1.0 ) ca[m] = -1.0;
           if ( ca[m] >  1.0 ) ca[m] =  1.0;
-          a[m] = acos(ca[m]);
+          a[m] = std::acos(ca[m]);
         }
 
       //  Compute areas = a1 + a2 + a3 - (M-2) * pi. here for a unit sphere:
@@ -179,27 +153,27 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
  * Licensed under the University of Illinois-NCSA License.
  */
 static double
-mod_tri_area(const double *restrict u, const double *restrict v, const double *restrict w)
+mod_tri_area(const double *u, const double *v, const double *w)
 {
   double tmp_vec[3];
 
   cross_product(u, v, tmp_vec);
-  double sina = sqrt(norm(tmp_vec));
-  double a = asin(sina);
+  double sina = std::sqrt(norm(tmp_vec));
+  double a = std::asin(sina);
 
   cross_product(u, w, tmp_vec);
-  double sinb = sqrt(norm(tmp_vec));
-  double b = asin(sinb);
+  double sinb = std::sqrt(norm(tmp_vec));
+  double b = std::asin(sinb);
 
   cross_product(w, v, tmp_vec);
-  double sinc = sqrt(norm(tmp_vec));
-  double c = asin(sinc);
+  double sinc = std::sqrt(norm(tmp_vec));
+  double c = std::asin(sinc);
 
   double s = 0.5 * (a + b + c);
 
-  double t = tan(s * 0.5) * tan((s - a) * 0.5) * tan((s - b) * 0.5) * tan((s - c) * 0.5);
+  double t = std::tan(s * 0.5) * std::tan((s - a) * 0.5) * std::tan((s - b) * 0.5) * std::tan((s - c) * 0.5);
 
-  double area = fabs(4.0 * atan(sqrt(fabs(t))));
+  double area = std::fabs(4.0 * std::atan(std::sqrt(std::fabs(t))));
 
   return area;
 }
@@ -263,8 +237,7 @@ mod_huiliers_area2(int num_corners, double *cell_corner_lon, double *cell_corner
 
   for (int i = 1; i < num_corners; i++)
     {
-      if (IS_EQUAL(cell_corner_lon[i], cell_corner_lon[i - 1]) && IS_EQUAL(cell_corner_lat[i], cell_corner_lat[i - 1]))
-        continue;
+      if (IS_EQUAL(cell_corner_lon[i], cell_corner_lon[i - 1]) && IS_EQUAL(cell_corner_lat[i], cell_corner_lat[i - 1])) continue;
 
       // points that make up a side of cell
       lonlat_to_xyz(cell_corner_lon[i], cell_corner_lat[i], pnt3);
@@ -287,53 +260,175 @@ mod_huiliers_area2(int num_corners, double *cell_corner_lon, double *cell_corner
   return sum;
 }
 
+static void
+getLonLatCorner(size_t nx, size_t idx, const double *grid_corner_lon, const double *grid_corner_lat, double *lons, double *lats)
+{
+  size_t j = idx / nx;
+  size_t i = idx - j * nx;
+
+  lons[0] = grid_corner_lon[2 * i];
+  lons[1] = grid_corner_lon[2 * i + 1];
+  lons[2] = grid_corner_lon[2 * i + 1];
+  lons[3] = grid_corner_lon[2 * i];
+
+  if (grid_corner_lat[2 * j + 1] > grid_corner_lat[2 * j])
+    {
+      lats[0] = grid_corner_lat[2 * j];
+      lats[1] = grid_corner_lat[2 * j];
+      lats[2] = grid_corner_lat[2 * j + 1];
+      lats[3] = grid_corner_lat[2 * j + 1];
+    }
+  else
+    {
+      lats[0] = grid_corner_lat[2 * j + 1];
+      lats[1] = grid_corner_lat[2 * j + 1];
+      lats[2] = grid_corner_lat[2 * j];
+      lats[3] = grid_corner_lat[2 * j];
+    }
+}
+
+static int
+gridGenAreaReg2D(const int gridID, double *area, bool lweights)
+{
+  // lweights = true : area used only to calculate weights
+
+  int status = 0;
+
+  if (gridInqXvals(gridID, nullptr) == 0 && gridInqYvals(gridID, nullptr) == 0 && gridInqXbounds(gridID, nullptr) == 0
+      && gridInqYbounds(gridID, nullptr) == 0)
+    {
+      cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
+      return 1;
+    }
+
+  char unitstr[CDI_MAX_NAME];
+  auto gridsize = gridInqSize(gridID);
+  auto nlon = gridInqXsize(gridID);
+  auto nlat = gridInqYsize(gridID);
+  if (!nlon) nlon = 1;
+  if (!nlat) nlat = 1;
+  std::vector<double> grid_corner_lon(nlon * 2);
+  std::vector<double> grid_corner_lat(nlat * 2);
+
+  if (gridInqXbounds(gridID, nullptr))
+    {
+      gridInqXbounds(gridID, grid_corner_lon.data());
+      gridInqXunits(gridID, unitstr);
+    }
+  else if (lweights && nlon == 1)
+    {
+      grid_corner_lon[0] = 0;
+      grid_corner_lon[1] = .01;
+      strcpy(unitstr, "radian");
+    }
+  else if (nlon == 1)
+    {
+      return 1;
+    }
+  else
+    {
+      std::vector<double> grid_center_lon(nlon);
+      gridInqXvals(gridID, grid_center_lon.data());
+      grid_gen_bounds(nlon, grid_center_lon, grid_corner_lon);
+      gridInqXunits(gridID, unitstr);
+    }
+
+  if (gridInqYbounds(gridID, nullptr))
+    {
+      gridInqYbounds(gridID, grid_corner_lat.data());
+      gridInqYunits(gridID, unitstr);
+    }
+  else if (lweights && nlat == 1)
+    {
+      grid_corner_lat[0] = 0;
+      grid_corner_lat[1] = .01;
+      strcpy(unitstr, "radian");
+    }
+  else if (nlat == 1)
+    {
+      return 1;
+    }
+  else
+    {
+      std::vector<double> grid_center_lat(nlat);
+      gridInqYvals(gridID, grid_center_lat.data());
+      grid_gen_bounds(nlat, grid_center_lat, grid_corner_lat);
+      grid_check_lat_borders(nlat * 2, grid_corner_lat.data());
+      gridInqYunits(gridID, unitstr);
+    }
+
+  grid_to_radian(unitstr, nlon * 2, grid_corner_lon.data(), "grid cell corner longitudes");
+  grid_to_radian(unitstr, nlat * 2, grid_corner_lat.data(), "grid cell corner latitudes");
+
+  int nv = 4;
+  double findex = 0;
+
+  Progress::init();
+
+#ifdef _OPENMP
+#pragma omp parallel for default(none) shared(gridsize, nlon, findex, area, nv, grid_corner_lon, grid_corner_lat)
+#endif
+  for (size_t i = 0; i < gridsize; ++i)
+    {
+#ifdef _OPENMP
+#pragma omp atomic
+#endif
+      findex++;
+      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
+
+      double lons[4], lats[4];
+      getLonLatCorner(nlon, i, grid_corner_lon.data(), grid_corner_lat.data(), lons, lats);
+      area[i] = mod_huiliers_area(nv, lons, lats);
+    }
+
+  Progress::update(0, 1, 1);
+
+  return status;
+}
+
 int
-gridGenArea(int gridID, double *area)
+gridGenAreaReg2Dweights(const int gridID, double *area)
+{
+  bool lweights = true;
+  return gridGenAreaReg2D(gridID, area, lweights);
+}
+
+static int
+gridGenAreaUnstruct(int gridID, double *area)
 {
   int status = 0;
-  bool lgrid_gen_bounds = false;
   bool lgriddestroy = false;
 
-  size_t gridsize = gridInqSize(gridID);
+  auto gridsize = gridInqSize(gridID);
   int gridtype = gridInqType(gridID);
-  int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-
-  if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN && projtype != CDI_PROJ_RLL && projtype != CDI_PROJ_LAEA
-      && projtype != CDI_PROJ_SINU && projtype != CDI_PROJ_LCC && gridtype != GRID_GME && gridtype != GRID_CURVILINEAR
-      && gridtype != GRID_UNSTRUCTURED)
-    {
-      cdoAbort("Internal error! Unsupported gridtype: %s", gridNamePtr(gridtype));
-    }
 
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
-      if (gridtype == GRID_GME)
+      if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
         {
           lgriddestroy = true;
           gridID = gridToUnstructured(gridID, 1);
-          /*
-          grid_mask = (int*) Malloc(gridsize*sizeof(int));
-          gridInqMaskGME(gridID, grid_mask);
-          if ( grid_mask ) Free(grid_mask);
-          */
         }
       else
         {
           lgriddestroy = true;
           gridID = gridToCurvilinear(gridID, 1);
-          lgrid_gen_bounds = true;
         }
     }
 
   if (gridtype == GRID_UNSTRUCTURED)
     {
-      if (gridInqYvals(gridID, NULL) == 0 || gridInqXvals(gridID, NULL) == 0)
+      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
         {
           if (gridInqNumber(gridID) > 0)
             {
               lgriddestroy = true;
               gridID = referenceToGrid(gridID);
-              if (gridID == -1) return 1;
+              if (gridID == -1)
+                {
+                  cdoWarning("Reference to source grid not found!");
+                  return 1;
+                }
             }
         }
     }
@@ -342,20 +437,16 @@ gridGenArea(int gridID, double *area)
 
   size_t nv = (gridtype == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-  if (gridInqYvals(gridID, NULL) == 0 || gridInqXvals(gridID, NULL) == 0)
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
     {
-      cdoWarning("Computation of grid cell area weights failed, grid cell "
-                 "center coordinates missing!");
-      status = 1;
-      return status;
+      cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
+      return 1;
     }
 
   if (nv == 0)
     {
-      cdoWarning("Computation of grid cell area weights failed, grid cell "
-                 "corner coordinates missing!");
-      status = 1;
-      return status;
+      cdoWarning("Computation of grid cell area weights failed, grid cell corner coordinates missing!");
+      return 1;
     }
 
   char xunitstr[CDI_MAX_NAME];
@@ -372,28 +463,15 @@ gridGenArea(int gridID, double *area)
   std::vector<double> grid_corner_lon(nv * gridsize);
   std::vector<double> grid_corner_lat(nv * gridsize);
 
-  if (gridInqYbounds(gridID, NULL) && gridInqXbounds(gridID, NULL))
+  if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
     {
       gridInqXbounds(gridID, grid_corner_lon.data());
       gridInqYbounds(gridID, grid_corner_lat.data());
     }
   else
     {
-      if (lgrid_gen_bounds)
-        {
-          size_t nlon = gridInqXsize(gridID);
-          size_t nlat = gridInqYsize(gridID);
-          double dlon = 0;
-          if (nlon == 1) dlon = 1;
-
-          grid_cell_center_to_bounds_X2D(xunitstr, nlon, nlat, grid_center_lon.data(), grid_corner_lon.data(), dlon);
-          grid_cell_center_to_bounds_Y2D(yunitstr, nlon, nlat, grid_center_lat.data(), grid_corner_lat.data());
-        }
-      else
-        {
-          status = 1;
-          return status;
-        }
+      status = 1;
+      return status;
     }
 
   grid_to_radian(xunitstr, gridsize, grid_center_lon.data(), "grid1 center longitudes");
@@ -406,34 +484,58 @@ gridGenArea(int gridID, double *area)
 
   double findex = 0;
 
-  progressInit();
+  Progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, gridsize, area, nv, grid_corner_lon, grid_corner_lat, \
-                                              grid_center_lon, grid_center_lat)
+#pragma omp parallel for default(none) \
+    shared(gridsize, nv, findex, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-      int lprogress = 1;
-      if (cdo_omp_get_thread_num() != 0) lprogress = 0;
-
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (lprogress) progressStatus(0, 1, findex / gridsize);
+      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
 
-      // area[i] = mod_cell_area(nv, grid_corner_lon+i*nv, grid_corner_lat+i*nv);
       if (nv <= 4)
         area[i] = mod_huiliers_area(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv]);
       else
-        area[i] = mod_huiliers_area2(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv],
-                                     grid_center_lon[i], grid_center_lat[i]);
+        area[i]
+            = mod_huiliers_area2(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv], grid_center_lon[i], grid_center_lat[i]);
     }
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
-  if (cdoVerbose)
+  return status;
+}
+
+int
+gridGenArea(const int gridID, double *area)
+{
+  int status = 0;
+
+  const auto gridsize = gridInqSize(gridID);
+  const auto gridtype = gridInqType(gridID);
+  const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+
+  if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)
+    {
+      bool lweights = false;
+      status = gridGenAreaReg2D(gridID, area, lweights);
+    }
+  else if (projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC
+           || projtype == CDI_PROJ_STERE || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
+           || gridtype == GRID_GAUSSIAN_REDUCED)
+    {
+      status = gridGenAreaUnstruct(gridID, area);
+    }
+  else
+    {
+      cdoAbort("Internal error! Unsupported gridtype: %s", gridNamePtr(gridtype));
+    }
+
+  if (gridVerbose)
     {
       double total_area = 0;
       for (size_t i = 0; i < gridsize; ++i) total_area += area[i];
