@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,10 +16,15 @@
 */
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
-#include "util.h" /* progressStatus */
-#include "cdoOptions.h"
+
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_math.h"
+#include <mpim_grid.h>
+#include "cdo_options.h"
+#include "interpol.h"
+#include "progress.h"
+#include "cimdOmp.h"
 
 /**
 * Find the interval i-1 .. i in which an element x fits and return i, the
@@ -125,12 +130,12 @@ rect_grid_search(size_t *ii, size_t *jj, double x, double y, size_t nxm, size_t
   int lfound = 0;
 
   *jj = find_element(y, nym, ym);
-  if (*jj >= nym && fabs(ym[0] - y) < rtol) *jj = 1;  // fix rounding errors
+  if (*jj >= nym && std::fabs(ym[0] - y) < rtol) *jj = 1;  // fix rounding errors
 
   if (*jj < nym)
     {
       *ii = find_element(x, nxm, xm);
-      if (*ii >= nxm && fabs(xm[0] - x) < rtol) *ii = 1;  // fix rounding errors
+      if (*ii >= nxm && std::fabs(xm[0] - x) < rtol) *ii = 1;  // fix rounding errors
 
       if (*ii < nxm) lfound = 1;
     }
@@ -145,7 +150,7 @@ rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, co
   *imin = nxm;
   *imax = -1;
 
-  bool lascend = (xm[0] < xm[nxm - 1]);
+  const bool lascend = (xm[0] < xm[nxm - 1]);
 
   long i1 = find_element(xmin, nxm, xm);
   long i2 = find_element(xmax, nxm, xm);
@@ -196,7 +201,7 @@ intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const double
   double value = 0;
 
   for (jj = 1; jj < nym; jj++)
-    if (y >= MIN(ym[jj - 1], ym[jj]) && y <= MAX(ym[jj - 1], ym[jj])) break;
+    if (y >= cdo::min(ym[jj - 1], ym[jj]) && y <= cdo::max(ym[jj - 1], ym[jj])) break;
 
   for (ii = 1; ii < nxm; ii++)
     if (x >= xm[ii - 1] && x <= xm[ii]) break;
@@ -213,30 +218,29 @@ intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const double
 }
 
 static void
-intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const double *restrict fieldm, const double *restrict xm,
-           const double *restrict ym, size_t gridsize2, double *field, const double *restrict x, const double *restrict y)
+intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const double *restrict fieldm,
+           const std::vector<double> &xm, const std::vector<double> &ym, size_t gridsize2, double *field,
+           const std::vector<double> &x, const std::vector<double> &y)
 {
   size_t nlon1 = nxm;
   double findex = 0;
 
   if (lon_is_circular) nlon1--;
-  size_t gridsize1 = nlon1 * nym;
+  const size_t gridsize1 = nlon1 * nym;
 
   std::vector<bool> grid1_mask(gridsize1);
   for (size_t jj = 0; jj < nym; ++jj)
     for (size_t ii = 0; ii < nlon1; ++ii) grid1_mask[jj * nlon1 + ii] = !DBL_IS_EQUAL(fieldm[jj * nlon1 + ii], missval);
 
-  progressInit();
+  Progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, \
-                                              lon_is_circular, grid1_mask)
+#pragma omp parallel for default(none) \
+    shared(findex, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, lon_is_circular, grid1_mask)
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
       size_t src_add[4]; /*  address for the four source points    */
-      int lprogress = 1;
-      if (cdo_omp_get_thread_num() != 0) lprogress = 0;
 
       field[i] = missval;
 
@@ -244,10 +248,10 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
 #pragma omp atomic
 #endif
       findex++;
-      if (lprogress) progressStatus(0, 1, findex / gridsize2);
+      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize2);
 
       size_t ii, jj;
-      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm, ym);
+      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm.data(), ym.data());
 
       if (lfound)
         {
@@ -279,27 +283,14 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
         }
     }
 
-  progressStatus(0, 1, 1);
-}
-
-double
-intlin(double x, double y1, double x1, double y2, double x2)
-{
-  /*
-    xlin - lineare interpolation
-
-    Uwe Schulzweida  04/05/1995
-  */
-  double value = (y2 * (x - x1) + y1 * (x2 - x)) / (x2 - x1);
-
-  return value;
+  Progress::update(0, 1, 1);
 }
 
 void
 intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 {
   /*
-    xlinarr - lineare interpolation over 1D array
+    intlinarr - lineare interpolation over 1D array
 
     Uwe Schulzweida  04/05/1995
   */
@@ -309,17 +300,17 @@ intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 }
 
 void
-intgridbil(Field *field1, Field *field2)
+intgridbil(Field &field1, Field &field2)
 {
   char xunits[CDI_MAX_NAME];
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
-  double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
-  double missval = field1->missval;
+  auto gridID1 = field1.grid;
+  auto gridID2 = field2.grid;
+  auto &array1 = field1.vec;
+  auto &array2 = field2.vec;
+  const auto missval = field1.missval;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  auto nlon1 = gridInqXsize(gridID1);
+  auto nlat1 = gridInqYsize(gridID1);
 
   int lon_is_circular = 0;
 
@@ -327,19 +318,19 @@ intgridbil(Field *field1, Field *field2)
   if (grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2)) lgeorefgrid = false;
 
   double **array1_2D = (double **) Malloc(nlat1 * sizeof(double *));
-  for (size_t ilat = 0; ilat < nlat1; ilat++) array1_2D[ilat] = array1 + ilat * nlon1;
+  for (size_t ilat = 0; ilat < nlat1; ilat++) array1_2D[ilat] = array1.data() + ilat * nlon1;
 
   if (lgeorefgrid)
     {
-      if (!(gridInqXvals(gridID1, NULL) && gridInqYvals(gridID1, NULL))) cdoAbort("Source grid has no coordinate values!");
+      if (!(gridInqXvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr))) cdoAbort("Source grid has no coordinate values!");
 
       lon_is_circular = gridIsCircular(gridID1);
 
       if (lon_is_circular) nlon1 += 1;
     }
 
-  std::vector<double> lon1(nlon1);
-  std::vector<double> lat1(nlat1);
+  Varray<double> lon1(nlon1);
+  Varray<double> lat1(nlat1);
   gridInqXvals(gridID1, lon1.data());
   gridInqYvals(gridID1, lat1.data());
 
@@ -355,8 +346,8 @@ intgridbil(Field *field1, Field *field2)
       if (lon_is_circular) lon1[nlon1 - 1] = lon1[0] + 2 * M_PI;
     }
 
-  size_t xsize2 = gridInqXsize(gridID2);
-  size_t ysize2 = gridInqYsize(gridID2);
+  const auto xsize2 = gridInqXsize(gridID2);
+  const auto ysize2 = gridInqYsize(gridID2);
 
   if (xsize2 == 1 && ysize2 == 1)
     {
@@ -393,12 +384,12 @@ intgridbil(Field *field1, Field *field2)
       if (lon2 < lon1[0] || lon2 > lon1[nlon1 - 1])
         cdoAbort("Longitude %f out of bounds (%f to %f)!", lon2, lon1[0], lon1[nlon1 - 1]);
 
-      if (lat2 < MIN(lat1[0], lat1[nlat1 - 1]) || lat2 > MAX(lat1[0], lat1[nlat1 - 1]))
+      if (lat2 < cdo::min(lat1[0], lat1[nlat1 - 1]) || lat2 > cdo::max(lat1[0], lat1[nlat1 - 1]))
         cdoAbort("Latitude %f out of bounds (%f to %f)!", lat2, lat1[0], lat1[nlat1 - 1]);
 
-      *array2 = intlinarr2p(nlon1, nlat1, array1_2D, lon1.data(), lat1.data(), lon2, lat2);
+      array2[0] = intlinarr2p(nlon1, nlat1, array1_2D, lon1.data(), lat1.data(), lon2, lat2);
       /*
-      printf("%5d %f %f %f\n", index++, lon2, lat2, *array2);
+      printf("%5d %f %f %f\n", index++, lon2, lat2, array2[0]);
       */
     }
   else
@@ -410,13 +401,14 @@ intgridbil(Field *field1, Field *field2)
           if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
             gridID2 = gridToCurvilinear(gridID2, 0);
 
-          if (!(gridInqXvals(gridID2, NULL) && gridInqYvals(gridID2, NULL))) cdoAbort("Target grid has no coordinate values!");
+          if (!(gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID2, nullptr)))
+            cdoAbort("Target grid has no coordinate values!");
         }
 
-      size_t gridsize2 = gridInqSize(gridID2);
+      const auto gridsize2 = gridInqSize(gridID2);
 
-      std::vector<double> xvals2(gridsize2);
-      std::vector<double> yvals2(gridsize2);
+      Varray<double> xvals2(gridsize2);
+      Varray<double> yvals2(gridsize2);
 
       if (lgeorefgrid)
         {
@@ -436,8 +428,8 @@ intgridbil(Field *field1, Field *field2)
         }
       else
         {
-          std::vector<double> xcoord(xsize2);
-          std::vector<double> ycoord(ysize2);
+          Varray<double> xcoord(xsize2);
+          Varray<double> ycoord(ysize2);
 
           gridInqXvals(gridID2, xcoord.data());
           gridInqYvals(gridID2, ycoord.data());
@@ -450,10 +442,9 @@ intgridbil(Field *field1, Field *field2)
               }
         }
 
-      intlinarr2(missval, lon_is_circular, nlon1, nlat1, array1, lon1.data(), lat1.data(), gridsize2, array2, xvals2.data(),
-                 yvals2.data());
+      intlinarr2(missval, lon_is_circular, nlon1, nlat1, array1.data(), lon1, lat1, gridsize2, array2.data(), xvals2, yvals2);
 
-      field2->nmiss = arrayNumMV(gridsize2, array2, missval);
+      field2.nmiss = varrayNumMV(gridsize2, array2, missval);
     }
 
   Free(array1_2D);
@@ -461,7 +452,7 @@ intgridbil(Field *field1, Field *field2)
 
 /* source code from pingo */
 void
-interpolate(Field *field1, Field *field2)
+interpolate(Field &field1, Field &field2)
 {
   int i;
   double *lono_array, *lato_array, *lono, *lato;
@@ -469,9 +460,7 @@ interpolate(Field *field1, Field *field2)
   // size_t gridsize_i
   size_t gridsize_o;
   int gridIDi;
-  double *arrayIn;
   int gridIDo;
-  double *arrayOut;
   double missval;
   long ilon, ilat, nxlon, nxlat, olon, olat;
   long l11, l12, l21, l22, l1, l2;
@@ -488,11 +477,11 @@ interpolate(Field *field1, Field *field2)
   double faclon1, faclon2, faclat1, faclat2;
   int nlon, nlat, out_nlon, out_nlat;
 
-  gridIDi = field1->grid;
-  gridIDo = field2->grid;
-  arrayIn = field1->ptr;
-  arrayOut = field2->ptr;
-  missval = field1->missval;
+  gridIDi = field1.grid;
+  gridIDo = field2.grid;
+  auto &arrayIn = field1.vec;
+  auto &arrayOut = field2.vec;
+  missval = field1.missval;
 
   // gridsize_i = gridInqSize(gridIDi);
   gridsize_o = gridInqSize(gridIDo);
@@ -507,9 +496,9 @@ interpolate(Field *field1, Field *field2)
   lon = lon_array + 1;
   lat = lat_array + 1;
 
-  if (!(gridInqXvals(gridIDi, NULL) && gridInqYvals(gridIDi, NULL))) cdoAbort("Source grid has no values");
+  if (!(gridInqXvals(gridIDi, nullptr) && gridInqYvals(gridIDi, nullptr))) cdoAbort("Source grid has no values");
 
-  if (!(gridInqXvals(gridIDo, NULL) && gridInqYvals(gridIDo, NULL))) cdoAbort("Target grid has no values");
+  if (!(gridInqXvals(gridIDo, nullptr) && gridInqYvals(gridIDo, nullptr))) cdoAbort("Target grid has no values");
 
   gridInqXvals(gridIDi, lon);
   gridInqYvals(gridIDi, lat);
@@ -641,7 +630,7 @@ interpolate(Field *field1, Field *field2)
   xlat[2 * nlat] = (lat[nlat - 1] + lat[nlat]) / 2;
 
   in0 = (double **) Malloc(nlat * sizeof(double *));
-  for (ilat = 0; ilat < nlat; ilat++) in0[ilat] = arrayIn + ilat * nlon;
+  for (ilat = 0; ilat < nlat; ilat++) in0[ilat] = arrayIn.data() + ilat * nlon;
 
   std::vector<long> ilon11(out_nlon);
   std::vector<long> ilon12(out_nlon);
@@ -657,8 +646,8 @@ interpolate(Field *field1, Field *field2)
       volon1 = (lono[olon - 1] + lono[olon]) / 2;
       volon2 = (lono[olon] + lono[olon + 1]) / 2;
       if (IS_EQUAL(volon1, volon2)) volon2 += 360;
-      volon2 -= 360 * floor((volon1 - xlon[0]) / 360);
-      volon1 -= 360 * floor((volon1 - xlon[0]) / 360);
+      volon2 -= 360 * std::floor((volon1 - xlon[0]) / 360);
+      volon1 -= 360 * std::floor((volon1 - xlon[0]) / 360);
       volon21[olon] = volon1;
       volon22[olon] = volon2;
       for (l21 = 0; l21 < nxlon && xlon[l21] < volon1; l21++)
@@ -727,7 +716,7 @@ interpolate(Field *field1, Field *field2)
     }
 
   xout = (double **) Malloc(out_nlat * sizeof(double *));
-  for (olat = 0; olat < out_nlat; olat++) xout[olat] = arrayOut + olat * out_nlon;
+  for (olat = 0; olat < out_nlat; olat++) xout[olat] = arrayOut.data() + olat * out_nlon;
 
   wrap_around = nlon > 1 && (lon[nlon - 1] >= lon[-1] + 360 - 0.001 || lon[nlon] >= lon[0] + 360 - 0.001);
 
@@ -917,9 +906,9 @@ interpolate(Field *field1, Field *field2)
                           b21 = a11 + (a12 - a11) * faclon1 + (a21 - a11) * faclat2 + (a22 - a12 - a21 + a11) * faclon1 * faclat2;
                           b22 = a11 + (a12 - a11) * faclon2 + (a21 - a11) * faclat2 + (a22 - a12 - a21 + a11) * faclon2 * faclat2;
                         }
-                      wsum += (vlon2 - vlon1) * (sin(vlat2) - sin(vlat1));
-                      t = 2 * sin((vlat2 + vlat1) / 2) * sin((vlat2 - vlat1) / 2) / (vlat2 - vlat1);
-                      sum += (vlon2 - vlon1) / 2 * ((b11 + b12) * (t - sin(vlat1)) + (b21 + b22) * (sin(vlat2) - t));
+                      wsum += (vlon2 - vlon1) * (std::sin(vlat2) - std::sin(vlat1));
+                      t = 2 * std::sin((vlat2 + vlat1) / 2) * std::sin((vlat2 - vlat1) / 2) / (vlat2 - vlat1);
+                      sum += (vlon2 - vlon1) / 2 * ((b11 + b12) * (t - std::sin(vlat1)) + (b21 + b22) * (std::sin(vlat2) - t));
                     }
                 }
             }
@@ -927,7 +916,7 @@ interpolate(Field *field1, Field *field2)
         }
     }
 
-  field2->nmiss = arrayNumMV(gridsize_o, arrayOut, missval);
+  field2.nmiss = varrayNumMV(gridsize_o, arrayOut, missval);
 
   Free(lon_array);
   Free(lat_array);
