@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,22 +15,56 @@
   GNU General Public License for more details.
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "error.h"
 #include "modules.h"
-#include <cdi.h>
 #include "util_string.h"
+#include "cdo_output.h"
 
-#include <dirent.h>
-#include <dlfcn.h>
-#include <regex>
-#include <set>
-#include <string>
-// for std::sort()
-#include <algorithm>
+#include <cstring>
+#include <algorithm>  // for std::sort()
+
+/* removes '-' from operator string and returns copy of parameter operatorCommand */
+/***
+  vector for library handles for loaded custom modules
+  */
+std::vector<void *> custom_modules_lib_handles;
+
+/***
+  Maps operator names to their module names
+ */
+std::map<std::string, std::string> modules_map;
+
+/***
+  Contains added modules as values and their names as key
+  */
+std::map<std::string, module_t> modules;
+
+/***
+  Key: operator alias / Value: operator original name
+ */
+std::map<std::string, std::string> aliases;
+static const char *
+getOperatorName(const char *operatorCommand)
+{
+  char *operatorName = nullptr;
+
+  if (operatorCommand)
+    {
+      if (operatorCommand[0] == '-') operatorCommand++;
+      const char *commapos = strchr(operatorCommand, ',');
+      const size_t len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
+
+      operatorName = (char *) malloc(len + 1);
+
+      if (operatorName != nullptr)
+        {
+          memcpy(operatorName, operatorCommand, len);
+        }
+      else{return nullptr;}
+      operatorName[len] = '\0';
+    }
+
+  return operatorName;
+}
 
 /**
  * @param a pointer to a string/substring
@@ -40,10 +74,8 @@
  * @retval true if a is similar to b
  * @retval false if a is not similar to b
  *
- * Recursive function for finding substrings of a operator name that match other
- * operators.
+ * Recursive function for finding substrings of a operator name that match other operators.
  */
-
 static bool
 similar(const char *a, const char *b, unsigned long alen, unsigned long blen)
 {
@@ -85,16 +117,12 @@ similar(std::string original, std::string other)
  * @retval false if not
  */
 static bool
-operator_name_exists(std::string operatorName)
+operator_name_exists(const std::string &operatorName)
 {
-  if (modules_map.find(operatorName) != modules_map.end())
-    {
-      return true;
-    }
-  if (aliases.find(operatorName) != aliases.end())
-    {
-      return true;
-    }
+  if (modules_map.find(operatorName) != modules_map.end()) return true;
+
+  if (aliases.find(operatorName) != aliases.end()) return true;
+
   return false;
 }
 
@@ -103,6 +131,7 @@ operator_name_exists(std::string operatorName)
  * @retval true if #modules contains \a moduleName
  * @retval false if not
  */
+/*
 static bool
 module_map_contains(std::string moduleName)
 {
@@ -112,11 +141,11 @@ module_map_contains(std::string moduleName)
     }
   else
     {
-      Error("Module %s not found", moduleName.c_str());
+      cdoAbort("Module %s not found", moduleName.c_str());
     }
   return false;
 }
-
+*/
 /***
  * function for finding similar operator names for the given string
  * @param operatorName operator name to find similar operators for
@@ -124,17 +153,18 @@ module_map_contains(std::string moduleName)
  * with a max lenght of 75 characters
  */
 static std::string
-find_similar(std::string operatorName)
+find_similar(const std::string &operatorName)
 {
   std::string found_similar_operators = "";
   unsigned long lines = 1;
   unsigned long line_length = 105;
+
   if (operatorName != "")
     {
-      // Searching for simlar operator names in operator to module map
-      for (auto str : modules_map)
+      // Searching for similar operator names in operator to module map
+      for (const auto &str : modules_map)
         {
-          if (similar(string2lower(operatorName), str.first))
+          if (similar(stringToLower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -146,9 +176,9 @@ find_similar(std::string operatorName)
             }
         }
       // Searching for similar operator names in aliases to original map
-      for (auto str : aliases)
+      for (const auto &str : aliases)
         {
-          if (similar(string2lower(operatorName), str.first))
+          if (similar(stringToLower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -160,6 +190,7 @@ find_similar(std::string operatorName)
             }
         }
     }
+
   return found_similar_operators;
 }
 
@@ -178,15 +209,16 @@ find_similar(std::string operatorName)
  *  \note If \p operatorName is a file name the program will exit.
  */
 static bool
-check_operator(std::string operatorName)
+check_operator(const std::string &operatorName)
 {
   if (operator_name_exists(operatorName))
     {
       return true;
     }
   else if (operatorName == "")
-    Error("Operator name missing!");
-
+    {
+      cdoAbort("Operator name missing!");
+    }
   else
     {
       // Checking if the operatorname is an existing file name
@@ -194,8 +226,8 @@ check_operator(std::string operatorName)
       if (fp)
         {
           fclose(fp);
-          fprintf(stderr, "Use commandline option -h for help.");
-          Error("Operator missing, %s is a file on disk!", operatorName.c_str());
+          fprintf(stderr, "Use commandline option -h for help.\n");
+          cdoAbort("Operator missing, %s is a file on disk!", operatorName.c_str());
         }
       // Operator is no filename
       // Checking for similar operators
@@ -211,11 +243,32 @@ check_operator(std::string operatorName)
         {
           fprintf(stderr, "(not found)\n");
         }
+
       exit(EXIT_FAILURE);
     }
+
   return false;
 }
 
+void
+registerOperatorsFromModules()
+{
+  for (auto &x : modules)
+    {
+      for (auto y : x.second.operators)
+        {
+          if (!operator_name_exists(y))
+            {
+              modules_map[y] = x.first;
+            }
+          else
+            {
+              cdoAbort("Tried to add operator but the operator name already exists");
+            }
+        }
+    }
+}
+
 /***
  * Adds a module and its operators to cdo.
  * Adds the module to modules
@@ -224,12 +277,12 @@ check_operator(std::string operatorName)
  * @note: if an error happens while adding the new module cdo will exit.
  */
 void
-add_module(std::string module_name, module_t new_module)
+add_module(const std::string &module_name, const module_t new_module)
 {
   if (modules.find(module_name) == modules.end())
     {
       modules[module_name] = new_module;
-      for (std::string operatorName : new_module.operators)
+      for (const std::string &operatorName : new_module.operators)
         {
           // if the operator name is not already in the map or in the aliases
           if (!operator_name_exists(operatorName))
@@ -238,50 +291,49 @@ add_module(std::string module_name, module_t new_module)
             }
           else
             {
-              Error("Tried to add operator but the operator name already exists");
+              cdoAbort("Tried to add operator but the operator name already exists");
             }
         }
     }
   else
     {
-      Error("Module %s name already exists", module_name.c_str());
+      cdoAbort("Module %s name already exists", module_name.c_str());
     }
 }
 
 /**
- * adds an key value pair to #modules_map with alias as key and originals name
- * as value
+ * adds an key value pair to #modules_map with alias as key and originals name as value
  * @param alias new alias to be added
  * @param original original operator name
  */
 int
-add_alias(std::string alias, std::string original)
+add_alias(const std::string &alias, const std::string &original)
 {
   auto iter_original = modules_map.find(original);
   auto iter_alias = aliases.find(alias);
 
   if (iter_alias != aliases.end())
     {
-      Warning("alias %s could not be added: it already exists", alias.c_str());
+      printf("WARNING: alias %s could not be added: it already exists", alias.c_str());
       return -1;
     }
 
   if (iter_original == modules_map.end())
     {
-      Error("alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
+      printf("ERROR: alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
       return -2;
     }
+
   if (modules_map.find(alias) != modules_map.end())
     {
-      Error("alias %s could not be added: alias name already exists as an "
-            "operator",
-            alias.c_str());
+      printf("ERROR alias %s could not be added: alias name already exists as an operator", alias.c_str());
+      return -3;
     }
+
   aliases[alias] = original;
 
   return 0;
 }
-/* clang-format on */
 
 /***
  * returns the module of given operator
@@ -292,7 +344,7 @@ add_alias(std::string alias, std::string original)
  *      std::string -> name of the operators module
  */
 static std::string
-get_module_name_to(std::string operatorName)
+get_module_name_to(const std::string &operatorName)
 {
   // if not true the programm will exit in function check_operator
   if (check_operator(operatorName))
@@ -310,7 +362,7 @@ get_module_name_to(std::string operatorName)
     {
       // TODO: see if depricated since no operator can be added without a
       // module
-      Error("Module for >%s< not installed", operatorName.c_str());
+      cdoAbort("Module for >%s< not installed", operatorName.c_str());
     }
   // Only to quell the warning. Function should never reach this.
   return "";
@@ -323,11 +375,13 @@ get_module_name_to(std::string operatorName)
  * @param operatorName name of the operator
  * @returns the function of the #module_t of \a operatorName
  */
+/* not used
 void *(*operatorModule(const char *operatorName))(void *)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].func;
 }
+*/
 
 /***
  * returns help for given operator name
@@ -336,10 +390,9 @@ void *(*operatorModule(const char *operatorName))(void *)
  * @return vector of strings containing the help
  */
 std::vector<std::string>
-operatorHelp(std::string operatorName)
+operatorHelp(const std::string &operatorName)
 {
-  std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].help;
+  return modules[get_module_name_to(operatorName)].help;
 }
 
 /***
@@ -348,13 +401,14 @@ operatorHelp(std::string operatorName)
  * @param operatorName operator name
  * @retval short
  */
+/* not used
 int
 operatorStreamInCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].streamInCnt;
 }
-
+*/
 /***
  * Returns the number of output streams a given operator has.
  * returns -1 if obase is used
@@ -362,13 +416,14 @@ operatorStreamInCnt(const char *operatorName)
  * @return 1 for CDI_REAL, 2 for CDI_COMP (complex), 3 for CDI_REAL and CDI_COMP
  * @retval short
  */
+/* not used
 int
 operatorStreamOutCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].streamOutCnt;
 }
-
+*/
 /***
  * Returns the number type this operator works with
  * @param operatorName operator name
@@ -382,8 +437,7 @@ operatorStreamNumber(const char *operatorName)
 }
 
 /***
- * Creates a sorted vector with all operator names and alisases excluding all
- * modules that are marked as internal
+ * Creates a sorted vector with all operator names and alisases excluding all modules that are marked as internal
  * @return a sorted std::vector containing all operator names and aliases
  * excluding all operators which modules are marked as internal
  */
@@ -392,7 +446,7 @@ get_sorted_operator_name_list()
 {
 
   std::vector<std::string> names;
-  for (std::pair<std::string, std::string> operator_module_names_pair : modules_map)
+  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
     {
       if (modules[operator_module_names_pair.second].mode == 1)
         {
@@ -400,11 +454,12 @@ get_sorted_operator_name_list()
         }
     }
   // adding operators names from alias_map
-  for (std::pair<std::string, std::string> alias : aliases)
+  for (const std::pair<std::string, std::string> &alias : aliases)
     {
       names.push_back(alias.first);
     }
   std::sort(names.begin(), names.end());
+
   return names;
 }
 
@@ -412,7 +467,7 @@ std::vector<std::string>
 get_no_output_operator_list()
 {
   std::vector<std::string> names;
-  for (std::pair<std::string, std::string> operator_module_names_pair : modules_map)
+  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
     {
       if (modules[operator_module_names_pair.second].mode == 1 && modules[operator_module_names_pair.second].streamOutCnt == 0)
         {
@@ -421,7 +476,7 @@ get_no_output_operator_list()
     }
   // adding operators names from alias_map
   std::string original;
-  for (std::pair<std::string, std::string> alias : aliases)
+  for (const std::pair<std::string, std::string> &alias : aliases)
     {
       original = alias.second;
       if (modules[modules_map[original]].mode == 1 && modules[modules_map[original]].streamOutCnt == 0)
@@ -430,6 +485,7 @@ get_no_output_operator_list()
         }
     }
   std::sort(names.begin(), names.end());
+
   return names;
 }
 
@@ -443,7 +499,7 @@ operatorPrintAll(void)
   std::vector<std::string> sorted_operator_names = get_sorted_operator_name_list();
 
   std::cout << tab;
-  for (auto operatorName : sorted_operator_names)
+  for (const auto &operatorName : sorted_operator_names)
     {
       if (number_of_chars > 85)
         {
@@ -463,17 +519,16 @@ operatorPrintAll(void)
   loads all custom modules in a specified folder
   @param folder_path custom modules folder
 */
-#ifdef CUSTOM_MODULES
 void
 load_custom_modules(std::string folder_path)
 {
   DIR *dir = opendir(folder_path.c_str());
   std::string file_path;
   std::regex library_regex("(.*\\.so)");
-  if (dir != NULL)
+  if (dir != nullptr)
     {
       struct dirent *ent = readdir(dir);
-      while (ent != NULL)
+      while (ent != nullptr)
         {
           if (std::regex_match(ent->d_name, library_regex))
             {
@@ -521,7 +576,7 @@ load_custom_module(std::string file_path)
   init_custom_module();
   std::cout << "loaded custom module from '" << file_path << "'" << std::endl;
 }
-#endif
+
 /***
   closes the handles for the loaded custum modules
 */
@@ -536,18 +591,16 @@ close_library_handles()
 #endif
 
 // helper function for setting the spacing in operatorPrintList
-std::string
-get_spacing_for(int p_space, std::string str)
+static std::string
+get_spacing_for(int p_space, const std::string &str)
 {
   std::string spacing = "";
-  for (int i = str.size(); i <= p_space; i++)
-    {
-      spacing += " ";
-    }
+  for (int i = str.size(); i <= p_space; i++) spacing += " ";
   return spacing;
 }
-std::string
-get_operator_description(std::string p_current_op_name, std::vector<std::string> help)
+
+static std::string
+get_operator_description(const std::string &p_current_op_name, const std::vector<std::string> &help)
 {
   std::string description = "";
   unsigned long cur_help_idx;
@@ -590,7 +643,6 @@ get_operator_description(std::string p_current_op_name, std::vector<std::string>
     }
   else
     {
-
       line = help.at(++operator_section);
       // search the operator section for current operator line
       while (line.find(p_current_op_name + " ") == std::string::npos && !line.empty() && operator_section < help.size() - 1)
@@ -598,15 +650,17 @@ get_operator_description(std::string p_current_op_name, std::vector<std::string>
           line = help.at(++operator_section);
         }
       // if operator line found save description for later use
-      if (!line.empty() && line.find(p_current_op_name + " ") != std::string::npos)
+      if (!line.empty() && line.find("    " + p_current_op_name + " ") != std::string::npos)
         {
           auto op_name_start = line.find_first_not_of(" \t");
 
           description = line.substr(line.find_first_not_of(" \t", op_name_start + p_current_op_name.size()), line.size());
         }
     }
+
   return description;
 }
+
 /***
  * Prints all operator names and their short descriptions
  * Aliases are listed and point to their original operator name.
@@ -627,18 +681,17 @@ operatorPrintList(bool print_no_output)
       output_list = get_sorted_operator_name_list();
     }
 
-  unsigned long list_length = output_list.size();
+  auto list_length = output_list.size();
   module_t *current_module;
 
   // help variables
 
-  for (unsigned long out_list_idx = 0; out_list_idx < list_length; out_list_idx++)
+  for (size_t out_list_idx = 0; out_list_idx < list_length; out_list_idx++)
     {
       std::string current_op_name = output_list[out_list_idx];
       current_module = &modules[get_module_name_to(current_op_name)];
       if (aliases.find(current_op_name) != aliases.end())
         {
-
           output_list[out_list_idx] += std::string(get_spacing_for(16, current_op_name) + "--> " + aliases[current_op_name]);
         }
       else if (!current_module->help.empty())
@@ -652,13 +705,13 @@ operatorPrintList(bool print_no_output)
       output_list[out_list_idx] += get_spacing_for(90, output_list[out_list_idx]) + in_out_info;
     }
   // print generated output list
-  for (std::string str : output_list)
+  for (const std::string &str : output_list)
     {
       std::cout << str << std::endl;
     }
 }
 
-bool
+static bool
 is_alias(const char *operatorName)
 {
   return (aliases.find(std::string(operatorName)) != aliases.end());
@@ -667,18 +720,15 @@ is_alias(const char *operatorName)
 const char *
 get_original(const char *operatorName)
 {
-  char *original = NULL;
-  if (is_alias(operatorName))
+  const char *name = getOperatorName(operatorName);
+  if (is_alias(name))
     {
-      std::string opName = aliases[std::string(operatorName)];
-      original = (char *) malloc(opName.size());
+      std::string opName = aliases[std::string(name)];
+      char *original = (char *) malloc(opName.size() + 1);
       strcpy(original, opName.c_str());
+      return original;
     }
-  else
-    {
-      return operatorName;
-    }
-  return original;
+  return name;
 }
 
 module_t &
