@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,15 +25,17 @@
 #include <pthread.h>
 #endif
 
-typedef struct
+#include <cstddef>
+
+struct read_arg_t
 {
-  int streamID;
+  CdoStreamID streamID;
   int *varID, *levelID;
   size_t *nmiss;
   double *array;
-} read_arg_t;
+};
 
-typedef struct
+struct par_io_t
 {
   int varID, levelID;
   size_t nmiss;
@@ -45,8 +47,8 @@ typedef struct
   pthread_t thrID;
   pthread_attr_t attr;
 #endif
-} par_io_t;
+};
 
-void parReadRecord(int streamID, int *varID, int *levelID, double *array, size_t *nmiss, par_io_t *parIO);
+void parReadRecord(CdoStreamID streamID, int *varID, int *levelID, double *array, size_t *nmiss, par_io_t *parIO);
 
 #endif /* PAR_IO_H */
