@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,13 +14,15 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include <math.h>
+
+#include <cmath>
 #include <string.h>
-#include "util.h"
+
+#include "compare.h"
 #include "percentiles.h"
 #include "nth_element.h"
 #include "util_string.h"
-#include "exception.h"
+#include "cdo_output.h"
 
 enum percentile_methods
 {
@@ -28,6 +30,7 @@ enum percentile_methods
   NIST,
   NUMPY
 };
+
 enum interpolation_methods
 {
   LINEAR = 1,
@@ -42,7 +45,7 @@ static int interpolation_method = LINEAR;
 static double
 percentile_nrank(double *array, size_t len, double pn)
 {
-  size_t irank = (size_t) ceil(len * (pn / 100.0));
+  size_t irank = (size_t) std::ceil(len * (pn / 100.0));
   if (irank < 1) irank = 1;
   if (irank > len) irank = len;
   return nth_element(array, len, irank - 1);
@@ -51,9 +54,9 @@ percentile_nrank(double *array, size_t len, double pn)
 static double
 percentile_nist(double *array, size_t len, double pn)
 {
-  double rank = (len + 1) * (pn / 100.0);
-  size_t k = (size_t) rank;
-  double d = rank - k;
+  const double rank = (len + 1) * (pn / 100.0);
+  const size_t k = (size_t) rank;
+  const double d = rank - k;
   double percentil = 0;
   if (k == 0)
     percentil = nth_element(array, len, 0);
@@ -61,8 +64,8 @@ percentile_nist(double *array, size_t len, double pn)
     percentil = nth_element(array, len, len - 1);
   else
     {
-      double vk1 = nth_element(array, len, k);
-      double vk = nth_element(array, len, k - 1);
+      const double vk1 = nth_element(array, len, k);
+      const double vk = nth_element(array, len, k - 1);
       percentil = vk + d * (vk1 - vk);
     }
 
@@ -72,9 +75,9 @@ percentile_nist(double *array, size_t len, double pn)
 static double
 percentile_numpy(double *array, size_t len, double pn)
 {
-  double rank = (len - 1) * (pn / 100.0) + 1;
-  size_t k = (size_t) rank;
-  double d = rank - k;
+  const double rank = (len - 1) * (pn / 100.0) + 1;
+  const size_t k = (size_t) rank;
+  const double d = rank - k;
   double percentil = 0;
   if (k == 0)
     percentil = nth_element(array, len, 0);
@@ -84,19 +87,18 @@ percentile_numpy(double *array, size_t len, double pn)
     {
       if (interpolation_method == LINEAR)
         {
-          double vk1 = nth_element(array, len, k);
-          double vk = nth_element(array, len, k - 1);
+          const auto vk1 = nth_element(array, len, k);
+          const auto vk = nth_element(array, len, k - 1);
           percentil = vk + d * (vk1 - vk);
         }
       else
         {
           size_t irank = 0;
-          if (interpolation_method == LOWER)
-            irank = (size_t) floor(rank);
-          else if (interpolation_method == HIGHER)
-            irank = (size_t) ceil(rank);
-          else if (interpolation_method == NEAREST)
-            irank = (size_t) lround(rank);
+          // clang-format off
+          if      (interpolation_method == LOWER)   irank = (size_t) std::floor(rank);
+          else if (interpolation_method == HIGHER)  irank = (size_t) std::ceil(rank);
+          else if (interpolation_method == NEAREST) irank = (size_t) std::lround(rank);
+          // clang-format on
           // numpy is using around(), with rounds to the nearest even value
 
           if (irank < 1) irank = 1;
@@ -114,61 +116,54 @@ percentile(double *array, size_t len, double pn)
 {
   double percentil = 0;
 
-  if (percentile_method == NRANK)
-    percentil = percentile_nrank(array, len, pn);
-  else if (percentile_method == NIST)
-    percentil = percentile_nist(array, len, pn);
-  else if (percentile_method == NUMPY)
-    percentil = percentile_numpy(array, len, pn);
-  else
-    cdoAbort("Internal error: percentile method %d not implemented!", percentile_method);
+  // clang-format off
+  if      (percentile_method == NRANK)  percentil = percentile_nrank(array, len, pn);
+  else if (percentile_method == NIST)   percentil = percentile_nist(array, len, pn);
+  else if (percentile_method == NUMPY)  percentil = percentile_numpy(array, len, pn);
+  else cdoAbort("Internal error: percentile method %d not implemented!", percentile_method);
+  // clang-format on
 
   return percentil;
 }
 
 void
-percentile_set_method(const char *methodstr)
+percentile_set_method(const std::string &methodstr)
 {
-  char *methodname = strdup(methodstr);
-  strtolower(methodname);
+  auto methodname = stringToLower(methodstr);
 
-  if (strcmp("nrank", methodname) == 0)
-    percentile_method = NRANK;
-  else if (strcmp("nist", methodname) == 0)
-    percentile_method = NIST;
-  else if (strcmp("numpy", methodname) == 0)
-    percentile_method = NUMPY;
-  else if (strcmp("numpy_linear", methodname) == 0)
+  // clang-format off
+  if      ("nrank" == methodname) percentile_method = NRANK;
+  else if ("nist"  == methodname) percentile_method = NIST;
+  else if ("numpy" == methodname) percentile_method = NUMPY;
+  else if ("numpy_linear" == methodname)
     {
       percentile_method = NUMPY;
       interpolation_method = LINEAR;
     }
-  else if (strcmp("numpy_lower", methodname) == 0)
+  else if ("numpy_lower" == methodname)
     {
       percentile_method = NUMPY;
       interpolation_method = LOWER;
     }
-  else if (strcmp("numpy_higher", methodname) == 0)
+  else if ("numpy_higher" == methodname)
     {
       percentile_method = NUMPY;
       interpolation_method = HIGHER;
     }
-  else if (strcmp("numpy_nearest", methodname) == 0)
+  else if ("numpy_nearest" == methodname)
     {
       percentile_method = NUMPY;
       interpolation_method = NEAREST;
     }
   else
-    cdoAbort("Percentile method %s not available!", methodstr);
+    cdoAbort("Percentile method %s not available!", methodstr.c_str());
+  // clang-format on
 }
 
 void
 percentile_check_number(double pn)
 {
-  if (pn < 0 || pn > 100)
-    cdoAbort("Percentile number %g out of range! Percentiles must be in the "
-             "range [0,100].",
-             pn);
+  if (pn < 0 || pn > 100) cdoAbort("Percentile number %g out of range! Percentiles must be in the range [0,100].", pn);
 }
 
 /*
@@ -199,6 +194,5 @@ done
 import numpy as np
 np.version.version
 a=np.array([15, 20, 35, 40, 50, 55])
-for p in [30, 40, 50, 75, 100] : print np.percentile(a, p,
-interpolation='linear')
+for p in [30, 40, 50, 75, 100] : print np.percentile(a, p, interpolation='linear')
 */
