@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 #ifndef PROCESS_H
 #define PROCESS_H
 
-#include "pstream.h"
+#include "cdoStream.h"
 #include "modules.h"
 
 #include <vector>
@@ -27,17 +27,16 @@
 #include <set>
 
 constexpr int MAX_PROCESS = 128;
-constexpr int MAX_STREAM = 64;
 constexpr int MAX_OPERATOR = 128;
-constexpr int MAX_OARGC = 4096;
-constexpr int MAX_FILES = 65536;
 
-struct cdoTimes
+enum class ProcessStatus
 {
-  double s_utime, s_stime;
-  double e_utime, e_stime;
-  double c_cputime = 0, c_usertime = 0, c_systime = 0;
-  double p_cputime = 0, p_usertime = 0, p_systime = 0;
+  Ok = 0,
+  UnlimitedIOCounts = -1,
+  MissInput = -2,
+  MissOutput = -3,
+  TooManyStreams = -4,
+  TooFewStreams = -5,
 };
 
 struct oper_t
@@ -48,32 +47,25 @@ struct oper_t
   const char *enter;
 };
 
-class ProcessType
+class Process
 {
 public:
   int m_ID;
   int m_posInParent;
 #ifdef HAVE_LIBPTHREAD
   pthread_t threadID;
-  int l_threadID;
 #endif
   short nchild;
-  std::vector<ProcessType *> childProcesses;
-  std::vector<ProcessType *> parentProcesses;
-  std::vector<PstreamType *> inputStreams;
-  std::vector<PstreamType *> outputStreams;
+  std::vector<Process *> childProcesses;
+  std::vector<Process *> parentProcesses;
+  std::vector<CdoStreamID> inputStreams;
+  std::vector<CdoStreamID> outputStreams;
   int nChildActive = 0;
 
-  double s_utime;
-  double s_stime;
-  double a_utime;
-  double a_stime;
-  double cputime;
+  double startTime;
 
-  size_t m_nvals;
   short nvars;
 
-  std::set<int> timesteps;
   int ntimesteps;
   int m_streamCnt;
   char const *m_operatorCommand;
@@ -81,16 +73,15 @@ public:
   char const *operatorArg;
   char prompt[64];
   short m_noper;
-  bool m_isActive; /*TEMP*/  // not used right now, maybe later (12.Jan.2018)
+  bool m_isActive;
+  const char *m_obase;
 
   module_t m_module;
-  std::vector<char *> m_oargv;
-  /*TEMP*/  // remove m_oargc and replace usage with
-            // m_oargv.size()/operatorArgc()
+  std::vector<std::string> m_oargv;
   int m_oargc;
   oper_t oper[MAX_OPERATOR];
 
-  ProcessType(int p_ID, const char *p_operatorNamme, const char *operatorCommand);
+  Process(int p_ID, const char *p_operatorNamme, const char *operatorCommand);
 
   pthread_t run();
 
@@ -102,6 +93,11 @@ public:
    * returns the number of out streams this process currently has.
    */
   int getOutStreamCnt();
+
+  /**
+   * returns full command through which this process was created including operator name and operator arguments
+   */
+  const char *getCommand();
   /**
    * Splits the operator arguments stores them.
    * Operator arguments are stored as char arrays and appended with
@@ -111,11 +107,11 @@ public:
   /**
    * Adds a Process as child and creates and adds a new pipe stream.
    */
-  void addChild(ProcessType *child_process);
+  void addChild(Process *child_process);
   /**
    * Adds a Process as parent and adds the parents input stream to out streams.
    */
-  void addParent(ProcessType *parent_process);
+  void addParent(Process *parent_process);
   /**
    * Compares the wanted and current stream counts.
    * @return if the wanted count is -1 this function always returns false.
@@ -138,8 +134,6 @@ public:
    * Adds and creates a new file pstream to the out streams
    */
   void addPipeOutStream();
-  void addNvals(size_t p_nvals);
-  void query_user_exit(const char *argument);
   void inqUserInputForOpArg(const char *enter);
   /**
    * Adds an operator to the process
@@ -150,16 +144,28 @@ public:
    */
   int getOperatorID();
   void setInactive();
-  const char *inqPrompt();
-  cdoTimes getTimes(int p_processNums);
+  const char *inqPrompt() const;
   void printProcessedValues();
-  void printBenchmarks(cdoTimes p_times, char *p_memstring);
-  int checkStreamCnt();
+  void printBenchmarks(double p_walltime, const char *p_memstring);
+  ProcessStatus checkStreamCnt();
+  void validate();
+  void handleProcessErr(ProcessStatus p_proErr);
+
+  bool hasOutStream(const CdoStreamID p_streamPtr);
+  bool hasInStream(const CdoStreamID p_streamPtr);
+  bool hasNoPipes();
+  size_t inqNvals();
+  const char *getOutStreamName();
+  size_t getOperArgc();
+  std::string getArgv(int idx);
+  std::string getObase();
 
 private:
   void initProcess();
-  ProcessType();
+  Process();
   void defPrompt();
 };
 
+void setProcessNum(int p_num);
+
 #endif /* PROCESS_H */
