@@ -15,7 +15,9 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+
+#include "dmemory.h"
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
@@ -137,26 +139,26 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
     }
   else
     {
-      x1 = rns * TWO * sin(pi4 - HALF * beglat) * cos(beglon);
-      y1 = TWO * sin(pi4 - HALF * beglat) * sin(beglon);
+      x1 = rns * TWO * std::sin(pi4 - HALF * beglat) * std::cos(beglon);
+      y1 = TWO * std::sin(pi4 - HALF * beglat) * std::sin(beglon);
       *luse_last = true;
     }
 
-  x2 = rns * TWO * sin(pi4 - HALF * endlat) * cos(endlon);
-  y2 = TWO * sin(pi4 - HALF * endlat) * sin(endlon);
+  x2 = rns * TWO * std::sin(pi4 - HALF * endlat) * std::cos(endlon);
+  y2 = TWO * std::sin(pi4 - HALF * endlat) * std::sin(endlon);
 
   for (n = 0; n < srch_corners * num_srch_cells; ++n)
     {
-      srch_corner_x[n] = rns * TWO * sin(pi4 - HALF * srch_corner_lat[n]) * cos(srch_corner_lon[n]);
-      srch_corner_y[n] = TWO * sin(pi4 - HALF * srch_corner_lat[n]) * sin(srch_corner_lon[n]);
+      srch_corner_x[n] = rns * TWO * std::sin(pi4 - HALF * srch_corner_lat[n]) * std::cos(srch_corner_lon[n]);
+      srch_corner_y[n] = TWO * std::sin(pi4 - HALF * srch_corner_lat[n]) * std::sin(srch_corner_lon[n]);
     }
 
   begx = x1;
   begy = y1;
   endx = x2;
   endy = y2;
-  begsegx = rns * TWO * sin(pi4 - HALF * begseg[0]) * cos(begseg[1]);
-  begsegy = TWO * sin(pi4 - HALF * begseg[0]) * sin(begseg[1]);
+  begsegx = rns * TWO * std::sin(pi4 - HALF * begseg[0]) * std::cos(begseg[1]);
+  begsegy = TWO * std::sin(pi4 - HALF * begseg[0]) * std::sin(begseg[1]);
   *intrsct_x = endx;
   *intrsct_y = endy;
 
@@ -392,9 +394,9 @@ after_srch_loop:
                   if (*intrsct_lon < ZERO) *intrsct_lon = *intrsct_lon + PI2;
 
                   if (std::fabs(*intrsct_x) > 1.e-10)
-                    *intrsct_lat = (pi4 - asin(rns * HALF * (*intrsct_x) / cos(*intrsct_lon))) * TWO;
+                    *intrsct_lat = (pi4 - std::asin(rns * HALF * (*intrsct_x) / std::cos(*intrsct_lon))) * TWO;
                   else if (std::fabs(*intrsct_y) > 1.e-10)
-                    *intrsct_lat = (pi4 - asin(HALF * (*intrsct_y) / sin(*intrsct_lon))) * TWO;
+                    *intrsct_lat = (pi4 - std::asin(HALF * (*intrsct_y) / std::sin(*intrsct_lon))) * TWO;
                   else
                     *intrsct_lat = TWO * pi4;
 
@@ -979,10 +981,10 @@ line_integral(double *weights, double in_phi1, double in_phi2, double theta1, do
 
   /*  Weights for the general case based on a trapezoidal approx to the integrals. */
 
-  sinth1 = sin(theta1);
-  sinth2 = sin(theta2);
-  costh1 = cos(theta1);
-  costh2 = cos(theta2);
+  sinth1 = std::sin(theta1);
+  sinth2 = std::sin(theta2);
+  costh1 = std::cos(theta1);
+  costh2 = std::cos(theta2);
 
   dphi = in_phi1 - in_phi2;
   if (dphi > PI)
@@ -1148,21 +1150,21 @@ normalize_weights(RemapGrid *tgt_grid, RemapVars &rv, double *src_centroid_lat,
 {
   /* Include centroids in weights and normalize using destination area if requested */
   long num_links = rv.num_links;
-  long src_cell_add; /* current linear address for source grid cell   */
-  long tgt_cell_add; /* current linear address for target grid cell   */
+  // src_cell_add: current linear address for source grid cell
+  // tgt_cell_add: current linear address for target grid cell
   double *weights = &rv.wts[0];
   double norm_factor = 0; /* factor for normalizing wts */
 
   if (rv.normOpt == NormOpt::DESTAREA)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, \
-                                              src_centroid_lon) private(src_cell_add, tgt_cell_add, norm_factor)
+#pragma omp parallel for default(none) \
+    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(norm_factor)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
-          tgt_cell_add = rv.tgt_cell_add[n];
+          const auto src_cell_add = rv.src_cell_add[n];
+          const auto tgt_cell_add = rv.tgt_cell_add[n];
           norm_factor = IS_NOT_EQUAL(tgt_grid->cell_area[tgt_cell_add], 0) ? ONE / tgt_grid->cell_area[tgt_cell_add] : ZERO;
           norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
         }
@@ -1170,13 +1172,13 @@ normalize_weights(RemapGrid *tgt_grid, RemapVars &rv, double *src_centroid_lat,
   else if (rv.normOpt == NormOpt::FRACAREA)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, \
-                                              src_centroid_lon) private(src_cell_add, tgt_cell_add, norm_factor)
+#pragma omp parallel for default(none) \
+    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(norm_factor)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
-          tgt_cell_add = rv.tgt_cell_add[n];
+          const auto src_cell_add = rv.src_cell_add[n];
+          const auto tgt_cell_add = rv.tgt_cell_add[n];
           norm_factor = IS_NOT_EQUAL(tgt_grid->cell_frac[tgt_cell_add], 0) ? ONE / tgt_grid->cell_frac[tgt_cell_add] : ZERO;
           norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
         }
@@ -1184,17 +1186,39 @@ normalize_weights(RemapGrid *tgt_grid, RemapVars &rv, double *src_centroid_lat,
   else if (rv.normOpt == NormOpt::NONE)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, \
-                                              src_centroid_lon) private(src_cell_add, norm_factor)
+#pragma omp parallel for default(none) \
+    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(norm_factor)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
+          const auto src_cell_add = rv.src_cell_add[n];
           norm_factor = ONE;
           norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
         }
     }
 }
+static
+void srvout(const char *fname, int nlon, int nlat, double *data)
+{
+  int ihead[8];
+
+  ihead[0] = 99;
+  ihead[1] = 0;
+  ihead[2] = 0;
+  ihead[3] = 0;
+  ihead[4] = nlon;
+  ihead[5] = nlat;
+  ihead[6] = 0;
+  ihead[7] = 0;
+
+  FILE *fp = fopen(fname, "w");
+  fprintf(fp, "%d %d %d %d %d %d %d %d\n", ihead[0], ihead[1], ihead[2], ihead[3], ihead[4], ihead[5], ihead[6], ihead[7]);
+
+  for (int i = 0; i < nlon*nlat; ++i ) fprintf(fp, "%g\n", data[i]);
+  
+  fclose(fp);
+}
+
 
 /*
   -----------------------------------------------------------------------
@@ -1282,14 +1306,14 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   srch_corners = tgt_num_cell_corners;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, num_wts, src_centroid_lon, src_centroid_lat, grid_store, rv, \
-                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, max_srch_cells,     \
-                                              src_num_cell_corners, srch_corners, src_grid, tgt_grid, tgt_grid_size,        \
-                                              src_grid_size, srch_add)
+#pragma omp parallel for default(none)                                                                                    \
+    shared(findex, rsearch, num_wts, src_centroid_lon, src_centroid_lat, grid_store, rv, Options::cdoVerbose, max_subseg, \
+           srch_corner_lat, srch_corner_lon, max_srch_cells, src_num_cell_corners, srch_corners, src_grid, tgt_grid,      \
+           tgt_grid_size, src_grid_size, srch_add)
 #endif
   for (long src_cell_add = 0; src_cell_add < src_grid_size; ++src_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
@@ -1478,14 +1502,14 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, num_wts, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv, \
-                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, max_srch_cells,     \
-                                              tgt_num_cell_corners, srch_corners, src_grid, tgt_grid, tgt_grid_size,        \
-                                              src_grid_size, srch_add)
+#pragma omp parallel for default(none)                                                                                    \
+    shared(findex, rsearch, num_wts, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv, Options::cdoVerbose, max_subseg, \
+           srch_corner_lat, srch_corner_lon, max_srch_cells, tgt_num_cell_corners, srch_corners, src_grid, tgt_grid,      \
+           tgt_grid_size, src_grid_size, srch_add)
 #endif
   for (long tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
@@ -1676,12 +1700,18 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   /* Finish centroid computation */
 
+  // int nlon = src_grid->dims[0];
+  // int nlat = src_grid->dims[1];
+  //srvout("clat1.srv", nlon, nlat, src_centroid_lat);
+  //srvout("clon1.srv", nlon, nlat, src_centroid_lon);
   for (long n = 0; n < src_grid_size; ++n)
     if (IS_NOT_EQUAL(src_grid->cell_area[n], 0))
       {
         src_centroid_lat[n] /= src_grid->cell_area[n];
         src_centroid_lon[n] /= src_grid->cell_area[n];
       }
+  //srvout("clat2.srv", nlon, nlat, src_centroid_lat);
+  //srvout("clon2.srv", nlon, nlat, src_centroid_lon);
 
   for (long n = 0; n < tgt_grid_size; ++n)
     if (IS_NOT_EQUAL(tgt_grid->cell_area[n], 0))
