@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,11 +15,14 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include "remap.h"
 #include "remap_store_link.h"
-#include "cdoOptions.h"
+#include "cdo_options.h"
+#include "progress.h"
+#include "cimdOmp.h"
 
 // Interpolation using a distance-weighted average
 
@@ -27,16 +30,16 @@
 void
 remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
 {
-  RemapGrid *src_grid = rsearch.srcGrid;
-  RemapGrid *tgt_grid = rsearch.tgtGrid;
+  auto src_grid = rsearch.srcGrid;
+  auto tgt_grid = rsearch.tgtGrid;
 
-  if (cdoVerbose) cdoPrint("Called %s()", __func__);
+  if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  progressInit();
+  Progress::init();
 
   // Compute mappings from source to target grid
 
-  size_t tgt_grid_size = tgt_grid->size;
+  auto tgt_grid_size = tgt_grid->size;
 
   std::vector<WeightLinks> weightLinks(tgt_grid_size);
   weightLinksAlloc(numNeighbors, tgt_grid_size, weightLinks);
@@ -44,24 +47,25 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  double start = cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   // Loop over destination grid
 
   double findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, weightLinks, numNeighbors, src_grid, tgt_grid, tgt_grid_size, knnWeights)
+#pragma omp parallel for default(none) \
+    shared(findex, rsearch, weightLinks, numNeighbors, src_grid, tgt_grid, tgt_grid_size, knnWeights)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progressStatus(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
 
       weightLinks[tgt_cell_add].nlinks = 0;
 
@@ -74,7 +78,7 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
       remapSearchPoints(rsearch, plon, plat, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_grid->mask.data());
+      const size_t nadds = knnWeights[ompthID].computeWeights(src_grid->mask);
 
       for (size_t n = 0; n < nadds; ++n)
         if (knnWeights[ompthID].m_mask[n]) tgt_grid->cell_frac[tgt_cell_add] = 1.0;
@@ -83,55 +87,63 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
       storeWeightLinks(0, nadds, knnWeights[ompthID].m_addr.data(), knnWeights[ompthID].m_dist.data(), tgt_cell_add, weightLinks);
     }
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
   if (rsearch.gps) gridPointSearchDelete(rsearch.gps);
-  rsearch.gps = NULL;
+  rsearch.gps = nullptr;
 
   weightLinksToRemapLinks(0, tgt_grid_size, weightLinks, rv);
 
   if (numNeighbors == 1) rv.links_per_value = numNeighbors;
 
-  if (cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // remapDistwgtWeights
 
 void
 remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict src_array, double *restrict tgt_array,
              double missval)
 {
-  RemapGrid *src_grid = rsearch.srcGrid;
-  RemapGrid *tgt_grid = rsearch.tgtGrid;
+  auto src_grid = rsearch.srcGrid;
+  auto tgt_grid = rsearch.tgtGrid;
 
-  if (cdoVerbose) cdoPrint("Called %s()", __func__);
+  if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  progressInit();
+  Progress::init();
 
   // Compute mappings from source to target grid
 
-  size_t tgt_grid_size = tgt_grid->size;
+  auto tgt_grid_size = tgt_grid->size;
+  auto src_grid_size = src_grid->size;
+
+  std::vector<int> src_grid_mask(src_grid_size);
+#ifdef _OPENMP
+#pragma omp parallel for default(none) schedule(static) shared(src_grid_size, src_array, src_grid_mask, missval)
+#endif
+  for (size_t i = 0; i < src_grid_size; ++i)
+    src_grid_mask[i] = !DBL_IS_EQUAL(src_array[i], missval);
 
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  double start = cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   // Loop over destination grid
 
   double findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, \
-                                              tgt_array, missval, knnWeights)
+#pragma omp parallel for default(none) \
+    shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, tgt_array, missval, src_grid_mask, knnWeights)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progressStatus(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
 
       tgt_array[tgt_cell_add] = missval;
 
@@ -144,31 +156,31 @@ remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict s
       remapSearchPoints(rsearch, plon, plat, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_grid->mask.data());
+      const size_t nadds = knnWeights[ompthID].computeWeights(src_grid_mask);
       if (nadds) tgt_array[tgt_cell_add] = knnWeights[ompthID].arrayWeightsSum(src_array);
     }
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
-  if (cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // remapDistwgt
 
 void remapInit(remapType &remap);
 
 void
-intgriddis(Field *field1, Field *field2, size_t numNeighbors)
+intgriddis(Field &field1, Field &field2, size_t numNeighbors)
 {
-  RemapMethod mapType = RemapMethod::DISTWGT;
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
-  double src_missval = field1->missval;
-  double tgt_missval = field2->missval;
-  double *src_array = field1->ptr;
-  double *tgt_array = field2->ptr;
+  auto mapType = RemapMethod::DISTWGT;
+  auto gridID1 = field1.grid;
+  auto gridID2 = field2.grid;
+  auto src_missval = field1.missval;
+  auto tgt_missval = field2.missval;
+  auto src_array = field1.vec.data();
+  auto tgt_array = field2.vec.data();
 
-  if (cdoVerbose) cdoPrint("Called %s()", __func__);
+  if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  progressInit();
+  Progress::init();
 
   // Interpolate from source to target grid
 
@@ -178,8 +190,8 @@ intgriddis(Field *field1, Field *field2, size_t numNeighbors)
   bool remap_extrapolate = false;
   remapInitGrids(mapType, remap_extrapolate, gridID1, remap.src_grid, gridID2, remap.tgt_grid);
 
-  size_t src_grid_size = remap.src_grid.size;
-  size_t tgt_grid_size = remap.tgt_grid.size;
+  auto src_grid_size = remap.src_grid.size;
+  auto tgt_grid_size = remap.tgt_grid.size;
 
   std::vector<int> src_mask(src_grid_size);
   for (size_t i = 0; i < src_grid_size; ++i) src_mask[i] = !DBL_IS_EQUAL(src_array[i], src_missval);
@@ -191,7 +203,7 @@ intgriddis(Field *field1, Field *field2, size_t numNeighbors)
 
   remapSearchInit(mapType, remap.search, remap.src_grid, remap.tgt_grid);
 
-  double start = cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   // Loop over destination grid
 
@@ -200,10 +212,10 @@ intgriddis(Field *field1, Field *field2, size_t numNeighbors)
 
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
       findex++;
-      if (ompthID == 0) progressStatus(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
 
       tgt_array[tgt_cell_add] = tgt_missval;
 
@@ -216,20 +228,20 @@ intgriddis(Field *field1, Field *field2, size_t numNeighbors)
       remapSearchPoints(remap.search, plon, plat, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_mask.data());
+      const auto nadds = knnWeights[ompthID].computeWeights(src_mask);
       if (nadds)
         tgt_array[tgt_cell_add] = knnWeights[ompthID].arrayWeightsSum(src_array);
       else
         nmiss++;
     }
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
-  field2->nmiss = nmiss;
+  field2.nmiss = nmiss;
 
   remapGridFree(remap.src_grid);
   remapGridFree(remap.tgt_grid);
   remapSearchFree(remap.search);
 
-  if (cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // intgriddis
