@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,8 +26,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "grid.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "griddes.h"
+#include <mpim_grid.h>
 #include "remap.h"
 #include "commandline.h"
 
@@ -80,40 +82,38 @@ void
 remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType submapType, int numNeighbors, int remapOrder,
                     RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
 {
-// Writes remap data to a NetCDF file using SCRIP conventions
+  // Writes remap data to a NetCDF file using SCRIP conventions
 
 #ifdef HAVE_LIBNETCDF
 
-  // Local variables
-
-  int nc_file_id;           /* id for NetCDF file                       */
-  int nc_srcgrdsize_id;     /* id for source grid size                  */
-  int nc_dstgrdsize_id;     /* id for destination grid size             */
-  int nc_srcgrdcorn_id = 0; /* id for number of source grid corners     */
-  int nc_dstgrdcorn_id = 0; /* id for number of dest grid corners       */
-  int nc_srcgrdrank_id;     /* id for source grid rank                  */
-  int nc_dstgrdrank_id;     /* id for dest grid rank                    */
-  int nc_numlinks_id;       /* id for number of links in mapping        */
-  int nc_numwgts_id;        /* id for number of weights for mapping     */
-  int nc_srcgrddims_id;     /* id for source grid dimensions            */
-  int nc_dstgrddims_id;     /* id for dest grid dimensions              */
-  int nc_srcgrdcntrlat_id;  /* id for source grid center latitude       */
-  int nc_dstgrdcntrlat_id;  /* id for dest grid center latitude         */
-  int nc_srcgrdcntrlon_id;  /* id for source grid center longitude      */
-  int nc_dstgrdcntrlon_id;  /* id for dest grid center longitude        */
-  int nc_srcgrdimask_id;    /* id for source grid mask                  */
-  int nc_dstgrdimask_id;    /* id for dest grid mask                    */
-  int nc_srcgrdcrnrlat_id;  /* id for latitude of source grid corners   */
-  int nc_srcgrdcrnrlon_id;  /* id for longitude of source grid corners  */
-  int nc_dstgrdcrnrlat_id;  /* id for latitude of dest grid corners     */
-  int nc_dstgrdcrnrlon_id;  /* id for longitude of dest grid corners    */
-  int nc_srcgrdarea_id;     /* id for area of source grid cells         */
-  int nc_dstgrdarea_id;     /* id for area of dest grid cells           */
-  int nc_srcgrdfrac_id;     /* id for area fraction on source grid      */
-  int nc_dstgrdfrac_id;     /* id for area fraction on dest grid        */
-  int nc_srcadd_id;         /* id for map source address                */
-  int nc_dstadd_id;         /* id for map destination address           */
-  int nc_rmpmatrix_id;      /* id for remapping matrix                  */
+  int nc_file_id = -1;          /* id for NetCDF file                       */
+  int nc_srcgrdsize_id = -1;    /* id for source grid size                  */
+  int nc_dstgrdsize_id = -1;    /* id for destination grid size             */
+  int nc_srcgrdcorn_id = 0;     /* id for number of source grid corners     */
+  int nc_dstgrdcorn_id = 0;     /* id for number of dest grid corners       */
+  int nc_srcgrdrank_id = -1;    /* id for source grid rank                  */
+  int nc_dstgrdrank_id = -1;    /* id for dest grid rank                    */
+  int nc_numlinks_id = -1;      /* id for number of links in mapping        */
+  int nc_numwgts_id = -1;       /* id for number of weights for mapping     */
+  int nc_srcgrddims_id = -1;    /* id for source grid dimensions            */
+  int nc_dstgrddims_id = -1;    /* id for dest grid dimensions              */
+  int nc_srcgrdcntrlat_id = -1; /* id for source grid center latitude       */
+  int nc_dstgrdcntrlat_id = -1; /* id for dest grid center latitude         */
+  int nc_srcgrdcntrlon_id = -1; /* id for source grid center longitude      */
+  int nc_dstgrdcntrlon_id = -1; /* id for dest grid center longitude        */
+  int nc_srcgrdimask_id = -1;   /* id for source grid mask                  */
+  int nc_dstgrdimask_id = -1;   /* id for dest grid mask                    */
+  int nc_srcgrdcrnrlat_id = -1; /* id for latitude of source grid corners   */
+  int nc_srcgrdcrnrlon_id = -1; /* id for longitude of source grid corners  */
+  int nc_dstgrdcrnrlat_id = -1; /* id for latitude of dest grid corners     */
+  int nc_dstgrdcrnrlon_id = -1; /* id for longitude of dest grid corners    */
+  int nc_srcgrdarea_id = -1;    /* id for area of source grid cells         */
+  int nc_dstgrdarea_id = -1;    /* id for area of dest grid cells           */
+  int nc_srcgrdfrac_id = -1;    /* id for area fraction on source grid      */
+  int nc_dstgrdfrac_id = -1;    /* id for area fraction on dest grid        */
+  int nc_srcadd_id = -1;        /* id for map source address                */
+  int nc_dstadd_id = -1;        /* id for map destination address           */
+  int nc_rmpmatrix_id = -1;     /* id for remapping matrix                  */
 
   int nc_dims2_id[2]; /* NetCDF ids for 2d array dims             */
 
@@ -138,7 +138,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
   switch (mapType)
     {
-    case RemapMethod::CONSERV:
+    case RemapMethod::CONSERV_SCRIP:
       lgridarea = true;
       if (submapType == SubmapType::LAF)
         {
@@ -150,7 +150,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
           strcpy(map_method, "Conservative remapping");
           break;
         }
-    case RemapMethod::CONSERV_YAC:
+    case RemapMethod::CONSERV:
       lgridarea = true;
       /*
       if ( submapType == SubmapType::LAF )
@@ -187,7 +187,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
     if (tgt_grid.lneed_cell_corners) nele2 += tgt_grid.num_cell_corners * 2 * 8;
     size_t filesize = src_grid.size * nele1 + tgt_grid.size * nele2 + nlinks * (4 + 4 + rv.num_wts * 8);
 
-    if (cdoVerbose)
+    if (Options::cdoVerbose)
       {
         cdoPrint("Number of remap links:       %zu", nlinks);
         cdoPrint("Filesize for remap weights: ~%zu", filesize);
@@ -200,23 +200,21 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
         if (nlinks > maxlinks || filesize > 8 * maxlinks || gridsize_max > 0x7FFFFC00)
           {
 #ifdef HAVE_NETCDF4
-            if (cdoVerbose) cdoPrint("Store weights and links to NetCDF4!");
+            if (Options::cdoVerbose) cdoPrint("Store weights and links to NetCDF4!");
             writemode |= NC_NETCDF4;
             if (gridsize_max > 0x7FFFFC00)
               sizetype = NC_UINT64;
             else
               writemode |= NC_CLASSIC_MODEL;
 #else
-            cdoPrint("Number of remap links %zu exceeds maximum of %zu and "
-                     "NetCDF 4 is not available!",
-                     nlinks, maxlinks);
+            cdoPrint("Number of remap links %zu exceeds maximum of %zu and NetCDF 4 is not available!", nlinks, maxlinks);
 #endif
           }
         else
           {
 #if defined(NC_64BIT_OFFSET)
             writemode |= NC_64BIT_OFFSET;
-            if (cdoVerbose) cdoPrint("Store weights and links to NetCDF2!");
+            if (Options::cdoVerbose) cdoPrint("Store weights and links to NetCDF2!");
 #else
             cdoPrint("Filesize for remap weights maybe too large!");
 #endif
@@ -237,7 +235,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "map_method", strlen(map_method), map_method));
 
   // Remap order
-  if (mapType == RemapMethod::CONSERV && submapType == SubmapType::NONE)
+  if (mapType == RemapMethod::CONSERV_SCRIP && submapType == SubmapType::NONE)
     nce(nc_put_att_int(nc_file_id, NC_GLOBAL, "remap_order", NC_INT, 1L, &remapOrder));
 
   // File convention
@@ -262,7 +260,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
       nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "history", strlen(history), history));
     }
 
-  if (CDO_Version_Info) nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "CDO", (int) strlen(cdoComment()) + 1, cdoComment()));
+  if (Options::VersionInfo) nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "CDO", (int) strlen(cdoComment()) + 1, cdoComment()));
 
   // Prepare NetCDF dimension info
 
@@ -271,10 +269,8 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nce(nc_def_dim(nc_file_id, "dst_grid_size", tgt_grid.size, &nc_dstgrdsize_id));
 
   // Define grid corner dimension
-  if (src_grid.lneed_cell_corners)
-    nce(nc_def_dim(nc_file_id, "src_grid_corners", src_grid.num_cell_corners, &nc_srcgrdcorn_id));
-  if (tgt_grid.lneed_cell_corners)
-    nce(nc_def_dim(nc_file_id, "dst_grid_corners", tgt_grid.num_cell_corners, &nc_dstgrdcorn_id));
+  if (src_grid.lneed_cell_corners) nce(nc_def_dim(nc_file_id, "src_grid_corners", src_grid.num_cell_corners, &nc_srcgrdcorn_id));
+  if (tgt_grid.lneed_cell_corners) nce(nc_def_dim(nc_file_id, "dst_grid_corners", tgt_grid.num_cell_corners, &nc_dstgrdcorn_id));
 
   // Define grid rank dimension
   nce(nc_def_dim(nc_file_id, "src_grid_rank", src_grid.rank, &nc_srcgrdrank_id));
@@ -441,8 +437,8 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 /*****************************************************************************/
 
 #ifdef HAVE_LIBNETCDF
-static RemapMethod
-getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int *remapOrder)
+RemapMethod
+getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remapOrder)
 {
   // Map method
   size_t attlen;
@@ -452,19 +448,19 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int *remap
   map_method[attlen] = 0;
 
   *submapType = SubmapType::NONE;
-  *remapOrder = 1;
+  remapOrder = 1;
 
   RemapMethod mapType = RemapMethod::UNDEF;
   if (cmpstr(map_method, "Conservative") == 0)
     {
       if (cmpstr(map_method, "Conservative remapping using clipping on sphere") == 0)
-        mapType = RemapMethod::CONSERV_YAC;
-      else
         mapType = RemapMethod::CONSERV;
+      else
+        mapType = RemapMethod::CONSERV_SCRIP;
 
       int iatt;
-      int status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
-      if (status == NC_NOERR) *remapOrder = iatt;
+      const int status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
+      if (status == NC_NOERR) remapOrder = iatt;
     }
   else if (cmpstr(map_method, "Bilinear") == 0)
     mapType = RemapMethod::BILINEAR;
@@ -482,7 +478,7 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int *remap
     }
   else if (cmpstr(map_method, "Largest") == 0)
     {
-      mapType = RemapMethod::CONSERV;
+      mapType = RemapMethod::CONSERV_SCRIP;
       *submapType = SubmapType::LAF;
     }
   else
@@ -491,7 +487,7 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int *remap
       cdoAbort("Invalid Map Type");
     }
 
-  if (cdoVerbose) cdoPrint("mapType = %s", map_method);
+  if (Options::cdoVerbose) cdoPrint("mapType = %s", map_method);
 
   return mapType;
 }
@@ -499,49 +495,47 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int *remap
 
 void
 remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMethod *mapType, SubmapType *submapType,
-                   int *numNeighbors, int *remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
+                   int *numNeighbors, int &remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
 {
-// The routine reads a NetCDF file to extract remapping info in SCRIP format
+  // The routine reads a NetCDF file to extract remapping info in SCRIP format
 
 #ifdef HAVE_LIBNETCDF
 
-  // Local variables
-
   bool lgridarea = false;
   int status;
-  int nc_srcgrdsize_id;    /* id for source grid size                  */
-  int nc_dstgrdsize_id;    /* id for destination grid size             */
-  int nc_srcgrdcorn_id;    /* id for number of source grid corners     */
-  int nc_dstgrdcorn_id;    /* id for number of dest grid corners       */
-  int nc_srcgrdrank_id;    /* id for source grid rank                  */
-  int nc_dstgrdrank_id;    /* id for dest grid rank                    */
-  int nc_numlinks_id;      /* id for number of links in mapping        */
-  int nc_numwgts_id;       /* id for number of weights for mapping     */
-  int nc_srcgrddims_id;    /* id for source grid dimensions            */
-  int nc_dstgrddims_id;    /* id for dest grid dimensions              */
-  int nc_srcgrdcntrlat_id; /* id for source grid center latitude       */
-  int nc_dstgrdcntrlat_id; /* id for dest grid center latitude         */
-  int nc_srcgrdcntrlon_id; /* id for source grid center longitude      */
-  int nc_dstgrdcntrlon_id; /* id for dest grid center longitude        */
-  int nc_srcgrdimask_id;   /* id for source grid mask                  */
-  int nc_dstgrdimask_id;   /* id for dest grid mask                    */
-  int nc_srcgrdcrnrlat_id; /* id for latitude of source grid corners   */
-  int nc_srcgrdcrnrlon_id; /* id for longitude of source grid corners  */
-  int nc_dstgrdcrnrlat_id; /* id for latitude of dest grid corners     */
-  int nc_dstgrdcrnrlon_id; /* id for longitude of dest grid corners    */
-  int nc_srcgrdarea_id;    /* id for area of source grid cells         */
-  int nc_dstgrdarea_id;    /* id for area of dest grid cells           */
-  int nc_srcgrdfrac_id;    /* id for area fraction on source grid      */
-  int nc_dstgrdfrac_id;    /* id for area fraction on dest grid        */
-  int nc_srcadd_id;        /* id for map source address                */
-  int nc_dstadd_id;        /* id for map destination address           */
-  int nc_rmpmatrix_id;     /* id for remapping matrix                  */
+  int nc_srcgrdsize_id;         /* id for source grid size                  */
+  int nc_dstgrdsize_id;         /* id for destination grid size             */
+  int nc_srcgrdcorn_id;         /* id for number of source grid corners     */
+  int nc_dstgrdcorn_id;         /* id for number of dest grid corners       */
+  int nc_srcgrdrank_id;         /* id for source grid rank                  */
+  int nc_dstgrdrank_id;         /* id for dest grid rank                    */
+  int nc_numlinks_id;           /* id for number of links in mapping        */
+  int nc_numwgts_id;            /* id for number of weights for mapping     */
+  int nc_srcgrddims_id;         /* id for source grid dimensions            */
+  int nc_dstgrddims_id;         /* id for dest grid dimensions              */
+  int nc_srcgrdcntrlat_id;      /* id for source grid center latitude       */
+  int nc_dstgrdcntrlat_id;      /* id for dest grid center latitude         */
+  int nc_srcgrdcntrlon_id;      /* id for source grid center longitude      */
+  int nc_dstgrdcntrlon_id;      /* id for dest grid center longitude        */
+  int nc_srcgrdimask_id;        /* id for source grid mask                  */
+  int nc_dstgrdimask_id;        /* id for dest grid mask                    */
+  int nc_srcgrdcrnrlat_id = -1; /* id for latitude of source grid corners   */
+  int nc_srcgrdcrnrlon_id = -1; /* id for longitude of source grid corners  */
+  int nc_dstgrdcrnrlat_id = -1; /* id for latitude of dest grid corners     */
+  int nc_dstgrdcrnrlon_id = -1; /* id for longitude of dest grid corners    */
+  int nc_srcgrdarea_id;         /* id for area of source grid cells         */
+  int nc_dstgrdarea_id;         /* id for area of dest grid cells           */
+  int nc_srcgrdfrac_id;         /* id for area fraction on source grid      */
+  int nc_dstgrdfrac_id;         /* id for area fraction on dest grid        */
+  int nc_srcadd_id;             /* id for map source address                */
+  int nc_dstadd_id;             /* id for map destination address           */
+  int nc_rmpmatrix_id;          /* id for remapping matrix                  */
 
   char map_name[1024];
-  char normalize_opt[64]; /* character string for normalization option */
-  char convention[64];    /* character string for output convention    */
-  char src_grid_name[64]; /* grid name for source grid                 */
-  char tgt_grid_name[64]; /* grid name for dest   grid                 */
+  char normalize_opt[64];       /* character string for normalization option */
+  char convention[64];          /* character string for output convention    */
+  char src_grid_name[64];       /* grid name for source grid                 */
+  char tgt_grid_name[64];       /* grid name for dest   grid                 */
   char src_grid_units[64];
   char tgt_grid_units[64];
   size_t attlen, dimlen;
@@ -550,8 +544,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   // Open file and read some global information
 
-  // nce(nc_open(interp_file, NC_NOWRITE, &nc_file_id));
-  int nc_file_id = cdf_openread(interp_file);
+  int nc_file_id = cdo_cdf_openread(interp_file);
 
   // Map name
 
@@ -559,7 +552,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "title", &attlen));
   map_name[attlen] = 0;
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     {
       cdoPrint("Reading remapping: %s", map_name);
       cdoPrint("From file: %s", interp_file);
@@ -568,9 +561,9 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   // Map Tyoe
   *mapType = getMapType(nc_file_id, submapType, numNeighbors, remapOrder);
 
-  if (*mapType == RemapMethod::CONSERV) lgridarea = true;
+  if (*mapType == RemapMethod::CONSERV_SCRIP) lgridarea = true;
 
-  remapVarsInit(*mapType, rv);
+  remapVarsInit(*mapType, remapOrder, rv);
 
   rv.mapType = *mapType;
   rv.links_per_value = -1;
@@ -581,29 +574,28 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "normalization", &attlen));
   normalize_opt[attlen] = 0;
 
-  if (strcmp(normalize_opt, "none") == 0)
-    rv.normOpt = NormOpt::NONE;
-  else if (strcmp(normalize_opt, "fracarea") == 0)
-    rv.normOpt = NormOpt::FRACAREA;
-  else if (strcmp(normalize_opt, "destarea") == 0)
-    rv.normOpt = NormOpt::DESTAREA;
+  // clang-format off
+  if      (cstrIsEqual(normalize_opt, "none"))     rv.normOpt = NormOpt::NONE;
+  else if (cstrIsEqual(normalize_opt, "fracarea")) rv.normOpt = NormOpt::FRACAREA;
+  else if (cstrIsEqual(normalize_opt, "destarea")) rv.normOpt = NormOpt::DESTAREA;
   else
     {
       cdoPrint("normalize_opt = %s", normalize_opt);
       cdoAbort("Invalid normalization option");
     }
+  // clang-format on
 
-  if (cdoVerbose) cdoPrint("normalize_opt = %s", normalize_opt);
+  if (Options::cdoVerbose) cdoPrint("normalize_opt = %s", normalize_opt);
 
   // File convention
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "conventions", convention));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "conventions", &attlen));
   convention[attlen] = 0;
 
-  if (strcmp(convention, "SCRIP") != 0)
+  if (!cstrIsEqual(convention, "SCRIP"))
     {
       cdoPrint("convention = %s", convention);
-      if (strcmp(convention, "NCAR-CSM") == 0)
+      if (cstrIsEqual(convention, "NCAR-CSM"))
         cdoAbort("Unsupported file convention!");
       else
         cdoAbort("Unknown file convention!");
@@ -621,19 +613,18 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "dest_grid", &attlen));
   tgt_grid_name[attlen] = 0;
 
-  if (cdoVerbose) cdoPrint("Remapping between: %s and %s", src_grid_name, tgt_grid_name);
+  if (Options::cdoVerbose) cdoPrint("Remapping between: %s and %s", src_grid_name, tgt_grid_name);
 
   // Initialize remapgrid structure
   remapGridInit(src_grid);
   remapGridInit(tgt_grid);
 
   // Read dimension information
-
   nce(nc_inq_dimid(nc_file_id, "src_grid_size", &nc_srcgrdsize_id));
   nce(nc_inq_dimlen(nc_file_id, nc_srcgrdsize_id, &dimlen));
   src_grid.size = dimlen;
   /*
-  if (  src_grid.size != gridInqSize(gridID1) )
+  if ( src_grid.size != gridInqSize(gridID1) )
     cdoAbort("Source grids have different size!");
   */
   nce(nc_inq_dimid(nc_file_id, "dst_grid_size", &nc_dstgrdsize_id));
@@ -709,30 +700,28 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   nce(nc_inq_varid(nc_file_id, "src_grid_imask", &nc_srcgrdimask_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_center_lat", &nc_srcgrdcntrlat_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_center_lon", &nc_srcgrdcntrlon_id));
+
   if (src_grid.num_cell_corners)
     {
       nce(nc_inq_varid(nc_file_id, "src_grid_corner_lat", &nc_srcgrdcrnrlat_id));
       nce(nc_inq_varid(nc_file_id, "src_grid_corner_lon", &nc_srcgrdcrnrlon_id));
     }
-  if (lgridarea)
-    {
-      nce(nc_inq_varid(nc_file_id, "src_grid_area", &nc_srcgrdarea_id));
-    }
+
+  if (lgridarea) nce(nc_inq_varid(nc_file_id, "src_grid_area", &nc_srcgrdarea_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_frac", &nc_srcgrdfrac_id));
 
   nce(nc_inq_varid(nc_file_id, "dst_grid_dims", &nc_dstgrddims_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_imask", &nc_dstgrdimask_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_center_lat", &nc_dstgrdcntrlat_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_center_lon", &nc_dstgrdcntrlon_id));
+
   if (tgt_grid.num_cell_corners)
     {
       nce(nc_inq_varid(nc_file_id, "dst_grid_corner_lat", &nc_dstgrdcrnrlat_id));
       nce(nc_inq_varid(nc_file_id, "dst_grid_corner_lon", &nc_dstgrdcrnrlon_id));
     }
-  if (lgridarea)
-    {
-      nce(nc_inq_varid(nc_file_id, "dst_grid_area", &nc_dstgrdarea_id));
-    }
+
+  if (lgridarea) nce(nc_inq_varid(nc_file_id, "dst_grid_area", &nc_dstgrdarea_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_frac", &nc_dstgrdfrac_id));
 
   nce(nc_inq_varid(nc_file_id, "src_address", &nc_srcadd_id));
@@ -767,10 +756,8 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
       nce(nc_inq_attlen(nc_file_id, nc_srcgrdcrnrlat_id, "units", &attlen));
       src_grid_units[attlen] = 0;
 
-      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lon,
-                     "source grid corner lon");
-      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lat,
-                     "source grid corner lat");
+      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lon, "source grid corner lon");
+      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lat, "source grid corner lat");
     }
 
   if (lgridarea) nce(nc_get_var_double(nc_file_id, nc_srcgrdarea_id, &src_grid.cell_area[0]));
@@ -802,10 +789,8 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
       nce(nc_inq_attlen(nc_file_id, nc_dstgrdcrnrlat_id, "units", &attlen));
       tgt_grid_units[attlen] = 0;
 
-      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lon,
-                     "target grid corner lon");
-      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lat,
-                     "target grid corner lat");
+      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lon, "target grid corner lon");
+      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lat, "target grid corner lat");
     }
 
   if (lgridarea) nce(nc_get_var_double(nc_file_id, nc_dstgrdarea_id, &tgt_grid.cell_area[0]));
@@ -827,8 +812,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
     }
 
   // Close input file
-
-  nce(nc_close(nc_file_id));
+  cdo_cdf_close(nc_file_id);
 
 #else
   cdoAbort("NetCDF support not compiled in!");
