@@ -16,52 +16,56 @@
 */
 
 #include <cassert>
+#include <cmath>
 
-#include <cdo_int.h>
+
+#include "dmemory.h"
 #include "param_conversion.h"
 #include "sellist.h"
 #include "util_wildcards.h"
+#include "cdo_output.h"
 
 // #define SELDEBUG 1
 
-static int
-get_ival(const char *intstr, int idefault, int istart, int iend, int *ilast)
+static inline void
+parameterError(const char *name, const char *string, const char *endptr)
 {
-  int i;
-  int ival = idefault;
-
-  for (i = istart; i < iend; i++)
-    {
-      if (!(isdigit(intstr[i]) || intstr[i] == '-'))
-        {
-          if (intstr[i] == '/')
-            ival = parameter2intlist(intstr + i + 1);
-          else
-            fprintf(stderr, "Syntax error in >%.*s<! Character %c not allowed.\n", iend, intstr, intstr[i]);
-          break;
-        }
-    }
-
-  *ilast = i;
-
-  return ival;
+  cdoAbort("%s parameter >%s< contains invalid character at position %d!", name, string, (int) (endptr - string + 1));
 }
 
 void
-split_intstring(const char *intstr, int *first, int *last, int *inc)
+split_intstring(const std::string &intstr, int &first, int &last, int &inc)
 {
-  int istrlen = strlen(intstr);
-  *first = parameter2intlist(intstr);
-  *last = *first;
-  *inc = 1;
-
-  int i, start = 1;
-  *last = get_ival(intstr, *first, start, istrlen, &i);
-
-  if (i < istrlen)
+    split_intstring(intstr.c_str(), first, last, inc);
+}
+void
+split_intstring(const char *intstr, int &first, int &last, int &inc)
+{
+  auto startptr = intstr;
+  char *endptr = nullptr;
+  int ival = (int) strtol(startptr, &endptr, 10);
+  if (*endptr != 0 && *endptr != '/' && (endptr - startptr) == 0) parameterError("Integer", startptr, endptr);
+  first = ival;
+  last = ival;
+  inc = 1;
+  
+  if (*endptr == '/')
     {
-      start = i + 1;
-      *inc = get_ival(intstr, 1, start, istrlen, &i);
+      startptr = endptr + 1;
+      endptr = nullptr;
+      ival = (int) strtol(startptr, &endptr, 10);
+      if (*endptr != 0 && *endptr != '/' && (endptr - startptr) == 0) parameterError("Integer", startptr, endptr);
+      last = ival;
+      if (first > last) inc = -1;
+
+      if (*endptr == '/')
+        {
+          startptr = endptr + 1;
+          endptr = nullptr;
+          ival = (int) strtol(startptr, &endptr, 10);
+          if (*endptr != 0 && (endptr - startptr) == 0) parameterError("Integer", startptr, endptr);
+          inc = ival;
+        }
     }
 }
 
@@ -77,7 +81,7 @@ sellist_init(SelectList &sellist, KVList &kvlist)
       e.key = kv.key;
       e.nvalues = kv.nvalues;
       e.values.resize(kv.nvalues);
-      for (int i = 0; i < kv.nvalues; ++i) e.values[i] = kv.values[i];
+      for (int k = 0; k < kv.nvalues; ++k) e.values[k] = kv.values[k];
 #ifdef SELDEBUG
       printf("%s =", e.key.c_str());
       for (int ii = 0; ii < e.nvalues; ++ii) printf(" '%s'", e.values[ii].c_str());
@@ -86,7 +90,7 @@ sellist_init(SelectList &sellist, KVList &kvlist)
       ++i;
     }
 
-  for (int i = 0; i < (int)sellist.size(); ++i)
+  for (int i = 0; i < (int) sellist.size(); ++i)
     {
       SelectEntry &e = sellist[i];
       e.flag = nullptr;
@@ -102,7 +106,7 @@ sellist_init(SelectList &sellist, KVList &kvlist)
 void
 sellist_destroy(SelectList &sellist)
 {
-  for (int i = 0; i < (int)sellist.size(); ++i)
+  for (int i = 0; i < (int) sellist.size(); ++i)
     {
       SelectEntry &e = sellist[i];
       if (e.txt) Free(e.txt);
@@ -114,7 +118,7 @@ sellist_destroy(SelectList &sellist)
 void
 sellist_verify(SelectList &sellist)
 {
-  for (int i = 0; i < (int)sellist.size(); ++i)
+  for (int i = 0; i < (int) sellist.size(); ++i)
     {
       const SelectEntry &e = sellist[i];
       if (e.type == 0) cdoAbort("Unsupported selection keyword: '%s'!", e.key.c_str());
@@ -126,17 +130,17 @@ sellist_add(SelectList &sellist, const char *txt, const char *name, int type)
 {
   int idx = -1;
 
-  for (int i = 0; i < (int)sellist.size(); ++i)
+  for (int i = 0; i < (int) sellist.size(); ++i)
     {
-      const char *key = sellist[i].key.c_str();
-      if (cstrIsEqual(key, name))
+      const auto &key = sellist[i].key;
+      if (key == name)
         {
           idx = i;
           break;
         }
     }
 
-  if (idx >= 0 && idx < (int)sellist.size())
+  if (idx >= 0 && idx < (int) sellist.size())
     {
       SelectEntry &e = sellist[idx];
       e.type = type;
@@ -153,13 +157,12 @@ sellist_add(SelectList &sellist, const char *txt, const char *name, int type)
 
       int j = 0;
       const int nvalues = e.nvalues;
-      for (int i = 0; i < nvalues; ++i)
-        switch (type)
+      for (int i = 0; i < nvalues; ++i) switch (type)
           {
           case SELLIST_INT:
             {
               int first, last, inc;
-              split_intstring(e.values[i].c_str(), &first, &last, &inc);
+              split_intstring(e.values[i].c_str(), first, last, inc);
 
               if (first == last)
                 {
@@ -198,8 +201,7 @@ sellist_add(SelectList &sellist, const char *txt, const char *name, int type)
       if (e.nvalues) e.flag = (bool *) Calloc(e.nvalues, sizeof(bool));
 #ifdef SELDEBUG
       printf("%s =", e.key.c_str());
-      for (int i = 0; i < e.nvalues; ++i)
-        switch (type)
+      for (int i = 0; i < e.nvalues; ++i) switch (type)
           {
           case SELLIST_INT: printf(" %d", ((int *) e.cvalues)[i]); break;
           case SELLIST_FLT: printf(" %g", ((double *) e.cvalues)[i]); break;
@@ -215,21 +217,20 @@ sellist_add(SelectList &sellist, const char *txt, const char *name, int type)
 int
 sellist_nvalues(const SelectList &sellist, const int idx)
 {
-  return (idx >= 0 && idx < (int)sellist.size()) ? sellist[idx].nvalues : 0;
+  return (idx >= 0 && idx < (int) sellist.size()) ? sellist[idx].nvalues : 0;
 }
 
 void
 sellist_check_flag(const SelectList &sellist, const int idx)
 {
-  if (idx < 0 || idx >= (int)sellist.size()) return;
+  if (idx < 0 || idx >= (int) sellist.size()) return;
 
   const int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
     {
       const SelectEntry &e = sellist[idx];
       for (int i = 0; i < nvalues; ++i)
-        if (!e.flag[i])
-          switch (e.type)
+        if (!e.flag[i]) switch (e.type)
             {
             case SELLIST_INT: cdoWarning("%s >%d< not found!", e.txt, ((int *) e.cvalues)[i]); break;
             case SELLIST_FLT: cdoWarning("%s >%g< not found!", e.txt, ((double *) e.cvalues)[i]); break;
@@ -243,7 +244,7 @@ sellist_check(SelectList &sellist, int idx, void *par)
 {
   bool found = false;
 
-  if (idx < 0 || idx >= (int)sellist.size()) return found;
+  if (idx < 0 || idx >= (int) sellist.size()) return found;
 
   int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -286,7 +287,7 @@ sellist_check_date(SelectList &sellist, int idx, const char *par)
 {
   bool found = false;
 
-  if (idx < 0 || idx >= (int)sellist.size()) return found;
+  if (idx < 0 || idx >= (int) sellist.size()) return found;
 
   int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -319,7 +320,7 @@ sellist_check_season(SelectList &sellist, int idx, int month)
   assert(month >= 1 && month <= 12);
   bool found = false;
 
-  if (idx < 0 || idx >= (int)sellist.size()) return found;
+  if (idx < 0 || idx >= (int) sellist.size()) return found;
 
   const int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -345,7 +346,7 @@ sellist_check_season(SelectList &sellist, int idx, int month)
 void
 sellist_def_flag(SelectList &sellist, int idx, int vindex, bool flag)
 {
-  if (idx < 0 || idx >= (int)sellist.size()) return;
+  if (idx < 0 || idx >= (int) sellist.size()) return;
 
   const int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -358,7 +359,7 @@ sellist_def_flag(SelectList &sellist, int idx, int vindex, bool flag)
 void
 sellist_get_val(const SelectList &sellist, int idx, int vindex, void *val)
 {
-  if (idx < 0 || idx >= (int)sellist.size()) return;
+  if (idx < 0 || idx >= (int) sellist.size()) return;
 
   const int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -379,7 +380,7 @@ sellist_get_val(const SelectList &sellist, int idx, int vindex, void *val)
 void
 sellist_def_val(SelectList &sellist, int idx, int vindex, void *val)
 {
-  if (idx < 0 || idx >= (int)sellist.size()) return;
+  if (idx < 0 || idx >= (int) sellist.size()) return;
 
   const int nvalues = sellist_nvalues(sellist, idx);
   if (nvalues)
@@ -415,7 +416,7 @@ sellist_print(const SelectList &sellist)
     {
       // printf("Parameter list: %s\n", sellist->
       printf("Num  Name             Type  Size  Entries\n");
-      for (int idx = 0; idx < (int)sellist.size(); ++idx)
+      for (int idx = 0; idx < (int) sellist.size(); ++idx)
         {
           const SelectEntry &e = sellist[idx];
           printf("%3d  %-16s %4d  %4d ", idx + 1, e.key.c_str(), e.type, e.nvalues);
